package Module::CoreList;
use strict;
use vars qw/$VERSION %released %patchlevel %version %families/;
$VERSION = '2.02';

=head1 NAME

Module::CoreList - what modules shipped with versions of perl

=head1 SYNOPSIS

 use Module::CoreList;

 print $Module::CoreList::version{5.00503}{CPAN}; # prints 1.48

 print Module::CoreList->first_release('File::Spec');       # prints 5.00503
 print Module::CoreList->first_release('File::Spec', 0.82); # prints 5.006001

 print join ", ", @{ $Module::CoreList::families{5.005} };
    # prints "5.005, 5.00503, 5.00504"

 print join " ", @{ $Module::CoreList::patchlevel{5.008001} };
    # prints "maint-5.8 21377"

=head1 DESCRIPTION

Module::CoreList contains the hash of hashes
%Module::CoreList::version, this is keyed on perl version as indicated
in $].  The second level hash is module => version pairs.

Note, it is possible for the version of a module to be unspecified,
whereby the value is undef, so use C<exists $version{$foo}{$bar}> if
that's what you're testing for.

It also contains %Module::CoreList::released hash, which has ISO
formatted versions of the release dates, as gleaned from L<perlhist>.

New, in 1.96 is also the %Module::CoreList::families hash, which
clusters known perl releases by their major versions.

In 2.01 %Module::CoreList::patchlevel contains the branch and patchlevel
corresponding to the specified perl version in the Perforce repository where
the perl sources are kept.

=head1 CAVEATS

Module::CoreList currently only covers the 5.000, 5.001, 5.002, 5.003_07,
5.004, 5.004_05, 5.005, 5.005_03, 5.005_04, 5.6.0, 5.6.1, 5.6.2, 5.7.3, 5.8.0,
5.8.1, 5.8.2, 5.8.3, 5.8.4, 5.8.5, 5.8.6, 5.8.7, 5.9.0, 5.9.1 and 5.9.2
releases of perl.

Producing this information can be rather time consuming so patches are
welcomed for other versions.

=head1 HISTORY

Moved to Changes file.

=head1 AUTHOR

Richard Clamp E<lt>richardc@unixbeard.netE<gt>

Currently maintained by the perl 5 porters E<lt>perl5-porters@perl.orgE<gt>.

=head1 COPYRIGHT

Copyright (C) 2002-2005 Richard Clamp.  All Rights Reserved.

This module is free software; you can redistribute it and/or modify it
under the same terms as Perl itself.

=head1 SEE ALSO

L<Module::Info>, L<perl>

=cut

my $dumpinc = 0;
sub import {
    my $self = shift;
    my $what = shift || '';
    if ($what eq 'dumpinc') {
        $dumpinc = 1;
    }
}

END {
    print "---INC---\n", join "\n" => keys %INC
      if $dumpinc;
}


sub first_release {
    my ($discard, $module, $version) = @_;

    my @perls = $version
        ? grep { exists $version{$_}{ $module } &&
                        $version{$_}{ $module } >= $version } keys %version
        : grep { exists $version{$_}{ $module }             } keys %version;

    return unless @perls;
    return (sort { $released{$a} cmp $released{$b} } @perls)[0];
}


# when things escaped
%released = (
    5.000    => '1994-10-17',
    5.001    => '1995-03-14',
    5.002    => '1996-02-96',
    5.00307  => '1996-10-10',
    5.004    => '1997-05-15',
    5.005    => '1998-07-22',
    5.00503  => '1999-03-28',
    5.00405  => '1999-04-29',
    5.006    => '2000-03-22',
    5.006001 => '2001-04-08',
    5.006002 => '2003-11-15',
    5.007003 => '2002-03-05',
    5.008    => '2002-07-19',
    5.008001 => '2003-09-25',
    5.008002 => '2003-11-05',
    5.009    => '2003-10-27',
    5.008003 => '2004-01-14',
    5.00504  => '2004-02-23',
    5.009001 => '2004-03-16',
    5.008004 => '2004-04-21',
    5.008005 => '2004-07-19',
    5.008006 => '2004-11-27',
    5.009002 => '2005-04-01',
    5.008007 => '2005-05-30',
   );

# perforce branches and patch levels
%patchlevel = (
    5.005    => [perl => 1647],
    5.00503  => ['maint-5.005' => 3198],
    5.00405  => ['maint-5.004' => 999],
    5.006    => [perl => 5899],
    5.006001 => ['maint-5.6' => 9654],
    5.006002 => ['maint-5.6' => 21727],
    5.007003 => [perl => 15039],
    5.008    => [perl => 17637],
    5.008001 => ['maint-5.8' => 21377],
    5.008002 => ['maint-5.8' => 21670],
    5.009    => [perl => 21539],
    5.008003 => ['maint-5.8' => 22151],
    5.00504  => ['maint-5.005' => 22270],
    5.009001 => [perl => 22506],
    5.008004 => ['maint-5.8' => 22729],
    5.008005 => ['maint-5.8' => 23139],
    5.008006 => ['maint-5.8' => 23552],
    5.009002 => [perl => 24131],
    5.008007 => ['maint-5.8' => 24641],
);
   
for my $version ( sort { $a <=> $b } keys %released ) {
    my $family = int ($version * 1000) / 1000;
    push @{ $families{ $family }} , $version;
}


%version = (
    5.000 => {
        'AnyDBM_File'           => undef,  # lib/AnyDBM_File.pm
        'AutoLoader'            => undef,  # lib/AutoLoader.pm
        'AutoSplit'             => undef,  # lib/AutoSplit.pm
        'Benchmark'             => undef,  # lib/Benchmark.pm
        'Carp'                  => undef,  # lib/Carp.pm
        'Cwd'                   => undef,  # lib/Cwd.pm
        'DB_File'               => undef,  # ext/DB_File/DB_File.pm
        'DynaLoader'            => undef,  # ext/DynaLoader/DynaLoader.pm
        'English'               => undef,  # lib/English.pm
        'Env'                   => undef,  # lib/Env.pm
        'Exporter'              => undef,  # lib/Exporter.pm
        'ExtUtils::MakeMaker'   => undef,  # lib/ExtUtils/MakeMaker.pm
        'Fcntl'                 => undef,  # ext/Fcntl/Fcntl.pm
        'File::Basename'        => undef,  # lib/File/Basename.pm
        'File::CheckTree'       => undef,  # lib/File/CheckTree.pm
        'File::Find'            => undef,  # lib/File/Find.pm
        'FileHandle'            => undef,  # lib/FileHandle.pm
        'GDBM_File'             => undef,  # ext/GDBM_File/GDBM_File.pm
        'Getopt::Long'          => undef,  # lib/Getopt/Long.pm
        'Getopt::Std'           => undef,  # lib/Getopt/Std.pm
        'I18N::Collate'         => undef,  # lib/I18N/Collate.pm
        'IPC::Open2'            => undef,  # lib/IPC/Open2.pm
        'IPC::Open3'            => undef,  # lib/IPC/Open3.pm
        'Math::BigFloat'        => undef,  # lib/Math/BigFloat.pm
        'Math::BigInt'          => undef,  # lib/Math/BigInt.pm
        'Math::Complex'         => undef,  # lib/Math/Complex.pm
        'NDBM_File'             => undef,  # ext/NDBM_File/NDBM_File.pm
        'Net::Ping'             => undef,  # lib/Net/Ping.pm
        'ODBM_File'             => undef,  # ext/ODBM_File/ODBM_File.pm
        'POSIX'                 => undef,  # ext/POSIX/POSIX.pm
        'SDBM_File'             => undef,  # ext/SDBM_File/SDBM_File.pm
        'Search::Dict'          => undef,  # lib/Search/Dict.pm
        'Shell'                 => undef,  # lib/Shell.pm
        'Socket'                => undef,  # ext/Socket/Socket.pm
        'Sys::Hostname'         => undef,  # lib/Sys/Hostname.pm
        'Sys::Syslog'           => undef,  # lib/Sys/Syslog.pm
        'Term::Cap'             => undef,  # lib/Term/Cap.pm
        'Term::Complete'        => undef,  # lib/Term/Complete.pm
        'Test::Harness'         => undef,  # lib/Test/Harness.pm
        'Text::Abbrev'          => undef,  # lib/Text/Abbrev.pm
        'Text::ParseWords'      => undef,  # lib/Text/ParseWords.pm
        'Text::Soundex'         => undef,  # lib/Text/Soundex.pm
        'Text::Tabs'            => undef,  # lib/Text/Tabs.pm
        'TieHash'               => undef,  # lib/TieHash.pm
        'Time::Local'           => undef,  # lib/Time/Local.pm
        'integer'               => undef,  # lib/integer.pm
        'less'                  => undef,  # lib/less.pm
        'sigtrap'               => undef,  # lib/sigtrap.pm
        'strict'                => undef,  # lib/strict.pm
        'subs'                  => undef,  # lib/subs.pm
    },
    5.001 => {
        'AnyDBM_File'           => undef,  # lib/AnyDBM_File.pm
        'AutoLoader'            => undef,  # lib/AutoLoader.pm
        'AutoSplit'             => undef,  # lib/AutoSplit.pm
        'Benchmark'             => undef,  # lib/Benchmark.pm
        'Carp'                  => undef,  # lib/Carp.pm
        'Cwd'                   => undef,  # lib/Cwd.pm
        'DB_File'               => undef,  # ext/DB_File/DB_File.pm
        'DynaLoader'            => undef,  # ext/DynaLoader/DynaLoader.pm
        'English'               => undef,  # lib/English.pm
        'Env'                   => undef,  # lib/Env.pm
        'Exporter'              => undef,  # lib/Exporter.pm
        'ExtUtils::Liblist'     => undef,  # lib/ExtUtils/Liblist.pm
        'ExtUtils::MakeMaker'   => undef,  # lib/ExtUtils/MakeMaker.pm
        'ExtUtils::Manifest'    => undef,  # lib/ExtUtils/Manifest.pm
        'ExtUtils::Mkbootstrap' => undef,  # lib/ExtUtils/Mkbootstrap.pm
        'Fcntl'                 => undef,  # ext/Fcntl/Fcntl.pm
        'File::Basename'        => undef,  # lib/File/Basename.pm
        'File::CheckTree'       => undef,  # lib/File/CheckTree.pm
        'File::Find'            => undef,  # lib/File/Find.pm
        'File::Path'            => undef,  # lib/File/Path.pm
        'FileHandle'            => undef,  # lib/FileHandle.pm
        'GDBM_File'             => undef,  # ext/GDBM_File/GDBM_File.pm
        'Getopt::Long'          => undef,  # lib/Getopt/Long.pm
        'Getopt::Std'           => undef,  # lib/Getopt/Std.pm
        'I18N::Collate'         => undef,  # lib/I18N/Collate.pm
        'IPC::Open2'            => undef,  # lib/IPC/Open2.pm
        'IPC::Open3'            => undef,  # lib/IPC/Open3.pm
        'Math::BigFloat'        => undef,  # lib/Math/BigFloat.pm
        'Math::BigInt'          => undef,  # lib/Math/BigInt.pm
        'Math::Complex'         => undef,  # lib/Math/Complex.pm
        'NDBM_File'             => undef,  # ext/NDBM_File/NDBM_File.pm
        'Net::Ping'             => undef,  # lib/Net/Ping.pm
        'ODBM_File'             => undef,  # ext/ODBM_File/ODBM_File.pm
        'POSIX'                 => undef,  # ext/POSIX/POSIX.pm
        'SDBM_File'             => undef,  # ext/SDBM_File/SDBM_File.pm
        'Search::Dict'          => undef,  # lib/Search/Dict.pm
        'Shell'                 => undef,  # lib/Shell.pm
        'Socket'                => undef,  # ext/Socket/Socket.pm
        'SubstrHash'            => undef,  # lib/SubstrHash.pm
        'Sys::Hostname'         => undef,  # lib/Sys/Hostname.pm
        'Sys::Syslog'           => undef,  # lib/Sys/Syslog.pm
        'Term::Cap'             => undef,  # lib/Term/Cap.pm
        'Term::Complete'        => undef,  # lib/Term/Complete.pm
        'Test::Harness'         => undef,  # lib/Test/Harness.pm
        'Text::Abbrev'          => undef,  # lib/Text/Abbrev.pm
        'Text::ParseWords'      => undef,  # lib/Text/ParseWords.pm
        'Text::Soundex'         => undef,  # lib/Text/Soundex.pm
        'Text::Tabs'            => undef,  # lib/Text/Tabs.pm
        'TieHash'               => undef,  # lib/TieHash.pm
        'Time::Local'           => undef,  # lib/Time/Local.pm
        'integer'               => undef,  # lib/integer.pm
        'less'                  => undef,  # lib/less.pm
        'lib'                   => undef,  # lib/lib.pm
        'sigtrap'               => undef,  # lib/sigtrap.pm
        'strict'                => undef,  # lib/strict.pm
        'subs'                  => undef,  # lib/subs.pm
    },
    5.002 => {
        'AnyDBM_File'           => undef,  # lib/AnyDBM_File.pm
        'AutoLoader'            => undef,  # lib/AutoLoader.pm
        'AutoSplit'             => undef,  # lib/AutoSplit.pm
        'Benchmark'             => undef,  # lib/Benchmark.pm
        'Carp'                  => undef,  # lib/Carp.pm
        'Cwd'                   => undef,  # lib/Cwd.pm
        'DB_File'               => '1.01',  # ext/DB_File/DB_File.pm
        'Devel::SelfStubber'    => '1.01',  # lib/Devel/SelfStubber.pm
        'DirHandle'             => undef,  # lib/DirHandle.pm
        'DynaLoader'            => '1.00',  # ext/DynaLoader/DynaLoader.pm
        'English'               => undef,  # lib/English.pm
        'Env'                   => undef,  # lib/Env.pm
        'Exporter'              => undef,  # lib/Exporter.pm
        'ExtUtils::Install'     => undef,  # lib/ExtUtils/Install.pm
        'ExtUtils::Liblist'     => undef,  # lib/ExtUtils/Liblist.pm
        'ExtUtils::MM_OS2'      => undef,  # lib/ExtUtils/MM_OS2.pm
        'ExtUtils::MM_Unix'     => undef,  # lib/ExtUtils/MM_Unix.pm
        'ExtUtils::MM_VMS'      => undef,  # lib/ExtUtils/MM_VMS.pm
        'ExtUtils::MakeMaker'   => '5.21',  # lib/ExtUtils/MakeMaker.pm
        'ExtUtils::Manifest'    => '1.22',  # lib/ExtUtils/Manifest.pm
        'ExtUtils::Mkbootstrap' => undef,  # lib/ExtUtils/Mkbootstrap.pm
        'ExtUtils::Mksymlists'  => '1.00',  # lib/ExtUtils/Mksymlists.pm
        'Fcntl'                 => '1.00',  # ext/Fcntl/Fcntl.pm
        'File::Basename'        => undef,  # lib/File/Basename.pm
        'File::CheckTree'       => undef,  # lib/File/CheckTree.pm
        'File::Copy'            => '1.5',  # lib/File/Copy.pm
        'File::Find'            => undef,  # lib/File/Find.pm
        'File::Path'            => '1.01',  # lib/File/Path.pm
        'FileCache'             => undef,  # lib/FileCache.pm
        'FileHandle'            => '1.00',  # ext/FileHandle/FileHandle.pm
        'GDBM_File'             => '1.00',  # ext/GDBM_File/GDBM_File.pm
        'Getopt::Long'          => '2.01',  # lib/Getopt/Long.pm
        'Getopt::Std'           => undef,  # lib/Getopt/Std.pm
        'I18N::Collate'         => undef,  # lib/I18N/Collate.pm
        'IPC::Open2'            => undef,  # lib/IPC/Open2.pm
        'IPC::Open3'            => undef,  # lib/IPC/Open3.pm
        'Math::BigFloat'        => undef,  # lib/Math/BigFloat.pm
        'Math::BigInt'          => undef,  # lib/Math/BigInt.pm
        'Math::Complex'         => undef,  # lib/Math/Complex.pm
        'NDBM_File'             => '1.00',  # ext/NDBM_File/NDBM_File.pm
        'Net::Ping'             => '1',  # lib/Net/Ping.pm
        'ODBM_File'             => '1.00',  # ext/ODBM_File/ODBM_File.pm
        'POSIX'                 => '1.00',  # ext/POSIX/POSIX.pm
        'Pod::Functions'        => undef,  # lib/Pod/Functions.pm
        'Pod::Text'             => undef,  # lib/Pod/Text.pm
        'SDBM_File'             => '1.00',  # ext/SDBM_File/SDBM_File.pm
        'Safe'                  => '1.00',  # ext/Safe/Safe.pm
        'Search::Dict'          => undef,  # lib/Search/Dict.pm
        'SelectSaver'           => undef,  # lib/SelectSaver.pm
        'SelfLoader'            => '1.06',  # lib/SelfLoader.pm
        'Shell'                 => undef,  # lib/Shell.pm
        'Socket'                => '1.5',  # ext/Socket/Socket.pm
        'Symbol'                => undef,  # lib/Symbol.pm
        'Sys::Hostname'         => undef,  # lib/Sys/Hostname.pm
        'Sys::Syslog'           => undef,  # lib/Sys/Syslog.pm
        'Term::Cap'             => undef,  # lib/Term/Cap.pm
        'Term::Complete'        => undef,  # lib/Term/Complete.pm
        'Term::ReadLine'        => undef,  # lib/Term/ReadLine.pm
        'Test::Harness'         => '1.07',  # lib/Test/Harness.pm
        'Text::Abbrev'          => undef,  # lib/Text/Abbrev.pm
        'Text::ParseWords'      => undef,  # lib/Text/ParseWords.pm
        'Text::Soundex'         => undef,  # lib/Text/Soundex.pm
        'Text::Tabs'            => undef,  # lib/Text/Tabs.pm
        'Text::Wrap'            => undef,  # lib/Text/Wrap.pm
        'Tie::Hash'             => undef,  # lib/Tie/Hash.pm
        'Tie::Scalar'           => undef,  # lib/Tie/Scalar.pm
        'Tie::SubstrHash'       => undef,  # lib/Tie/SubstrHash.pm
        'Time::Local'           => undef,  # lib/Time/Local.pm
        'diagnostics'           => undef,  # lib/diagnostics.pm
        'integer'               => undef,  # lib/integer.pm
        'less'                  => undef,  # lib/less.pm
        'lib'                   => undef,  # lib/lib.pm
        'overload'              => undef,  # lib/overload.pm
        'sigtrap'               => undef,  # lib/sigtrap.pm
        'strict'                => undef,  # lib/strict.pm
        'subs'                  => undef,  # lib/subs.pm
        'vars'                  => undef,  # lib/vars.pm
    },
    5.00307 => {
        'AnyDBM_File'           => undef, #./lib/AnyDBM_File.pm
        'AutoLoader'            => undef, #./lib/AutoLoader.pm
        'AutoSplit'             => undef, #./lib/AutoSplit.pm
        'Benchmark'             => undef, #./lib/Benchmark.pm
        'Carp'                  => undef, #./lib/Carp.pm
        'Config'                => undef,
        'Cwd'                   => undef, #./lib/Cwd.pm
        'DB_File'               => '1.03', #./lib/DB_File.pm
        'Devel::SelfStubber'    => '1.01', #./lib/Devel/SelfStubber.pm
        'diagnostics'           => undef, #./lib/diagnostics.pm
        'DirHandle'             => undef, #./lib/DirHandle.pm
        'DynaLoader'            => '1.00', #./ext/DynaLoader/DynaLoader.pm
        'English'               => undef, #./lib/English.pm
        'Env'                   => undef, #./lib/Env.pm
        'Exporter'              => undef, #./lib/Exporter.pm
        'ExtUtils::Embed'       => '1.18', #./lib/ExtUtils/Embed.pm
        'ExtUtils::Install'     => '1.15 ', #./lib/ExtUtils/Install.pm
        'ExtUtils::Liblist'     => '1.20 ', #./lib/ExtUtils/Liblist.pm
        'ExtUtils::MakeMaker'   => '5.38', #./lib/ExtUtils/MakeMaker.pm
        'ExtUtils::Manifest'    => '1.27', #./lib/ExtUtils/Manifest.pm
        'ExtUtils::Mkbootstrap' => '1.13 ', #./lib/ExtUtils/Mkbootstrap.pm
        'ExtUtils::Mksymlists'  => '1.12 ', #./lib/ExtUtils/Mksymlists.pm
        'ExtUtils::MM_OS2'      => undef, #./lib/ExtUtils/MM_OS2.pm
        'ExtUtils::MM_Unix'     => '1.107 ', #./lib/ExtUtils/MM_Unix.pm
        'ExtUtils::MM_VMS'      => undef, #./lib/ExtUtils/MM_VMS.pm
        'ExtUtils::testlib'     => '1.11 ', #./lib/ExtUtils/testlib.pm
        'Fatal'                 => undef, #./lib/Fatal.pm
        'Fcntl'                 => '1.00', #./ext/Fcntl/Fcntl.pm
        'File::Basename'        => '2.4', #./lib/File/Basename.pm
        'File::CheckTree'       => undef, #./lib/File/CheckTree.pm
        'File::Copy'            => '1.5', #./lib/File/Copy.pm
        'File::Find'            => undef, #./lib/File/Find.pm
        'File::Path'            => '1.01', #./lib/File/Path.pm
        'FileCache'             => undef, #./lib/FileCache.pm
        'FileHandle'            => '1.00', #./ext/FileHandle/FileHandle.pm
        'FindBin'               => '1.04', #./lib/FindBin.pm
        'GDBM_File'             => '1.00', #./ext/GDBM_File/GDBM_File.pm
        'Getopt::Long'          => '2.04', #./lib/Getopt/Long.pm
        'Getopt::Std'           => undef, #./lib/Getopt/Std.pm
        'I18N::Collate'         => undef, #./lib/I18N/Collate.pm
        'integer'               => undef, #./lib/integer.pm
        'IO'                    => undef, #./ext/IO/IO.pm
        'IO::File'              => '1.05', #./ext/IO/lib/IO/File.pm
        'IO::Handle'            => '1.12', #./ext/IO/lib/IO/Handle.pm
        'IO::Pipe'              => '1.07', #./ext/IO/lib/IO/Pipe.pm
        'IO::Seekable'          => '1.05', #./ext/IO/lib/IO/Seekable.pm
        'IO::Select'            => '1.09', #./ext/IO/lib/IO/Select.pm
        'IO::Socket'            => '1.13', #./ext/IO/lib/IO/Socket.pm
        'IPC::Open2'            => undef, #./lib/IPC/Open2.pm
        'IPC::Open3'            => undef, #./lib/IPC/Open3.pm
        'less'                  => undef, #./lib/less.pm
        'lib'                   => undef, #./lib/lib.pm
        'Math::BigFloat'        => undef, #./lib/Math/BigFloat.pm
        'Math::BigInt'          => undef, #./lib/Math/BigInt.pm
        'Math::Complex'         => undef, #./lib/Math/Complex.pm
        'NDBM_File'             => '1.00', #./ext/NDBM_File/NDBM_File.pm
        'Net::Ping'             => '1.01', #./lib/Net/Ping.pm
        'ODBM_File'             => '1.00', #./ext/ODBM_File/ODBM_File.pm
        'Opcode'                => '1.01', #./ext/Opcode/Opcode.pm
        'ops'                   => undef, #./ext/Opcode/ops.pm
        'OS2::ExtAttr'          => '0.01', #./os2/OS2/ExtAttr/ExtAttr.pm
        'OS2::PrfDB'            => '0.02', #./os2/OS2/PrfDB/PrfDB.pm
        'OS2::Process'          => undef, #./os2/OS2/Process/Process.pm
        'OS2::REXX'             => undef, #./os2/OS2/REXX/REXX.pm
        'overload'              => undef, #./lib/overload.pm
        'Pod::Functions'        => undef, #./lib/Pod/Functions.pm
        'Pod::Text'             => undef, #./lib/Pod/Text.pm
        'POSIX'                 => '1.00', #./ext/POSIX/POSIX.pm
        'Safe'                  => '2.06', #./ext/Opcode/Safe.pm
        'SDBM_File'             => '1.00', #./ext/SDBM_File/SDBM_File.pm
        'Search::Dict'          => undef, #./lib/Search/Dict.pm
        'SelectSaver'           => undef, #./lib/SelectSaver.pm
        'SelfLoader'            => '1.06', #./lib/SelfLoader.pm
        'Shell'                 => undef, #./lib/Shell.pm
        'sigtrap'               => '1.01', #./lib/sigtrap.pm
        'Socket'                => '1.5', #./ext/Socket/Socket.pm
        'strict'                => undef, #./lib/strict.pm
        'subs'                  => undef, #./lib/subs.pm
        'Symbol'                => undef, #./lib/Symbol.pm
        'Sys::Hostname'         => undef, #./lib/Sys/Hostname.pm
        'Sys::Syslog'           => undef, #./lib/Sys/Syslog.pm
        'Term::Cap'             => undef, #./lib/Term/Cap.pm
        'Term::Complete'        => undef, #./lib/Term/Complete.pm
        'Term::ReadLine'        => undef, #./lib/Term/ReadLine.pm
        'Test::Harness'         => '1.13', #./lib/Test/Harness.pm
        'Text::Abbrev'          => undef, #./lib/Text/Abbrev.pm
        'Text::ParseWords'      => undef, #./lib/Text/ParseWords.pm
        'Text::Soundex'         => undef, #./lib/Text/Soundex.pm
        'Text::Tabs'            => '96.051501', #./lib/Text/Tabs.pm
        'Text::Wrap'            => '96.041801', #./lib/Text/Wrap.pm
        'Tie::Hash'             => undef, #./lib/Tie/Hash.pm
        'Tie::Scalar'           => undef, #./lib/Tie/Scalar.pm
        'Tie::SubstrHash'       => undef, #./lib/Tie/SubstrHash.pm
        'Time::Local'           => undef, #./lib/Time/Local.pm
        'UNIVERSAL'             => undef, #./lib/UNIVERSAL.pm
        'vars'                  => undef, #./lib/vars.pm
        'VMS::Filespec'         => undef, #./vms/ext/Filespec.pm
        'VMS::Stdio'            => '2.0', #./vms/ext/Stdio/Stdio.pm
    },
    5.004   => {
        'AnyDBM_File'           => undef, #./lib/AnyDBM_File.pm
        'AutoLoader'            => undef, #./lib/AutoLoader.pm
        'AutoSplit'             => undef, #./lib/AutoSplit.pm
        'autouse'               => '1.01', #./lib/autouse.pm
        'Benchmark'             => undef, #./lib/Benchmark.pm
        'blib'                  => undef, #./lib/blib.pm
        'Bundle::CPAN'          => '0.02', #./lib/Bundle/CPAN.pm
        'Carp'                  => undef, #./lib/Carp.pm
        'CGI'                   => '2.36', #./lib/CGI.pm
        'CGI::Apache'           => '1.01', #./lib/CGI/Apache.pm
        'CGI::Carp'             => '1.06', #./lib/CGI/Carp.pm
        'CGI::Fast'             => '1.00a', #./lib/CGI/Fast.pm
        'CGI::Push'             => '1.00', #./lib/CGI/Push.pm
        'CGI::Switch'           => '0.05', #./lib/CGI/Switch.pm
        'Class::Struct'         => undef, #./lib/Class/Struct.pm
        'Config'                => undef,
        'constant'              => '1.00', #./lib/constant.pm
        'CPAN'                  => '1.2401', #./lib/CPAN.pm
        'CPAN::FirstTime'       => '1.18 ', #./lib/CPAN/FirstTime.pm
        'CPAN::Nox'             => undef, #./lib/CPAN/Nox.pm
        'Cwd'                   => '2.00', #./lib/Cwd.pm
        'DB_File'               => '1.14', #./ext/DB_File/DB_File.pm
        'Devel::SelfStubber'    => '1.01', #./lib/Devel/SelfStubber.pm
        'diagnostics'           => undef, #./lib/diagnostics.pm
        'DirHandle'             => undef, #./lib/DirHandle.pm
        'DynaLoader'            => '1.02', #./ext/DynaLoader/DynaLoader.pm
        'English'               => undef, #./lib/English.pm
        'Env'                   => undef, #./lib/Env.pm
        'Exporter'              => undef, #./lib/Exporter.pm
        'ExtUtils::Command'     => '1.00', #./lib/ExtUtils/Command.pm
        'ExtUtils::Embed'       => '1.2501', #./lib/ExtUtils/Embed.pm
        'ExtUtils::Install'     => '1.16 ', #./lib/ExtUtils/Install.pm
        'ExtUtils::Liblist'     => '1.2201 ', #./lib/ExtUtils/Liblist.pm
        'ExtUtils::MakeMaker'   => '5.4002', #./lib/ExtUtils/MakeMaker.pm
        'ExtUtils::Manifest'    => '1.33 ', #./lib/ExtUtils/Manifest.pm
        'ExtUtils::Mkbootstrap' => '1.13 ', #./lib/ExtUtils/Mkbootstrap.pm
        'ExtUtils::Mksymlists'  => '1.13 ', #./lib/ExtUtils/Mksymlists.pm
        'ExtUtils::MM_OS2'      => undef, #./lib/ExtUtils/MM_OS2.pm
        'ExtUtils::MM_Unix'     => '1.114 ', #./lib/ExtUtils/MM_Unix.pm
        'ExtUtils::MM_VMS'      => undef, #./lib/ExtUtils/MM_VMS.pm
        'ExtUtils::MM_Win32'    => undef, #./lib/ExtUtils/MM_Win32.pm
        'ExtUtils::testlib'     => '1.11 ', #./lib/ExtUtils/testlib.pm
        'ExtUtils::XSSymSet'    => '1.0', #./vms/ext/XSSymSet.pm
        'Fcntl'                 => '1.03', #./ext/Fcntl/Fcntl.pm
        'File::Basename'        => '2.5', #./lib/File/Basename.pm
        'File::CheckTree'       => undef, #./lib/File/CheckTree.pm
        'File::Compare'         => '1.1001', #./lib/File/Compare.pm
        'File::Copy'            => '2.02', #./lib/File/Copy.pm
        'File::Find'            => undef, #./lib/File/Find.pm
        'File::Path'            => '1.04', #./lib/File/Path.pm
        'File::stat'            => undef, #./lib/File/stat.pm
        'FileCache'             => undef, #./lib/FileCache.pm
        'FileHandle'            => '2.00', #./lib/FileHandle.pm
        'FindBin'               => '1.04', #./lib/FindBin.pm
        'GDBM_File'             => '1.00', #./ext/GDBM_File/GDBM_File.pm
        'Getopt::Long'          => '2.10', #./lib/Getopt/Long.pm
        'Getopt::Std'           => undef, #./lib/Getopt/Std.pm
        'I18N::Collate'         => undef, #./lib/I18N/Collate.pm
        'integer'               => undef, #./lib/integer.pm
        'IO'                    => undef, #./ext/IO/IO.pm
        'IO::File'              => '1.0602', #./ext/IO/lib/IO/File.pm
        'IO::Handle'            => '1.1504', #./ext/IO/lib/IO/Handle.pm
        'IO::Pipe'              => '1.0901', #./ext/IO/lib/IO/Pipe.pm
        'IO::Seekable'          => '1.06', #./ext/IO/lib/IO/Seekable.pm
        'IO::Select'            => '1.10', #./ext/IO/lib/IO/Select.pm
        'IO::Socket'            => '1.1602', #./ext/IO/lib/IO/Socket.pm
        'IPC::Open2'            => '1.01', #./lib/IPC/Open2.pm
        'IPC::Open3'            => '1.0101', #./lib/IPC/Open3.pm
        'less'                  => undef, #./lib/less.pm
        'lib'                   => undef, #./lib/lib.pm
        'locale'                => undef, #./lib/locale.pm
        'Math::BigFloat'        => undef, #./lib/Math/BigFloat.pm
        'Math::BigInt'          => undef, #./lib/Math/BigInt.pm
        'Math::Complex'         => '1.01', #./lib/Math/Complex.pm
        'Math::Trig'            => '1', #./lib/Math/Trig.pm
        'NDBM_File'             => '1.00', #./ext/NDBM_File/NDBM_File.pm
        'Net::hostent'          => undef, #./lib/Net/hostent.pm
        'Net::netent'           => undef, #./lib/Net/netent.pm
        'Net::Ping'             => '2.02', #./lib/Net/Ping.pm
        'Net::protoent'         => undef, #./lib/Net/protoent.pm
        'Net::servent'          => undef, #./lib/Net/servent.pm
        'ODBM_File'             => '1.00', #./ext/ODBM_File/ODBM_File.pm
        'Opcode'                => '1.04', #./ext/Opcode/Opcode.pm
        'ops'                   => undef, #./ext/Opcode/ops.pm
        'Safe'                  => '2.06', #./ext/Opcode/Safe.pm
        'OS2::ExtAttr'          => '0.01', #./os2/OS2/ExtAttr/ExtAttr.pm
        'OS2::PrfDB'            => '0.02', #./os2/OS2/PrfDB/PrfDB.pm
        'OS2::Process'          => undef, #./os2/OS2/Process/Process.pm
        'OS2::REXX'             => undef, #./os2/OS2/REXX/REXX.pm
        'overload'              => undef, #./lib/overload.pm
        'Pod::Functions'        => undef, #./lib/Pod/Functions.pm
        'Pod::Html'             => undef, #./lib/Pod/Html.pm
        'Pod::Text'             => '1.0203', #./lib/Pod/Text.pm
        'POSIX'                 => '1.02', #./ext/POSIX/POSIX.pm
        'SDBM_File'             => '1.00', #./ext/SDBM_File/SDBM_File.pm
        'Search::Dict'          => undef, #./lib/Search/Dict.pm
        'SelectSaver'           => undef, #./lib/SelectSaver.pm
        'SelfLoader'            => '1.07', #./lib/SelfLoader.pm
        'Shell'                 => undef, #./lib/Shell.pm
        'sigtrap'               => '1.02', #./lib/sigtrap.pm
        'Socket'                => '1.6', #./ext/Socket/Socket.pm
        'strict'                => undef, #./lib/strict.pm
        'subs'                  => undef, #./lib/subs.pm
        'Symbol'                => '1.02', #./lib/Symbol.pm
        'Sys::Hostname'         => undef, #./lib/Sys/Hostname.pm
        'Sys::Syslog'           => undef, #./lib/Sys/Syslog.pm
        'Term::Cap'             => undef, #./lib/Term/Cap.pm
        'Term::Complete'        => undef, #./lib/Term/Complete.pm
        'Term::ReadLine'        => undef, #./lib/Term/ReadLine.pm
        'Test::Harness'         => '1.1502', #./lib/Test/Harness.pm
        'Text::Abbrev'          => undef, #./lib/Text/Abbrev.pm
        'Text::ParseWords'      => undef, #./lib/Text/ParseWords.pm
        'Text::Soundex'         => undef, #./lib/Text/Soundex.pm
        'Text::Tabs'            => '96.121201', #./lib/Text/Tabs.pm
        'Text::Wrap'            => '97.011701', #./lib/Text/Wrap.pm
        'Tie::Hash'             => undef, #./lib/Tie/Hash.pm
        'Tie::RefHash'          => undef, #./lib/Tie/RefHash.pm
        'Tie::Scalar'           => undef, #./lib/Tie/Scalar.pm
        'Tie::SubstrHash'       => undef, #./lib/Tie/SubstrHash.pm
        'Time::gmtime'          => '1.01', #./lib/Time/gmtime.pm
        'Time::Local'           => undef, #./lib/Time/Local.pm
        'Time::localtime'       => '1.01', #./lib/Time/localtime.pm
        'Time::tm'              => undef, #./lib/Time/tm.pm
        'UNIVERSAL'             => undef, #./lib/UNIVERSAL.pm
        'User::grent'           => undef, #./lib/User/grent.pm
        'User::pwent'           => undef, #./lib/User/pwent.pm
        'vars'                  => undef, #./lib/vars.pm
        'VMS::DCLsym'           => '1.01', #./vms/ext/DCLsym/DCLsym.pm
        'VMS::Filespec'         => undef, #./vms/ext/Filespec.pm
        'VMS::Stdio'            => '2.02', #./vms/ext/Stdio/Stdio.pm
        'vmsish'                => undef, #./vms/ext/vmsish.pm
    },
    5.005   => {
        'AnyDBM_File'           => undef, #./lib/AnyDBM_File.pm
        'attrs'                 => '1.0', #./ext/attrs/attrs.pm
        'AutoLoader'            => undef, #./lib/AutoLoader.pm
        'AutoSplit'             => '1.0302', #./lib/AutoSplit.pm
        'autouse'               => '1.01', #./lib/autouse.pm
        'B'                     => undef, #./ext/B/B.pm
        'B::Asmdata'            => undef, #./ext/B/B/Asmdata.pm
        'B::Assembler'          => undef, #./ext/B/B/Assembler.pm
        'B::Bblock'             => undef, #./ext/B/B/Bblock.pm
        'B::Bytecode'           => undef, #./ext/B/B/Bytecode.pm
        'B::C'                  => undef, #./ext/B/B/C.pm
        'B::CC'                 => undef, #./ext/B/B/CC.pm
        'B::Debug'              => undef, #./ext/B/B/Debug.pm
        'B::Deparse'            => '0.56', #./ext/B/B/Deparse.pm
        'B::Disassembler'       => undef, #./ext/B/B/Disassembler.pm
        'B::Lint'               => undef, #./ext/B/B/Lint.pm
        'B::Showlex'            => undef, #./ext/B/B/Showlex.pm
        'B::Stackobj'           => undef, #./ext/B/B/Stackobj.pm
        'B::Terse'              => undef, #./ext/B/B/Terse.pm
        'B::Xref'               => undef, #./ext/B/B/Xref.pm
        'base'                  => undef, #./lib/base.pm
        'Benchmark'             => undef, #./lib/Benchmark.pm
        'blib'                  => '1.00', #./lib/blib.pm
        'Carp'                  => undef, #./lib/Carp.pm
        'CGI'                   => '2.42', #./lib/CGI.pm
        'CGI::Apache'           => '1.1', #./lib/CGI/Apache.pm
        'CGI::Carp'             => '1.101', #./lib/CGI/Carp.pm
        'CGI::Cookie'           => '1.06', #./lib/CGI/Cookie.pm
        'CGI::Fast'             => '1.00a', #./lib/CGI/Fast.pm
        'CGI::Push'             => '1.01', #./lib/CGI/Push.pm
        'CGI::Switch'           => '0.06', #./lib/CGI/Switch.pm
        'Class::Struct'         => undef, #./lib/Class/Struct.pm
        'Config'                => undef,
        'constant'              => '1.00', #./lib/constant.pm
        'CPAN'                  => '1.3901', #./lib/CPAN.pm
        'CPAN::FirstTime'       => '1.29 ', #./lib/CPAN/FirstTime.pm
        'CPAN::Nox'             => undef, #./lib/CPAN/Nox.pm
        'Cwd'                   => '2.01', #./lib/Cwd.pm
        'Data::Dumper'          => '2.09', #./ext/Data/Dumper/Dumper.pm
        'DB_File'               => '1.60', #./ext/DB_File/DB_File.pm
        'Devel::SelfStubber'    => '1.01', #./lib/Devel/SelfStubber.pm
        'DynaLoader'            => '1.03',
        'diagnostics'           => undef, #./lib/diagnostics.pm
        'DirHandle'             => undef, #./lib/DirHandle.pm
        'English'               => undef, #./lib/English.pm
        'Env'                   => undef, #./lib/Env.pm
        'Exporter'              => undef, #./lib/Exporter.pm
        'ExtUtils::Command'     => '1.01', #./lib/ExtUtils/Command.pm
        'ExtUtils::Embed'       => '1.2505', #./lib/ExtUtils/Embed.pm
        'ExtUtils::Install'     => '1.28 ', #./lib/ExtUtils/Install.pm
        'ExtUtils::Installed'   => '0.02', #./lib/ExtUtils/Installed.pm
        'ExtUtils::Liblist'     => '1.25 ', #./lib/ExtUtils/Liblist.pm
        'ExtUtils::MakeMaker'   => '5.4301', #./lib/ExtUtils/MakeMaker.pm
        'ExtUtils::Manifest'    => '1.33 ', #./lib/ExtUtils/Manifest.pm
        'ExtUtils::Mkbootstrap' => '1.13 ', #./lib/ExtUtils/Mkbootstrap.pm
        'ExtUtils::Mksymlists'  => '1.17 ', #./lib/ExtUtils/Mksymlists.pm
        'ExtUtils::MM_OS2'      => undef, #./lib/ExtUtils/MM_OS2.pm
        'ExtUtils::MM_Unix'     => '1.12601 ', #./lib/ExtUtils/MM_Unix.pm
        'ExtUtils::MM_VMS'      => undef, #./lib/ExtUtils/MM_VMS.pm
        'ExtUtils::MM_Win32'    => undef, #./lib/ExtUtils/MM_Win32.pm
        'ExtUtils::Packlist'    => '0.03', #./lib/ExtUtils/Packlist.pm
        'ExtUtils::testlib'     => '1.11 ', #./lib/ExtUtils/testlib.pm
        'ExtUtils::XSSymSet'    => '1.0', #./vms/ext/XSSymSet.pm
        'Fatal'                 => '1.02', #./lib/Fatal.pm
        'Fcntl'                 => '1.03', #./ext/Fcntl/Fcntl.pm
        'fields'                => '0.02', #./lib/fields.pm
        'File::Basename'        => '2.6', #./lib/File/Basename.pm
        'File::CheckTree'       => undef, #./lib/File/CheckTree.pm
        'File::Compare'         => '1.1001', #./lib/File/Compare.pm
        'File::Copy'            => '2.02', #./lib/File/Copy.pm
        'File::DosGlob'         => undef, #./lib/File/DosGlob.pm
        'File::Find'            => undef, #./lib/File/Find.pm
        'File::Path'            => '1.0401', #./lib/File/Path.pm
        'File::Spec'            => '0.6', #./lib/File/Spec.pm
        'File::Spec::Mac'       => '1.0', #./lib/File/Spec/Mac.pm
        'File::Spec::OS2'       => undef, #./lib/File/Spec/OS2.pm
        'File::Spec::Unix'      => undef, #./lib/File/Spec/Unix.pm
        'File::Spec::VMS'       => undef, #./lib/File/Spec/VMS.pm
        'File::Spec::Win32'     => undef, #./lib/File/Spec/Win32.pm
        'File::stat'            => undef, #./lib/File/stat.pm
        'FileCache'             => undef, #./lib/FileCache.pm
        'FileHandle'            => '2.00', #./lib/FileHandle.pm
        'FindBin'               => '1.41', #./lib/FindBin.pm
        'GDBM_File'             => '1.00', #./ext/GDBM_File/GDBM_File.pm
        'Getopt::Long'          => '2.17', #./lib/Getopt/Long.pm
        'Getopt::Std'           => undef, #./lib/Getopt/Std.pm
        'I18N::Collate'         => undef, #./lib/I18N/Collate.pm
        'integer'               => undef, #./lib/integer.pm
        'IO'                    => undef, #./ext/IO/IO.pm
        'IO::File'              => '1.06021', #./ext/IO/lib/IO/File.pm
        'IO::Handle'            => '1.1505', #./ext/IO/lib/IO/Handle.pm
        'IO::Pipe'              => '1.0901', #./ext/IO/lib/IO/Pipe.pm
        'IO::Seekable'          => '1.06', #./ext/IO/lib/IO/Seekable.pm
        'IO::Select'            => '1.10', #./ext/IO/lib/IO/Select.pm
        'IO::Socket'            => '1.1603', #./ext/IO/lib/IO/Socket.pm
        'IPC::Open2'            => '1.01', #./lib/IPC/Open2.pm
        'IPC::Open3'            => '1.0102', #./lib/IPC/Open3.pm
        'IPC::Msg'              => '1.00', #./ext/IPC/SysV/Msg.pm
        'IPC::Semaphore'        => '1.00', #./ext/IPC/SysV/Semaphore.pm
        'IPC::SysV'             => '1.03', #./ext/IPC/SysV/SysV.pm
        'less'                  => undef, #./lib/less.pm
        'lib'                   => undef, #./lib/lib.pm
        'locale'                => undef, #./lib/locale.pm
        'Math::BigFloat'        => undef, #./lib/Math/BigFloat.pm
        'Math::BigInt'          => undef, #./lib/Math/BigInt.pm
        'Math::Complex'         => '1.25', #./lib/Math/Complex.pm
        'Math::Trig'            => '1', #./lib/Math/Trig.pm
        'NDBM_File'             => '1.01', #./ext/NDBM_File/NDBM_File.pm
        'Net::hostent'          => undef, #./lib/Net/hostent.pm
        'Net::netent'           => undef, #./lib/Net/netent.pm
        'Net::Ping'             => '2.02', #./lib/Net/Ping.pm
        'Net::protoent'         => undef, #./lib/Net/protoent.pm
        'Net::servent'          => undef, #./lib/Net/servent.pm
        'O'                     => undef, #./ext/B/O.pm
        'ODBM_File'             => '1.00', #./ext/ODBM_File/ODBM_File.pm
        'Opcode'                => '1.04', #./ext/Opcode/Opcode.pm
        'ops'                   => undef, #./ext/Opcode/ops.pm
        'Safe'                  => '2.06', #./ext/Opcode/Safe.pm
        'OS2::ExtAttr'          => '0.01', #./os2/OS2/ExtAttr/ExtAttr.pm
        'OS2::PrfDB'            => '0.02', #./os2/OS2/PrfDB/PrfDB.pm
        'OS2::Process'          => '0.2', #./os2/OS2/Process/Process.pm
        'OS2::REXX'             => undef, #./os2/OS2/REXX/REXX.pm
        'overload'              => undef, #./lib/overload.pm
        'Pod::Functions'        => undef, #./lib/Pod/Functions.pm
        'Pod::Html'             => '1.01', #./lib/Pod/Html.pm
        'Pod::Text'             => '1.0203', #./lib/Pod/Text.pm
        'POSIX'                 => '1.02', #./ext/POSIX/POSIX.pm
        're'                    => '0.02', #./ext/re/re.pm
        'SDBM_File'             => '1.00', #./ext/SDBM_File/SDBM_File.pm
        'Search::Dict'          => undef, #./lib/Search/Dict.pm
        'SelectSaver'           => undef, #./lib/SelectSaver.pm
        'SelfLoader'            => '1.08', #./lib/SelfLoader.pm
        'Shell'                 => undef, #./lib/Shell.pm
        'sigtrap'               => '1.02', #./lib/sigtrap.pm
        'Socket'                => '1.7', #./ext/Socket/Socket.pm
        'strict'                => '1.01', #./lib/strict.pm
        'subs'                  => undef, #./lib/subs.pm
        'Symbol'                => '1.02', #./lib/Symbol.pm
        'Sys::Hostname'         => undef, #./lib/Sys/Hostname.pm
        'Sys::Syslog'           => undef, #./lib/Sys/Syslog.pm
        'Term::Cap'             => undef, #./lib/Term/Cap.pm
        'Term::Complete'        => undef, #./lib/Term/Complete.pm
        'Term::ReadLine'        => undef, #./lib/Term/ReadLine.pm
        'Test'                  => '1.04', #./lib/Test.pm
        'Test::Harness'         => '1.1602', #./lib/Test/Harness.pm
        'Text::Abbrev'          => undef, #./lib/Text/Abbrev.pm
        'Text::ParseWords'      => '3.1', #./lib/Text/ParseWords.pm
        'Text::Soundex'         => undef, #./lib/Text/Soundex.pm
        'Text::Tabs'            => '96.121201', #./lib/Text/Tabs.pm
        'Text::Wrap'            => '97.02', #./lib/Text/Wrap.pm
        'Thread'                => '1.0', #./ext/Thread/Thread.pm
        'Thread::Queue'         => undef, #./ext/Thread/Thread/Queue.pm
        'Thread::Semaphore'     => undef, #./ext/Thread/Thread/Semaphore.pm
        'Thread::Signal'        => undef, #./ext/Thread/Thread/Signal.pm
        'Thread::Specific'      => undef, #./ext/Thread/Thread/Specific.pm
        'Tie::Array'            => '1.00', #./lib/Tie/Array.pm
        'Tie::Handle'           => undef, #./lib/Tie/Handle.pm
        'Tie::Hash'             => undef, #./lib/Tie/Hash.pm
        'Tie::RefHash'          => undef, #./lib/Tie/RefHash.pm
        'Tie::Scalar'           => undef, #./lib/Tie/Scalar.pm
        'Tie::SubstrHash'       => undef, #./lib/Tie/SubstrHash.pm
        'Time::gmtime'          => '1.01', #./lib/Time/gmtime.pm
        'Time::Local'           => undef, #./lib/Time/Local.pm
        'Time::localtime'       => '1.01', #./lib/Time/localtime.pm
        'Time::tm'              => undef, #./lib/Time/tm.pm
        'UNIVERSAL'             => undef, #./lib/UNIVERSAL.pm
        'User::grent'           => undef, #./lib/User/grent.pm
        'User::pwent'           => undef, #./lib/User/pwent.pm
        'vars'                  => undef, #./lib/vars.pm
        'VMS::DCLsym'           => '1.01', #./vms/ext/DCLsym/DCLsym.pm
        'VMS::Filespec'         => undef, #./vms/ext/Filespec.pm
        'VMS::Stdio'            => '2.1', #./vms/ext/Stdio/Stdio.pm
        'vmsish'                => undef, #./vms/ext/vmsish.pm
    },
    5.00503   => {
        'AnyDBM_File'           => undef,
        'attrs'                 => '1.0',
        'AutoLoader'            => undef,
        'AutoSplit'             => 1.0303,
        'autouse'               => 1.01,
        'B::Asmdata'            => undef,
        'B::Assembler'          => undef,
        'B::Bblock'             => undef,
        'B::Bytecode'           => undef,
        'B::C'                  => undef,
        'B::CC'                 => undef,
        'B::Debug'              => undef,
        'B::Deparse'            => 0.56,
        'B::Disassembler'       => undef,
        'B::Lint'               => undef,
        'B'                     => undef,
        'B::Showlex'            => undef,
        'B::Stackobj'           => undef,
        'B::Terse'              => undef,
        'B::Xref'               => undef,
        'base'                  => undef,
        'Benchmark'             => undef,
        'blib'                  => '1.00',
        'Carp'                  => undef,
        'CGI'                   => 2.46,
        'CGI::Apache'           => 1.1,
        'CGI::Carp'             => 1.13,
        'CGI::Cookie'           => 1.06,
        'CGI::Fast'             => 1.01,
        'CGI::Push'             => 1.01,
        'CGI::Switch'           => 0.06,
        'Class::Struct'         => undef,
        'Config'                => undef,
        'constant'              => '1.00',
        'CPAN::FirstTime'       => 1.36 ,
        'CPAN'                  => 1.48,
        'CPAN::Nox'             => '1.00',
        'Cwd'                   => 2.01,
        'Data::Dumper'          => 2.101,
        'DB_File'               => 1.65,
        'Devel::SelfStubber'    => 1.01,
        'diagnostics'           => undef,
        'DirHandle'             => undef,
        'Dumpvalue'             => undef,
        'DynaLoader'            => 1.03,
        'English'               => undef,
        'Env'                   => undef,
        'Exporter'              => undef,
        'ExtUtils::Command'     => 1.01,
        'ExtUtils::Embed'       => 1.2505,
        'ExtUtils::Install'     => 1.28 ,
        'ExtUtils::Installed'   => 0.02,
        'ExtUtils::Liblist'     => 1.25 ,
        'ExtUtils::MakeMaker'   => 5.4302,
        'ExtUtils::Manifest'    => 1.33 ,
        'ExtUtils::Mkbootstrap' => 1.14 ,
        'ExtUtils::Mksymlists'  => 1.17 ,
        'ExtUtils::MM_OS2'      => undef,
        'ExtUtils::MM_Unix'     => 1.12602 ,
        'ExtUtils::MM_VMS'      => undef,
        'ExtUtils::MM_Win32'    => undef,
        'ExtUtils::Packlist'    => 0.03,
        'ExtUtils::testlib'     => 1.11 ,
        'ExtUtils::XSSymSet'    => '1.0',
        'Fatal'                 => 1.02,
        'Fcntl'                 => 1.03,
        'fields'                => 0.02,
        'File::Basename'        => 2.6,
        'File::CheckTree'       => undef,
        'File::Compare'         => 1.1001,
        'File::Copy'            => 2.02,
        'File::DosGlob'         => undef,
        'File::Find'            => undef,
        'File::Path'            => 1.0401,
        'File::Spec'            => 0.6,
        'File::Spec::Mac'       => '1.0',
        'File::Spec::OS2'       => undef,
        'File::Spec::Unix'      => undef,
        'File::Spec::VMS'       => undef,
        'File::Spec::Win32'     => undef,
        'File::stat'            => undef,
        'FileCache'             => undef,
        'FileHandle'            => '2.00',
        'FindBin'               => 1.42,
        'GDBM_File'             => '1.00',
        'Getopt::Long'          => 2.19,
        'Getopt::Std'           => 1.01,
        'I18N::Collate'         => undef,
        'integer'               => undef,
        'IO'                    => undef,
        'IO::File'              => 1.06021,
        'IO::Handle'            => 1.1505,
        'IO::Pipe'              => 1.0902,
        'IO::Seekable'          => 1.06,
        'IO::Select'            => '1.10',
        'IO::Socket'            => 1.1603,
        'IPC::Msg'              => '1.00',
        'IPC::Open2'            => 1.01,
        'IPC::Open3'            => 1.0103,
        'IPC::Semaphore'        => '1.00',
        'IPC::SysV'             => 1.03,
        'less'                  => undef,
        'lib'                   => undef,
        'locale'                => undef,
        'Math::BigFloat'        => undef,
        'Math::BigInt'          => undef,
        'Math::Complex'         => 1.26,
        'Math::Trig'            => 1,
        'NDBM_File'             => 1.01,
        'Net::hostent'          => undef,
        'Net::netent'           => undef,
        'Net::Ping'             => 2.02,
        'Net::protoent'         => undef,
        'Net::servent'          => undef,
        'O'                     => undef,
        'ODBM_File'             => '1.00',
        'Opcode'                => 1.04,
        'ops'                   => undef,
        'OS2::ExtAttr'          => 0.01,
        'OS2::PrfDB'            => 0.02,
        'OS2::Process'          => 0.2,
        'OS2::REXX'             => undef,
        'overload'              => undef,
        'Pod::Functions'        => undef,
        'Pod::Html'             => 1.01,
        'Pod::Text'             => 1.0203,
        'POSIX'                 => 1.02,
        're'                    => 0.02,
        'Safe'                  => 2.06,
        'SDBM_File'             => '1.00',
        'Search::Dict'          => undef,
        'SelectSaver'           => undef,
        'SelfLoader'            => 1.08,
        'Shell'                 => undef,
        'sigtrap'               => 1.02,
        'Socket'                => 1.7,
        'strict'                => 1.01,
        'subs'                  => undef,
        'Symbol'                => 1.02,
        'Sys::Hostname'         => undef,
        'Sys::Syslog'           => undef,
        'Term::Cap'             => undef,
        'Term::Complete'        => undef,
        'Term::ReadLine'        => undef,
        'Test'                  => 1.122,
        'Test::Harness'         => 1.1602,
        'Text::Abbrev'          => undef,
        'Text::ParseWords'      => 3.1,
        'Text::Soundex'         => undef,
        'Text::Tabs'            => 96.121201,
        'Text::Wrap'            => 98.112902,
        'Thread'                => '1.0',
        'Thread::Queue'         => undef,
        'Thread::Semaphore'     => undef,
        'Thread::Specific'      => undef,
        'Thread::Signal'        => undef,
        'Tie::Array'            => '1.00',
        'Tie::Handle'           => undef,
        'Tie::Hash'             => undef,
        'Tie::RefHash'          => undef,
        'Tie::Scalar'           => undef,
        'Tie::SubstrHash'       => undef,
        'Time::gmtime'          => 1.01,
        'Time::Local'           => undef,
        'Time::localtime'       => 1.01,
        'Time::tm'              => undef,
        'UNIVERSAL'             => undef,
        'User::grent'           => undef,
        'User::pwent'           => undef,
        'vars'                  => undef,
        'VMS::DCLsym'           => 1.01,
        'VMS::Filespec'         => undef,
        'VMS::Stdio'            => 2.1,
        'vmsish'                => undef,
    },
    5.00405   => {
        'AnyDBM_File'           => undef, #./lib/AnyDBM_File.pm
        'attrs'                 => '0.1', #./lib/attrs.pm
        'AutoLoader'            => '5.56', #./lib/AutoLoader.pm
        'AutoSplit'             => '1.0303', #./lib/AutoSplit.pm
        'autouse'               => '1.01', #./lib/autouse.pm
        'base'                  => undef, #./lib/base.pm
        'Benchmark'             => undef, #./lib/Benchmark.pm
        'blib'                  => '1.00', #./lib/blib.pm
        'Bundle::CPAN'          => '0.03', #./lib/Bundle/CPAN.pm
        'Carp'                  => undef, #./lib/Carp.pm
        'CGI'                   => '2.42', #./lib/CGI.pm
        'CGI::Apache'           => '1.1', #./lib/CGI/Apache.pm
        'CGI::Carp'             => '1.10', #./lib/CGI/Carp.pm
        'CGI::Cookie'           => '1.06', #./lib/CGI/Cookie.pm
        'CGI::Fast'             => '1.00a', #./lib/CGI/Fast.pm
        'CGI::Push'             => '1.01', #./lib/CGI/Push.pm
        'CGI::Switch'           => '0.06', #./lib/CGI/Switch.pm
        'Class::Struct'         => undef, #./lib/Class/Struct.pm
        'Config'                => undef,
        'constant'              => '1.00', #./lib/constant.pm
        'CPAN'                  => '1.40', #./lib/CPAN.pm
        'CPAN::FirstTime'       => '1.30 ', #./lib/CPAN/FirstTime.pm
        'CPAN::Nox'             => undef, #./lib/CPAN/Nox.pm
        'Cwd'                   => '2.01', #./lib/Cwd.pm
        'DB_File'               => '1.15', #./ext/DB_File/DB_File.pm
        'Devel::SelfStubber'    => '1.01', #./lib/Devel/SelfStubber.pm
        'diagnostics'           => undef, #./lib/diagnostics.pm
        'DirHandle'             => undef, #./lib/DirHandle.pm
        'DynaLoader'            => '1.03',
        'English'               => undef, #./lib/English.pm
        'Env'                   => undef, #./lib/Env.pm
        'Exporter'              => undef, #./lib/Exporter.pm
        'ExtUtils::Command'     => '1.01', #./lib/ExtUtils/Command.pm
        'ExtUtils::Embed'       => '1.2505', #./lib/ExtUtils/Embed.pm
        'ExtUtils::Install'     => '1.28 ', #./lib/ExtUtils/Install.pm
        'ExtUtils::Liblist'     => '1.25 ', #./lib/ExtUtils/Liblist.pm
        'ExtUtils::MakeMaker'   => '5.42', #./lib/ExtUtils/MakeMaker.pm
        'ExtUtils::Manifest'    => '1.33 ', #./lib/ExtUtils/Manifest.pm
        'ExtUtils::Mkbootstrap' => '1.14 ', #./lib/ExtUtils/Mkbootstrap.pm
        'ExtUtils::Mksymlists'  => '1.16 ', #./lib/ExtUtils/Mksymlists.pm
        'ExtUtils::MM_OS2'      => undef, #./lib/ExtUtils/MM_OS2.pm
        'ExtUtils::MM_Unix'     => '1.118 ', #./lib/ExtUtils/MM_Unix.pm
        'ExtUtils::MM_VMS'      => undef, #./lib/ExtUtils/MM_VMS.pm
        'ExtUtils::MM_Win32'    => undef, #./lib/ExtUtils/MM_Win32.pm
        'ExtUtils::testlib'     => '1.11 ', #./lib/ExtUtils/testlib.pm
        'ExtUtils::XSSymSet'    => '1.0', #./vms/ext/XSSymSet.pm
        'Fcntl'                 => '1.03', #./ext/Fcntl/Fcntl.pm
        'File::Basename'        => '2.6', #./lib/File/Basename.pm
        'File::CheckTree'       => undef, #./lib/File/CheckTree.pm
        'File::Compare'         => '1.1001', #./lib/File/Compare.pm
        'File::Copy'            => '2.02', #./lib/File/Copy.pm
        'File::DosGlob'         => undef, #./lib/File/DosGlob.pm
        'File::Find'            => undef, #./lib/File/Find.pm
        'File::Path'            => '1.0402', #./lib/File/Path.pm
        'File::Spec'            => '0.6', #./lib/File/Spec.pm
        'File::Spec::Mac'       => '1.0', #./lib/File/Spec/Mac.pm
        'File::Spec::OS2'       => undef, #./lib/File/Spec/OS2.pm
        'File::Spec::Unix'      => undef, #./lib/File/Spec/Unix.pm
        'File::Spec::VMS'       => undef, #./lib/File/Spec/VMS.pm
        'File::Spec::Win32'     => undef, #./lib/File/Spec/Win32.pm
        'File::stat'            => undef, #./lib/File/stat.pm
        'FileCache'             => undef, #./lib/FileCache.pm
        'FileHandle'            => '2.00', #./lib/FileHandle.pm
        'FindBin'               => '1.41', #./lib/FindBin.pm
        'GDBM_File'             => '1.00', #./ext/GDBM_File/GDBM_File.pm
        'Getopt::Long'          => '2.19', #./lib/Getopt/Long.pm
        'Getopt::Std'           => undef, #./lib/Getopt/Std.pm
        'I18N::Collate'         => undef, #./lib/I18N/Collate.pm
        'integer'               => undef, #./lib/integer.pm
        'IO'                    => undef, #./ext/IO/IO.pm
        'IO::File'              => '1.06021', #./ext/IO/lib/IO/File.pm
        'IO::Handle'            => '1.1504', #./ext/IO/lib/IO/Handle.pm
        'IO::Pipe'              => '1.0901', #./ext/IO/lib/IO/Pipe.pm
        'IO::Seekable'          => '1.06', #./ext/IO/lib/IO/Seekable.pm
        'IO::Select'            => '1.10', #./ext/IO/lib/IO/Select.pm
        'IO::Socket'            => '1.1603', #./ext/IO/lib/IO/Socket.pm
        'IPC::Open2'            => '1.01', #./lib/IPC/Open2.pm
        'IPC::Open3'            => '1.0103', #./lib/IPC/Open3.pm
        'less'                  => undef, #./lib/less.pm
        'lib'                   => undef, #./lib/lib.pm
        'locale'                => undef, #./lib/locale.pm
        'Math::BigFloat'        => undef, #./lib/Math/BigFloat.pm
        'Math::BigInt'          => undef, #./lib/Math/BigInt.pm
        'Math::Complex'         => '1.25', #./lib/Math/Complex.pm
        'Math::Trig'            => '1', #./lib/Math/Trig.pm
        'NDBM_File'             => '1.01', #./ext/NDBM_File/NDBM_File.pm
        'Net::hostent'          => undef, #./lib/Net/hostent.pm
        'Net::netent'           => undef, #./lib/Net/netent.pm
        'Net::Ping'             => '2.02', #./lib/Net/Ping.pm
        'Net::protoent'         => undef, #./lib/Net/protoent.pm
        'Net::servent'          => undef, #./lib/Net/servent.pm
        'ODBM_File'             => '1.00', #./ext/ODBM_File/ODBM_File.pm
        'Opcode'                => '1.04', #./ext/Opcode/Opcode.pm
        'ops'                   => undef, #./ext/Opcode/ops.pm
        'OS2::ExtAttr'          => '0.01', #./os2/OS2/ExtAttr/ExtAttr.pm
        'OS2::PrfDB'            => '0.02', #./os2/OS2/PrfDB/PrfDB.pm
        'OS2::Process'          => undef, #./os2/OS2/Process/Process.pm
        'OS2::REXX'             => undef, #./os2/OS2/REXX/REXX.pm
        'overload'              => undef, #./lib/overload.pm
        'Pod::Functions'        => undef, #./lib/Pod/Functions.pm
        'Pod::Html'             => '1.0101', #./lib/Pod/Html.pm
        'Pod::Text'             => '1.0204', #./lib/Pod/Text.pm
        'POSIX'                 => '1.02', #./ext/POSIX/POSIX.pm
        're'                    => undef, #./lib/re.pm
        'Safe'                  => '2.06', #./ext/Opcode/Safe.pm
        'SDBM_File'             => '1.00', #./ext/SDBM_File/SDBM_File.pm
        'Search::Dict'          => undef, #./lib/Search/Dict.pm
        'SelectSaver'           => undef, #./lib/SelectSaver.pm
        'SelfLoader'            => '1.08', #./lib/SelfLoader.pm
        'Shell'                 => undef, #./lib/Shell.pm
        'sigtrap'               => '1.02', #./lib/sigtrap.pm
        'Socket'                => '1.7', #./ext/Socket/Socket.pm
        'strict'                => '1.01', #./lib/strict.pm
        'subs'                  => undef, #./lib/subs.pm
        'Symbol'                => '1.02', #./lib/Symbol.pm
        'Sys::Hostname'         => undef, #./lib/Sys/Hostname.pm
        'Sys::Syslog'           => undef, #./lib/Sys/Syslog.pm
        'Term::Cap'             => undef, #./lib/Term/Cap.pm
        'Term::Complete'        => undef, #./lib/Term/Complete.pm
        'Term::ReadLine'        => undef, #./lib/Term/ReadLine.pm
        'Test'                  => '1.04', #./lib/Test.pm
        'Test::Harness'         => '1.1602', #./lib/Test/Harness.pm
        'Text::Abbrev'          => undef, #./lib/Text/Abbrev.pm
        'Text::ParseWords'      => '3.1001', #./lib/Text/ParseWords.pm
        'Text::Soundex'         => undef, #./lib/Text/Soundex.pm
        'Text::Tabs'            => '96.121201', #./lib/Text/Tabs.pm
        'Text::Wrap'            => '98.112902', #./lib/Text/Wrap.pm
        'Tie::Handle'           => undef, #./lib/Tie/Handle.pm
        'Tie::Hash'             => undef, #./lib/Tie/Hash.pm
        'Tie::RefHash'          => undef, #./lib/Tie/RefHash.pm
        'Tie::Scalar'           => undef, #./lib/Tie/Scalar.pm
        'Tie::SubstrHash'       => undef, #./lib/Tie/SubstrHash.pm
        'Time::gmtime'          => '1.01', #./lib/Time/gmtime.pm
        'Time::Local'           => undef, #./lib/Time/Local.pm
        'Time::localtime'       => '1.01', #./lib/Time/localtime.pm
        'Time::tm'              => undef, #./lib/Time/tm.pm
        'UNIVERSAL'             => undef, #./lib/UNIVERSAL.pm
        'User::grent'           => undef, #./lib/User/grent.pm
        'User::pwent'           => undef, #./lib/User/pwent.pm
        'vars'                  => undef, #./lib/vars.pm
        'VMS::DCLsym'           => '1.01', #./vms/ext/DCLsym/DCLsym.pm
        'VMS::Filespec'         => undef, #./vms/ext/Filespec.pm
        'VMS::Stdio'            => '2.02', #./vms/ext/Stdio/Stdio.pm
        'vmsish'                => undef, #./vms/ext/vmsish.pm
    },

        5.00504 => {
        'AnyDBM_File'           => undef,  #lib/AnyDBM_File.pm
        'attrs'                 => '1.0',  #lib/attrs.pm
        'AutoLoader'            => undef,  #lib/AutoLoader.pm
        'AutoSplit'             => '1.0303',  #lib/AutoSplit.pm
        'autouse'               => '1.01',  #lib/autouse.pm
        'base'                  => undef,  #lib/base.pm
        'B::Asmdata'            => undef,  #lib/B/Asmdata.pm
        'B::Assembler'          => undef,  #lib/B/Assembler.pm
        'B::Bblock'             => undef,  #lib/B/Bblock.pm
        'B::Bytecode'           => undef,  #lib/B/Bytecode.pm
        'B::CC'                 => undef,  #lib/B/CC.pm
        'B::C'                  => undef,  #lib/B/C.pm
        'B::Debug'              => undef,  #lib/B/Debug.pm
        'B::Deparse'            => '0.56',  #lib/B/Deparse.pm
        'B::Disassembler'       => undef,  #lib/B/Disassembler.pm
        'Benchmark'             => undef,  #lib/Benchmark.pm
        'blib'                  => '1.00',  #lib/blib.pm
        'B::Lint'               => undef,  #lib/B/Lint.pm
        'B::Showlex'            => undef,  #lib/B/Showlex.pm
        'B::Stackobj'           => undef,  #lib/B/Stackobj.pm
        'B::Terse'              => undef,  #lib/B/Terse.pm
        'B'                     => undef,  #lib/B.pm
        'B::Xref'               => undef,  #lib/B/Xref.pm
        'Carp'                  => undef,  #lib/Carp.pm
        'CGI'                   => '2.46',  #lib/CGI.pm
        'CGI::Apache'           => '1.1',  #lib/CGI/Apache.pm
        'CGI::Carp'             => '1.13',  #lib/CGI/Carp.pm
        'CGI::Cookie'           => '1.06',  #lib/CGI/Cookie.pm
        'CGI::Fast'             => '1.01',  #lib/CGI/Fast.pm
        'CGI::Push'             => '1.01',  #lib/CGI/Push.pm
        'CGI::Switch'           => '0.06',  #lib/CGI/Switch.pm
        'Class::Struct'         => undef,  #lib/Class/Struct.pm
        'Config'                => undef,  #lib/Config.pm
        'constant'              => '1.00',  #lib/constant.pm
        'CPAN'                  => '1.48',  #lib/CPAN.pm
        'CPAN::FirstTime'       => '1.36 ',  #lib/CPAN/FirstTime.pm
        'CPAN::Nox'             => '1.00',  #lib/CPAN/Nox.pm
        'Cwd'                   => '2.01',  #lib/Cwd.pm
        'Data::Dumper'          => '2.101',  #lib/Data/Dumper.pm
        'DB_File'               => '1.807',  #lib/DB_File.pm
        'Devel::SelfStubber'    => '1.01',  #lib/Devel/SelfStubber.pm
        'diagnostics'           => undef,  #lib/diagnostics.pm
        'DirHandle'             => undef,  #lib/DirHandle.pm
        'Dumpvalue'             => undef,  #lib/Dumpvalue.pm
        'DynaLoader'            => '1.03',  #lib/DynaLoader.pm
        'English'               => undef,  #lib/English.pm
        'Env'                   => undef,  #lib/Env.pm
        'Errno'                 => '1.111',  #lib/Errno.pm
        'Exporter'              => undef,  #lib/Exporter.pm
        'ExtUtils::Command'     => '1.01',  #lib/ExtUtils/Command.pm
        'ExtUtils::Embed'       => '1.2505',  #lib/ExtUtils/Embed.pm
        'ExtUtils::Install'     => '1.28 ',  #lib/ExtUtils/Install.pm
        'ExtUtils::Installed'   => '0.02',  #lib/ExtUtils/Installed.pm
        'ExtUtils::Liblist'     => '1.25 ',  #lib/ExtUtils/Liblist.pm
        'ExtUtils::MakeMaker'   => '5.4302',  #lib/ExtUtils/MakeMaker.pm
        'ExtUtils::Manifest'    => '1.33 ',  #lib/ExtUtils/Manifest.pm
        'ExtUtils::Miniperl'    => undef,  #lib/ExtUtils/Miniperl.pm
        'ExtUtils::Mkbootstrap' => '1.14 ',  #lib/ExtUtils/Mkbootstrap.pm
        'ExtUtils::Mksymlists'  => '1.17 ',  #lib/ExtUtils/Mksymlists.pm
        'ExtUtils::MM_OS2'      => undef,  #lib/ExtUtils/MM_OS2.pm
        'ExtUtils::MM_Unix'     => '1.12602 ',  #lib/ExtUtils/MM_Unix.pm
        'ExtUtils::MM_VMS'      => undef,  #lib/ExtUtils/MM_VMS.pm
        'ExtUtils::MM_Win32'    => undef,  #lib/ExtUtils/MM_Win32.pm
        'ExtUtils::Packlist'    => '0.03',  #lib/ExtUtils/Packlist.pm
        'ExtUtils::testlib'     => '1.11 ',  #lib/ExtUtils/testlib.pm
        'ExtUtils::XSSymSet'    => '1.0',  #vms/ext/XSSymSet.pm
        'Fatal'                 => '1.02',  #lib/Fatal.pm
        'Fcntl'                 => '1.03',  #lib/Fcntl.pm
        'fields'                => '0.02',  #lib/fields.pm
        'File::Basename'        => '2.6',  #lib/File/Basename.pm
        'FileCache'             => undef,  #lib/FileCache.pm
        'File::CheckTree'       => undef,  #lib/File/CheckTree.pm
        'File::Compare'         => '1.1002',  #lib/File/Compare.pm
        'File::Copy'            => '2.02',  #lib/File/Copy.pm
        'File::DosGlob'         => undef,  #lib/File/DosGlob.pm
        'File::Find'            => undef,  #lib/File/Find.pm
        'FileHandle'            => '2.00',  #lib/FileHandle.pm
        'File::Path'            => '1.0401',  #lib/File/Path.pm
        'File::Spec'            => '0.8',  #lib/File/Spec.pm
        'File::Spec::Functions' => undef,  #lib/File/Spec/Functions.pm
        'File::Spec::Mac'       => undef,  #lib/File/Spec/Mac.pm
        'File::Spec::OS2'       => undef,  #lib/File/Spec/OS2.pm
        'File::Spec::Unix'      => undef,  #lib/File/Spec/Unix.pm
        'File::Spec::VMS'       => undef,  #lib/File/Spec/VMS.pm
        'File::Spec::Win32'     => undef,  #lib/File/Spec/Win32.pm
        'File::stat'            => undef,  #lib/File/stat.pm
        'FindBin'               => '1.42',  #lib/FindBin.pm
        'GDBM_File'             => '1.00',  #lib/GDBM_File.pm
        'Getopt::Long'          => '2.20',  #lib/Getopt/Long.pm
        'Getopt::Std'           => '1.01',  #lib/Getopt/Std.pm
        'I18N::Collate'         => undef,  #lib/I18N/Collate.pm
        'integer'               => undef,  #lib/integer.pm
        'IO::File'              => '1.06021',  #lib/IO/File.pm
        'IO::Handle'            => '1.1505',  #lib/IO/Handle.pm
        'IO::Pipe'              => '1.0902',  #lib/IO/Pipe.pm
        'IO::Seekable'          => '1.06',  #lib/IO/Seekable.pm
        'IO::Select'            => '1.10',  #lib/IO/Select.pm
        'IO::Socket'            => '1.1603',  #lib/IO/Socket.pm
        'IO'                    => undef,  #lib/IO.pm
        'IPC::Msg'              => '1.00',  #lib/IPC/Msg.pm
        'IPC::Open2'            => '1.01',  #lib/IPC/Open2.pm
        'IPC::Open3'            => '1.0103',  #lib/IPC/Open3.pm
        'IPC::Semaphore'        => '1.00',  #lib/IPC/Semaphore.pm
        'IPC::SysV'             => '1.03',  #lib/IPC/SysV.pm
        'less'                  => undef,  #lib/less.pm
        'lib'                   => undef,  #lib/lib.pm
        'locale'                => undef,  #lib/locale.pm
        'Math::BigFloat'        => undef,  #lib/Math/BigFloat.pm
        'Math::BigInt'          => undef,  #lib/Math/BigInt.pm
        'Math::Complex'         => '1.26',  #lib/Math/Complex.pm
        'Math::Trig'            => '1',  #lib/Math/Trig.pm
        'NDBM_File'             => '1.01',  #ext/NDBM_File/NDBM_File.pm
        'Net::hostent'          => undef,  #lib/Net/hostent.pm
        'Net::netent'           => undef,  #lib/Net/netent.pm
        'Net::Ping'             => '2.02',  #lib/Net/Ping.pm
        'Net::protoent'         => undef,  #lib/Net/protoent.pm
        'Net::servent'          => undef,  #lib/Net/servent.pm
        'ODBM_File'             => '1.00', #ext/ODBM_File/ODBM_File.pm
        'Opcode'                => '1.04',  #lib/Opcode.pm
        'ops'                   => undef,  #lib/ops.pm
        'O'                     => undef,  #lib/O.pm
        'OS2::ExtAttr'          => '0.01',  #os2/OS2/ExtAttr/ExtAttr.pm
        'OS2::PrfDB'            => '0.02',  #os2/OS2/PrfDB/PrfDB.pm
        'OS2::Process'          => '0.2',  #os2/OS2/Process/Process.pm
        'OS2::REXX'             => undef,  #os2/OS2/REXX/REXX.pm
        'overload'              => undef,  #lib/overload.pm
        'Pod::Functions'        => undef,  #lib/Pod/Functions.pm
        'Pod::Html'             => '1.02',  #lib/Pod/Html.pm
        'Pod::Text'             => '1.0203',  #lib/Pod/Text.pm
        'POSIX'                 => '1.02',  #lib/POSIX.pm
        're'                    => '0.02',  #lib/re.pm
        'Safe'                  => '2.06',  #lib/Safe.pm
        'SDBM_File'             => '1.00',  #lib/SDBM_File.pm
        'Search::Dict'          => undef,  #lib/Search/Dict.pm
        'SelectSaver'           => undef,  #lib/SelectSaver.pm
        'SelfLoader'            => '1.08',  #lib/SelfLoader.pm
        'Shell'                 => undef,  #lib/Shell.pm
        'sigtrap'               => '1.02',  #lib/sigtrap.pm
        'Socket'                => '1.7',  #lib/Socket.pm
        'strict'                => '1.01',  #lib/strict.pm
        'subs'                  => undef,  #lib/subs.pm
        'Symbol'                => '1.02',  #lib/Symbol.pm
        'Sys::Hostname'         => undef,  #lib/Sys/Hostname.pm
        'Sys::Syslog'           => undef,  #lib/Sys/Syslog.pm
        'Term::Cap'             => undef,  #lib/Term/Cap.pm
        'Term::Complete'        => undef,  #lib/Term/Complete.pm
        'Term::ReadLine'        => undef,  #lib/Term/ReadLine.pm
        'Test'                  => '1.122',  #lib/Test.pm
        'Test::Harness'         => '1.1602',  #lib/Test/Harness.pm
        'Text::Abbrev'          => undef,  #lib/Text/Abbrev.pm
        'Text::ParseWords'      => '3.1',  #lib/Text/ParseWords.pm
        'Text::Soundex'         => undef,  #lib/Text/Soundex.pm
        'Text::Tabs'            => '96.121201',  #lib/Text/Tabs.pm
        'Text::Wrap'            => '98.112902',  #lib/Text/Wrap.pm
        'Thread'                => '1.0',  #ext/Thread/Thread.pm
        'Thread::Queue'         => undef,  #ext/Thread/Thread/Queue.pm
        'Thread::Semaphore'     => undef,  #ext/Thread/Thread/Semaphore.pm
        'Thread::Signal'        => undef,  #ext/Thread/Thread/Signal.pm
        'Thread::Specific'      => undef,  #ext/Thread/Thread/Specific.pm
        'Tie::Array'            => '1.00',  #lib/Tie/Array.pm
        'Tie::Handle'           => undef,  #lib/Tie/Handle.pm
        'Tie::Hash'             => undef,  #lib/Tie/Hash.pm
        'Tie::RefHash'          => undef,  #lib/Tie/RefHash.pm
        'Tie::Scalar'           => undef,  #lib/Tie/Scalar.pm
        'Tie::SubstrHash'       => undef,  #lib/Tie/SubstrHash.pm
        'Time::gmtime'          => '1.01',  #lib/Time/gmtime.pm
        'Time::localtime'       => '1.01',  #lib/Time/localtime.pm
        'Time::Local'           => undef,  #lib/Time/Local.pm
        'Time::tm'              => undef,  #lib/Time/tm.pm
        'UNIVERSAL'             => undef,  #lib/UNIVERSAL.pm
        'User::grent'           => undef,  #lib/User/grent.pm
        'User::pwent'           => undef,  #lib/User/pwent.pm
        'vars'                  => undef,  #lib/vars.pm
        'VMS::DCLsym'           => '1.01',  #vms/ext/DCLsym/DCLsym.pm
        'VMS::Filespec'         => undef,  #vms/ext/Filespec.pm
        'VMS::Stdio'            => '2.1',  #vms/ext/Stdio/Stdio.pm
        'vmsish'                => undef,  #vms/ext/vmsish.pm
        },

    5.006   => {
        'AnyDBM_File'           => undef, #./lib/AnyDBM_File.pm
        'AutoLoader'            => '5.57', #./lib/AutoLoader.pm
        'AutoSplit'             => '1.0305', #./lib/AutoSplit.pm
        'B'                     => undef, #./ext/B/B.pm
        'B::Asmdata'            => undef, #./ext/B/B/Asmdata.pm
        'B::Assembler'          => undef, #./ext/B/B/Assembler.pm
        'B::Bblock'             => undef, #./ext/B/B/Bblock.pm
        'B::Bytecode'           => undef, #./ext/B/B/Bytecode.pm
        'B::C'                  => undef, #./ext/B/B/C.pm
        'B::CC'                 => undef, #./ext/B/B/CC.pm
        'B::Debug'              => undef, #./ext/B/B/Debug.pm
        'B::Deparse'            => '0.59', #./ext/B/B/Deparse.pm
        'B::Disassembler'       => undef, #./ext/B/B/Disassembler.pm
        'B::Lint'               => undef, #./ext/B/B/Lint.pm
        'B::Showlex'            => undef, #./ext/B/B/Showlex.pm
        'B::Stackobj'           => undef, #./ext/B/B/Stackobj.pm
        'B::Stash'              => undef, #./ext/B/B/Stash.pm
        'B::Terse'              => undef, #./ext/B/B/Terse.pm
        'B::Xref'               => undef, #./ext/B/B/Xref.pm
        'Benchmark'             => '1', #./lib/Benchmark.pm
        'ByteLoader'            => '0.03', #./ext/ByteLoader/ByteLoader.pm
        'CGI'                   => '2.56', #./lib/CGI.pm
        'CGI::Apache'           => undef, #./lib/CGI/Apache.pm
        'CGI::Carp'             => '1.14', #./lib/CGI/Carp.pm
        'CGI::Cookie'           => '1.12', #./lib/CGI/Cookie.pm
        'CGI::Fast'             => '1.02', #./lib/CGI/Fast.pm
        'CGI::Pretty'           => '1.03', #./lib/CGI/Pretty.pm
        'CGI::Push'             => '1.01', #./lib/CGI/Push.pm
        'CGI::Switch'           => undef, #./lib/CGI/Switch.pm
        'CPAN'                  => '1.52', #./lib/CPAN.pm
        'CPAN::FirstTime'       => '1.38 ', #./lib/CPAN/FirstTime.pm
        'CPAN::Nox'             => '1.00', #./lib/CPAN/Nox.pm
        'Carp'                  => undef, #./lib/Carp.pm
        'Carp::Heavy'           => undef, #./lib/Carp/Heavy.pm
        'Class::Struct'         => '0.58', #./lib/Class/Struct.pm
        'Config'                => undef,
        'Cwd'                   => '2.02', #./lib/Cwd.pm
        'DB'                    => '1.0', #./lib/DB.pm
        'DB_File'               => '1.72', #./ext/DB_File/DB_File.pm
        'Data::Dumper'          => '2.101', #./ext/Data/Dumper/Dumper.pm
        'Devel::DProf'          => '20000000.00_00', #./ext/Devel/DProf/DProf.pm
        'Devel::Peek'           => '1.00_01', #./ext/Devel/Peek/Peek.pm
        'Devel::SelfStubber'    => '1.01', #./lib/Devel/SelfStubber.pm
        'DirHandle'             => undef, #./lib/DirHandle.pm
        'Dumpvalue'             => undef, #./lib/Dumpvalue.pm
        'DynaLoader'            => '1.04',
        'English'               => undef, #./lib/English.pm
        'Env'                   => undef, #./lib/Env.pm
        'Exporter'              => '5.562', #./lib/Exporter.pm
        'Exporter::Heavy'       => undef, #./lib/Exporter/Heavy.pm
        'ExtUtils::Command'     => '1.01', #./lib/ExtUtils/Command.pm
        'ExtUtils::Embed'       => '1.2505', #./lib/ExtUtils/Embed.pm
        'ExtUtils::Install'     => '1.28 ', #./lib/ExtUtils/Install.pm
        'ExtUtils::Installed'   => '0.02', #./lib/ExtUtils/Installed.pm
        'ExtUtils::Liblist'     => '1.25 ', #./lib/ExtUtils/Liblist.pm
        'ExtUtils::MM_Cygwin'   => undef, #./lib/ExtUtils/MM_Cygwin.pm
        'ExtUtils::MM_OS2'      => undef, #./lib/ExtUtils/MM_OS2.pm
        'ExtUtils::MM_Unix'     => '1.12603 ', #./lib/ExtUtils/MM_Unix.pm
        'ExtUtils::MM_VMS'      => undef, #./lib/ExtUtils/MM_VMS.pm
        'ExtUtils::MM_Win32'    => undef, #./lib/ExtUtils/MM_Win32.pm
        'ExtUtils::MakeMaker'   => '5.45', #./lib/ExtUtils/MakeMaker.pm
        'ExtUtils::Manifest'    => '1.33 ', #./lib/ExtUtils/Manifest.pm
        'ExtUtils::Mkbootstrap' => '1.14 ', #./lib/ExtUtils/Mkbootstrap.pm
        'ExtUtils::Mksymlists'  => '1.17 ', #./lib/ExtUtils/Mksymlists.pm
        'ExtUtils::Packlist'    => '0.03', #./lib/ExtUtils/Packlist.pm
        'ExtUtils::XSSymSet'    => '1.0', #./vms/ext/XSSymSet.pm
        'ExtUtils::testlib'     => '1.11 ', #./lib/ExtUtils/testlib.pm
        'Fatal'                 => '1.02', #./lib/Fatal.pm
        'Fcntl'                 => '1.03', #./ext/Fcntl/Fcntl.pm
        'File::Basename'        => '2.6', #./lib/File/Basename.pm
        'File::CheckTree'       => undef, #./lib/File/CheckTree.pm
        'File::Compare'         => '1.1002', #./lib/File/Compare.pm
        'File::Copy'            => '2.03', #./lib/File/Copy.pm
        'File::DosGlob'         => undef, #./lib/File/DosGlob.pm
        'File::Find'            => undef, #./lib/File/Find.pm
        'File::Glob'            => '0.991', #./ext/File/Glob/Glob.pm
        'File::Path'            => '1.0403', #./lib/File/Path.pm
        'File::Spec'            => '0.8', #./lib/File/Spec.pm
        'File::Spec::Functions' => undef, #./lib/File/Spec/Functions.pm
        'File::Spec::Mac'       => undef, #./lib/File/Spec/Mac.pm
        'File::Spec::OS2'       => undef, #./lib/File/Spec/OS2.pm
        'File::Spec::Unix'      => undef, #./lib/File/Spec/Unix.pm
        'File::Spec::VMS'       => undef, #./lib/File/Spec/VMS.pm
        'File::Spec::Win32'     => undef, #./lib/File/Spec/Win32.pm
        'File::stat'            => undef, #./lib/File/stat.pm
        'FileCache'             => undef, #./lib/FileCache.pm
        'FileHandle'            => '2.00', #./lib/FileHandle.pm
        'FindBin'               => '1.42', #./lib/FindBin.pm
        'GDBM_File'             => '1.03', #./ext/GDBM_File/GDBM_File.pm
        'Getopt::Long'          => '2.23', #./lib/Getopt/Long.pm
        'Getopt::Std'           => '1.02', #./lib/Getopt/Std.pm
        'I18N::Collate'         => undef, #./lib/I18N/Collate.pm
        'IO'                    => '1.20', #./ext/IO/IO.pm
        'IO::Dir'               => '1.03', #./ext/IO/lib/IO/Dir.pm
        'IO::File'              => '1.08', #./ext/IO/lib/IO/File.pm
        'IO::Handle'            => '1.21', #./ext/IO/lib/IO/Handle.pm
        'IO::Pipe'              => '1.121', #./ext/IO/lib/IO/Pipe.pm
        'IO::Poll'              => '0.01', #./ext/IO/lib/IO/Poll.pm
        'IO::Seekable'          => '1.08', #./ext/IO/lib/IO/Seekable.pm
        'IO::Select'            => '1.14', #./ext/IO/lib/IO/Select.pm
        'IO::Socket'            => '1.26', #./ext/IO/lib/IO/Socket.pm
        'IO::Socket::INET'      => '1.25', #./ext/IO/lib/IO/Socket/INET.pm
        'IO::Socket::UNIX'      => '1.20', #./ext/IO/lib/IO/Socket/UNIX.pm
        'IPC::Open2'            => '1.01', #./lib/IPC/Open2.pm
        'IPC::Open3'            => '1.0103', #./lib/IPC/Open3.pm
        'IPC::Msg'              => '1.00', #./ext/IPC/SysV/Msg.pm
        'IPC::Semaphore'        => '1.00', #./ext/IPC/SysV/Semaphore.pm
        'IPC::SysV'             => '1.03', #./ext/IPC/SysV/SysV.pm
        'JNI'                   => '0.01', #./jpl/JNI/JNI.pm
        'JPL::AutoLoader'       => undef, #./jpl/JPL/AutoLoader.pm
        'JPL::Class'            => undef, #./jpl/JPL/Class.pm
        'JPL::Compile'          => undef, #./jpl/JPL/Compile.pm
        'Math::BigFloat'        => undef, #./lib/Math/BigFloat.pm
        'Math::BigInt'          => undef, #./lib/Math/BigInt.pm
        'Math::Complex'         => '1.26', #./lib/Math/Complex.pm
        'Math::Trig'            => '1', #./lib/Math/Trig.pm
        'NDBM_File'             => '1.03', #./ext/NDBM_File/NDBM_File.pm
        'Net::Ping'             => '2.02', #./lib/Net/Ping.pm
        'Net::hostent'          => undef, #./lib/Net/hostent.pm
        'Net::netent'           => undef, #./lib/Net/netent.pm
        'Net::protoent'         => undef, #./lib/Net/protoent.pm
        'Net::servent'          => undef, #./lib/Net/servent.pm
        'O'                     => undef, #./ext/B/O.pm
        'ODBM_File'             => '1.02', #./ext/ODBM_File/ODBM_File.pm
        'OS2::ExtAttr'          => '0.01', #./os2/OS2/ExtAttr/ExtAttr.pm
        'OS2::PrfDB'            => '0.02', #./os2/OS2/PrfDB/PrfDB.pm
        'OS2::Process'          => '0.2', #./os2/OS2/Process/Process.pm
        'OS2::REXX'             => undef, #./os2/OS2/REXX/REXX.pm
        'OS2::DLL'              => undef, #./os2/OS2/REXX/DLL/DLL.pm
        'Opcode'                => '1.04', #./ext/Opcode/Opcode.pm
        'POSIX'                 => '1.03', #./ext/POSIX/POSIX.pm
        'Pod::Checker'          => '1.098', #./lib/Pod/Checker.pm
        'Pod::Find'             => '0.12', #./lib/Pod/Find.pm
        'Pod::Functions'        => undef, #./lib/Pod/Functions.pm
        'Pod::Html'             => '1.03', #./lib/Pod/Html.pm
        'Pod::InputObjects'     => '1.12', #./lib/Pod/InputObjects.pm
        'Pod::Man'              => '1.02', #./lib/Pod/Man.pm
        'Pod::ParseUtils'       => '0.2', #./lib/Pod/ParseUtils.pm
        'Pod::Parser'           => '1.12', #./lib/Pod/Parser.pm
        'Pod::Plainer'          => '0.01', #./lib/Pod/Plainer.pm
        'Pod::Select'           => '1.12', #./lib/Pod/Select.pm
        'Pod::Text'             => '2.03', #./lib/Pod/Text.pm
        'Pod::Text::Color'      => '0.05', #./lib/Pod/Text/Color.pm
        'Pod::Text::Termcap'    => '0.04', #./lib/Pod/Text/Termcap.pm
        'Pod::Usage'            => '1.12', #./lib/Pod/Usage.pm
        'SDBM_File'             => '1.02', #./ext/SDBM_File/SDBM_File.pm
        'Safe'                  => '2.06', #./ext/Opcode/Safe.pm
        'Search::Dict'          => undef, #./lib/Search/Dict.pm
        'SelectSaver'           => undef, #./lib/SelectSaver.pm
        'SelfLoader'            => '1.0901', #./lib/SelfLoader.pm
        'Shell'                 => '0.2', #./lib/Shell.pm
        'Socket'                => '1.72', #./ext/Socket/Socket.pm
        'Symbol'                => '1.02', #./lib/Symbol.pm
        'Sys::Hostname'         => '1.1', #./ext/Sys/Hostname/Hostname.pm
        'Sys::Syslog'           => '0.01', #./ext/Sys/Syslog/Syslog.pm
        'Term::ANSIColor'       => '1.01', #./lib/Term/ANSIColor.pm
        'Term::Cap'             => undef, #./lib/Term/Cap.pm
        'Term::Complete'        => undef, #./lib/Term/Complete.pm
        'Term::ReadLine'        => undef, #./lib/Term/ReadLine.pm
        'Test'                  => '1.13', #./lib/Test.pm
        'Test::Harness'         => '1.1604', #./lib/Test/Harness.pm
        'Text::Abbrev'          => undef, #./lib/Text/Abbrev.pm
        'Text::ParseWords'      => '3.2', #./lib/Text/ParseWords.pm
        'Text::Soundex'         => '1.0', #./lib/Text/Soundex.pm
        'Text::Tabs'            => '98.112801', #./lib/Text/Tabs.pm
        'Text::Wrap'            => '98.112902', #./lib/Text/Wrap.pm
        'Thread'                => '1.0', #./ext/Thread/Thread.pm
        'Thread::Queue'         => undef, #./ext/Thread/Thread/Queue.pm
        'Thread::Semaphore'     => undef, #./ext/Thread/Thread/Semaphore.pm
        'Thread::Signal'        => undef, #./ext/Thread/Thread/Signal.pm
        'Thread::Specific'      => undef, #./ext/Thread/Thread/Specific.pm
        'Tie::Array'            => '1.01', #./lib/Tie/Array.pm
        'Tie::Handle'           => '1.0', #./lib/Tie/Handle.pm
        'Tie::Hash'             => undef, #./lib/Tie/Hash.pm
        'Tie::RefHash'          => undef, #./lib/Tie/RefHash.pm
        'Tie::Scalar'           => undef, #./lib/Tie/Scalar.pm
        'Tie::SubstrHash'       => undef, #./lib/Tie/SubstrHash.pm
        'Time::Local'           => undef, #./lib/Time/Local.pm
        'Time::gmtime'          => '1.01', #./lib/Time/gmtime.pm
        'Time::localtime'       => '1.01', #./lib/Time/localtime.pm
        'Time::tm'              => undef, #./lib/Time/tm.pm
        'UNIVERSAL'             => undef, #./lib/UNIVERSAL.pm
        'User::grent'           => undef, #./lib/User/grent.pm
        'User::pwent'           => undef, #./lib/User/pwent.pm
        'VMS::DCLsym'           => '1.01', #./vms/ext/DCLsym/DCLsym.pm
        'VMS::Filespec'         => undef, #./vms/ext/Filespec.pm
        'VMS::Stdio'            => '2.2', #./vms/ext/Stdio/Stdio.pm
        'XSLoader'              => '0.01',
        'attributes'            => '0.03', #./lib/attributes.pm
        'attrs'                 => '1.0', #./ext/attrs/attrs.pm
        'autouse'               => '1.02', #./lib/autouse.pm
        'base'                  => '1.01', #./lib/base.pm
        'blib'                  => '1.00', #./lib/blib.pm
        'bytes'                 => undef, #./lib/bytes.pm
        'charnames'             => undef, #./lib/charnames.pm
        'constant'              => '1.02', #./lib/constant.pm
        'diagnostics'           => '1.0', #./lib/diagnostics.pm
        'fields'                => '1.01', #./lib/fields.pm
        'filetest'              => undef, #./lib/filetest.pm
        'integer'               => undef, #./lib/integer.pm
        'less'                  => undef, #./lib/less.pm
        'lib'                   => '0.5564', #./lib/lib.pm
        'locale'                => undef, #./lib/locale.pm
        'open'                  => undef, #./lib/open.pm
        'ops'                   => undef, #./ext/Opcode/ops.pm
        'overload'              => undef, #./lib/overload.pm
        're'                    => '0.02', #./ext/re/re.pm
        'sigtrap'               => '1.02', #./lib/sigtrap.pm
        'strict'                => '1.01', #./lib/strict.pm
        'subs'                  => undef, #./lib/subs.pm
        'utf8'                  => undef, #./lib/utf8.pm
        'vars'                  => undef, #./lib/vars.pm
        'vmsish'                => undef, #./vms/ext/vmsish.pm
        'warnings'              => undef, #./lib/warnings.pm
        'warnings::register'    => undef, #./lib/warnings/register.pm
    },
    5.006001   => {
        'AnyDBM_File'           => undef,
        'attributes'            => 0.03,
        'attrs'                 => '1.0',
        'AutoLoader'            => 5.58,
        'AutoSplit'             => 1.0305,
        'autouse'               => 1.02,
        'B::Asmdata'            => undef,
        'B::Assembler'          => 0.02,
        'B::Bblock'             => undef,
        'B::Bytecode'           => undef,
        'B::C'                  => undef,
        'B::CC'                 => undef,
        'B::Concise'            => 0.51,
        'B::Debug'              => undef,
        'B::Deparse'            => 0.6,
        'B::Disassembler'       => undef,
        'B::Lint'               => undef,
        'B'                     => undef,
        'B::Showlex'            => undef,
        'B::Stackobj'           => undef,
        'B::Stash'              => undef,
        'B::Terse'              => undef,
        'B::Xref'               => undef,
        'base'                  => 1.01,
        'Benchmark'             => 1,
        'blib'                  => '1.00',
        'ByteLoader'            => 0.04,
        'bytes'                 => undef,
        'Carp'                  => undef,
        'Carp::Heavy'           => undef,
        'CGI'                   => 2.752,
        'CGI::Apache'           => undef,
        'CGI::Carp'             => '1.20',
        'CGI::Cookie'           => 1.18,
        'CGI::Fast'             => 1.02,
        'CGI::Pretty'           => 1.05,
        'CGI::Push'             => 1.04,
        'CGI::Switch'           => undef,
        'CGI::Util'             => 1.1,
        'charnames'             => undef,
        'Class::Struct'         => 0.59,
        'Config'                => undef,
        'constant'              => 1.02,
        'CPAN::FirstTime'       => 1.53 ,
        'CPAN'                  => '1.59_54',
        'CPAN::Nox'             => '1.00',
        'Cwd'                   => 2.04,
        'Data::Dumper'          => 2.102,
        'DB'                    => '1.0',
        'DB_File'               => 1.75,
        'Devel::DProf'          => '20000000.00_00',
        'Devel::Peek'           => '1.00_01',
        'Devel::SelfStubber'    => 1.01,
        'diagnostics'           => '1.0', # really v1.0, but that causes breakage
        'DirHandle'             => undef,
        'Dumpvalue'             => undef,
        'DynaLoader'            => 1.04,
        'English'               => undef,
        'Env'                   => undef,
        'Exporter'              => 5.562,
        'Exporter::Heavy'       => undef,
        'ExtUtils::Command'     => 1.01,
        'ExtUtils::Embed'       => 1.2505,
        'ExtUtils::Install'     => 1.28 ,
        'ExtUtils::Installed'   => 0.02,
        'ExtUtils::Liblist'     => 1.26 ,
        'ExtUtils::MakeMaker'   => 5.45,
        'ExtUtils::Manifest'    => 1.33 ,
        'ExtUtils::Mkbootstrap' => 1.14 ,
        'ExtUtils::Mksymlists'  => 1.17 ,
        'ExtUtils::MM_Cygwin'   => undef,
        'ExtUtils::MM_OS2'      => undef,
        'ExtUtils::MM_Unix'     => 1.12603 ,
        'ExtUtils::MM_VMS'      => undef,
        'ExtUtils::MM_Win32'    => undef,
        'ExtUtils::Packlist'    => 0.03,
        'ExtUtils::testlib'     => 1.11 ,
        'ExtUtils::XSSymSet'    => '1.0',
        'Fatal'                 => 1.02,
        'Fcntl'                 => 1.03,
        'fields'                => 1.01,
        'File::Basename'        => 2.6,
        'File::CheckTree'       => undef,
        'File::Compare'         => 1.1002,
        'File::Copy'            => 2.03,
        'File::DosGlob'         => undef,
        'File::Find'            => undef,
        'File::Glob'            => 0.991,
        'File::Path'            => 1.0404,
        'File::Spec'            => 0.82,
        'File::Spec::Epoc'      => undef,
        'File::Spec::Functions' => 1.1,
        'File::Spec::Mac'       => 1.2,
        'File::Spec::OS2'       => 1.1,
        'File::Spec::Unix'      => 1.2,
        'File::Spec::VMS'       => 1.1,
        'File::Spec::Win32'     => 1.2,
        'File::stat'            => undef,
        'File::Temp'            => 0.12,
        'FileCache'             => undef,
        'FileHandle'            => '2.00',
        'filetest'              => undef,
        'FindBin'               => 1.42,
        'GDBM_File'             => 1.05,
        'Getopt::Long'          => 2.25,
        'Getopt::Std'           => 1.02,
        'I18N::Collate'         => undef,
        'integer'               => undef,
        'IO'                    => '1.20',
        'IO::Dir'               => 1.03,
        'IO::File'              => 1.08,
        'IO::Handle'            => 1.21,
        'IO::Pipe'              => 1.121,
        'IO::Poll'              => 0.05,
        'IO::Seekable'          => 1.08,
        'IO::Select'            => 1.14,
        'IO::Socket'            => 1.26,
        'IO::Socket::INET'      => 1.25,
        'IO::Socket::UNIX'      => '1.20',
        'IPC::Msg'              => '1.00',
        'IPC::Open2'            => 1.01,
        'IPC::Open3'            => 1.0103,
        'IPC::Semaphore'        => '1.00',
        'IPC::SysV'             => 1.03,
        'JNI'                   => 0.1,
        'JPL::AutoLoader'       => undef,
        'JPL::Class'            => undef,
        'JPL::Compile'          => undef,
        'less'                  => undef,
        'lib'                   => 0.5564,
        'locale'                => undef,
        'Math::BigFloat'        => 0.02,
        'Math::BigInt'          => 0.01,
        'Math::Complex'         => 1.31,
        'Math::Trig'            => 1,
        'NDBM_File'             => 1.04,
        'Net::hostent'          => undef,
        'Net::netent'           => undef,
        'Net::Ping'             => 2.02,
        'Net::protoent'         => undef,
        'Net::servent'          => undef,
        'O'                     => undef,
        'ODBM_File'             => 1.03,
        'Opcode'                => 1.04,
        'open'                  => undef,
        'ops'                   => undef,
        'OS2::DLL'              => undef,
        'OS2::ExtAttr'          => 0.01,
        'OS2::PrfDB'            => 0.02,
        'OS2::Process'          => 0.2,
        'OS2::REXX'             => '1.00',
        'overload'              => undef,
        'Pod::Checker'          => 1.2,
        'Pod::Find'             => 0.21,
        'Pod::Functions'        => undef,
        'Pod::Html'             => 1.03,
        'Pod::LaTeX'            => 0.53,
        'Pod::Man'              => 1.15,
        'Pod::InputObjects'     => 1.13,
        'Pod::Parser'           => 1.13,
        'Pod::ParseUtils'       => 0.22,
        'Pod::Plainer'          => 0.01,
        'Pod::Select'           => 1.13,
        'Pod::Text'             => 2.08,
        'Pod::Text::Color'      => 0.06,
        'Pod::Text::Overstrike' => 1.01,
        'Pod::Text::Termcap'    => 1,
        'Pod::Usage'            => 1.14,
        'POSIX'                 => 1.03,
        're'                    => 0.02,
        'Safe'                  => 2.06,
        'SDBM_File'             => 1.03,
        'Search::Dict'          => undef,
        'SelectSaver'           => undef,
        'SelfLoader'            => 1.0902,
        'Shell'                 => 0.3,
        'sigtrap'               => 1.02,
        'Socket'                => 1.72,
        'strict'                => 1.01,
        'subs'                  => undef,
        'Symbol'                => 1.02,
        'Sys::Hostname'         => 1.1,
        'Sys::Syslog'           => 0.01,
        'Term::ANSIColor'       => 1.03,
        'Term::Cap'             => undef,
        'Term::Complete'        => undef,
        'Term::ReadLine'        => undef,
        'Test'                  => 1.15,
        'Test::Harness'         => 1.1604,
        'Text::Abbrev'          => undef,
        'Text::ParseWords'      => 3.2,
        'Text::Soundex'         => '1.0',
        'Text::Tabs'            => 98.112801,
        'Text::Wrap'            => 2001.0131,
        'Thread'                => '1.0',
        'Thread::Queue'         => undef,
        'Thread::Semaphore'     => undef,
        'Thread::Signal'        => undef,
        'Thread::Specific'      => undef,
        'Tie::Array'            => 1.01,
        'Tie::Handle'           => '4.0',
        'Tie::Hash'             => undef,
        'Tie::RefHash'          => 1.3,
        'Tie::Scalar'           => undef,
        'Tie::SubstrHash'       => undef,
        'Time::gmtime'          => 1.01,
        'Time::Local'           => undef,
        'Time::localtime'       => 1.01,
        'Time::tm'              => undef,
        'UNIVERSAL'             => undef,
        'User::grent'           => undef,
        'User::pwent'           => undef,
        'utf8'                  => undef,
        'vars'                  => undef,
        'VMS::DCLsym'           => 1.01,
        'VMS::Filespec'         => undef,
        'VMS::Stdio'            => 2.2,
        'vmsish'                => undef,
        'warnings'              => undef,
        'warnings::register'    => undef,
        'XSLoader'              => '0.01',
    },
    5.006002 => {
        'AnyDBM_File'           => undef,  #lib/AnyDBM_File.pm
        'attributes'            => '0.03',  #lib/attributes.pm
        'attrs'                 => '1.0',  #lib/attrs.pm
        'AutoLoader'            => '5.58',  #lib/AutoLoader.pm
        'AutoSplit'             => '1.0305',  #lib/AutoSplit.pm
        'autouse'               => '1.02',  #lib/autouse.pm
        'B'                     => undef,  #lib/B.pm
        'B::Asmdata'            => undef,  #lib/B/Asmdata.pm
        'B::Assembler'          => '0.02',  #lib/B/Assembler.pm
        'B::Bblock'             => undef,  #lib/B/Bblock.pm
        'B::Bytecode'           => undef,  #lib/B/Bytecode.pm
        'B::C'                  => undef,  #lib/B/C.pm
        'B::CC'                 => undef,  #lib/B/CC.pm
        'B::Concise'            => '0.51',  #lib/B/Concise.pm
        'B::Debug'              => undef,  #lib/B/Debug.pm
        'B::Deparse'            => '0.6',  #lib/B/Deparse.pm
        'B::Disassembler'       => undef,  #lib/B/Disassembler.pm
        'B::Lint'               => undef,  #lib/B/Lint.pm
        'B::Showlex'            => undef,  #lib/B/Showlex.pm
       'B::Stackobj'           => undef,  #lib/B/Stackobj.pm
        'B::Stash'              => undef,  #lib/B/Stash.pm
        'B::Terse'              => undef,  #lib/B/Terse.pm
        'B::Xref'               => undef,  #lib/B/Xref.pm
        'base'                  => '1.01',  #lib/base.pm
        'Benchmark'             => '1',  #lib/Benchmark.pm
        'blib'                  => '1.00',  #lib/blib.pm
        'ByteLoader'            => '0.04',  #lib/ByteLoader.pm
        'bytes'                 => undef,  #lib/bytes.pm
        'Carp'                  => undef,  #lib/Carp.pm
        'Carp::Heavy'           => undef,  #lib/Carp/Heavy.pm
        'CGI'                   => '2.752',  #lib/CGI.pm
        'CGI::Apache'           => undef,  #lib/CGI/Apache.pm
        'CGI::Carp'             => '1.20',  #lib/CGI/Carp.pm
        'CGI::Cookie'           => '1.18',  #lib/CGI/Cookie.pm
        'CGI::Fast'             => '1.02',  #lib/CGI/Fast.pm
        'CGI::Pretty'           => '1.05',  #lib/CGI/Pretty.pm
        'CGI::Push'             => '1.04',  #lib/CGI/Push.pm
        'CGI::Switch'           => undef,  #lib/CGI/Switch.pm
        'CGI::Util'             => '1.1',  #lib/CGI/Util.pm
        'charnames'             => undef,  #lib/charnames.pm
        'Class::Struct'         => '0.59',  #lib/Class/Struct.pm
        'Config'                => undef,  #lib/Config.pm
        'constant'              => '1.02',  #lib/constant.pm
        'CPAN'                  => '1.59_54',  #lib/CPAN.pm
        'CPAN::FirstTime'       => '1.53 ',  #lib/CPAN/FirstTime.pm
        'CPAN::Nox'             => '1.00',  #lib/CPAN/Nox.pm
        'Cwd'                   => '2.04', #lib/Cwd.pm
        'Data::Dumper'          => '2.121', #lib/Data/Dumper.pm
        'DB'                    => '1.0', #lib/DB.pm
        'DB_File'               => '1.806', #lib/DB_File.pm
        'Devel::DProf'          => '20000000.00_00', #lib/Devel/DProf.pm
        'Devel::Peek'           => '1.00_01', #lib/Devel/Peek.pm
        'Devel::SelfStubber'    => '1.01', #lib/Devel/SelfStubber.pm
        'diagnostics'           => '1.0', #lib/diagnostics.pm
        'DirHandle'             => undef, #lib/DirHandle.pm
        'Dumpvalue'             => undef, #lib/Dumpvalue.pm
        'DynaLoader'            => '1.04', #lib/DynaLoader.pm
        'English'               => undef, #lib/English.pm
        'Env'                   => undef, #lib/Env.pm
        'Errno'                 => '1.111', #lib/Errno.pm
        'Exporter'              => '5.562', #lib/Exporter.pm
        'Exporter::Heavy'       => undef, #lib/Exporter/Heavy.pm
        'ExtUtils::Command'     => '1.05', #lib/ExtUtils/Command.pm
        'ExtUtils::Command::MM' => '0.03', #lib/ExtUtils/Command/MM.pm
        'ExtUtils::Embed'       => '1.2505', #lib/ExtUtils/Embed.pm
        'ExtUtils::Install'     => '1.32', #lib/ExtUtils/Install.pm
        'ExtUtils::Installed'   => '0.08', #lib/ExtUtils/Installed.pm
        'ExtUtils::Liblist'     => '1.01', #lib/ExtUtils/Liblist.pm
        'ExtUtils::Liblist::Kid'=> '1.3', #lib/ExtUtils/Liblist/Kid.pm
        'ExtUtils::MakeMaker'   => '6.17', #lib/ExtUtils/MakeMaker.pm
        'ExtUtils::MakeMaker::bytes'=> '0.01', #lib/ExtUtils/MakeMaker/bytes.pm
        'ExtUtils::MakeMaker::vmsish'=> '0.01', #lib/ExtUtils/MakeMaker/vmsish.pm
        'ExtUtils::Manifest'    => '1.42', #lib/ExtUtils/Manifest.pm
        'ExtUtils::Miniperl'    => undef, #lib/ExtUtils/Miniperl.pm
        'ExtUtils::Mkbootstrap' => '1.15', #lib/ExtUtils/Mkbootstrap.pm
        'ExtUtils::Mksymlists'  => '1.19', #lib/ExtUtils/Mksymlists.pm
        'ExtUtils::MM'          => '0.04', #lib/ExtUtils/MM.pm
        'ExtUtils::MM_Any'      => '0.07', #lib/ExtUtils/MM_Any.pm
        'ExtUtils::MM_BeOS'     => '1.04', #lib/ExtUtils/MM_BeOS.pm
        'ExtUtils::MM_Cygwin'   => '1.06', #lib/ExtUtils/MM_Cygwin.pm
        'ExtUtils::MM_DOS'      => '0.02', #lib/ExtUtils/MM_DOS.pm
        'ExtUtils::MM_MacOS'    => '1.07', #lib/ExtUtils/MM_MacOS.pm
        'ExtUtils::MM_NW5'      => '2.06', #lib/ExtUtils/MM_NW5.pm
        'ExtUtils::MM_OS2'      => '1.04', #lib/ExtUtils/MM_OS2.pm
        'ExtUtils::MM_Unix'     => '1.42', #lib/ExtUtils/MM_Unix.pm
        'ExtUtils::MM_UWIN'     => '0.02', #lib/ExtUtils/MM_UWIN.pm
        'ExtUtils::MM_VMS'      => '5.70', #lib/ExtUtils/MM_VMS.pm
        'ExtUtils::MM_Win32'    => '1.09', #lib/ExtUtils/MM_Win32.pm
        'ExtUtils::MM_Win95'    => '0.03', #lib/ExtUtils/MM_Win95.pm
        'ExtUtils::MY'          => '0.01', #lib/ExtUtils/MY.pm
        'ExtUtils::Packlist'    => '0.04', #lib/ExtUtils/Packlist.pm
        'ExtUtils::testlib'     => '1.15', #lib/ExtUtils/testlib.pm
        'ExtUtils::XSSymSet'    => '1.0',  #vms/ext/XSSymSet.pm
        'Fatal'                 => '1.02', #lib/Fatal.pm
        'Fcntl'                 => '1.03', #lib/Fcntl.pm
        'fields'                => '1.01', #lib/fields.pm
        'File::Basename'        => '2.6', #lib/File/Basename.pm
        'File::CheckTree'       => undef, #lib/File/CheckTree.pm
        'File::Compare'         => '1.1002', #lib/File/Compare.pm
        'File::Copy'            => '2.03', #lib/File/Copy.pm
        'File::DosGlob'         => undef, #lib/File/DosGlob.pm
        'File::Find'            => undef, #lib/File/Find.pm
        'File::Glob'            => '0.991', #lib/File/Glob.pm
        'File::Path'            => '1.0404', #lib/File/Path.pm
        'File::Spec'            => '0.86', #lib/File/Spec.pm
        'File::Spec::Cygwin'    => '1.1', #lib/File/Spec/Cygwin.pm
        'File::Spec::Epoc'      => '1.1', #lib/File/Spec/Epoc.pm
        'File::Spec::Functions' => '1.3', #lib/File/Spec/Functions.pm
        'File::Spec::Mac'       => '1.4', #lib/File/Spec/Mac.pm
        'File::Spec::OS2'       => '1.2', #lib/File/Spec/OS2.pm
        'File::Spec::Unix'      => '1.5', #lib/File/Spec/Unix.pm
        'File::Spec::VMS'       => '1.4', #lib/File/Spec/VMS.pm
        'File::Spec::Win32'     => '1.4', #lib/File/Spec/Win32.pm
        'File::stat'            => undef, #lib/File/stat.pm
        'File::Temp'            => '0.14', #lib/File/Temp.pm
        'FileCache'             => undef, #lib/FileCache.pm
        'FileHandle'            => '2.00', #lib/FileHandle.pm
        'filetest'              => undef, #lib/filetest.pm
        'FindBin'               => '1.42', #lib/FindBin.pm
        'GDBM_File'             => '1.05', #ext/GDBM_File/GDBM_File.pm
        'Getopt::Long'          => '2.25', #lib/Getopt/Long.pm
        'Getopt::Std'           => '1.02', #lib/Getopt/Std.pm
        'I18N::Collate'         => undef, #lib/I18N/Collate.pm
        'if'                    => '0.03', #lib/if.pm
        'integer'               => undef, #lib/integer.pm
        'IO'                    => '1.20', #lib/IO.pm
        'IO::Dir'               => '1.03', #lib/IO/Dir.pm
        'IO::File'              => '1.08', #lib/IO/File.pm
        'IO::Handle'            => '1.21', #lib/IO/Handle.pm
        'IO::Pipe'              => '1.121', #lib/IO/Pipe.pm
        'IO::Poll'              => '0.05', #lib/IO/Poll.pm
        'IO::Seekable'          => '1.08', #lib/IO/Seekable.pm
        'IO::Select'            => '1.14', #lib/IO/Select.pm
        'IO::Socket'            => '1.26', #lib/IO/Socket.pm
        'IO::Socket::INET'      => '1.25', #lib/IO/Socket/INET.pm
        'IO::Socket::UNIX'      => '1.20', #lib/IO/Socket/UNIX.pm
        'IPC::Msg'              => '1.00', #lib/IPC/Msg.pm
        'IPC::Open2'            => '1.01', #lib/IPC/Open2.pm
        'IPC::Open3'            => '1.0103', #lib/IPC/Open3.pm
        'IPC::Semaphore'        => '1.00', #lib/IPC/Semaphore.pm
        'IPC::SysV'             => '1.03', #lib/IPC/SysV.pm
        'JNI'                   => '0.1', #jpl/JNI/JNI.pm
        'JPL::AutoLoader'       => undef, #jpl/JPL/AutoLoader.pm
        'JPL::Class'            => undef, #jpl/JPL/Class.pm
        'JPL::Compile'          => undef, #jpl/JPL/Compile.pm
        'less'                  => undef, #lib/less.pm
        'lib'                   => '0.5564', #lib/lib.pm
        'locale'                => undef, #lib/locale.pm
        'Math::BigFloat'        => '0.02', #lib/Math/BigFloat.pm
        'Math::BigInt'          => '0.01', #lib/Math/BigInt.pm
        'Math::Complex'         => '1.31', #lib/Math/Complex.pm
        'Math::Trig'            => '1', #lib/Math/Trig.pm
        'NDBM_File'             => '1.04',  #ext/NDBM_File/NDBM_File.pm
        'Net::hostent'          => undef, #lib/Net/hostent.pm
        'Net::netent'           => undef, #lib/Net/netent.pm
        'Net::Ping'             => '2.02', #lib/Net/Ping.pm
        'Net::protoent'         => undef, #lib/Net/protoent.pm
        'Net::servent'          => undef, #lib/Net/servent.pm
        'O'                     => undef, #lib/O.pm
        'ODBM_File'             => '1.03', #ext/ODBM_File/ODBM_File.pm
        'Opcode'                => '1.04', #lib/Opcode.pm
        'open'                  => undef, #lib/open.pm
        'ops'                   => '1.00', #lib/ops.pm
        'OS2::DLL'              => undef, #os2/OS2/REXX/DLL/DLL.pm
        'OS2::ExtAttr'          => '0.01', #os2/OS2/ExtAttr/ExtAttr.pm
        'OS2::PrfDB'            => '0.02', #os2/OS2/PrfDB/PrfDB.pm
        'OS2::Process'          => '0.2', #os2/OS2/Process/Process.pm
        'OS2::REXX'             => '1.00', #os2/OS2/REXX/REXX.pm
        'overload'              => undef, #lib/overload.pm
        'Pod::Checker'          => '1.2', #lib/Pod/Checker.pm
        'Pod::Find'             => '0.21', #lib/Pod/Find.pm
        'Pod::Functions'        => undef, #lib/Pod/Functions.pm
        'Pod::Html'             => '1.03', #lib/Pod/Html.pm
        'Pod::InputObjects'     => '1.13', #lib/Pod/InputObjects.pm
        'Pod::LaTeX'            => '0.53', #lib/Pod/LaTeX.pm
        'Pod::Man'              => '1.15', #lib/Pod/Man.pm
        'Pod::Parser'           => '1.13', #lib/Pod/Parser.pm
        'Pod::ParseUtils'       => '0.22', #lib/Pod/ParseUtils.pm
        'Pod::Plainer'          => '0.01', #lib/Pod/Plainer.pm
        'Pod::Select'           => '1.13', #lib/Pod/Select.pm
        'Pod::Text'             => '2.08', #lib/Pod/Text.pm
        'Pod::Text::Color'      => '0.06', #lib/Pod/Text/Color.pm
        'Pod::Text::Overstrike' => '1.01', #lib/Pod/Text/Overstrike.pm
        'Pod::Text::Termcap'    => '1', #lib/Pod/Text/Termcap.pm
        'Pod::Usage'            => '1.14', #lib/Pod/Usage.pm
        'POSIX'                 => '1.03', #lib/POSIX.pm
        're'                    => '0.02', #lib/re.pm
        'Safe'                  => '2.10', #lib/Safe.pm
        'SDBM_File'             => '1.03', #lib/SDBM_File.pm
        'Search::Dict'          => undef, #lib/Search/Dict.pm
        'SelectSaver'           => undef, #lib/SelectSaver.pm
        'SelfLoader'            => '1.0902', #lib/SelfLoader.pm
        'Shell'                 => '0.3', #lib/Shell.pm
        'sigtrap'               => '1.02', #lib/sigtrap.pm
        'Socket'                => '1.72', #lib/Socket.pm
        'strict'                => '1.01', #lib/strict.pm
        'subs'                  => undef, #lib/subs.pm
        'Symbol'                => '1.02', #lib/Symbol.pm
        'Sys::Hostname'         => '1.1', #lib/Sys/Hostname.pm
        'Sys::Syslog'           => '0.01', #lib/Sys/Syslog.pm
        'Term::ANSIColor'       => '1.03', #lib/Term/ANSIColor.pm
        'Term::Cap'             => undef, #lib/Term/Cap.pm
        'Term::Complete'        => undef, #lib/Term/Complete.pm
        'Term::ReadLine'        => undef, #lib/Term/ReadLine.pm
        'Test'                  => '1.24', #lib/Test.pm
        'Test::Builder'         => '0.17', #lib/Test/Builder.pm
        'Test::Harness'         => '2.30', #lib/Test/Harness.pm
        'Test::Harness::Assert' => '0.01', #lib/Test/Harness/Assert.pm
        'Test::Harness::Iterator'=> '0.01', #lib/Test/Harness/Iterator.pm
        'Test::Harness::Straps' => '0.15', #lib/Test/Harness/Straps.pm
        'Test::More'            => '0.47', #lib/Test/More.pm
        'Test::Simple'          => '0.47', #lib/Test/Simple.pm
        'Text::Abbrev'          => undef, #lib/Text/Abbrev.pm
        'Text::ParseWords'      => '3.2', #lib/Text/ParseWords.pm
        'Text::Soundex'         => '1.0', #lib/Text/Soundex.pm
        'Text::Tabs'            => '98.112801', #lib/Text/Tabs.pm
        'Text::Wrap'            => '2001.0131', #lib/Text/Wrap.pm
        'Thread'                => '1.0', #ext/Thread/Thread.pm
        'Thread::Queue'         => undef, #ext/Thread/Thread/Queue.pm
        'Thread::Semaphore'     => undef, #ext/Thread/Thread/Semaphore.pm
        'Thread::Signal'        => undef, #ext/Thread/Thread/Signal.pm
        'Thread::Specific'      => undef, #ext/Thread/Thread/Specific.pm
        'Tie::Array'            => '1.01', #lib/Tie/Array.pm
        'Tie::Handle'           => '4.0', #lib/Tie/Handle.pm
        'Tie::Hash'             => undef, #lib/Tie/Hash.pm
        'Tie::RefHash'          => '1.3', #lib/Tie/RefHash.pm
        'Tie::Scalar'           => undef, #lib/Tie/Scalar.pm
        'Tie::SubstrHash'       => undef, #lib/Tie/SubstrHash.pm
        'Time::gmtime'          => '1.01', #lib/Time/gmtime.pm
        'Time::Local'           => undef, #lib/Time/Local.pm
        'Time::localtime'       => '1.01', #lib/Time/localtime.pm
        'Time::tm'              => undef, #lib/Time/tm.pm
        'UNIVERSAL'             => undef, #lib/UNIVERSAL.pm
        'User::grent'           => undef, #lib/User/grent.pm
        'User::pwent'           => undef, #lib/User/pwent.pm
        'utf8'                  => undef, #lib/utf8.pm
        'vars'                  => undef, #lib/vars.pm
        'VMS::DCLsym'           => '1.01', #vms/ext/DCLsym/DCLsym.pm
        'VMS::Filespec'         => undef, #vms/ext/Filespec.pm
        'VMS::Stdio'            => '2.2', #vms/ext/Stdio/Stdio.pm
        'vmsish'                => undef, #vms/ext/vmsish.pm
        'warnings'              => undef, #lib/warnings.pm
        'warnings::register'    => undef, #lib/warnings/register.pm
        'XSLoader'              => '0.01', #lib/XSLoader.pm
   },

    5.007003   => {
        'AnyDBM_File'           => '1.00',
        'Attribute::Handlers'   => '0.76',
        'attributes'            => '0.04_01',
        'attrs'                 => '1.01',
        'AutoLoader'            => '5.59',
        'AutoSplit'             => '1.0307',
        'autouse'               => '1.03',
        'B::Asmdata'            => '1.00',
        'B::Assembler'          => '0.04',
        'B::Bblock'             => '1.00',
        'B::Bytecode'           => '1.00',
        'B::C'                  => '1.01',
        'B::CC'                 => '1.00',
        'B::Concise'            => '0.52',
        'B::Debug'              => '1.00',
        'B::Deparse'            => '0.63',
        'B::Disassembler'       => '1.01',
        'B::Lint'               => '1.00',
        'B'                     => '1.00',
        'B::Showlex'            => '1.00',
        'B::Stackobj'           => '1.00',
        'B::Stash'              => '1.00',
        'B::Terse'              => '1.00',
        'B::Xref'               => '1.00',
        'base'                  => '1.02',
        'Benchmark'             => '1.04',
        'blib'                  => '1.01',
        'ByteLoader'            => '0.04',
        'bytes'                 => '1.00',
        'Carp'                  => '1.01',
        'Carp::Heavy'           => undef,
        'CGI'                   => '2.80',
        'CGI::Apache'           => '1.00',
        'CGI::Carp'             => '1.22',
        'CGI::Cookie'           => '1.20',
        'CGI::Fast'             => '1.04',
        'CGI::Pretty'           => '1.05_00',
        'CGI::Push'             => '1.04',
        'CGI::Switch'           => '1.00',
        'CGI::Util'             => '1.3',
        'charnames'             => '1.01',
        'Class::ISA'            => '0.32',
        'Class::Struct'         => '0.61',
        'Config'                => undef,
        'constant'              => '1.04',
        'CPAN::FirstTime'       => '1.54 ',
        'CPAN'                  => '1.59_56',
        'CPAN::Nox'             => '1.00_01',
        'Cwd'                   => '2.06',
        'Data::Dumper'          => '2.12',
        'DB'                    => '1.0',
        'DB_File'               => '1.804',
        'Devel::DProf'          => '20000000.00_01',
        'Devel::Peek'           => '1.00_03',
        'Devel::PPPort'         => '2.0002',
        'Devel::SelfStubber'    => '1.03',
        'diagnostics'           => '1.1',
        'Digest'                => '1.00',
        'Digest::MD5'           => '2.16',
        'DirHandle'             => '1.00',
        'Dumpvalue'             => '1.10',
        'DynaLoader'            => 1.04,
        'Encode'                => '0.40',
        'Encode::CN'            => '0.02',
        'Encode::CN::HZ'        => undef,
        'Encode::Encoding'      => '0.02',
        'Encode::Internal'      => '0.30',
        'Encode::iso10646_1'    => '0.30',
        'Encode::JP'            => '0.02',
        'Encode::JP::Constants' => '1.02',
        'Encode::JP::H2Z'       => '0.77',
        'Encode::JP::ISO_2022_JP' => undef,
        'Encode::JP::JIS'       => undef,
        'Encode::JP::Tr'        => '0.77',
        'Encode::KR'            => '0.02',
        'Encode::Tcl'           => '1.01',
        'Encode::Tcl::Escape'   => '1.01',
        'Encode::Tcl::Extended' => '1.01',
        'Encode::Tcl::HanZi'    => '1.01',
        'Encode::Tcl::Table'    => '1.01',
        'Encode::TW'            => '0.02',
        'Encode::Unicode'       => '0.30',
        'Encode::usc2_le'       => '0.30',
        'Encode::utf8'          => '0.30',
        'Encode::XS'            => '0.40',
        'encoding'              => '1.00',
        'English'               => '1.00',
        'Env'                   => '1.00',
        'Exporter'              => '5.566',
        'Exporter::Heavy'       => '5.562',
        'ExtUtils::Command'     => '1.02',
        'ExtUtils::Constant'    => '0.11',
        'ExtUtils::Embed'       => '1.250601',
        'ExtUtils::Install'     => '1.29',
        'ExtUtils::Installed'   => '0.04',
        'ExtUtils::Liblist'     => '1.2701',
        'ExtUtils::MakeMaker'   => '5.48_03',
        'ExtUtils::Manifest'    => '1.35',
        'ExtUtils::Mkbootstrap' => '1.1401',
        'ExtUtils::Mksymlists'  => '1.18',
        'ExtUtils::MM_BeOS'     => '1.00',
        'ExtUtils::MM_Cygwin'   => '1.00',
        'ExtUtils::MM_OS2'      => '1.00',
        'ExtUtils::MM_Unix'     => '1.12607',
        'ExtUtils::MM_VMS'      => '5.56',
        'ExtUtils::MM_Win32'    => '1.00_02',
        'ExtUtils::Packlist'    => '0.04',
        'ExtUtils::testlib'     => '1.1201',
        'ExtUtils::XSSymSet'    => '1.0',
        'Fatal'                 => '1.03',
        'Fcntl'                 => '1.04',
        'fields'                => '1.02',
        'File::Basename'        => '2.71',
        'File::CheckTree'       => '4.1',
        'File::Compare'         => '1.1003',
        'File::Copy'            => '2.05',
        'File::DosGlob'         => '1.00',
        'File::Find'            => '1.04',
        'File::Glob'            => '1.01',
        'File::Path'            => '1.05',
        'File::Spec'            => '0.83',
        'File::Spec::Cygwin'    => '1.0',
        'File::Spec::Epoc'      => '1.00',
        'File::Spec::Functions' => '1.2',
        'File::Spec::Mac'       => '1.3',
        'File::Spec::OS2'       => '1.1',
        'File::Spec::Unix'      => '1.4',
        'File::Spec::VMS'       => '1.2',
        'File::Spec::Win32'     => '1.3',
        'File::stat'            => '1.00',
        'File::Temp'            => '0.13',
        'FileCache'             => '1.00',
        'FileHandle'            => '2.01',
        'filetest'              => '1.00',
        'Filter::Simple'        => '0.77',
        'Filter::Util::Call'    => '1.06',
        'FindBin'               => '1.43',
        'GDBM_File'             => '1.06',
        'Getopt::Long'          => '2.28',
        'Getopt::Std'           => '1.03',
        'I18N::Collate'         => '1.00',
        'I18N::Langinfo'        => '0.01',
        'I18N::LangTags'        => '0.27',
        'I18N::LangTags::List'  => '0.25',
        'if'                    => '0.01',
        'integer'               => '1.00',
        'IO'                    => '1.20',
        'IO::Dir'               => '1.03_00',
        'IO::File'              => '1.09',
        'IO::Handle'            => '1.21_00',
        'IO::Pipe'              => '1.122',
        'IO::Poll'              => '0.06',
        'IO::Seekable'          => '1.08_00',
        'IO::Select'            => '1.15',
        'IO::Socket'            => '1.27',
        'IO::Socket::INET'      => '1.26',
        'IO::Socket::UNIX'      => '1.20_00',
        'IPC::Msg'              => '1.00_00',
        'IPC::Open2'            => '1.01',
        'IPC::Open3'            => '1.0104',
        'IPC::Semaphore'        => '1.00_00',
        'IPC::SysV'             => '1.03_00',
        'JNI'                   => '0.1',
        'JPL::AutoLoader'       => undef,
        'JPL::Class'            => undef,
        'JPL::Compile'          => undef,
        'less'                  => '0.01',
        'lib'                   => '0.5564',
        'List::Util'            => '1.06_00',
        'locale'                => '1.00',
        'Locale::Constants'     => '2.01',
        'Locale::Country'       => '2.01',
        'Locale::Currency'      => '2.01',
        'Locale::Language'      => '2.01',
        'Locale::Maketext'      => '1.03',
        'Locale::Script'        => '2.01',
        'Math::BigFloat'        => '1.30',
        'Math::BigInt'          => '1.54',
        'Math::BigInt::Calc'    => '0.25',
        'Math::Complex'         => '1.34',
        'Math::Trig'            => '1.01',
        'Memoize'               => '0.66',
        'Memoize::AnyDBM_File'  => '0.65',
        'Memoize::Expire'       => '0.66',
        'Memoize::ExpireFile'   => '0.65',
        'Memoize::ExpireTest'   => '0.65',
        'Memoize::NDBM_File'    => '0.65',
        'Memoize::SDBM_File'    => '0.65',
        'Memoize::Storable'     => '0.65',
        'MIME::Base64'          => '2.12',
        'MIME::QuotedPrint'     => '2.03',
        'NDBM_File'             => '1.04',
        'Net::Cmd'              => '2.21',
        'Net::Config'           => '1.10',
        'Net::Domain'           => '2.17',
        'Net::FTP'              => '2.64',
        'Net::FTP::A'           => '1.15',
        'Net::FTP::dataconn'    => '0.10',
        'Net::FTP::E'           => '0.01',
        'Net::FTP::I'           => '1.12',
        'Net::FTP::L'           => '0.01',
        'Net::hostent'          => '1.00',
        'Net::netent'           => '1.00',
        'Net::Netrc'            => '2.12',
        'Net::NNTP'             => '2.21',
        'Net::Ping'             => '2.12',
        'Net::POP3'             => '2.23',
        'Net::protoent'         => '1.00',
        'Net::servent'          => '1.00',
        'Net::SMTP'             => '2.21',
        'Net::Time'             => '2.09',
        'NEXT'                  => '0.50',
        'O'                     => '1.00',
        'ODBM_File'             => '1.03',
        'Opcode'                => '1.05',
        'open'                  => '1.01',
        'ops'                   => '1.00',
        'OS2::DLL'              => '1.00',
        'OS2::ExtAttr'          => '0.01',
        'OS2::PrfDB'            => '0.02',
        'OS2::Process'          => '1.0',
        'OS2::REXX'             => '1.01',
        'overload'              => '1.00',
        'PerlIO'                => '1.00',
        'PerlIO::Scalar'        => '0.01',
        'PerlIO::Via'           => '0.01',
        'Pod::Checker'          => '1.3',
        'Pod::Find'             => '0.22',
        'Pod::Functions'        => '1.01',
        'Pod::Html'             => '1.04',
        'Pod::LaTeX'            => '0.54',
        'Pod::Man'              => '1.32',
        'Pod::InputObjects'     => '1.13',
        'Pod::ParseLink'        => '1.05',
        'Pod::Parser'           => '1.13',
        'Pod::ParseUtils'       => '0.22',
        'Pod::Plainer'          => '0.01',
        'Pod::Select'           => '1.13',
        'Pod::Text'             => '2.18',
        'Pod::Text::Color'      => '1.03',
        'Pod::Text::Overstrike' => '1.08',
        'Pod::Text::Termcap'    => '1.09',
        'Pod::Usage'            => '1.14',
        'POSIX'                 => '1.05',
        're'                    => '0.03',
        'Safe'                  => '2.07',
        'Scalar::Util'          => undef,
        'SDBM_File'             => '1.03',
        'Search::Dict'          => '1.02',
        'SelectSaver'           => '1.00',
        'SelfLoader'            => '1.0903',
        'Shell'                 => '0.4',
        'sigtrap'               => '1.02',
        'Socket'                => '1.75',
        'sort'                  => '1.00',
        'Storable'              => '1.015',
        'strict'                => '1.02',
        'subs'                  => '1.00',
        'Switch'                => '2.06',
        'Symbol'                => '1.04',
        'Sys::Hostname'         => '1.1',
        'Sys::Syslog'           => '0.02',
        'Term::ANSIColor'       => '1.04',
        'Term::Cap'             => '1.07',
        'Term::Complete'        => '1.4',
        'Term::ReadLine'        => '1.00',
        'Test'                  => '1.18',
        'Test::Builder'         => '0.11',
        'Test::Harness'         => '2.01',
        'Test::Harness::Assert' => '0.01',
        'Test::Harness::Iterator'=> '0.01',
        'Test::Harness::Straps' => '0.08',
        'Test::More'            => '0.41',
        'Test::Simple'          => '0.41',
        'Text::Abbrev'          => '1.00',
        'Text::Balanced'        => '1.89',
        'Text::ParseWords'      => '3.21',
        'Text::Soundex'         => '1.01',
        'Text::Tabs'            => '98.112801',
        'Text::Wrap'            => '2001.0929',
        'Thread'                => '2.00',
        'Thread::Queue'         => '1.00',
        'Thread::Semaphore'     => '1.00',
        'Thread::Signal'        => '1.00',
        'Thread::Specific'      => '1.00',
        'threads'               => '0.05',
        'threads::shared'       => '0.90',
        'Tie::Array'            => '1.02',
        'Tie::File'             => '0.17',
        'Tie::Hash'             => '1.00',
        'Tie::Handle'           => '4.1',
        'Tie::Memoize'          => '1.0',
        'Tie::RefHash'          => '1.3_00',
        'Tie::Scalar'           => '1.00',
        'Tie::SubstrHash'       => '1.00',
        'Time::gmtime'          => '1.02',
        'Time::HiRes'           => '1.20_00',
        'Time::Local'           => '1.04',
        'Time::localtime'       => '1.02',
        'Time::tm'              => '1.00',
        'Unicode::Collate'      => '0.10',
        'Unicode::Normalize'    => '0.14',
        'Unicode::UCD'          => '0.2',
        'UNIVERSAL'             => '1.00',
        'User::grent'           => '1.00',
        'User::pwent'           => '1.00',
        'utf8'                  => '1.00',
        'vars'                  => '1.01',
        'VMS::DCLsym'           => '1.02',
        'VMS::Filespec'         => '1.1',
        'VMS::Stdio'            => '2.3',
        'vmsish'                => '1.00',
        'warnings'              => '1.00',
        'warnings::register'    => '1.00',
        'XS::Typemap'           => '0.01',
        'XSLoader'              => '0.01',
    },

    5.008   => {
        'AnyDBM_File'           => '1.00', #./lib/AnyDBM_File.pm
        'Attribute::Handlers'   => '0.77', #./lib/Attribute/Handlers.pm
        'attributes'            => '0.05', #./lib/attributes.pm
        'attrs'                 => '1.01', #./ext/attrs/attrs.pm
        'AutoLoader'            => '5.59', #./lib/AutoLoader.pm
        'AutoSplit'             => '1.0307', #./lib/AutoSplit.pm
        'autouse'               => '1.03', #./lib/autouse.pm
        'B'                     => '1.01', #./ext/B/B.pm
        'B::Asmdata'            => '1.00', #./ext/B/B/Asmdata.pm
        'B::Assembler'          => '0.04', #./ext/B/B/Assembler.pm
        'B::Bblock'             => '1.00', #./ext/B/B/Bblock.pm
        'B::Bytecode'           => '1.00', #./ext/B/B/Bytecode.pm
        'B::C'                  => '1.01', #./ext/B/B/C.pm
        'B::CC'                 => '1.00', #./ext/B/B/CC.pm
        'B::Concise'            => '0.52', #./ext/B/B/Concise.pm
        'B::Debug'              => '1.00', #./ext/B/B/Debug.pm
        'B::Deparse'            => '0.63', #./ext/B/B/Deparse.pm
        'B::Disassembler'       => '1.01', #./ext/B/B/Disassembler.pm
        'B::Lint'               => '1.01', #./ext/B/B/Lint.pm
        'B::Showlex'            => '1.00', #./ext/B/B/Showlex.pm
        'B::Stackobj'           => '1.00', #./ext/B/B/Stackobj.pm
        'B::Stash'              => '1.00', #./ext/B/B/Stash.pm
        'B::Terse'              => '1.00', #./ext/B/B/Terse.pm
        'B::Xref'               => '1.01', #./ext/B/B/Xref.pm
        'base'                  => '1.03', #./lib/base.pm
        'Benchmark'             => '1.04', #./lib/Benchmark.pm
        'bigint'                => '0.02', #./lib/bigint.pm
        'bignum'                => '0.11', #./lib/bignum.pm
        'bigrat'                => '0.04', #./lib/bigrat.pm
        'blib'                  => '1.02', #./lib/blib.pm
        'ByteLoader'            => '0.04', #./ext/ByteLoader/ByteLoader.pm
        'bytes'                 => '1.00', #./lib/bytes.pm
        'Carp'                  => '1.01', #./lib/Carp.pm
        'Carp::Heavy'           => 'undef', #./lib/Carp/Heavy.pm
        'CGI'                   => '2.81', #./lib/CGI.pm
        'CGI::Apache'           => '1.00', #./lib/CGI/Apache.pm
        'CGI::Carp'             => '1.23', #./lib/CGI/Carp.pm
        'CGI::Cookie'           => '1.20', #./lib/CGI/Cookie.pm
        'CGI::Fast'             => '1.04', #./lib/CGI/Fast.pm
        'CGI::Pretty'           => '1.05_00', #./lib/CGI/Pretty.pm
        'CGI::Push'             => '1.04', #./lib/CGI/Push.pm
        'CGI::Switch'           => '1.00', #./lib/CGI/Switch.pm
        'CGI::Util'             => '1.3', #./lib/CGI/Util.pm
        'charnames'             => '1.01', #./lib/charnames.pm
        'Class::ISA'            => '0.32', #./lib/Class/ISA.pm
        'Class::Struct'         => '0.61', #./lib/Class/Struct.pm
        'constant'              => '1.04', #./lib/constant.pm
        'Config'                => undef,
        'CPAN'                  => '1.61', #./lib/CPAN.pm
        'CPAN::FirstTime'       => '1.56 ', #./lib/CPAN/FirstTime.pm
        'CPAN::Nox'             => '1.02', #./lib/CPAN/Nox.pm
        'Cwd'                   => '2.06', #./lib/Cwd.pm
        'Data::Dumper'          => '2.12', #./ext/Data/Dumper/Dumper.pm
        'DB'                    => '1.0', #./lib/DB.pm
        'DB_File'               => '1.804', #./ext/DB_File/DB_File.pm
        'Devel::DProf'          => '20000000.00_01', #./ext/Devel/DProf/DProf.pm
        'Devel::Peek'           => '1.00_03', #./ext/Devel/Peek/Peek.pm
        'Devel::PPPort'         => '2.0002', #./ext/Devel/PPPort/PPPort.pm
        'Devel::SelfStubber'    => '1.03', #./lib/Devel/SelfStubber.pm
        'diagnostics'           => '1.1', #./lib/diagnostics.pm
        'Digest'                => '1.00', #./lib/Digest.pm
        'Digest::MD5'           => '2.20', #./ext/Digest/MD5/MD5.pm
        'DirHandle'             => '1.00', #./lib/DirHandle.pm
        'Dumpvalue'             => '1.11', #./lib/Dumpvalue.pm
        'DynaLoader'            => '1.04',
        'Encode'                => '1.75', #./ext/Encode/Encode.pm
        'Encode::Alias'         => '1.32', #./ext/Encode/lib/Encode/Alias.pm
        'Encode::Byte'          => '1.22', #./ext/Encode/Byte/Byte.pm
        'Encode::CJKConstants'  => '1.00', #./ext/Encode/lib/Encode/CJKConstants.pm
        'Encode::CN'            => '1.24', #./ext/Encode/CN/CN.pm
        'Encode::CN::HZ'        => '1.04', #./ext/Encode/lib/Encode/CN/HZ.pm
        'Encode::Config'        => '1.06', #./ext/Encode/lib/Encode/Config.pm
        'Encode::EBCDIC'        => '1.21', #./ext/Encode/EBCDIC/EBCDIC.pm
        'Encode::Encoder'       => '0.05', #./ext/Encode/lib/Encode/Encoder.pm
        'Encode::Encoding'      => '1.30', #./ext/Encode/lib/Encode/Encoding.pm
        'Encode::Guess'         => '1.06', #./ext/Encode/lib/Encode/Guess.pm
        'Encode::JP::H2Z'       => '1.02', #./ext/Encode/lib/Encode/JP/H2Z.pm
        'Encode::JP::JIS7'      => '1.08', #./ext/Encode/lib/Encode/JP/JIS7.pm
        'Encode::JP'            => '1.25', #./ext/Encode/JP/JP.pm
        'Encode::KR'            => '1.22', #./ext/Encode/KR/KR.pm
        'Encode::KR::2022_KR'   => '1.05', #./ext/Encode/lib/Encode/KR/2022_KR.pm
        'Encode::MIME::Header'  => '1.05', #./ext/Encode/lib/Encode/MIME/Header.pm
        'Encode::Symbol'        => '1.22', #./ext/Encode/Symbol/Symbol.pm
        'Encode::TW'            => '1.26', #./ext/Encode/TW/TW.pm
        'Encode::Unicode'       => '1.37', #./ext/Encode/Unicode/Unicode.pm
        'encoding'              => '1.35', #./ext/Encode/encoding.pm
        'English'               => '1.00', #./lib/English.pm
        'Env'                   => '1.00', #./lib/Env.pm
        'Exporter'              => '5.566', #./lib/Exporter.pm
        'Exporter::Heavy'       => '5.566', #./lib/Exporter/Heavy.pm
        'ExtUtils::Command'     => '1.04', #./lib/ExtUtils/Command.pm
        'ExtUtils::Command::MM' => '0.01', #./lib/ExtUtils/Command/MM.pm
        'ExtUtils::Constant'    => '0.12', #./lib/ExtUtils/Constant.pm
        'ExtUtils::Embed'       => '1.250601', #./lib/ExtUtils/Embed.pm
        'ExtUtils::Install'     => '1.29', #./lib/ExtUtils/Install.pm
        'ExtUtils::Installed'   => '0.06', #./lib/ExtUtils/Installed.pm
        'ExtUtils::Liblist'     => '1.00', #./lib/ExtUtils/Liblist.pm
        'ExtUtils::Liblist::Kid'=> '1.29', #./lib/ExtUtils/Liblist/Kid.pm
        'ExtUtils::MakeMaker'   => '6.03', #./lib/ExtUtils/MakeMaker.pm
        'ExtUtils::Manifest'    => '1.38', #./lib/ExtUtils/Manifest.pm
        'ExtUtils::Mkbootstrap' => '1.15', #./lib/ExtUtils/Mkbootstrap.pm
        'ExtUtils::Mksymlists'  => '1.19', #./lib/ExtUtils/Mksymlists.pm
        'ExtUtils::MM'          => '0.04', #./lib/ExtUtils/MM.pm
        'ExtUtils::MM_Any'      => '0.04', #./lib/ExtUtils/MM_Any.pm
        'ExtUtils::MM_BeOS'     => '1.03', #./lib/ExtUtils/MM_BeOS.pm
        'ExtUtils::MM_Cygwin'   => '1.04', #./lib/ExtUtils/MM_Cygwin.pm
        'ExtUtils::MM_DOS'      => '0.01', #./lib/ExtUtils/MM_DOS.pm
        'ExtUtils::MM_MacOS'    => '1.03', #./lib/ExtUtils/MM_MacOS.pm
        'ExtUtils::MM_NW5'      => '2.05', #./lib/ExtUtils/MM_NW5.pm
        'ExtUtils::MM_OS2'      => '1.03', #./lib/ExtUtils/MM_OS2.pm
        'ExtUtils::MM_Unix'     => '1.33', #./lib/ExtUtils/MM_Unix.pm
        'ExtUtils::MM_UWIN'     => '0.01', #./lib/ExtUtils/MM_UWIN.pm
        'ExtUtils::MM_VMS'      => '5.65', #./lib/ExtUtils/MM_VMS.pm
        'ExtUtils::MM_Win32'    => '1.05', #./lib/ExtUtils/MM_Win32.pm
        'ExtUtils::MM_Win95'    => '0.02', #./lib/ExtUtils/MM_Win95.pm
        'ExtUtils::MY'          => '0.01', #./lib/ExtUtils/MY.pm
        'ExtUtils::Packlist'    => '0.04', #./lib/ExtUtils/Packlist.pm
        'ExtUtils::testlib'     => '1.15', #./lib/ExtUtils/testlib.pm
        'ExtUtils::XSSymSet'    => '1.0', #./vms/ext/XSSymSet.pm
        'Fatal'                 => '1.03', #./lib/Fatal.pm
        'Fcntl'                 => '1.04', #./ext/Fcntl/Fcntl.pm
        'fields'                => '1.02', #./lib/fields.pm
        'File::Basename'        => '2.71', #./lib/File/Basename.pm
        'File::CheckTree'       => '4.2', #./lib/File/CheckTree.pm
        'File::Compare'         => '1.1003', #./lib/File/Compare.pm
        'File::Copy'            => '2.05', #./lib/File/Copy.pm
        'File::DosGlob'         => '1.00', #./lib/File/DosGlob.pm
        'File::Find'            => '1.04', #./lib/File/Find.pm
        'File::Glob'            => '1.01', #./ext/File/Glob/Glob.pm
        'File::Path'            => '1.05', #./lib/File/Path.pm
        'File::Spec'            => '0.83', #./lib/File/Spec.pm
        'File::Spec::Cygwin'    => '1.0', #./lib/File/Spec/Cygwin.pm
        'File::Spec::Epoc'      => '1.00', #./lib/File/Spec/Epoc.pm
        'File::Spec::Functions' => '1.2', #./lib/File/Spec/Functions.pm
        'File::Spec::Mac'       => '1.3', #./lib/File/Spec/Mac.pm
        'File::Spec::OS2'       => '1.1', #./lib/File/Spec/OS2.pm
        'File::Spec::Unix'      => '1.4', #./lib/File/Spec/Unix.pm
        'File::Spec::VMS'       => '1.2', #./lib/File/Spec/VMS.pm
        'File::Spec::Win32'     => '1.3', #./lib/File/Spec/Win32.pm
        'File::stat'            => '1.00', #./lib/File/stat.pm
        'File::Temp'            => '0.13', #./lib/File/Temp.pm
        'FileCache'             => '1.021', #./lib/FileCache.pm
        'FileHandle'            => '2.01', #./lib/FileHandle.pm
        'filetest'              => '1.00', #./lib/filetest.pm
        'Filter::Simple'        => '0.78', #./lib/Filter/Simple.pm
        'Filter::Util::Call'    => '1.06', #./ext/Filter/Util/Call/Call.pm
        'FindBin'               => '1.43', #./lib/FindBin.pm
        'GDBM_File'             => '1.06', #./ext/GDBM_File/GDBM_File.pm
        'Getopt::Long'          => '2.32', #./lib/Getopt/Long.pm
        'Getopt::Std'           => '1.03', #./lib/Getopt/Std.pm
        'Hash::Util'            => '0.04', #./lib/Hash/Util.pm
        'I18N::Collate'         => '1.00', #./lib/I18N/Collate.pm
        'I18N::Langinfo'        => '0.01', #./ext/I18N/Langinfo/Langinfo.pm
        'I18N::LangTags'        => '0.27', #./lib/I18N/LangTags.pm
        'I18N::LangTags::List'  => '0.25', #./lib/I18N/LangTags/List.pm
        'if'                    => '0.01', #./lib/if.pm
        'integer'               => '1.00', #./lib/integer.pm
        'IO'                    => '1.20', #./ext/IO/IO.pm
        'IO::Dir'               => '1.03_00', #./ext/IO/lib/IO/Dir.pm
        'IO::File'              => '1.09', #./ext/IO/lib/IO/File.pm
        'IO::Handle'            => '1.21_00', #./ext/IO/lib/IO/Handle.pm
        'IO::Pipe'              => '1.122', #./ext/IO/lib/IO/Pipe.pm
        'IO::Poll'              => '0.06', #./ext/IO/lib/IO/Poll.pm
        'IO::Seekable'          => '1.08_00', #./ext/IO/lib/IO/Seekable.pm
        'IO::Select'            => '1.15', #./ext/IO/lib/IO/Select.pm
        'IO::Socket'            => '1.27', #./ext/IO/lib/IO/Socket.pm
        'IO::Socket::INET'      => '1.26', #./ext/IO/lib/IO/Socket/INET.pm
        'IO::Socket::UNIX'      => '1.20_00', #./ext/IO/lib/IO/Socket/UNIX.pm
        'IPC::Open2'            => '1.01', #./lib/IPC/Open2.pm
        'IPC::Open3'            => '1.0104', #./lib/IPC/Open3.pm
        'IPC::Msg'              => '1.00_00', #./ext/IPC/SysV/Msg.pm
        'IPC::Semaphore'        => '1.00_00', #./ext/IPC/SysV/Semaphore.pm
        'IPC::SysV'             => '1.03_00', #./ext/IPC/SysV/SysV.pm
        'JNI'                   => '0.1', #./jpl/JNI/JNI.pm
        'JPL::AutoLoader'       => undef, #./jpl/JPL/AutoLoader.pm
        'JPL::Class'            => undef, #./jpl/JPL/Class.pm
        'JPL::Compile'          => undef, #./jpl/JPL/Compile.pm
        'less'                  => '0.01', #./lib/less.pm
        'lib'                   => '0.5564',
        'List::Util'            => '1.07_00', #./ext/List/Util/lib/List/Util.pm
        'locale'                => '1.00', #./lib/locale.pm
        'Locale::Constants'     => '2.01', #./lib/Locale/Constants.pm
        'Locale::Country'       => '2.04', #./lib/Locale/Country.pm
        'Locale::Currency'      => '2.01', #./lib/Locale/Currency.pm
        'Locale::Language'      => '2.01', #./lib/Locale/Language.pm
        'Locale::Maketext'      => '1.03', #./lib/Locale/Maketext.pm
        'Locale::Script'        => '2.01', #./lib/Locale/Script.pm
        'Math::BigFloat'        => '1.35', #./lib/Math/BigFloat.pm
        'Math::BigFloat::Trace' => '0.01', #./lib/Math/BigFloat/Trace.pm
        'Math::BigInt'          => '1.60', #./lib/Math/BigInt.pm
        'Math::BigInt::Calc'    => '0.30', #./lib/Math/BigInt/Calc.pm
        'Math::BigInt::Trace'   => '0.01', #./lib/Math/BigInt/Trace.pm
        'Math::BigRat'          => '0.07', #./lib/Math/BigRat.pm
        'Math::Complex'         => '1.34', #./lib/Math/Complex.pm
        'Math::Trig'            => '1.01', #./lib/Math/Trig.pm
        'Memoize'               => '1.01', #./lib/Memoize.pm
        'Memoize::AnyDBM_File'  => '0.65', #./lib/Memoize/AnyDBM_File.pm
        'Memoize::Expire'       => '1.00', #./lib/Memoize/Expire.pm
        'Memoize::ExpireFile'   => '1.01', #./lib/Memoize/ExpireFile.pm
        'Memoize::ExpireTest'   => '0.65', #./lib/Memoize/ExpireTest.pm
        'Memoize::NDBM_File'    => '0.65', #./lib/Memoize/NDBM_File.pm
        'Memoize::SDBM_File'    => '0.65', #./lib/Memoize/SDBM_File.pm
        'Memoize::Storable'     => '0.65', #./lib/Memoize/Storable.pm
        'MIME::Base64'          => '2.12', #./ext/MIME/Base64/Base64.pm
        'MIME::QuotedPrint'     => '2.03', #./ext/MIME/Base64/QuotedPrint.pm
        'NDBM_File'             => '1.04', #./ext/NDBM_File/NDBM_File.pm
        'Net::Cmd'              => '2.21', #./lib/Net/Cmd.pm
        'Net::Config'           => '1.10', #./lib/Net/Config.pm
        'Net::Domain'           => '2.17', #./lib/Net/Domain.pm
        'Net::FTP'              => '2.65', #./lib/Net/FTP.pm
        'Net::FTP::A'           => '1.15', #./lib/Net/FTP/A.pm
        'Net::FTP::dataconn'    => '0.11', #./lib/Net/FTP/dataconn.pm
        'Net::FTP::E'           => '0.01', #./lib/Net/FTP/E.pm
        'Net::FTP::I'           => '1.12', #./lib/Net/FTP/I.pm
        'Net::FTP::L'           => '0.01', #./lib/Net/FTP/L.pm
        'Net::hostent'          => '1.00', #./lib/Net/hostent.pm
        'Net::netent'           => '1.00', #./lib/Net/netent.pm
        'Net::Netrc'            => '2.12', #./lib/Net/Netrc.pm
        'Net::NNTP'             => '2.21', #./lib/Net/NNTP.pm
        'Net::Ping'             => '2.19', #./lib/Net/Ping.pm
        'Net::POP3'             => '2.23', #./lib/Net/POP3.pm
        'Net::protoent'         => '1.00', #./lib/Net/protoent.pm
        'Net::servent'          => '1.00', #./lib/Net/servent.pm
        'Net::SMTP'             => '2.24', #./lib/Net/SMTP.pm
        'Net::Time'             => '2.09', #./lib/Net/Time.pm
        'NEXT'                  => '0.50', #./lib/NEXT.pm
        'O'                     => '1.00', #./ext/B/O.pm
        'ODBM_File'             => '1.03', #./ext/ODBM_File/ODBM_File.pm
        'Opcode'                => '1.05', #./ext/Opcode/Opcode.pm
        'open'                  => '1.01', #./lib/open.pm
        'ops'                   => '1.00', #./ext/Opcode/ops.pm
        'OS2::DLL'              => '1.00', #./os2/OS2/REXX/DLL/DLL.pm
        'OS2::ExtAttr'          => '0.01', #./os2/OS2/ExtAttr/ExtAttr.pm
        'OS2::PrfDB'            => '0.02', #./os2/OS2/PrfDB/PrfDB.pm
        'OS2::Process'          => '1.0', #./os2/OS2/Process/Process.pm
        'OS2::REXX'             => '1.01', #./os2/OS2/REXX/REXX.pm
        'overload'              => '1.00', #./lib/overload.pm
        'PerlIO'                => '1.01', #./lib/PerlIO.pm
        'PerlIO::encoding'      => '0.06', #./ext/PerlIO/encoding/encoding.pm
        'PerlIO::scalar'        => '0.01', #./ext/PerlIO/scalar/scalar.pm
        'PerlIO::via'           => '0.01', #./ext/PerlIO/via/via.pm
        'PerlIO::via::QuotedPrint'=> '0.04', #./lib/PerlIO/via/QuotedPrint.pm
        'Pod::Checker'          => '1.3', #./lib/Pod/Checker.pm
        'Pod::Find'             => '0.22', #./lib/Pod/Find.pm
        'Pod::Functions'        => '1.01', #./lib/Pod/Functions.pm
        'Pod::Html'             => '1.04', #./lib/Pod/Html.pm
        'Pod::InputObjects'     => '1.13', #./lib/Pod/InputObjects.pm
        'Pod::LaTeX'            => '0.54', #./lib/Pod/LaTeX.pm
        'Pod::Man'              => '1.33', #./lib/Pod/Man.pm
        'Pod::ParseLink'        => '1.05', #./lib/Pod/ParseLink.pm
        'Pod::Parser'           => '1.13', #./lib/Pod/Parser.pm
        'Pod::ParseUtils'       => '0.22', #./lib/Pod/ParseUtils.pm
        'Pod::Plainer'          => '0.01', #./lib/Pod/Plainer.pm
        'Pod::Select'           => '1.13', #./lib/Pod/Select.pm
        'Pod::Text'             => '2.19', #./lib/Pod/Text.pm
        'Pod::Text::Color'      => '1.03', #./lib/Pod/Text/Color.pm
        'Pod::Text::Overstrike' => '1.08', #./lib/Pod/Text/Overstrike.pm
        'Pod::Text::Termcap'    => '1.09', #./lib/Pod/Text/Termcap.pm
        'Pod::Usage'            => '1.14', #./lib/Pod/Usage.pm
        'POSIX'                 => '1.05', #./ext/POSIX/POSIX.pm
        're'                    => '0.03', #./ext/re/re.pm
        'Safe'                  => '2.07', #./ext/Opcode/Safe.pm
        'Scalar::Util'          => 'undef', #./ext/List/Util/lib/Scalar/Util.pm
        'SDBM_File'             => '1.03', #./ext/SDBM_File/SDBM_File.pm
        'Search::Dict'          => '1.02', #./lib/Search/Dict.pm
        'SelectSaver'           => '1.00', #./lib/SelectSaver.pm
        'SelfLoader'            => '1.0903', #./lib/SelfLoader.pm
        'Shell'                 => '0.4', #./lib/Shell.pm
        'sigtrap'               => '1.02', #./lib/sigtrap.pm
        'Socket'                => '1.75', #./ext/Socket/Socket.pm
        'sort'                  => '1.01', #./lib/sort.pm
        'Storable'              => '2.04', #./ext/Storable/Storable.pm
        'strict'                => '1.02', #./lib/strict.pm
        'subs'                  => '1.00', #./lib/subs.pm
        'Switch'                => '2.09', #./lib/Switch.pm
        'Symbol'                => '1.04', #./lib/Symbol.pm
        'Sys::Hostname'         => '1.1', #./ext/Sys/Hostname/Hostname.pm
        'Sys::Syslog'           => '0.03', #./ext/Sys/Syslog/Syslog.pm
        'Term::ANSIColor'       => '1.04', #./lib/Term/ANSIColor.pm
        'Term::Cap'             => '1.07', #./lib/Term/Cap.pm
        'Term::Complete'        => '1.4', #./lib/Term/Complete.pm
        'Term::ReadLine'        => '1.00', #./lib/Term/ReadLine.pm
        'Test'                  => '1.20', #./lib/Test.pm
        'Test::Builder'         => '0.15', #./lib/Test/Builder.pm
        'Test::Harness'         => '2.26', #./lib/Test/Harness.pm
        'Test::Harness::Assert' => '0.01', #./lib/Test/Harness/Assert.pm
        'Test::Harness::Iterator'=> '0.01', #./lib/Test/Harness/Iterator.pm
        'Test::Harness::Straps' => '0.14', #./lib/Test/Harness/Straps.pm
        'Test::More'            => '0.45', #./lib/Test/More.pm
        'Test::Simple'          => '0.45', #./lib/Test/Simple.pm
        'Text::Abbrev'          => '1.00', #./lib/Text/Abbrev.pm
        'Text::Balanced'        => '1.89', #./lib/Text/Balanced.pm
        'Text::ParseWords'      => '3.21', #./lib/Text/ParseWords.pm
        'Text::Soundex'         => '1.01', #./lib/Text/Soundex.pm
        'Text::Tabs'            => '98.112801', #./lib/Text/Tabs.pm
        'Text::Wrap'            => '2001.0929', #./lib/Text/Wrap.pm
        'Thread'                => '2.00', #./lib/Thread.pm
        'Thread::Queue'         => '2.00', #./lib/Thread/Queue.pm
        'Thread::Semaphore'     => '2.00', #./lib/Thread/Semaphore.pm
        'Thread::Signal'        => '1.00', #./ext/Thread/Thread/Signal.pm
        'Thread::Specific'      => '1.00', #./ext/Thread/Thread/Specific.pm
        'threads'               => '0.99', #./ext/threads/threads.pm
        'threads::shared'       => '0.90', #./ext/threads/shared/shared.pm
        'Tie::Array'            => '1.02', #./lib/Tie/Array.pm
        'Tie::File'             => '0.93', #./lib/Tie/File.pm
        'Tie::Handle'           => '4.1', #./lib/Tie/Handle.pm
        'Tie::Hash'             => '1.00', #./lib/Tie/Hash.pm
        'Tie::Memoize'          => '1.0', #./lib/Tie/Memoize.pm
        'Tie::RefHash'          => '1.30', #./lib/Tie/RefHash.pm
        'Tie::Scalar'           => '1.00', #./lib/Tie/Scalar.pm
        'Tie::SubstrHash'       => '1.00', #./lib/Tie/SubstrHash.pm
        'Time::gmtime'          => '1.02', #./lib/Time/gmtime.pm
        'Time::HiRes'           => '1.20_00', #./ext/Time/HiRes/HiRes.pm
        'Time::Local'           => '1.04', #./lib/Time/Local.pm
        'Time::localtime'       => '1.02', #./lib/Time/localtime.pm
        'Time::tm'              => '1.00', #./lib/Time/tm.pm
        'Unicode::Collate'      => '0.12', #./lib/Unicode/Collate.pm
        'Unicode::Normalize'    => '0.17', #./ext/Unicode/Normalize/Normalize.pm
        'Unicode::UCD'          => '0.2', #./lib/Unicode/UCD.pm
        'UNIVERSAL'             => '1.00', #./lib/UNIVERSAL.pm
        'User::grent'           => '1.00', #./lib/User/grent.pm
        'User::pwent'           => '1.00', #./lib/User/pwent.pm
        'utf8'                  => '1.00', #./lib/utf8.pm
        'vars'                  => '1.01', #./lib/vars.pm
        'VMS::DCLsym'           => '1.02', #./vms/ext/DCLsym/DCLsym.pm
        'VMS::Filespec'         => '1.1', #./vms/ext/Filespec.pm
        'VMS::Stdio'            => '2.3', #./vms/ext/Stdio/Stdio.pm
        'vmsish'                => '1.00', #./lib/vmsish.pm
        'warnings'              => '1.00', #./lib/warnings.pm
        'warnings::register'    => '1.00', #./lib/warnings/register.pm
        'XS::APItest'           => '0.01', #./ext/XS/APItest/APItest.pm
        'XS::Typemap'           => '0.01', #./ext/XS/Typemap/Typemap.pm
        'XSLoader'              => '0.01',
    },

    5.008001 => {
        'AnyDBM_File'           => '1.00', #./lib/AnyDBM_File.pm
        'Attribute::Handlers'   => '0.78', #./lib/Attribute/Handlers.pm
        'attributes'            => '0.06', #./lib/attributes.pm
        'attrs'                 => '1.01', #./lib/attrs.pm
        'AutoLoader'            => '5.60', #./lib/AutoLoader.pm
        'AutoSplit'             => '1.04', #./lib/AutoSplit.pm
        'autouse'               => '1.03', #./lib/autouse.pm
        'B'                     => '1.02', #./lib/B.pm
        'B::Asmdata'            => '1.01', #./lib/B/Asmdata.pm
        'B::Assembler'          => '0.06', #./lib/B/Assembler.pm
        'B::Bblock'             => '1.02', #./lib/B/Bblock.pm
        'B::Bytecode'           => '1.01', #./lib/B/Bytecode.pm
        'B::C'                  => '1.02', #./lib/B/C.pm
        'B::CC'                 => '1.00', #./lib/B/CC.pm
        'B::Concise'            => '0.56', #./lib/B/Concise.pm
        'B::Debug'              => '1.01', #./lib/B/Debug.pm
        'B::Deparse'            => '0.64', #./lib/B/Deparse.pm
        'B::Disassembler'       => '1.03', #./lib/B/Disassembler.pm
        'B::Lint'               => '1.02', #./lib/B/Lint.pm
        'B::Showlex'            => '1.00', #./lib/B/Showlex.pm
        'B::Stackobj'           => '1.00', #./lib/B/Stackobj.pm
        'B::Stash'              => '1.00', #./lib/B/Stash.pm
        'B::Terse'              => '1.02', #./lib/B/Terse.pm
        'B::Xref'               => '1.01', #./lib/B/Xref.pm
        'base'                  => '2.03', #./lib/base.pm
        'Benchmark'             => '1.051', #./lib/Benchmark.pm
        'bigint'                => '0.04', #./lib/bigint.pm
        'bignum'                => '0.14', #./lib/bignum.pm
        'bigrat'                => '0.06', #./lib/bigrat.pm
        'blib'                  => '1.02', #./lib/blib.pm
        'ByteLoader'            => '0.05', #./lib/ByteLoader.pm
        'bytes'                 => '1.01', #./lib/bytes.pm
        'Carp'                  => '1.01', #./lib/Carp.pm
        'Carp::Heavy'           => '1.01', #./lib/Carp/Heavy.pm
        'CGI'                   => '3.00', #./lib/CGI.pm
        'CGI::Apache'           => '1.00', #./lib/CGI/Apache.pm
        'CGI::Carp'             => '1.26', #./lib/CGI/Carp.pm
        'CGI::Cookie'           => '1.24', #./lib/CGI/Cookie.pm
        'CGI::Fast'             => '1.041', #./lib/CGI/Fast.pm
        'CGI::Pretty'           => '1.07_00', #./lib/CGI/Pretty.pm
        'CGI::Push'             => '1.04', #./lib/CGI/Push.pm
        'CGI::Switch'           => '1.00', #./lib/CGI/Switch.pm
        'CGI::Util'             => '1.31', #./lib/CGI/Util.pm
        'charnames'             => '1.02', #./lib/charnames.pm
        'Class::ISA'            => '0.32', #./lib/Class/ISA.pm
        'Class::Struct'         => '0.63', #./lib/Class/Struct.pm
        'Config'                => undef, #./lib/Config.pm
        'constant'              => '1.04', #./lib/constant.pm
        'CPAN'                  => '1.76_01', #./lib/CPAN.pm
        'CPAN::FirstTime'       => '1.60 ', #./lib/CPAN/FirstTime.pm
        'CPAN::Nox'             => '1.03', #./lib/CPAN/Nox.pm
        'Cwd'                   => '2.08', #./lib/Cwd.pm
        'Data::Dumper'          => '2.121', #./lib/Data/Dumper.pm
        'DB'                    => '1.0', #./lib/DB.pm
        'DB_File'               => '1.806', #./lib/DB_File.pm
        'Devel::DProf'          => '20030813.00', #./lib/Devel/DProf.pm
        'Devel::Peek'           => '1.01', #./lib/Devel/Peek.pm
        'Devel::PPPort'         => '2.007', #./lib/Devel/PPPort.pm
        'Devel::SelfStubber'    => '1.03', #./lib/Devel/SelfStubber.pm
        'diagnostics'           => '1.11', #./lib/diagnostics.pm
        'Digest'                => '1.02', #./lib/Digest.pm
        'Digest::MD5'           => '2.27', #./lib/Digest/MD5.pm
        'DirHandle'             => '1.00', #./lib/DirHandle.pm
        'Dumpvalue'             => '1.11', #./lib/Dumpvalue.pm
        'DynaLoader'            => '1.04', #./lib/DynaLoader.pm
        'Encode'                => '1.9801', #./lib/Encode.pm
        'Encode::Alias'         => '1.38', #./lib/Encode/Alias.pm
        'Encode::Byte'          => '1.23', #./lib/Encode/Byte.pm
        'Encode::CJKConstants'  => '1.02', #./lib/Encode/CJKConstants.pm
        'Encode::CN'            => '1.24', #./lib/Encode/CN.pm
        'Encode::CN::HZ'        => '1.05', #./lib/Encode/CN/HZ.pm
        'Encode::Config'        => '1.07', #./lib/Encode/Config.pm
        'Encode::EBCDIC'        => '1.21', #./lib/Encode/EBCDIC.pm
        'Encode::Encoder'       => '0.07', #./lib/Encode/Encoder.pm
        'Encode::Encoding'      => '1.33', #./lib/Encode/Encoding.pm
        'Encode::Guess'         => '1.09', #./lib/Encode/Guess.pm
        'Encode::JP'            => '1.25', #./lib/Encode/JP.pm
        'Encode::JP::H2Z'       => '1.02', #./lib/Encode/JP/H2Z.pm
        'Encode::JP::JIS7'      => '1.12', #./lib/Encode/JP/JIS7.pm
        'Encode::KR'            => '1.23', #./lib/Encode/KR.pm
        'Encode::KR::2022_KR'   => '1.06', #./lib/Encode/KR/2022_KR.pm
        'Encode::MIME::Header'  => '1.09', #./lib/Encode/MIME/Header.pm
        'Encode::Symbol'        => '1.22', #./lib/Encode/Symbol.pm
        'Encode::TW'            => '1.26', #./lib/Encode/TW.pm
        'Encode::Unicode'       => '1.40', #./lib/Encode/Unicode.pm
        'Encode::Unicode::UTF7' => '0.02', #./lib/Encode/Unicode/UTF7.pm
        'encoding'              => '1.47', #./lib/encoding.pm
        'English'               => '1.01', #./lib/English.pm
        'Env'                   => '1.00', #./lib/Env.pm
        'Errno'                 => '1.09_00', #./lib/Errno.pm
        'Exporter'              => '5.567', #./lib/Exporter.pm
        'Exporter::Heavy'       => '5.567', #./lib/Exporter/Heavy.pm
        'ExtUtils::Command'     => '1.05', #./lib/ExtUtils/Command.pm
        'ExtUtils::Command::MM' => '0.03', #./lib/ExtUtils/Command/MM.pm
        'ExtUtils::Constant'    => '0.14', #./lib/ExtUtils/Constant.pm
        'ExtUtils::Embed'       => '1.250601', #./lib/ExtUtils/Embed.pm
        'ExtUtils::Install'     => '1.32', #./lib/ExtUtils/Install.pm
        'ExtUtils::Installed'   => '0.08', #./lib/ExtUtils/Installed.pm
        'ExtUtils::Liblist'     => '1.01', #./lib/ExtUtils/Liblist.pm
        'ExtUtils::Liblist::Kid'=> '1.3', #./lib/ExtUtils/Liblist/Kid.pm
        'ExtUtils::MakeMaker'   => '6.17', #./lib/ExtUtils/MakeMaker.pm
        'ExtUtils::MakeMaker::bytes'=> '0.01', #./lib/ExtUtils/MakeMaker/bytes.pm
        'ExtUtils::MakeMaker::vmsish'=> '0.01', #./lib/ExtUtils/MakeMaker/vmsish.pm
        'ExtUtils::Manifest'    => '1.42', #./lib/ExtUtils/Manifest.pm
        'ExtUtils::Miniperl'    => undef, #./lib/ExtUtils/Miniperl.pm
        'ExtUtils::Mkbootstrap' => '1.15', #./lib/ExtUtils/Mkbootstrap.pm
        'ExtUtils::Mksymlists'  => '1.19', #./lib/ExtUtils/Mksymlists.pm
        'ExtUtils::MM'          => '0.04', #./lib/ExtUtils/MM.pm
        'ExtUtils::MM_Any'      => '0.07', #./lib/ExtUtils/MM_Any.pm
        'ExtUtils::MM_BeOS'     => '1.04', #./lib/ExtUtils/MM_BeOS.pm
        'ExtUtils::MM_Cygwin'   => '1.06', #./lib/ExtUtils/MM_Cygwin.pm
        'ExtUtils::MM_DOS'      => '0.02', #./lib/ExtUtils/MM_DOS.pm
        'ExtUtils::MM_MacOS'    => '1.07', #./lib/ExtUtils/MM_MacOS.pm
        'ExtUtils::MM_NW5'      => '2.06', #./lib/ExtUtils/MM_NW5.pm
        'ExtUtils::MM_OS2'      => '1.04', #./lib/ExtUtils/MM_OS2.pm
        'ExtUtils::MM_Unix'     => '1.42', #./lib/ExtUtils/MM_Unix.pm
        'ExtUtils::MM_UWIN'     => '0.02', #./lib/ExtUtils/MM_UWIN.pm
        'ExtUtils::MM_VMS'      => '5.70', #./lib/ExtUtils/MM_VMS.pm
        'ExtUtils::MM_Win32'    => '1.09', #./lib/ExtUtils/MM_Win32.pm
        'ExtUtils::MM_Win95'    => '0.03', #./lib/ExtUtils/MM_Win95.pm
        'ExtUtils::MY'          => '0.01', #./lib/ExtUtils/MY.pm
        'ExtUtils::Packlist'    => '0.04', #./lib/ExtUtils/Packlist.pm
        'ExtUtils::testlib'     => '1.15', #./lib/ExtUtils/testlib.pm
        'ExtUtils::XSSymSet'    => '1.0',  #./vms/ext/XSSymSet.pm
        'Fatal'                 => '1.03', #./lib/Fatal.pm
        'Fcntl'                 => '1.05', #./lib/Fcntl.pm
        'fields'                => '2.03', #./lib/fields.pm
        'File::Basename'        => '2.72', #./lib/File/Basename.pm
        'File::CheckTree'       => '4.2', #./lib/File/CheckTree.pm
        'File::Compare'         => '1.1003', #./lib/File/Compare.pm
        'File::Copy'            => '2.06', #./lib/File/Copy.pm
        'File::DosGlob'         => '1.00', #./lib/File/DosGlob.pm
        'File::Find'            => '1.05', #./lib/File/Find.pm
        'File::Glob'            => '1.02', #./lib/File/Glob.pm
        'File::Path'            => '1.06', #./lib/File/Path.pm
        'File::Spec'            => '0.86', #./lib/File/Spec.pm
        'File::Spec::Cygwin'    => '1.1', #./lib/File/Spec/Cygwin.pm
        'File::Spec::Epoc'      => '1.1', #./lib/File/Spec/Epoc.pm
        'File::Spec::Functions' => '1.3', #./lib/File/Spec/Functions.pm
        'File::Spec::Mac'       => '1.4', #./lib/File/Spec/Mac.pm
        'File::Spec::OS2'       => '1.2', #./lib/File/Spec/OS2.pm
        'File::Spec::Unix'      => '1.5', #./lib/File/Spec/Unix.pm
        'File::Spec::VMS'       => '1.4', #./lib/File/Spec/VMS.pm
        'File::Spec::Win32'     => '1.4', #./lib/File/Spec/Win32.pm
        'File::stat'            => '1.00', #./lib/File/stat.pm
        'File::Temp'            => '0.14', #./lib/File/Temp.pm
        'FileCache'             => '1.03', #./lib/FileCache.pm
        'FileHandle'            => '2.01', #./lib/FileHandle.pm
        'filetest'              => '1.01', #./lib/filetest.pm
        'Filter::Simple'        => '0.78', #./lib/Filter/Simple.pm
        'Filter::Util::Call'    => '1.0601', #./lib/Filter/Util/Call.pm
        'FindBin'               => '1.43', #./lib/FindBin.pm
        'GDBM_File'             => '1.07', #./ext/GDBM_File/GDBM_File.pm
        'Getopt::Long'          => '2.34', #./lib/Getopt/Long.pm
        'Getopt::Std'           => '1.04', #./lib/Getopt/Std.pm
        'Hash::Util'            => '0.05', #./lib/Hash/Util.pm
        'I18N::Collate'         => '1.00', #./lib/I18N/Collate.pm
        'I18N::Langinfo'        => '0.02', #./lib/I18N/Langinfo.pm
        'I18N::LangTags'        => '0.28', #./lib/I18N/LangTags.pm
        'I18N::LangTags::List'  => '0.26', #./lib/I18N/LangTags/List.pm
        'if'                    => '0.03', #./lib/if.pm
        'integer'               => '1.00', #./lib/integer.pm
        'IO'                    => '1.21', #./lib/IO.pm
        'IO::Dir'               => '1.04', #./lib/IO/Dir.pm
        'IO::File'              => '1.10', #./lib/IO/File.pm
        'IO::Handle'            => '1.23', #./lib/IO/Handle.pm
        'IO::Pipe'              => '1.122', #./lib/IO/Pipe.pm
        'IO::Poll'              => '0.06', #./lib/IO/Poll.pm
        'IO::Seekable'          => '1.09', #./lib/IO/Seekable.pm
        'IO::Select'            => '1.16', #./lib/IO/Select.pm
        'IO::Socket'            => '1.28', #./lib/IO/Socket.pm
        'IO::Socket::INET'      => '1.27', #./lib/IO/Socket/INET.pm
        'IO::Socket::UNIX'      => '1.21', #./lib/IO/Socket/UNIX.pm
        'IPC::Msg'              => '1.02', #./lib/IPC/Msg.pm
        'IPC::Open2'            => '1.01', #./lib/IPC/Open2.pm
        'IPC::Open3'            => '1.0105', #./lib/IPC/Open3.pm
        'IPC::Semaphore'        => '1.02', #./lib/IPC/Semaphore.pm
        'IPC::SysV'             => '1.04', #./lib/IPC/SysV.pm
        'JNI'                   => '0.2', #./jpl/JNI/JNI.pm
        'JPL::AutoLoader'       => undef, #./jpl/JPL/AutoLoader.pm
        'JPL::Class'            => undef, #./jpl/JPL/Class.pm
        'JPL::Compile'          => undef, #./jpl/JPL/Compile.pm
        'less'                  => '0.01', #./lib/less.pm
        'lib'                   => '0.5565', #./lib/lib.pm
        'List::Util'            => '1.13', #./lib/List/Util.pm
        'locale'                => '1.00', #./lib/locale.pm
        'Locale::Constants'     => '2.01', #./lib/Locale/Constants.pm
        'Locale::Country'       => '2.61', #./lib/Locale/Country.pm
        'Locale::Currency'      => '2.21', #./lib/Locale/Currency.pm
        'Locale::Language'      => '2.21', #./lib/Locale/Language.pm
        'Locale::Maketext'      => '1.06', #./lib/Locale/Maketext.pm
        'Locale::Maketext::Guts'=> undef, #./lib/Locale/Maketext/Guts.pm
        'Locale::Maketext::GutsLoader'=> undef, #./lib/Locale/Maketext/GutsLoader.pm
        'Locale::Script'        => '2.21', #./lib/Locale/Script.pm
        'Math::BigFloat'        => '1.40', #./lib/Math/BigFloat.pm
        'Math::BigFloat::Trace' => '0.01', #./lib/Math/BigFloat/Trace.pm
        'Math::BigInt'          => '1.66', #./lib/Math/BigInt.pm
        'Math::BigInt::Calc'    => '0.36', #./lib/Math/BigInt/Calc.pm
        'Math::BigInt::Scalar'  => '0.11', #./lib/Math/BigInt/Scalar.pm
        'Math::BigInt::Trace'   => '0.01', #./lib/Math/BigInt/Trace.pm
        'Math::BigRat'          => '0.10', #./lib/Math/BigRat.pm
        'Math::Complex'         => '1.34', #./lib/Math/Complex.pm
        'Math::Trig'            => '1.02', #./lib/Math/Trig.pm
        'Memoize'               => '1.01', #./lib/Memoize.pm
        'Memoize::AnyDBM_File'  => '0.65', #./lib/Memoize/AnyDBM_File.pm
        'Memoize::Expire'       => '1.00', #./lib/Memoize/Expire.pm
        'Memoize::ExpireFile'   => '1.01', #./lib/Memoize/ExpireFile.pm
        'Memoize::ExpireTest'   => '0.65', #./lib/Memoize/ExpireTest.pm
        'Memoize::NDBM_File'    => '0.65', #./lib/Memoize/NDBM_File.pm
        'Memoize::SDBM_File'    => '0.65', #./lib/Memoize/SDBM_File.pm
        'Memoize::Storable'     => '0.65', #./lib/Memoize/Storable.pm
        'MIME::Base64'          => '2.20', #./lib/MIME/Base64.pm
        'MIME::QuotedPrint'     => '2.20', #./lib/MIME/QuotedPrint.pm
        'NDBM_File'             => '1.05', #./ext/NDBM_File/NDBM_File.pm
        'Net::Cmd'              => '2.24', #./lib/Net/Cmd.pm
        'Net::Config'           => '1.10', #./lib/Net/Config.pm
        'Net::Domain'           => '2.18', #./lib/Net/Domain.pm
        'Net::FTP'              => '2.71', #./lib/Net/FTP.pm
        'Net::FTP::A'           => '1.16', #./lib/Net/FTP/A.pm
        'Net::FTP::dataconn'    => '0.11', #./lib/Net/FTP/dataconn.pm
        'Net::FTP::E'           => '0.01', #./lib/Net/FTP/E.pm
        'Net::FTP::I'           => '1.12', #./lib/Net/FTP/I.pm
        'Net::FTP::L'           => '0.01', #./lib/Net/FTP/L.pm
        'Net::hostent'          => '1.01', #./lib/Net/hostent.pm
        'Net::netent'           => '1.00', #./lib/Net/netent.pm
        'Net::Netrc'            => '2.12', #./lib/Net/Netrc.pm
        'Net::NNTP'             => '2.22', #./lib/Net/NNTP.pm
        'Net::Ping'             => '2.31', #./lib/Net/Ping.pm
        'Net::POP3'             => '2.24', #./lib/Net/POP3.pm
        'Net::protoent'         => '1.00', #./lib/Net/protoent.pm
        'Net::servent'          => '1.01', #./lib/Net/servent.pm
        'Net::SMTP'             => '2.26', #./lib/Net/SMTP.pm
        'Net::Time'             => '2.09', #./lib/Net/Time.pm
        'NEXT'                  => '0.60', #./lib/NEXT.pm
        'O'                     => '1.00', #./lib/O.pm
        'ODBM_File'             => '1.04', #./ext/ODBM_File/ODBM_File.pm
        'Opcode'                => '1.05', #./lib/Opcode.pm
        'open'                  => '1.02', #./lib/open.pm
        'ops'                   => '1.00', #./lib/ops.pm
        'OS2::ExtAttr'          => '0.02', #./os2/OS2/ExtAttr/ExtAttr.pm
        'OS2::PrfDB'            => '0.03', #./os2/OS2/PrfDB/PrfDB.pm
        'OS2::Process'          => '1.01', #./os2/OS2/Process/Process.pm
        'OS2::DLL'              => '1.01', #./os2/OS2/REXX/DLL/DLL.pm
        'OS2::REXX'             => '1.02', #./os2/OS2/REXX/REXX.pm
        'overload'              => '1.01', #./lib/overload.pm
        'PerlIO'                => '1.02', #./lib/PerlIO.pm
        'PerlIO::encoding'      => '0.07', #./lib/PerlIO/encoding.pm
        'PerlIO::scalar'        => '0.02', #./lib/PerlIO/scalar.pm
        'PerlIO::via'           => '0.02', #./lib/PerlIO/via.pm
        'PerlIO::via::QuotedPrint'=> '0.05', #./lib/PerlIO/via/QuotedPrint.pm
        'Pod::Checker'          => '1.41', #./lib/Pod/Checker.pm
        'Pod::Find'             => '0.24', #./lib/Pod/Find.pm
        'Pod::Functions'        => '1.02', #./lib/Pod/Functions.pm
        'Pod::Html'             => '1.0501', #./lib/Pod/Html.pm
        'Pod::InputObjects'     => '1.14', #./lib/Pod/InputObjects.pm
        'Pod::LaTeX'            => '0.55', #./lib/Pod/LaTeX.pm
        'Pod::Man'              => '1.37', #./lib/Pod/Man.pm
        'Pod::ParseLink'        => '1.06', #./lib/Pod/ParseLink.pm
        'Pod::Parser'           => '1.13', #./lib/Pod/Parser.pm
        'Pod::ParseUtils'       => '0.3', #./lib/Pod/ParseUtils.pm
        'Pod::Perldoc'          => '3.10', #./lib/Pod/Perldoc.pm
        'Pod::Perldoc::BaseTo'  => undef, #./lib/Pod/Perldoc/BaseTo.pm
        'Pod::Perldoc::GetOptsOO'=> undef, #./lib/Pod/Perldoc/GetOptsOO.pm
        'Pod::Perldoc::ToChecker'=> undef, #./lib/Pod/Perldoc/ToChecker.pm
        'Pod::Perldoc::ToMan'   => undef, #./lib/Pod/Perldoc/ToMan.pm
        'Pod::Perldoc::ToNroff' => undef, #./lib/Pod/Perldoc/ToNroff.pm
        'Pod::Perldoc::ToPod'   => undef, #./lib/Pod/Perldoc/ToPod.pm
        'Pod::Perldoc::ToRtf'   => undef, #./lib/Pod/Perldoc/ToRtf.pm
        'Pod::Perldoc::ToText'  => undef, #./lib/Pod/Perldoc/ToText.pm
        'Pod::Perldoc::ToTk'    => 'undef', #./lib/Pod/Perldoc/ToTk.pm
        'Pod::Perldoc::ToXml'   => undef, #./lib/Pod/Perldoc/ToXml.pm
        'Pod::Plainer'          => '0.01', #./lib/Pod/Plainer.pm
        'Pod::PlainText'        => '2.01', #./lib/Pod/PlainText.pm
        'Pod::Select'           => '1.13', #./lib/Pod/Select.pm
        'Pod::Text'             => '2.21', #./lib/Pod/Text.pm
        'Pod::Text::Color'      => '1.04', #./lib/Pod/Text/Color.pm
        'Pod::Text::Overstrike' => '1.1', #./lib/Pod/Text/Overstrike.pm
        'Pod::Text::Termcap'    => '1.11', #./lib/Pod/Text/Termcap.pm
        'Pod::Usage'            => '1.16', #./lib/Pod/Usage.pm
        'POSIX'                 => '1.06', #./lib/POSIX.pm
        're'                    => '0.04', #./lib/re.pm
        'Safe'                  => '2.10', #./lib/Safe.pm
        'Scalar::Util'          => '1.13', #./lib/Scalar/Util.pm
        'SDBM_File'             => '1.04', #./lib/SDBM_File.pm
        'Search::Dict'          => '1.02', #./lib/Search/Dict.pm
        'SelectSaver'           => '1.00', #./lib/SelectSaver.pm
        'SelfLoader'            => '1.0904', #./lib/SelfLoader.pm
        'Shell'                 => '0.5', #./lib/Shell.pm
        'sigtrap'               => '1.02', #./lib/sigtrap.pm
        'Socket'                => '1.76', #./lib/Socket.pm
        'sort'                  => '1.02', #./lib/sort.pm
        'Storable'              => '2.08', #./lib/Storable.pm
        'strict'                => '1.03', #./lib/strict.pm
        'subs'                  => '1.00', #./lib/subs.pm
        'Switch'                => '2.10', #./lib/Switch.pm
        'Symbol'                => '1.05', #./lib/Symbol.pm
        'Sys::Hostname'         => '1.11', #./lib/Sys/Hostname.pm
        'Sys::Syslog'           => '0.04', #./lib/Sys/Syslog.pm
        'Term::ANSIColor'       => '1.07', #./lib/Term/ANSIColor.pm
        'Term::Cap'             => '1.08', #./lib/Term/Cap.pm
        'Term::Complete'        => '1.401', #./lib/Term/Complete.pm
        'Term::ReadLine'        => '1.01', #./lib/Term/ReadLine.pm
        'Test'                  => '1.24', #./lib/Test.pm
        'Test::Builder'         => '0.17', #./lib/Test/Builder.pm
        'Test::Harness'         => '2.30', #./lib/Test/Harness.pm
        'Test::Harness::Assert' => '0.01', #./lib/Test/Harness/Assert.pm
        'Test::Harness::Iterator'=> '0.01', #./lib/Test/Harness/Iterator.pm
        'Test::Harness::Straps' => '0.15', #./lib/Test/Harness/Straps.pm
        'Test::More'            => '0.47', #./lib/Test/More.pm
        'Test::Simple'          => '0.47', #./lib/Test/Simple.pm
        'Text::Abbrev'          => '1.01', #./lib/Text/Abbrev.pm
        'Text::Balanced'        => '1.95', #./lib/Text/Balanced.pm
        'Text::ParseWords'      => '3.21', #./lib/Text/ParseWords.pm
        'Text::Soundex'         => '1.01', #./lib/Text/Soundex.pm
        'Text::Tabs'            => '98.112801', #./lib/Text/Tabs.pm
        'Text::Wrap'            => '2001.09291', #./lib/Text/Wrap.pm
        'Thread'                => '2.00', #./lib/Thread.pm
        'Thread::Queue'         => '2.00', #./lib/Thread/Queue.pm
        'Thread::Semaphore'     => '2.01', #./lib/Thread/Semaphore.pm
        'Thread::Signal'        => '1.00', #./ext/Thread/Thread/Signal.pm
        'Thread::Specific'      => '1.00', #./ext/Thread/Thread/Specific.pm
        'threads'               => '1.00', #./lib/threads.pm
        'threads::shared'       => '0.91', #./lib/threads/shared.pm
        'Tie::Array'            => '1.03', #./lib/Tie/Array.pm
        'Tie::File'             => '0.97', #./lib/Tie/File.pm
        'Tie::Handle'           => '4.1', #./lib/Tie/Handle.pm
        'Tie::Hash'             => '1.00', #./lib/Tie/Hash.pm
        'Tie::Memoize'          => '1.0', #./lib/Tie/Memoize.pm
        'Tie::RefHash'          => '1.31', #./lib/Tie/RefHash.pm
        'Tie::Scalar'           => '1.00', #./lib/Tie/Scalar.pm
        'Tie::SubstrHash'       => '1.00', #./lib/Tie/SubstrHash.pm
        'Time::gmtime'          => '1.02', #./lib/Time/gmtime.pm
        'Time::HiRes'           => '1.51', #./lib/Time/HiRes.pm
        'Time::Local'           => '1.07', #./lib/Time/Local.pm
        'Time::localtime'       => '1.02', #./lib/Time/localtime.pm
        'Time::tm'              => '1.00', #./lib/Time/tm.pm
        'Unicode::Collate'      => '0.28', #./lib/Unicode/Collate.pm
        'Unicode::Normalize'    => '0.23', #./lib/Unicode/Normalize.pm
        'Unicode::UCD'          => '0.21', #./lib/Unicode/UCD.pm
        'UNIVERSAL'             => '1.01', #./lib/UNIVERSAL.pm
        'User::grent'           => '1.00', #./lib/User/grent.pm
        'User::pwent'           => '1.00', #./lib/User/pwent.pm
        'utf8'                  => '1.02', #./lib/utf8.pm
        'vars'                  => '1.01', #./lib/vars.pm
        'VMS::DCLsym'           => '1.02', #./vms/ext/DCLsym/DCLsym.pm
        'VMS::Filespec'         => '1.11', #./vms/ext/Filespec.pm
        'VMS::Stdio'            => '2.3', #./vms/ext/Stdio/Stdio.pm
        'vmsish'                => '1.01', #./lib/vmsish.pm
        'warnings'              => '1.03', #./lib/warnings.pm
        'warnings::register'    => '1.00', #./lib/warnings/register.pm
        'XS::APItest'           => '0.02', #./lib/XS/APItest.pm
        'XS::Typemap'           => '0.01', #./lib/XS/Typemap.pm
        'XSLoader'              => '0.02', #./lib/XSLoader.pm
    },

    5.008002 => {
        'AnyDBM_File' => '1.00',  #AnyDBM_File.pm
        'Attribute::Handlers' => 0.78, #Attribute\Handlers.pm
        'attributes' => 0.06,   #attributes.pm
        'attrs' => 1.01,        #attrs.pm
        'AutoLoader' => '5.60',   #AutoLoader.pm
        'AutoSplit' => 1.04,    #AutoSplit.pm
        'autouse' => 1.03,      #autouse.pm
        'B' => 1.02,            #B.pm
        'B::Asmdata' => 1.01,   #B\Asmdata.pm
        'B::Assembler' => 0.06, #B\Assembler.pm
        'B::Bblock' => 1.02,    #B\Bblock.pm
        'B::Bytecode' => 1.01,  #B\Bytecode.pm
        'B::C' => 1.02,         #B\C.pm
        'B::CC' => '1.00',        #B\CC.pm
        'B::Concise' => 0.56,   #B\Concise.pm
        'B::Debug' => 1.01,     #B\Debug.pm
        'B::Deparse' => 0.64,   #B\Deparse.pm
        'B::Disassembler' => 1.03, #B\Disassembler.pm
        'B::Lint' => 1.02,      #B\Lint.pm
        'B::Showlex' => '1.00',   #B\Showlex.pm
        'B::Stackobj' => '1.00',  #B\Stackobj.pm
        'B::Stash' => '1.00',     #B\Stash.pm
        'B::Terse' => 1.02,     #B\Terse.pm
        'B::Xref' => 1.01,      #B\Xref.pm
        'base' => 2.03,         #base.pm
        'Benchmark' => 1.051,   #Benchmark.pm
        'bigint' => 0.04,       #bigint.pm
        'bignum' => 0.14,       #bignum.pm
        'bigrat' => 0.06,       #bigrat.pm
        'blib' => 1.02,         #blib.pm
        'ByteLoader' => 0.05,   #ByteLoader.pm
        'bytes' => 1.01,        #bytes.pm
        'Carp' => 1.01,         #Carp.pm
        'Carp::Heavy' => 1.01,  #Carp\Heavy.pm
        'CGI' => '3.00',          #CGI.pm
        'CGI::Apache' => '1.00',  #CGI\Apache.pm
        'CGI::Carp' => 1.26,    #CGI\Carp.pm
        'CGI::Cookie' => 1.24,  #CGI\Cookie.pm
        'CGI::Fast' => 1.041,   #CGI\Fast.pm
        'CGI::Pretty' => '1.07_00', #CGI\Pretty.pm
        'CGI::Push' => 1.04,    #CGI\Push.pm
        'CGI::Switch' => '1.00',  #CGI\Switch.pm
        'CGI::Util' => 1.31,    #CGI\Util.pm
        'charnames' => 1.02,    #charnames.pm
        'Class::ISA' => 0.32,   #Class\ISA.pm
        'Class::Struct' => 0.63, #Class\Struct.pm
        'Config' => undef,      #Config.pm
        'constant' => 1.04,     #constant.pm
        'CPAN' => '1.76_01',      #CPAN.pm
        'CPAN::FirstTime' => '1.60 ', #CPAN\FirstTime.pm
        'CPAN::Nox' => 1.03,    #CPAN\Nox.pm
        'Cwd' => 2.08,          #Cwd.pm
        'Data::Dumper' => 2.121, #Data\Dumper.pm
        'DB' => '1.0',            #DB.pm
        'Devel::DProf' => '20030813.00', #Devel\DProf.pm
        'Devel::Peek' => 1.01,  #Devel\Peek.pm
        'Devel::PPPort' => 2.009, #Devel\PPPort.pm
        'Devel::SelfStubber' => 1.03, #Devel\SelfStubber.pm
        'diagnostics' => 1.11,  #diagnostics.pm
        'Digest' => 1.02,       #Digest.pm
        'Digest::MD5' => '2.30',  #Digest\MD5.pm
        'DirHandle' => '1.00',    #DirHandle.pm
        'Dumpvalue' => 1.11,    #Dumpvalue.pm
        'DynaLoader' => 1.04,   #DynaLoader.pm
        'Encode' => 1.9801,     #Encode.pm
        'Encode::Alias' => 1.38, #Encode\Alias.pm
        'Encode::Byte' => 1.23, #Encode\Byte.pm
        'Encode::CJKConstants' => 1.02, #Encode\CJKConstants.pm
        'Encode::CN' => 1.24,   #Encode\CN.pm
        'Encode::CN::HZ' => 1.05, #Encode\CN\HZ.pm
        'Encode::Config' => 1.07, #Encode\Config.pm
        'Encode::EBCDIC' => 1.21, #Encode\EBCDIC.pm
        'Encode::Encoder' => 0.07, #Encode\Encoder.pm
        'Encode::Encoding' => 1.33, #Encode\Encoding.pm
        'Encode::Guess' => 1.09, #Encode\Guess.pm
        'Encode::JP' => 1.25,   #Encode\JP.pm
        'Encode::JP::H2Z' => 1.02, #Encode\JP\H2Z.pm
        'Encode::JP::JIS7' => 1.12, #Encode\JP\JIS7.pm
        'Encode::KR' => 1.23,   #Encode\KR.pm
        'Encode::KR::2022_KR' => 1.06, #Encode\KR\2022_KR.pm
        'Encode::MIME::Header' => 1.09, #Encode\MIME\Header.pm
        'Encode::Symbol' => 1.22, #Encode\Symbol.pm
        'Encode::TW' => 1.26,   #Encode\TW.pm
        'Encode::Unicode' => '1.40', #Encode\Unicode.pm
        'Encode::Unicode::UTF7' => 0.02, #Encode\Unicode\UTF7.pm
        'encoding' => 1.47,     #encoding.pm
        'English' => 1.01,      #English.pm
        'Env' => '1.00',          #Env.pm
        'Errno' => '1.09_00',     #Errno.pm
        'Exporter' => 5.567,    #Exporter.pm
        'Exporter::Heavy' => 5.567, #Exporter\Heavy.pm
        'ExtUtils::Command' => 1.05, #ExtUtils\Command.pm
        'ExtUtils::Command::MM' => 0.03, #ExtUtils\Command\MM.pm
        'ExtUtils::Constant' => 0.14, #ExtUtils\Constant.pm
        'ExtUtils::Embed' => 1.250601, #ExtUtils\Embed.pm
        'ExtUtils::Install' => 1.32, #ExtUtils\Install.pm
        'ExtUtils::Installed' => 0.08, #ExtUtils\Installed.pm
        'ExtUtils::Liblist' => 1.01, #ExtUtils\Liblist.pm
        'ExtUtils::Liblist::Kid' => 1.3, #ExtUtils\Liblist\Kid.pm
        'ExtUtils::MakeMaker' => 6.17, #ExtUtils\MakeMaker.pm
        'ExtUtils::MakeMaker::bytes' => 0.01, #ExtUtils\MakeMaker\bytes.pm
        'ExtUtils::MakeMaker::vmsish' => 0.01, #ExtUtils\MakeMaker\vmsish.pm
        'ExtUtils::Manifest' => 1.42, #ExtUtils\Manifest.pm
        'ExtUtils::Miniperl' => undef, #ExtUtils\Miniperl.pm
        'ExtUtils::Mkbootstrap' => 1.15, #ExtUtils\Mkbootstrap.pm
        'ExtUtils::Mksymlists' => 1.19, #ExtUtils\Mksymlists.pm
        'ExtUtils::MM' => 0.04, #ExtUtils\MM.pm
        'ExtUtils::MM_Any' => 0.07, #ExtUtils\MM_Any.pm
        'ExtUtils::MM_BeOS' => 1.04, #ExtUtils\MM_BeOS.pm
        'ExtUtils::MM_Cygwin' => 1.06, #ExtUtils\MM_Cygwin.pm
        'ExtUtils::MM_DOS' => 0.02, #ExtUtils\MM_DOS.pm
        'ExtUtils::MM_MacOS' => 1.07, #ExtUtils\MM_MacOS.pm
        'ExtUtils::MM_NW5' => 2.06, #ExtUtils\MM_NW5.pm
        'ExtUtils::MM_OS2' => 1.04, #ExtUtils\MM_OS2.pm
        'ExtUtils::MM_Unix' => 1.42, #ExtUtils\MM_Unix.pm
        'ExtUtils::MM_UWIN' => 0.02, #ExtUtils\MM_UWIN.pm
        'ExtUtils::MM_VMS' => '5.70', #ExtUtils\MM_VMS.pm
        'ExtUtils::MM_Win32' => 1.09, #ExtUtils\MM_Win32.pm
        'ExtUtils::MM_Win95' => 0.03, #ExtUtils\MM_Win95.pm
        'ExtUtils::MY' => 0.01, #ExtUtils\MY.pm
        'ExtUtils::Packlist' => 0.04, #ExtUtils\Packlist.pm
        'ExtUtils::testlib' => 1.15, #ExtUtils\testlib.pm
        'ExtUtils::XSSymSet' => '1.0',  #vms\ext\XSSymSet.pm
        'Fatal' => 1.03,        #Fatal.pm
        'Fcntl' => 1.05,        #Fcntl.pm
        'fields' => 2.03,       #fields.pm
        'File::Basename' => 2.72, #File\Basename.pm
        'File::CheckTree' => 4.2, #File\CheckTree.pm
        'File::Compare' => 1.1003, #File\Compare.pm
        'File::Copy' => 2.06,   #File\Copy.pm
        'File::DosGlob' => '1.00', #File\DosGlob.pm
        'File::Find' => 1.05,   #File\Find.pm
        'File::Glob' => 1.02,   #File\Glob.pm
        'File::Path' => 1.06,   #File\Path.pm
        'File::Spec' => 0.86,   #File\Spec.pm
        'File::Spec::Cygwin' => 1.1, #File\Spec\Cygwin.pm
        'File::Spec::Epoc' => 1.1, #File\Spec\Epoc.pm
        'File::Spec::Functions' => 1.3, #File\Spec\Functions.pm
        'File::Spec::Mac' => 1.4, #File\Spec\Mac.pm
        'File::Spec::OS2' => 1.2, #File\Spec\OS2.pm
        'File::Spec::Unix' => 1.5, #File\Spec\Unix.pm
        'File::Spec::VMS' => 1.4, #File\Spec\VMS.pm
        'File::Spec::Win32' => 1.4, #File\Spec\Win32.pm
        'File::stat' => '1.00',   #File\stat.pm
        'File::Temp' => 0.14,   #File\Temp.pm
        'FileCache' => 1.03,    #FileCache.pm
        'FileHandle' => 2.01,   #FileHandle.pm
        'filetest' => 1.01,     #filetest.pm
        'Filter::Simple' => 0.78, #Filter\Simple.pm
        'Filter::Util::Call' => 1.0601, #Filter\Util\Call.pm
        'FindBin' => 1.43,      #FindBin.pm
        'GDBM_File' => '1.07', #ext\GDBM_File\GDBM_File.pm
        'Getopt::Long' => 2.34, #Getopt\Long.pm
        'Getopt::Std' => 1.04,  #Getopt\Std.pm
        'Hash::Util' => 0.05,   #Hash\Util.pm
        'I18N::Collate' => '1.00', #I18N\Collate.pm
        'I18N::Langinfo' => '0.02', #I18N\Langinfo.pm
        'I18N::LangTags' => 0.29, #I18N\LangTags.pm
        'I18N::LangTags::List' => 0.29, #I18N\LangTags\List.pm
        'if' => 0.03,           #if.pm
        'integer' => '1.00',      #integer.pm
        'IO' => 1.21,           #IO.pm
        'IO::Dir' => 1.04,      #IO\Dir.pm
        'IO::File' => '1.10',     #IO\File.pm
        'IO::Handle' => 1.23,   #IO\Handle.pm
        'IO::Pipe' => 1.122,    #IO\Pipe.pm
        'IO::Poll' => 0.06,     #IO\Poll.pm
        'IO::Seekable' => 1.09, #IO\Seekable.pm
        'IO::Select' => 1.16,   #IO\Select.pm
        'IO::Socket' => 1.28,   #IO\Socket.pm
        'IO::Socket::INET' => 1.27, #IO\Socket\INET.pm
        'IO::Socket::UNIX' => 1.21, #IO\Socket\UNIX.pm
        'IPC::Msg' => 1.02,     #IPC\Msg.pm
        'IPC::Open2' => 1.01,   #IPC\Open2.pm
        'IPC::Open3' => 1.0105, #IPC\Open3.pm
        'IPC::Semaphore' => 1.02, #IPC\Semaphore.pm
        'IPC::SysV' => 1.04,    #IPC\SysV.pm
        'JNI' => '0.2',         #jpl\JNI\JNI.pm
        'JPL::AutoLoader' => undef, #jpl\JPL\AutoLoader.pm
        'JPL::Class' => undef,  #jpl\JPL\Class.pm
        'JPL::Compile' => undef, #jpl\JPL\Compile.pm
        'less' => 0.01,         #less.pm
        'lib' => 0.5565,        #lib.pm
        'List::Util' => 1.13,   #List\Util.pm
        'locale' => '1.00',       #locale.pm
        'Locale::Constants' => 2.01, #Locale\Constants.pm
        'Locale::Country' => 2.61, #Locale\Country.pm
        'Locale::Currency' => 2.21, #Locale\Currency.pm
        'Locale::Language' => 2.21, #Locale\Language.pm
        'Locale::Maketext' => 1.06, #Locale\Maketext.pm
        'Locale::Maketext::Guts' => undef, #Locale\Maketext\Guts.pm
        'Locale::Maketext::GutsLoader' => undef, #Locale\Maketext\GutsLoader.pm
        'Locale::Script' => 2.21, #Locale\Script.pm
        'Math::BigFloat' => '1.40', #Math\BigFloat.pm
        'Math::BigFloat::Trace' => 0.01, #Math\BigFloat\Trace.pm
        'Math::BigInt' => 1.66, #Math\BigInt.pm
        'Math::BigInt::Calc' => 0.36, #Math\BigInt\Calc.pm
        'Math::BigInt::Scalar' => 0.11, #Math\BigInt\Scalar.pm
        'Math::BigInt::Trace' => 0.01, #Math\BigInt\Trace.pm
        'Math::BigRat' => '0.10', #Math\BigRat.pm
        'Math::Complex' => 1.34, #Math\Complex.pm
        'Math::Trig' => 1.02,   #Math\Trig.pm
        'Memoize' => 1.01,      #Memoize.pm
        'Memoize::AnyDBM_File' => 0.65, #Memoize\AnyDBM_File.pm
        'Memoize::Expire' => '1.00', #Memoize\Expire.pm
        'Memoize::ExpireFile' => 1.01, #Memoize\ExpireFile.pm
        'Memoize::ExpireTest' => 0.65, #Memoize\ExpireTest.pm
        'Memoize::NDBM_File' => 0.65, #Memoize\NDBM_File.pm
        'Memoize::SDBM_File' => 0.65, #Memoize\SDBM_File.pm
        'Memoize::Storable' => 0.65, #Memoize\Storable.pm
        'MIME::Base64' => 2.21, #MIME\Base64.pm
        'MIME::QuotedPrint' => 2.21, #MIME\QuotedPrint.pm
        'NDBM_File' => '1.05',  #ext\NDBM_File\NDBM_File.pm
        'Net::Cmd' => 2.24,     #Net\Cmd.pm
        'Net::Config' => '1.10',  #Net\Config.pm
        'Net::Domain' => 2.19,  #Net\Domain.pm
        'Net::FTP' => 2.72,     #Net\FTP.pm
        'Net::FTP::A' => 1.16,  #Net\FTP\A.pm
        'Net::FTP::dataconn' => 0.11, #Net\FTP\dataconn.pm
        'Net::FTP::E' => 0.01,  #Net\FTP\E.pm
        'Net::FTP::I' => 1.12,  #Net\FTP\I.pm
        'Net::FTP::L' => 0.01,  #Net\FTP\L.pm
        'Net::hostent' => 1.01, #Net\hostent.pm
        'Net::netent' => '1.00',  #Net\netent.pm
        'Net::Netrc' => 2.12,   #Net\Netrc.pm
        'Net::NNTP' => 2.22,    #Net\NNTP.pm
        'Net::Ping' => 2.31,    #Net\Ping.pm
        'Net::POP3' => 2.24,    #Net\POP3.pm
        'Net::protoent' => '1.00', #Net\protoent.pm
        'Net::servent' => 1.01, #Net\servent.pm
        'Net::SMTP' => 2.26,    #Net\SMTP.pm
        'Net::Time' => 2.09,    #Net\Time.pm
        'NEXT' => '0.60',         #NEXT.pm
        'O' => '1.00',            #O.pm
        'ODBM_File' => '1.04',  #ext\ODBM_File\ODBM_File.pm
        'Opcode' => 1.05,       #Opcode.pm
        'open' => 1.02,         #open.pm
        'ops' => '1.00',          #ops.pm
        'OS2::DLL' => '1.01',   #os2\OS2\REXX\DLL\DLL.pm
        'OS2::ExtAttr' => '0.02', #os2\OS2\ExtAttr\ExtAttr.pm
        'OS2::PrfDB' => '0.03', #os2\OS2\PrfDB\PrfDB.pm
        'OS2::Process' => '1.01', #os2\OS2\Process\Process.pm
        'OS2::REXX' => '1.02',  #os2\OS2\REXX\REXX.pm
        'overload' => 1.01,     #overload.pm
        'PerlIO' => 1.02,       #PerlIO.pm
        'PerlIO::encoding' => 0.07, #PerlIO\encoding.pm
        'PerlIO::scalar' => 0.02, #PerlIO\scalar.pm
        'PerlIO::via' => 0.02,  #PerlIO\via.pm
        'PerlIO::via::QuotedPrint' => 0.05, #PerlIO\via\QuotedPrint.pm
        'Pod::Checker' => 1.41, #Pod\Checker.pm
        'Pod::Find' => 0.24,    #Pod\Find.pm
        'Pod::Functions' => 1.02, #Pod\Functions.pm
        'Pod::Html' => 1.0501,  #Pod\Html.pm
        'Pod::InputObjects' => 1.14, #Pod\InputObjects.pm
        'Pod::LaTeX' => 0.55,   #Pod\LaTeX.pm
        'Pod::Man' => 1.37,     #Pod\Man.pm
        'Pod::ParseLink' => 1.06, #Pod\ParseLink.pm
        'Pod::Parser' => 1.13,  #Pod\Parser.pm
        'Pod::ParseUtils' => 0.3, #Pod\ParseUtils.pm
        'Pod::Perldoc' => 3.11, #Pod\Perldoc.pm
        'Pod::Perldoc::BaseTo' => undef, #Pod\Perldoc\BaseTo.pm
        'Pod::Perldoc::GetOptsOO' => undef, #Pod\Perldoc\GetOptsOO.pm
        'Pod::Perldoc::ToChecker' => undef, #Pod\Perldoc\ToChecker.pm
        'Pod::Perldoc::ToMan' => undef, #Pod\Perldoc\ToMan.pm
        'Pod::Perldoc::ToNroff' => undef, #Pod\Perldoc\ToNroff.pm
        'Pod::Perldoc::ToPod' => undef, #Pod\Perldoc\ToPod.pm
        'Pod::Perldoc::ToRtf' => undef, #Pod\Perldoc\ToRtf.pm
        'Pod::Perldoc::ToText' => undef, #Pod\Perldoc\ToText.pm
        'Pod::Perldoc::ToTk' => undef, #Pod\Perldoc\ToTk.pm
        'Pod::Perldoc::ToXml' => undef, #Pod\Perldoc\ToXml.pm
        'Pod::Plainer' => 0.01, #Pod\Plainer.pm
        'Pod::PlainText' => 2.01, #Pod\PlainText.pm
        'Pod::Select' => 1.13,  #Pod\Select.pm
        'Pod::Text' => 2.21,    #Pod\Text.pm
        'Pod::Text::Color' => 1.04, #Pod\Text\Color.pm
        'Pod::Text::Overstrike' => 1.1, #Pod\Text\Overstrike.pm
        'Pod::Text::Termcap' => 1.11, #Pod\Text\Termcap.pm
        'Pod::Usage' => 1.16,   #Pod\Usage.pm
        'POSIX' => 1.06,        #POSIX.pm
        're' => 0.04,           #re.pm
        'Safe' => '2.10',         #Safe.pm
        'Scalar::Util' => 1.13, #Scalar\Util.pm
        'SDBM_File' => 1.04,    #SDBM_File.pm
        'Search::Dict' => 1.02, #Search\Dict.pm
        'SelectSaver' => '1.00',  #SelectSaver.pm
        'SelfLoader' => 1.0904, #SelfLoader.pm
        'Shell' => 0.5,         #Shell.pm
        'sigtrap' => 1.02,      #sigtrap.pm
        'Socket' => 1.76,       #Socket.pm
        'sort' => 1.02,         #sort.pm
        'Storable' => 2.08,     #Storable.pm
        'strict' => 1.03,       #strict.pm
        'subs' => '1.00',         #subs.pm
        'Switch' => '2.10',       #Switch.pm
        'Symbol' => 1.05,       #Symbol.pm
        'Sys::Hostname' => 1.11, #Sys\Hostname.pm
        'Sys::Syslog' => '0.04', #ext\Sys\Syslog\Syslog.pm
        'Term::ANSIColor' => 1.07, #Term\ANSIColor.pm
        'Term::Cap' => 1.08,    #Term\Cap.pm
        'Term::Complete' => 1.401, #Term\Complete.pm
        'Term::ReadLine' => 1.01, #Term\ReadLine.pm
        'Test' => 1.24,         #Test.pm
        'Test::Builder' => 0.17, #Test\Builder.pm
        'Test::Harness' => '2.30', #Test\Harness.pm
        'Test::Harness::Assert' => 0.01, #Test\Harness\Assert.pm
        'Test::Harness::Iterator' => 0.01, #Test\Harness\Iterator.pm
        'Test::Harness::Straps' => 0.15, #Test\Harness\Straps.pm
        'Test::More' => 0.47,   #Test\More.pm
        'Test::Simple' => 0.47, #Test\Simple.pm
        'Text::Abbrev' => 1.01, #Text\Abbrev.pm
        'Text::Balanced' => 1.95, #Text\Balanced.pm
        'Text::ParseWords' => 3.21, #Text\ParseWords.pm
        'Text::Soundex' => 1.01, #Text\Soundex.pm
        'Text::Tabs' => 98.112801, #Text\Tabs.pm
        'Text::Wrap' => 2001.09291, #Text\Wrap.pm
        'Thread' => '2.00',       #Thread.pm
        'Thread::Queue' => '2.00', #Thread\Queue.pm
        'Thread::Semaphore' => 2.01, #Thread\Semaphore.pm
        'Thread::Signal' => '1.00', #Thread\Signal.pm
        'Thread::Specific' => '1.00', #Thread\Specific.pm
        'threads' => '1.00',      #threads.pm
        'threads::shared' => 0.91, #threads\shared.pm
        'Tie::Array' => 1.03,   #Tie\Array.pm
        'Tie::File' => 0.97,    #Tie\File.pm
        'Tie::Handle' => 4.1,   #Tie\Handle.pm
        'Tie::Hash' => '1.00',    #Tie\Hash.pm
        'Tie::Memoize' => '1.0',  #Tie\Memoize.pm
        'Tie::RefHash' => 1.31, #Tie\RefHash.pm
        'Tie::Scalar' => '1.00',  #Tie\Scalar.pm
        'Tie::SubstrHash' => '1.00', #Tie\SubstrHash.pm
        'Time::gmtime' => 1.02, #Time\gmtime.pm
        'Time::HiRes' => 1.52,  #Time\HiRes.pm
        'Time::Local' => 1.07,  #Time\Local.pm
        'Time::localtime' => 1.02, #Time\localtime.pm
        'Time::tm' => '1.00',     #Time\tm.pm
        'Unicode::Collate' => '0.30', #Unicode\Collate.pm
        'Unicode::Normalize' => 0.25, #Unicode\Normalize.pm
        'Unicode::UCD' => 0.21, #Unicode\UCD.pm
        'UNIVERSAL' => 1.01,    #UNIVERSAL.pm
        'User::grent' => '1.00',  #User\grent.pm
        'User::pwent' => '1.00',  #User\pwent.pm
        'utf8' => 1.02,         #utf8.pm
        'vars' => 1.01,         #vars.pm
        'VMS::DCLsym' => '1.02', #vms\ext\DCLsym\DCLsym.pm
        'VMS::Filespec' => '1.11', #vms\ext\Filespec.pm
        'VMS::Stdio' => '2.3',  #vms\ext\Stdio\Stdio.pm
        'vmsish' => 1.01,       #vmsish.pm
        'warnings' => 1.03,     #warnings.pm
        'warnings::register' => '1.00', #warnings\register.pm
        'XS::APItest' => 0.02,  #XS\APItest.pm
        'XS::Typemap' => 0.01,  #XS\Typemap.pm
        'XSLoader' => 0.02,     #XSLoader.pm
    },

        5.008003 => {
        'AnyDBM_File'           => '1.00',  #lib/AnyDBM_File.pm
        'Attribute::Handlers'   => '0.78',  #lib/Attribute/Handlers.pm
        'attributes'            => '0.06',  #lib/attributes.pm
        'attrs'                 => '1.01',  #lib/attrs.pm
        'AutoLoader'            => '5.60',  #lib/AutoLoader.pm
        'AutoSplit'             => '1.04',  #lib/AutoSplit.pm
        'autouse'               => '1.03',  #lib/autouse.pm
        'B'                     => '1.02',  #lib/B.pm
        'base'                  => '2.04',  #lib/base.pm
        'B::Asmdata'            => '1.01',  #lib/B/Asmdata.pm
        'B::Assembler'          => '0.06',  #lib/B/Assembler.pm
        'B::Bblock'             => '1.02',  #lib/B/Bblock.pm
        'B::Bytecode'           => '1.01',  #lib/B/Bytecode.pm
        'B::C'                  => '1.02',  #lib/B/C.pm
        'B::CC'                 => '1.00',  #lib/B/CC.pm
        'B::Concise'            => '0.56',  #lib/B/Concise.pm
        'B::Debug'              => '1.01',  #lib/B/Debug.pm
        'B::Deparse'            => '0.64',  #lib/B/Deparse.pm
        'B::Disassembler'       => '1.03',  #lib/B/Disassembler.pm
        'Benchmark'             => '1.052',  #lib/Benchmark.pm
        'bigint'                => '0.04',  #lib/bigint.pm
        'bignum'                => '0.14',  #lib/bignum.pm
        'bigrat'                => '0.06',  #lib/bigrat.pm
        'blib'                  => '1.02',  #lib/blib.pm
        'B::Lint'               => '1.02',  #lib/B/Lint.pm
        'B::Showlex'            => '1.00',  #lib/B/Showlex.pm
        'B::Stackobj'           => '1.00',  #lib/B/Stackobj.pm
        'B::Stash'              => '1.00',  #lib/B/Stash.pm
        'B::Terse'              => '1.02',  #lib/B/Terse.pm
        'B::Xref'               => '1.01',  #lib/B/Xref.pm
        'ByteLoader'            => '0.05',  #lib/ByteLoader.pm
        'bytes'                 => '1.01',  #lib/bytes.pm
        'Carp'                  => '1.01',  #lib/Carp.pm
        'Carp::Heavy'           => '1.01',  #lib/Carp/Heavy.pm
        'CGI'                   => '3.01',  #lib/CGI.pm
        'CGI::Apache'           => '1.00',  #lib/CGI/Apache.pm
        'CGI::Carp'             => '1.27',  #lib/CGI/Carp.pm
        'CGI::Cookie'           => '1.24',  #lib/CGI/Cookie.pm
        'CGI::Fast'             => '1.05',  #lib/CGI/Fast.pm
        'CGI::Pretty'           => '1.08',  #lib/CGI/Pretty.pm
        'CGI::Push'             => '1.04',  #lib/CGI/Push.pm
        'CGI::Switch'           => '1.00',  #lib/CGI/Switch.pm
        'CGI::Util'             => '1.4',  #lib/CGI/Util.pm
        'charnames'             => '1.02',  #lib/charnames.pm
        'Class::ISA'            => '0.32',  #lib/Class/ISA.pm
        'Class::Struct'         => '0.63',  #lib/Class/Struct.pm
        'Config'                => undef,  #lib/Config.pm
        'constant'              => '1.04',  #lib/constant.pm
        'CPAN'                  => '1.76_01',  #lib/CPAN.pm
        'CPAN::FirstTime'       => '1.60 ',  #lib/CPAN/FirstTime.pm
        'CPAN::Nox'             => '1.03',  #lib/CPAN/Nox.pm
        'Cwd'                   => '2.12',  #lib/Cwd.pm
        'Data::Dumper'          => '2.121',  #lib/Data/Dumper.pm
        'DB'                    => '1.0',  #lib/DB.pm
        'DB_File'               => '1.808',  #lib/DB_File.pm
        'Devel::DProf'          => '20030813.00',  #lib/Devel/DProf.pm
        'Devel::Peek'           => '1.01',  #lib/Devel/Peek.pm
        'Devel::PPPort'         => '2.011',  #lib/Devel/PPPort.pm
        'Devel::SelfStubber'    => '1.03',  #lib/Devel/SelfStubber.pm
        'diagnostics'           => '1.12',  #lib/diagnostics.pm
        'Digest'                => '1.05',  #lib/Digest.pm
        'Digest::base'          => '1.00',  #lib/Digest/base.pm
        'Digest::MD5'           => '2.33',  #lib/Digest/MD5.pm
        'DirHandle'             => '1.00',  #lib/DirHandle.pm
        'Dumpvalue'             => '1.11',  #lib/Dumpvalue.pm
        'DynaLoader'            => '1.04',  #lib/DynaLoader.pm
        'Encode'                => '1.99',  #lib/Encode.pm
        'Encode::Alias'         => '1.38',  #lib/Encode/Alias.pm
        'Encode::Byte'          => '1.23',  #lib/Encode/Byte.pm
        'Encode::CJKConstants'  => '1.02',  #lib/Encode/CJKConstants.pm
        'Encode::CN'            => '1.24',  #lib/Encode/CN.pm
        'Encode::CN::HZ'        => '1.05',  #lib/Encode/CN/HZ.pm
        'Encode::Config'        => '1.07',  #lib/Encode/Config.pm
        'Encode::EBCDIC'        => '1.21',  #lib/Encode/EBCDIC.pm
        'Encode::Encoder'       => '0.07',  #lib/Encode/Encoder.pm
        'Encode::Encoding'      => '1.33',  #lib/Encode/Encoding.pm
        'Encode::Guess'         => '1.09',  #lib/Encode/Guess.pm
        'Encode::JP'            => '1.25',  #lib/Encode/JP.pm
        'Encode::JP::H2Z'       => '1.02',  #lib/Encode/JP/H2Z.pm
        'Encode::JP::JIS7'      => '1.12',  #lib/Encode/JP/JIS7.pm
        'Encode::KR'            => '1.23',  #lib/Encode/KR.pm
        'Encode::KR::2022_KR'   => '1.06',  #lib/Encode/KR/2022_KR.pm
        'Encode::MIME::Header'  => '1.09',  #lib/Encode/MIME/Header.pm
        'Encode::Symbol'        => '1.22',  #lib/Encode/Symbol.pm
        'Encode::TW'            => '1.26',  #lib/Encode/TW.pm
        'Encode::Unicode'       => '1.40',  #lib/Encode/Unicode.pm
        'Encode::Unicode::UTF7' => '0.02',  #lib/Encode/Unicode/UTF7.pm
        'encoding'              => '1.48',  #lib/encoding.pm
        'English'               => '1.01',  #lib/English.pm
        'Env'                   => '1.00',  #lib/Env.pm
        'Errno'                 => '1.09_00',  #lib/Errno.pm
        'Exporter'              => '5.57',  #lib/Exporter.pm
        'Exporter::Heavy'       => '5.567',  #lib/Exporter/Heavy.pm
        'ExtUtils::Command'     => '1.05',  #lib/ExtUtils/Command.pm
        'ExtUtils::Command::MM' => '0.03',  #lib/ExtUtils/Command/MM.pm
        'ExtUtils::Constant'    => '0.14',  #lib/ExtUtils/Constant.pm
        'ExtUtils::Embed'       => '1.250601',  #lib/ExtUtils/Embed.pm
        'ExtUtils::Install'     => '1.32',  #lib/ExtUtils/Install.pm
        'ExtUtils::Installed'   => '0.08',  #lib/ExtUtils/Installed.pm
        'ExtUtils::Liblist'     => '1.01',  #lib/ExtUtils/Liblist.pm
        'ExtUtils::Liblist::Kid'=> '1.3',  #lib/ExtUtils/Liblist/Kid.pm
        'ExtUtils::MakeMaker'   => '6.17',  #lib/ExtUtils/MakeMaker.pm
        'ExtUtils::MakeMaker::bytes'=> '0.01',  #lib/ExtUtils/MakeMaker/bytes.pm
        'ExtUtils::MakeMaker::vmsish'=> '0.01',  #lib/ExtUtils/MakeMaker/vmsish.pm
        'ExtUtils::Manifest'    => '1.42',  #lib/ExtUtils/Manifest.pm
        'ExtUtils::Miniperl'    => undef,  #lib/ExtUtils/Miniperl.pm
        'ExtUtils::Mkbootstrap' => '1.15',  #lib/ExtUtils/Mkbootstrap.pm
        'ExtUtils::Mksymlists'  => '1.19',  #lib/ExtUtils/Mksymlists.pm
        'ExtUtils::MM'          => '0.04',  #lib/ExtUtils/MM.pm
        'ExtUtils::MM_Any'      => '0.07',  #lib/ExtUtils/MM_Any.pm
        'ExtUtils::MM_BeOS'     => '1.04',  #lib/ExtUtils/MM_BeOS.pm
        'ExtUtils::MM_Cygwin'   => '1.06',  #lib/ExtUtils/MM_Cygwin.pm
        'ExtUtils::MM_DOS'      => '0.02',  #lib/ExtUtils/MM_DOS.pm
        'ExtUtils::MM_MacOS'    => '1.07',  #lib/ExtUtils/MM_MacOS.pm
        'ExtUtils::MM_NW5'      => '2.06',  #lib/ExtUtils/MM_NW5.pm
        'ExtUtils::MM_OS2'      => '1.04',  #lib/ExtUtils/MM_OS2.pm
        'ExtUtils::MM_Unix'     => '1.42',  #lib/ExtUtils/MM_Unix.pm
        'ExtUtils::MM_UWIN'     => '0.02',  #lib/ExtUtils/MM_UWIN.pm
        'ExtUtils::MM_VMS'      => '5.70',  #lib/ExtUtils/MM_VMS.pm
        'ExtUtils::MM_Win32'    => '1.09',  #lib/ExtUtils/MM_Win32.pm
        'ExtUtils::MM_Win95'    => '0.03',  #lib/ExtUtils/MM_Win95.pm
        'ExtUtils::MY'          => '0.01',  #lib/ExtUtils/MY.pm
        'ExtUtils::Packlist'    => '0.04',  #lib/ExtUtils/Packlist.pm
        'ExtUtils::testlib'     => '1.15',  #lib/ExtUtils/testlib.pm
        'ExtUtils::XSSymSet'    => '1.0',  #vms/ext/XSSymSet.pm
        'Fatal'                 => '1.03',  #lib/Fatal.pm
        'Fcntl'                 => '1.05',  #lib/Fcntl.pm
        'fields'                => '2.03',  #lib/fields.pm
        'File::Basename'        => '2.72',  #lib/File/Basename.pm
        'FileCache'             => '1.03',  #lib/FileCache.pm
        'File::CheckTree'       => '4.3',  #lib/File/CheckTree.pm
        'File::Compare'         => '1.1003',  #lib/File/Compare.pm
        'File::Copy'            => '2.07',  #lib/File/Copy.pm
        'File::DosGlob'         => '1.00',  #lib/File/DosGlob.pm
        'File::Find'            => '1.06',  #lib/File/Find.pm
        'File::Glob'            => '1.02',  #lib/File/Glob.pm
        'FileHandle'            => '2.01',  #lib/FileHandle.pm
        'File::Path'            => '1.06',  #lib/File/Path.pm
        'File::Spec'            => '0.87',  #lib/File/Spec.pm
        'File::Spec::Cygwin'    => '1.1',  #lib/File/Spec/Cygwin.pm
        'File::Spec::Epoc'      => '1.1',  #lib/File/Spec/Epoc.pm
        'File::Spec::Functions' => '1.3',  #lib/File/Spec/Functions.pm
        'File::Spec::Mac'       => '1.4',  #lib/File/Spec/Mac.pm
        'File::Spec::OS2'       => '1.2',  #lib/File/Spec/OS2.pm
        'File::Spec::Unix'      => '1.5',  #lib/File/Spec/Unix.pm
        'File::Spec::VMS'       => '1.4',  #lib/File/Spec/VMS.pm
        'File::Spec::Win32'     => '1.4',  #lib/File/Spec/Win32.pm
        'File::stat'            => '1.00',  #lib/File/stat.pm
        'File::Temp'            => '0.14',  #lib/File/Temp.pm
        'filetest'              => '1.01',  #lib/filetest.pm
        'Filter::Simple'        => '0.78',  #lib/Filter/Simple.pm
        'Filter::Util::Call'    => '1.0601',  #lib/Filter/Util/Call.pm
        'FindBin'               => '1.44',  #lib/FindBin.pm
        'GDBM_File'             => '1.07',  #lib/GDBM_File.pm
        'Getopt::Long'          => '2.34',  #lib/Getopt/Long.pm
        'Getopt::Std'           => '1.05',  #lib/Getopt/Std.pm
        'Hash::Util'            => '0.05',  #lib/Hash/Util.pm
        'I18N::Collate'         => '1.00',  #lib/I18N/Collate.pm
        'I18N::Langinfo'        => '0.02',  #lib/I18N/Langinfo.pm
        'I18N::LangTags'        => '0.29',  #lib/I18N/LangTags.pm
        'I18N::LangTags::List'  => '0.29',  #lib/I18N/LangTags/List.pm
        'if'                    => '0.03',  #lib/if.pm
        'integer'               => '1.00',  #lib/integer.pm
        'IO'                    => '1.21',  #lib/IO.pm
        'IO::Dir'               => '1.04',  #lib/IO/Dir.pm
        'IO::File'              => '1.10',  #lib/IO/File.pm
        'IO::Handle'            => '1.23',  #lib/IO/Handle.pm
        'IO::Pipe'              => '1.122',  #lib/IO/Pipe.pm
        'IO::Poll'              => '0.06',  #lib/IO/Poll.pm
        'IO::Seekable'          => '1.09',  #lib/IO/Seekable.pm
        'IO::Select'            => '1.16',  #lib/IO/Select.pm
        'IO::Socket'            => '1.28',  #lib/IO/Socket.pm
        'IO::Socket::INET'      => '1.27',  #lib/IO/Socket/INET.pm
        'IO::Socket::UNIX'      => '1.21',  #lib/IO/Socket/UNIX.pm
        'IPC::Msg'              => '1.02',  #lib/IPC/Msg.pm
        'IPC::Open2'            => '1.01',  #lib/IPC/Open2.pm
        'IPC::Open3'            => '1.0105',  #lib/IPC/Open3.pm
        'IPC::Semaphore'        => '1.02',  #lib/IPC/Semaphore.pm
        'IPC::SysV'             => '1.04',  #lib/IPC/SysV.pm
        'JNI'                   => '0.2',  #jpl/JNI/JNI.pm
        'JPL::AutoLoader'       => undef,  #jpl/JPL/AutoLoader.pm
        'JPL::Class'            => undef,  #jpl/JPL/Class.pm
        'JPL::Compile'          => undef,  #jpl/JPL/Compile.pm
        'less'                  => '0.01',  #lib/less.pm
        'lib'                   => '0.5565',  #lib/lib.pm
        'List::Util'            => '1.13',  #lib/List/Util.pm
        'locale'                => '1.00',  #lib/locale.pm
        'Locale::Constants'     => '2.01',  #lib/Locale/Constants.pm
        'Locale::Country'       => '2.61',  #lib/Locale/Country.pm
        'Locale::Currency'      => '2.21',  #lib/Locale/Currency.pm
        'Locale::Language'      => '2.21',  #lib/Locale/Language.pm
        'Locale::Maketext'      => '1.06',  #lib/Locale/Maketext.pm
        'Locale::Maketext::GutsLoader'=> undef,  #lib/Locale/Maketext/GutsLoader.pm
        'Locale::Maketext::Guts'=> undef,  #lib/Locale/Maketext/Guts.pm
        'Locale::Script'        => '2.21',  #lib/Locale/Script.pm
        'Math::BigFloat'        => '1.42',  #lib/Math/BigFloat.pm
        'Math::BigFloat::Trace' => '0.01',  #lib/Math/BigFloat/Trace.pm
        'Math::BigInt'          => '1.68',  #lib/Math/BigInt.pm
        'Math::BigInt::Calc'    => '0.38',  #lib/Math/BigInt/Calc.pm
        'Math::BigInt::CalcEmu' => '0.02',  #lib/Math/BigInt/CalcEmu.pm
        'Math::BigInt::Trace'   => '0.01',  #lib/Math/BigInt/Trace.pm
        'Math::BigRat'          => '0.10',  #lib/Math/BigRat.pm
        'Math::Complex'         => '1.34',  #lib/Math/Complex.pm
        'Math::Trig'            => '1.02',  #lib/Math/Trig.pm
        'Memoize'               => '1.01',  #lib/Memoize.pm
        'Memoize::AnyDBM_File'  => '0.65',  #lib/Memoize/AnyDBM_File.pm
        'Memoize::Expire'       => '1.00',  #lib/Memoize/Expire.pm
        'Memoize::ExpireFile'   => '1.01',  #lib/Memoize/ExpireFile.pm
        'Memoize::ExpireTest'   => '0.65',  #lib/Memoize/ExpireTest.pm
        'Memoize::NDBM_File'    => '0.65',  #lib/Memoize/NDBM_File.pm
        'Memoize::SDBM_File'    => '0.65',  #lib/Memoize/SDBM_File.pm
        'Memoize::Storable'     => '0.65',  #lib/Memoize/Storable.pm
        'MIME::Base64'          => '2.21',  #lib/MIME/Base64.pm
        'MIME::QuotedPrint'     => '2.21',  #lib/MIME/QuotedPrint.pm
        'NDBM_File'             => '1.05',  #lib/NDBM_File.pm
        'Net::Cmd'              => '2.24',  #lib/Net/Cmd.pm
        'Net::Config'           => '1.10',  #lib/Net/Config.pm
        'Net::Domain'           => '2.19',  #lib/Net/Domain.pm
        'Net::FTP'              => '2.72',  #lib/Net/FTP.pm
        'Net::FTP::A'           => '1.16',  #lib/Net/FTP/A.pm
        'Net::FTP::dataconn'    => '0.11',  #lib/Net/FTP/dataconn.pm
        'Net::FTP::E'           => '0.01',  #lib/Net/FTP/E.pm
        'Net::FTP::I'           => '1.12',  #lib/Net/FTP/I.pm
        'Net::FTP::L'           => '0.01',  #lib/Net/FTP/L.pm
        'Net::hostent'          => '1.01',  #lib/Net/hostent.pm
        'Net::netent'           => '1.00',  #lib/Net/netent.pm
        'Net::Netrc'            => '2.12',  #lib/Net/Netrc.pm
        'Net::NNTP'             => '2.22',  #lib/Net/NNTP.pm
        'Net::Ping'             => '2.31',  #lib/Net/Ping.pm
        'Net::POP3'             => '2.24',  #lib/Net/POP3.pm
        'Net::protoent'         => '1.00',  #lib/Net/protoent.pm
        'Net::servent'          => '1.01',  #lib/Net/servent.pm
        'Net::SMTP'             => '2.26',  #lib/Net/SMTP.pm
        'Net::Time'             => '2.09',  #lib/Net/Time.pm
        'NEXT'                  => '0.60',  #lib/NEXT.pm
        'O'                     => '1.00',  #lib/O.pm
        'ODBM_File'             => '1.04',  #ext/ODBM_File/ODBM_File.pm
        'Opcode'                => '1.05',  #lib/Opcode.pm
        'open'                  => '1.02',  #lib/open.pm
        'ops'                   => '1.00',  #lib/ops.pm
        'OS2::DLL'              => '1.02',  #os2/OS2/REXX/DLL/DLL.pm
        'OS2::ExtAttr'          => '0.02',  #os2/OS2/ExtAttr/ExtAttr.pm
        'OS2::PrfDB'            => '0.03',  #os2/OS2/PrfDB/PrfDB.pm
        'OS2::Process'          => '1.01',  #os2/OS2/Process/Process.pm
        'OS2::REXX'             => '1.02',  #os2/OS2/REXX/REXX.pm
        'overload'              => '1.01',  #lib/overload.pm
        'PerlIO'                => '1.03',  #lib/PerlIO.pm
        'PerlIO::encoding'      => '0.07',  #lib/PerlIO/encoding.pm
        'PerlIO::scalar'        => '0.02',  #lib/PerlIO/scalar.pm
        'PerlIO::via'           => '0.02',  #lib/PerlIO/via.pm
        'PerlIO::via::QuotedPrint'=> '0.06',  #lib/PerlIO/via/QuotedPrint.pm
        'Pod::Checker'          => '1.41',  #lib/Pod/Checker.pm
        'Pod::Find'             => '0.24',  #lib/Pod/Find.pm
        'Pod::Functions'        => '1.02',  #lib/Pod/Functions.pm
        'Pod::Html'             => '1.0502',  #lib/Pod/Html.pm
        'Pod::InputObjects'     => '1.14',  #lib/Pod/InputObjects.pm
        'Pod::LaTeX'            => '0.55',  #lib/Pod/LaTeX.pm
        'Pod::Man'              => '1.37',  #lib/Pod/Man.pm
        'Pod::ParseLink'        => '1.06',  #lib/Pod/ParseLink.pm
        'Pod::Parser'           => '1.14',  #lib/Pod/Parser.pm
        'Pod::ParseUtils'       => '0.3',  #lib/Pod/ParseUtils.pm
        'Pod::Perldoc'          => '3.12',  #lib/Pod/Perldoc.pm
        'Pod::Perldoc::BaseTo'  => undef,  #lib/Pod/Perldoc/BaseTo.pm
        'Pod::Perldoc::GetOptsOO'=> undef,  #lib/Pod/Perldoc/GetOptsOO.pm
        'Pod::Perldoc::ToChecker'=> undef,  #lib/Pod/Perldoc/ToChecker.pm
        'Pod::Perldoc::ToMan'   => undef,  #lib/Pod/Perldoc/ToMan.pm
        'Pod::Perldoc::ToNroff' => undef,  #lib/Pod/Perldoc/ToNroff.pm
        'Pod::Perldoc::ToPod'   => undef,  #lib/Pod/Perldoc/ToPod.pm
        'Pod::Perldoc::ToRtf'   => undef,  #lib/Pod/Perldoc/ToRtf.pm
        'Pod::Perldoc::ToText'  => undef,  #lib/Pod/Perldoc/ToText.pm
        'Pod::Perldoc::ToTk'    => 'undef',  #lib/Pod/Perldoc/ToTk.pm
        'Pod::Perldoc::ToXml'   => undef,  #lib/Pod/Perldoc/ToXml.pm
        'Pod::Plainer'          => '0.01',  #lib/Pod/Plainer.pm
        'Pod::PlainText'        => '2.02',  #lib/Pod/PlainText.pm
        'Pod::Select'           => '1.13',  #lib/Pod/Select.pm
        'Pod::Text'             => '2.21',  #lib/Pod/Text.pm
        'Pod::Text::Color'      => '1.04',  #lib/Pod/Text/Color.pm
        'Pod::Text::Overstrike' => '1.1',  #lib/Pod/Text/Overstrike.pm
        'Pod::Text::Termcap'    => '1.11',  #lib/Pod/Text/Termcap.pm
        'Pod::Usage'            => '1.16',  #lib/Pod/Usage.pm
        'POSIX'                 => '1.07',  #lib/POSIX.pm
        're'                    => '0.04',  #lib/re.pm
        'Safe'                  => '2.10',  #lib/Safe.pm
        'Scalar::Util'          => '1.13',  #lib/Scalar/Util.pm
        'SDBM_File'             => '1.04',  #lib/SDBM_File.pm
        'Search::Dict'          => '1.02',  #lib/Search/Dict.pm
        'SelectSaver'           => '1.00',  #lib/SelectSaver.pm
        'SelfLoader'            => '1.0904',  #lib/SelfLoader.pm
        'Shell'                 => '0.5',  #lib/Shell.pm
        'sigtrap'               => '1.02',  #lib/sigtrap.pm
        'Socket'                => '1.76',  #lib/Socket.pm
        'sort'                  => '1.02',  #lib/sort.pm
        'Storable'              => '2.09',  #lib/Storable.pm
        'strict'                => '1.03',  #lib/strict.pm
        'subs'                  => '1.00',  #lib/subs.pm
        'Switch'                => '2.10',  #lib/Switch.pm
        'Symbol'                => '1.05',  #lib/Symbol.pm
        'Sys::Hostname'         => '1.11',  #lib/Sys/Hostname.pm
        'Sys::Syslog'           => '0.04',  #lib/Sys/Syslog.pm
        'Term::ANSIColor'       => '1.07',  #lib/Term/ANSIColor.pm
        'Term::Cap'             => '1.08',  #lib/Term/Cap.pm
        'Term::Complete'        => '1.401',  #lib/Term/Complete.pm
        'Term::ReadLine'        => '1.01',  #lib/Term/ReadLine.pm
        'Test'                  => '1.24',  #lib/Test.pm
        'Test::Builder'         => '0.17',  #lib/Test/Builder.pm
        'Test::Harness'         => '2.40',  #lib/Test/Harness.pm
        'Test::Harness::Assert' => '0.02',  #lib/Test/Harness/Assert.pm
        'Test::Harness::Iterator'=> '0.02',  #lib/Test/Harness/Iterator.pm
        'Test::Harness::Straps' => '0.19',  #lib/Test/Harness/Straps.pm
        'Test::More'            => '0.47',  #lib/Test/More.pm
        'Test::Simple'          => '0.47',  #lib/Test/Simple.pm
        'Text::Abbrev'          => '1.01',  #lib/Text/Abbrev.pm
        'Text::Balanced'        => '1.95',  #lib/Text/Balanced.pm
        'Text::ParseWords'      => '3.21',  #lib/Text/ParseWords.pm
        'Text::Soundex'         => '1.01',  #lib/Text/Soundex.pm
        'Text::Tabs'            => '98.112801',  #lib/Text/Tabs.pm
        'Text::Wrap'            => '2001.09291',  #lib/Text/Wrap.pm
        'Thread'                => '2.00',  #lib/Thread.pm
        'Thread::Queue'         => '2.00',  #lib/Thread/Queue.pm
        'threads'               => '1.01',  #lib/threads.pm
        'Thread::Semaphore'     => '2.01',  #lib/Thread/Semaphore.pm
        'Thread::Signal'        => '1.00', #./ext/Thread/Thread/Signal.pm
        'Thread::Specific'      => '1.00', #./ext/Thread/Thread/Specific.pm
        'threads::shared'       => '0.92',  #lib/threads/shared.pm
        'Tie::Array'            => '1.03',  #lib/Tie/Array.pm
        'Tie::File'             => '0.97',  #lib/Tie/File.pm
        'Tie::Handle'           => '4.1',  #lib/Tie/Handle.pm
        'Tie::Hash'             => '1.01',  #lib/Tie/Hash.pm
        'Tie::Memoize'          => '1.0',  #lib/Tie/Memoize.pm
        'Tie::RefHash'          => '1.31',  #lib/Tie/RefHash.pm
        'Tie::Scalar'           => '1.00',  #lib/Tie/Scalar.pm
        'Tie::SubstrHash'       => '1.00',  #lib/Tie/SubstrHash.pm
        'Time::gmtime'          => '1.02',  #lib/Time/gmtime.pm
        'Time::HiRes'           => '1.52',  #lib/Time/HiRes.pm
        'Time::Local'           => '1.07',  #lib/Time/Local.pm
        'Time::localtime'       => '1.02',  #lib/Time/localtime.pm
        'Time::tm'              => '1.00',  #lib/Time/tm.pm
        'Unicode::Collate'      => '0.33',  #lib/Unicode/Collate.pm
        'Unicode::Normalize'    => '0.28',  #lib/Unicode/Normalize.pm
        'Unicode::UCD'          => '0.21',  #lib/Unicode/UCD.pm
        'UNIVERSAL'             => '1.01',  #lib/UNIVERSAL.pm
        'User::grent'           => '1.00',  #lib/User/grent.pm
        'User::pwent'           => '1.00',  #lib/User/pwent.pm
        'utf8'                  => '1.02',  #lib/utf8.pm
        'vars'                  => '1.01',  #lib/vars.pm
        'VMS::DCLsym'           => '1.02',  #vms/ext/DCLsym/DCLsym.pm
        'VMS::Filespec'         => '1.11',  #vms/ext/Filespec.pm
        'VMS::Stdio'            => '2.3',  #vms/ext/Stdio/Stdio.pm
        'vmsish'                => '1.01',  #lib/vmsish.pm
        'warnings'              => '1.03',  #lib/warnings.pm
        'warnings::register'    => '1.00',  #lib/warnings/register.pm
        'XS::APItest'           => '0.03',  #lib/XS/APItest.pm
        'XSLoader'              => '0.02',  #lib/XSLoader.pm
        'XS::Typemap'           => '0.01',  #lib/XS/Typemap.pm
        },

    5.009 => {
        'AnyDBM_File'           => '1.00',  #lib/AnyDBM_File.pm
        'assertions'            => '0.01',  #lib/assertions.pm
        'assertions::activate'  => '0.01',  #lib/assertions/activate.pm
        'Attribute::Handlers'   => '0.78',  #lib/Attribute/Handlers.pm
        'attributes'            => '0.06',  #lib/attributes.pm
        'attrs'                 => '1.01',  #lib/attrs.pm
        'AutoLoader'            => '5.60',  #lib/AutoLoader.pm
        'AutoSplit'             => '1.04',  #lib/AutoSplit.pm
        'autouse'               => '1.03',  #lib/autouse.pm
        'B'                     => '1.03',  #lib/B.pm
        'B::Asmdata'            => '1.01',  #lib/B/Asmdata.pm
        'B::Assembler'          => '0.06',  #lib/B/Assembler.pm
        'B::Bblock'             => '1.02',  #lib/B/Bblock.pm
        'B::Bytecode'           => '1.01',  #lib/B/Bytecode.pm
        'B::C'                  => '1.03',  #lib/B/C.pm
        'B::CC'                 => '1.00',  #lib/B/CC.pm
        'B::Concise'            => '0.57',  #lib/B/Concise.pm
        'B::Debug'              => '1.01',  #lib/B/Debug.pm
        'B::Deparse'            => '0.65',  #lib/B/Deparse.pm
        'B::Disassembler'       => '1.03',  #lib/B/Disassembler.pm
        'B::Lint'               => '1.02',  #lib/B/Lint.pm
        'B::Showlex'            => '1.00',  #lib/B/Showlex.pm
        'B::Stackobj'           => '1.00',  #lib/B/Stackobj.pm
        'B::Stash'              => '1.00',  #lib/B/Stash.pm
        'B::Terse'              => '1.02',  #lib/B/Terse.pm
        'B::Xref'               => '1.01',  #lib/B/Xref.pm
        'base'                  => '2.03',  #lib/base.pm
        'Benchmark'             => '1.051',  #lib/Benchmark.pm
        'bigint'                => '0.04',  #lib/bigint.pm
        'bignum'                => '0.14',  #lib/bignum.pm
        'bigrat'                => '0.06',  #lib/bigrat.pm
        'blib'                  => '1.02',  #lib/blib.pm
        'ByteLoader'            => '0.05',  #lib/ByteLoader.pm
        'bytes'                 => '1.01',  #lib/bytes.pm
        'Carp'                  => '1.01',  #lib/Carp.pm
        'Carp::Heavy'           => '1.01',  #lib/Carp/Heavy.pm
        'CGI'                   => '3.00',  #lib/CGI.pm
        'CGI::Apache'           => '1.00',  #lib/CGI/Apache.pm
        'CGI::Carp'             => '1.26',  #lib/CGI/Carp.pm
        'CGI::Cookie'           => '1.24',  #lib/CGI/Cookie.pm
        'CGI::Fast'             => '1.041',  #lib/CGI/Fast.pm
        'CGI::Pretty'           => '1.07_00',  #lib/CGI/Pretty.pm
        'CGI::Push'             => '1.04',  #lib/CGI/Push.pm
        'CGI::Switch'           => '1.00',  #lib/CGI/Switch.pm
        'CGI::Util'             => '1.31',  #lib/CGI/Util.pm
        'charnames'             => '1.02',  #lib/charnames.pm
        'Class::ISA'            => '0.32',  #lib/Class/ISA.pm
        'Class::Struct'         => '0.63',  #lib/Class/Struct.pm
        'Config'                => undef,  #lib/Config.pm
        'constant'              => '1.04',  #lib/constant.pm
        'CPAN'                  => '1.76_01',  #lib/CPAN.pm
        'CPAN::FirstTime'       => '1.60 ',  #lib/CPAN/FirstTime.pm
        'CPAN::Nox'             => '1.03',  #lib/CPAN/Nox.pm
        'Cwd'                   => '2.08',  #lib/Cwd.pm
        'Data::Dumper'          => '2.121',  #lib/Data/Dumper.pm
        'DB'                    => '1.0',  #lib/DB.pm
        'DB_File'               => '1.806',  #lib/DB_File.pm
        'Devel::DProf'          => '20030813.00',  #lib/Devel/DProf.pm
        'Devel::Peek'           => '1.01',  #lib/Devel/Peek.pm
        'Devel::PPPort'         => '2.008',  #lib/Devel/PPPort.pm
        'Devel::SelfStubber'    => '1.03',  #lib/Devel/SelfStubber.pm
        'diagnostics'           => '1.11',  #lib/diagnostics.pm
        'Digest'                => '1.02',  #lib/Digest.pm
        'Digest::MD5'           => '2.30',  #lib/Digest/MD5.pm
        'DirHandle'             => '1.00',  #lib/DirHandle.pm
        'Dumpvalue'             => '1.11',  #lib/Dumpvalue.pm
        'DynaLoader'            => '1.04',  #lib/DynaLoader.pm
        'Encode'                => '1.9801',  #lib/Encode.pm
        'Encode::Alias'         => '1.38',  #lib/Encode/Alias.pm
        'Encode::Byte'          => '1.23',  #lib/Encode/Byte.pm
        'Encode::CJKConstants'  => '1.02',  #lib/Encode/CJKConstants.pm
        'Encode::CN'            => '1.24',  #lib/Encode/CN.pm
        'Encode::CN::HZ'        => '1.05',  #lib/Encode/CN/HZ.pm
        'Encode::Config'        => '1.07',  #lib/Encode/Config.pm
        'Encode::EBCDIC'        => '1.21',  #lib/Encode/EBCDIC.pm
        'Encode::Encoder'       => '0.07',  #lib/Encode/Encoder.pm
        'Encode::Encoding'      => '1.33',  #lib/Encode/Encoding.pm
        'Encode::Guess'         => '1.09',  #lib/Encode/Guess.pm
        'Encode::JP'            => '1.25',  #lib/Encode/JP.pm
        'Encode::JP::H2Z'       => '1.02',  #lib/Encode/JP/H2Z.pm
        'Encode::JP::JIS7'      => '1.12',  #lib/Encode/JP/JIS7.pm
        'Encode::KR'            => '1.23',  #lib/Encode/KR.pm
        'Encode::KR::2022_KR'   => '1.06',  #lib/Encode/KR/2022_KR.pm
        'Encode::MIME::Header'  => '1.09',  #lib/Encode/MIME/Header.pm
        'Encode::Symbol'        => '1.22',  #lib/Encode/Symbol.pm
        'Encode::TW'            => '1.26',  #lib/Encode/TW.pm
        'Encode::Unicode'       => '1.40',  #lib/Encode/Unicode.pm
        'Encode::Unicode::UTF7' => '0.02',  #lib/Encode/Unicode/UTF7.pm
        'encoding'              => '1.47',  #lib/encoding.pm
        'English'               => '1.02',  #lib/English.pm
        'Env'                   => '1.00',  #lib/Env.pm
        'Errno'                 => '1.09_00',  #lib/Errno.pm
        'Exporter'              => '5.567',  #lib/Exporter.pm
        'Exporter::Heavy'       => '5.567',  #lib/Exporter/Heavy.pm
        'ExtUtils::Command'     => '1.05',  #lib/ExtUtils/Command.pm
        'ExtUtils::Command::MM' => '0.03',  #lib/ExtUtils/Command/MM.pm
        'ExtUtils::Constant'    => '0.14',  #lib/ExtUtils/Constant.pm
        'ExtUtils::Embed'       => '1.250601',  #lib/ExtUtils/Embed.pm
        'ExtUtils::Install'     => '1.32',  #lib/ExtUtils/Install.pm
        'ExtUtils::Installed'   => '0.08',  #lib/ExtUtils/Installed.pm
        'ExtUtils::Liblist'     => '1.01',  #lib/ExtUtils/Liblist.pm
        'ExtUtils::Liblist::Kid'=> '1.3',  #lib/ExtUtils/Liblist/Kid.pm
        'ExtUtils::MakeMaker'   => '6.17',  #lib/ExtUtils/MakeMaker.pm
        'ExtUtils::MakeMaker::bytes'=> '0.01',  #lib/ExtUtils/MakeMaker/bytes.pm
        'ExtUtils::MakeMaker::vmsish'=> '0.01',  #lib/ExtUtils/MakeMaker/vmsish.pm
        'ExtUtils::Manifest'    => '1.42',  #lib/ExtUtils/Manifest.pm
        'ExtUtils::Miniperl'    => undef,  #lib/ExtUtils/Miniperl.pm
        'ExtUtils::Mkbootstrap' => '1.15',  #lib/ExtUtils/Mkbootstrap.pm
        'ExtUtils::Mksymlists'  => '1.19',  #lib/ExtUtils/Mksymlists.pm
        'ExtUtils::MM'          => '0.04',  #lib/ExtUtils/MM.pm
        'ExtUtils::MM_Any'      => '0.07',  #lib/ExtUtils/MM_Any.pm
        'ExtUtils::MM_BeOS'     => '1.04',  #lib/ExtUtils/MM_BeOS.pm
        'ExtUtils::MM_Cygwin'   => '1.06',  #lib/ExtUtils/MM_Cygwin.pm
        'ExtUtils::MM_DOS'      => '0.02',  #lib/ExtUtils/MM_DOS.pm
        'ExtUtils::MM_MacOS'    => '1.07',  #lib/ExtUtils/MM_MacOS.pm
        'ExtUtils::MM_NW5'      => '2.06',  #lib/ExtUtils/MM_NW5.pm
        'ExtUtils::MM_OS2'      => '1.04',  #lib/ExtUtils/MM_OS2.pm
        'ExtUtils::MM_Unix'     => '1.42',  #lib/ExtUtils/MM_Unix.pm
        'ExtUtils::MM_UWIN'     => '0.02',  #lib/ExtUtils/MM_UWIN.pm
        'ExtUtils::MM_VMS'      => '5.70',  #lib/ExtUtils/MM_VMS.pm
        'ExtUtils::MM_Win32'    => '1.09',  #lib/ExtUtils/MM_Win32.pm
        'ExtUtils::MM_Win95'    => '0.03',  #lib/ExtUtils/MM_Win95.pm
        'ExtUtils::MY'          => '0.01',  #lib/ExtUtils/MY.pm
        'ExtUtils::Packlist'    => '0.04',  #lib/ExtUtils/Packlist.pm
        'ExtUtils::testlib'     => '1.15',  #lib/ExtUtils/testlib.pm
        'ExtUtils::XSSymSet'    => '1.0',  #vms/ext/XSSymSet.pm
        'Fatal'                 => '1.04',  #lib/Fatal.pm
        'Fcntl'                 => '1.05',  #lib/Fcntl.pm
        'fields'                => '2.03',  #lib/fields.pm
        'File::Basename'        => '2.72',  #lib/File/Basename.pm
        'File::CheckTree'       => '4.2',  #lib/File/CheckTree.pm
        'File::Compare'         => '1.1003',  #lib/File/Compare.pm
        'File::Copy'            => '2.06',  #lib/File/Copy.pm
        'File::DosGlob'         => '1.00',  #lib/File/DosGlob.pm
        'File::Find'            => '1.05',  #lib/File/Find.pm
        'File::Glob'            => '1.02',  #lib/File/Glob.pm
        'File::Path'            => '1.06',  #lib/File/Path.pm
        'File::Spec'            => '0.86',  #lib/File/Spec.pm
        'File::Spec::Cygwin'    => '1.1',  #lib/File/Spec/Cygwin.pm
        'File::Spec::Epoc'      => '1.1',  #lib/File/Spec/Epoc.pm
        'File::Spec::Functions' => '1.3',  #lib/File/Spec/Functions.pm
        'File::Spec::Mac'       => '1.4',  #lib/File/Spec/Mac.pm
        'File::Spec::OS2'       => '1.2',  #lib/File/Spec/OS2.pm
        'File::Spec::Unix'      => '1.5',  #lib/File/Spec/Unix.pm
        'File::Spec::VMS'       => '1.4',  #lib/File/Spec/VMS.pm
        'File::Spec::Win32'     => '1.4',  #lib/File/Spec/Win32.pm
        'File::stat'            => '1.00',  #lib/File/stat.pm
        'File::Temp'            => '0.14',  #lib/File/Temp.pm
        'FileCache'             => '1.03',  #lib/FileCache.pm
        'FileHandle'            => '2.01',  #lib/FileHandle.pm
        'filetest'              => '1.01',  #lib/filetest.pm
        'Filter::Simple'        => '0.78',  #lib/Filter/Simple.pm
        'Filter::Util::Call'    => '1.0601',  #lib/Filter/Util/Call.pm
        'FindBin'               => '1.43',  #lib/FindBin.pm
        'GDBM_File'             => '1.07',  #ext/GDBM_File/GDBM_File.pm
        'Getopt::Long'          => '2.34',  #lib/Getopt/Long.pm
        'Getopt::Std'           => '1.04',  #lib/Getopt/Std.pm
        'Hash::Util'            => '0.05',  #lib/Hash/Util.pm
        'I18N::Collate'         => '1.00',  #lib/I18N/Collate.pm
        'I18N::Langinfo'        => '0.02',  #lib/I18N/Langinfo.pm
        'I18N::LangTags'        => '0.29',  #lib/I18N/LangTags.pm
        'I18N::LangTags::List'  => '0.29',  #lib/I18N/LangTags/List.pm
        'if'                    => '0.03',  #lib/if.pm
        'integer'               => '1.00',  #lib/integer.pm
        'IO'                    => '1.21',  #lib/IO.pm
        'IO::Dir'               => '1.04',  #lib/IO/Dir.pm
        'IO::File'              => '1.10',  #lib/IO/File.pm
        'IO::Handle'            => '1.23',  #lib/IO/Handle.pm
        'IO::Pipe'              => '1.122',  #lib/IO/Pipe.pm
        'IO::Poll'              => '0.06',  #lib/IO/Poll.pm
        'IO::Seekable'          => '1.09',  #lib/IO/Seekable.pm
        'IO::Select'            => '1.16',  #lib/IO/Select.pm
        'IO::Socket'            => '1.28',  #lib/IO/Socket.pm
        'IO::Socket::INET'      => '1.27',  #lib/IO/Socket/INET.pm
        'IO::Socket::UNIX'      => '1.21',  #lib/IO/Socket/UNIX.pm
        'IPC::Msg'              => '1.02',  #lib/IPC/Msg.pm
        'IPC::Open2'            => '1.01',  #lib/IPC/Open2.pm
        'IPC::Open3'            => '1.0105',  #lib/IPC/Open3.pm
        'IPC::Semaphore'        => '1.02',  #lib/IPC/Semaphore.pm
        'IPC::SysV'             => '1.04',  #lib/IPC/SysV.pm
        'JNI'                   => '0.2',  #jpl/JNI/JNI.pm
        'JPL::AutoLoader'       => undef,  #jpl/JPL/AutoLoader.pm
        'JPL::Class'            => undef,  #jpl/JPL/Class.pm
        'JPL::Compile'          => undef,  #jpl/JPL/Compile.pm
        'less'                  => '0.01',  #lib/less.pm
        'lib'                   => '0.5565',  #lib/lib.pm
        'List::Util'            => '1.13',  #lib/List/Util.pm
        'locale'                => '1.00',  #lib/locale.pm
        'Locale::Constants'     => '2.01',  #lib/Locale/Constants.pm
        'Locale::Country'       => '2.61',  #lib/Locale/Country.pm
        'Locale::Currency'      => '2.21',  #lib/Locale/Currency.pm
        'Locale::Language'      => '2.21',  #lib/Locale/Language.pm
        'Locale::Maketext'      => '1.06',  #lib/Locale/Maketext.pm
        'Locale::Maketext::Guts'=> undef,  #lib/Locale/Maketext/Guts.pm
        'Locale::Maketext::GutsLoader'=> undef,  #lib/Locale/Maketext/GutsLoader.pm
        'Locale::Script'        => '2.21',  #lib/Locale/Script.pm
        'Math::BigFloat'        => '1.40',  #lib/Math/BigFloat.pm
        'Math::BigFloat::Trace' => '0.01',  #lib/Math/BigFloat/Trace.pm
        'Math::BigInt'          => '1.66',  #lib/Math/BigInt.pm
        'Math::BigInt::Calc'    => '0.36',  #lib/Math/BigInt/Calc.pm
        'Math::BigInt::Scalar'  => '0.11',  #lib/Math/BigInt/Scalar.pm
        'Math::BigInt::Trace'   => '0.01',  #lib/Math/BigInt/Trace.pm
        'Math::BigRat'          => '0.10',  #lib/Math/BigRat.pm
        'Math::Complex'         => '1.34',  #lib/Math/Complex.pm
        'Math::Trig'            => '1.02',  #lib/Math/Trig.pm
        'Memoize'               => '1.01',  #lib/Memoize.pm
        'Memoize::AnyDBM_File'  => '0.65',  #lib/Memoize/AnyDBM_File.pm
        'Memoize::Expire'       => '1.00',  #lib/Memoize/Expire.pm
        'Memoize::ExpireFile'   => '1.01',  #lib/Memoize/ExpireFile.pm
        'Memoize::ExpireTest'   => '0.65',  #lib/Memoize/ExpireTest.pm
        'Memoize::NDBM_File'    => '0.65',  #lib/Memoize/NDBM_File.pm
        'Memoize::SDBM_File'    => '0.65',  #lib/Memoize/SDBM_File.pm
        'Memoize::Storable'     => '0.65',  #lib/Memoize/Storable.pm
        'MIME::Base64'          => '2.21',  #lib/MIME/Base64.pm
        'MIME::QuotedPrint'     => '2.21',  #lib/MIME/QuotedPrint.pm
        'NDBM_File'             => '1.05',  #ext/NDBM_File/NDBM_File.pm
        'Net::Cmd'              => '2.24',  #lib/Net/Cmd.pm
        'Net::Config'           => '1.10',  #lib/Net/Config.pm
        'Net::Domain'           => '2.19',  #lib/Net/Domain.pm
        'Net::FTP'              => '2.72',  #lib/Net/FTP.pm
        'Net::FTP::A'           => '1.16',  #lib/Net/FTP/A.pm
        'Net::FTP::dataconn'    => '0.11',  #lib/Net/FTP/dataconn.pm
        'Net::FTP::E'           => '0.01',  #lib/Net/FTP/E.pm
        'Net::FTP::I'           => '1.12',  #lib/Net/FTP/I.pm
        'Net::FTP::L'           => '0.01',  #lib/Net/FTP/L.pm
        'Net::hostent'          => '1.01',  #lib/Net/hostent.pm
        'Net::netent'           => '1.00',  #lib/Net/netent.pm
        'Net::Netrc'            => '2.12',  #lib/Net/Netrc.pm
        'Net::NNTP'             => '2.22',  #lib/Net/NNTP.pm
        'Net::Ping'             => '2.31',  #lib/Net/Ping.pm
        'Net::POP3'             => '2.24',  #lib/Net/POP3.pm
        'Net::protoent'         => '1.00',  #lib/Net/protoent.pm
        'Net::servent'          => '1.01',  #lib/Net/servent.pm
        'Net::SMTP'             => '2.26',  #lib/Net/SMTP.pm
        'Net::Time'             => '2.09',  #lib/Net/Time.pm
        'NEXT'                  => '0.60',  #lib/NEXT.pm
        'O'                     => '1.00',  #lib/O.pm
        'ODBM_File'             => '1.04',  #ext/ODBM_File/ODBM_File.pm
        'Opcode'                => '1.06',  #lib/Opcode.pm
        'open'                  => '1.02',  #lib/open.pm
        'ops'                   => '1.00',  #lib/ops.pm
        'OS2::DLL'              => '1.02',  #os2/OS2/REXX/DLL/DLL.pm
        'OS2::ExtAttr'          => '0.02',  #os2/OS2/ExtAttr/ExtAttr.pm
        'OS2::PrfDB'            => '0.03',  #os2/OS2/PrfDB/PrfDB.pm
        'OS2::Process'          => '1.01',  #os2/OS2/Process/Process.pm
        'OS2::REXX'             => '1.02',  #os2/OS2/REXX/REXX.pm
        'overload'              => '1.02',  #lib/overload.pm
        'PerlIO'                => '1.02',  #lib/PerlIO.pm
        'PerlIO::encoding'      => '0.07',  #lib/PerlIO/encoding.pm
        'PerlIO::scalar'        => '0.02',  #lib/PerlIO/scalar.pm
        'PerlIO::via'           => '0.02',  #lib/PerlIO/via.pm
        'PerlIO::via::QuotedPrint'=> '0.05',  #lib/PerlIO/via/QuotedPrint.pm
        'Pod::Checker'          => '1.41',  #lib/Pod/Checker.pm
        'Pod::Find'             => '0.24',  #lib/Pod/Find.pm
        'Pod::Functions'        => '1.02',  #lib/Pod/Functions.pm
        'Pod::Html'             => '1.0501',  #lib/Pod/Html.pm
        'Pod::InputObjects'     => '1.14',  #lib/Pod/InputObjects.pm
        'Pod::LaTeX'            => '0.55',  #lib/Pod/LaTeX.pm
        'Pod::Man'              => '1.37',  #lib/Pod/Man.pm
        'Pod::ParseLink'        => '1.06',  #lib/Pod/ParseLink.pm
        'Pod::Parser'           => '1.13',  #lib/Pod/Parser.pm
        'Pod::ParseUtils'       => '0.3',  #lib/Pod/ParseUtils.pm
        'Pod::Perldoc'          => '3.11',  #lib/Pod/Perldoc.pm
        'Pod::Perldoc::BaseTo'  => undef,  #lib/Pod/Perldoc/BaseTo.pm
        'Pod::Perldoc::GetOptsOO'=> undef,  #lib/Pod/Perldoc/GetOptsOO.pm
        'Pod::Perldoc::ToChecker'=> undef,  #lib/Pod/Perldoc/ToChecker.pm
        'Pod::Perldoc::ToMan'   => undef,  #lib/Pod/Perldoc/ToMan.pm
        'Pod::Perldoc::ToNroff' => undef,  #lib/Pod/Perldoc/ToNroff.pm
        'Pod::Perldoc::ToPod'   => undef,  #lib/Pod/Perldoc/ToPod.pm
        'Pod::Perldoc::ToRtf'   => undef,  #lib/Pod/Perldoc/ToRtf.pm
        'Pod::Perldoc::ToText'  => undef,  #lib/Pod/Perldoc/ToText.pm
        'Pod::Perldoc::ToTk'    => 'undef',  #lib/Pod/Perldoc/ToTk.pm
        'Pod::Perldoc::ToXml'   => undef,  #lib/Pod/Perldoc/ToXml.pm
        'Pod::Plainer'          => '0.01',  #lib/Pod/Plainer.pm
        'Pod::PlainText'        => '2.01',  #lib/Pod/PlainText.pm
        'Pod::Select'           => '1.13',  #lib/Pod/Select.pm
        'Pod::Text'             => '2.21',  #lib/Pod/Text.pm
        'Pod::Text::Color'      => '1.04',  #lib/Pod/Text/Color.pm
        'Pod::Text::Overstrike' => '1.1',  #lib/Pod/Text/Overstrike.pm
        'Pod::Text::Termcap'    => '1.11',  #lib/Pod/Text/Termcap.pm
        'Pod::Usage'            => '1.16',  #lib/Pod/Usage.pm
        'POSIX'                 => '1.06',  #lib/POSIX.pm
        're'                    => '0.04',  #lib/re.pm
        'Safe'                  => '2.10',  #lib/Safe.pm
        'Scalar::Util'          => '1.13',  #lib/Scalar/Util.pm
        'SDBM_File'             => '1.04',  #lib/SDBM_File.pm
        'Search::Dict'          => '1.02',  #lib/Search/Dict.pm
        'SelectSaver'           => '1.00',  #lib/SelectSaver.pm
        'SelfLoader'            => '1.0904',  #lib/SelfLoader.pm
        'Shell'                 => '0.5',  #lib/Shell.pm
        'sigtrap'               => '1.02',  #lib/sigtrap.pm
        'Socket'                => '1.76',  #lib/Socket.pm
        'sort'                  => '1.02',  #lib/sort.pm
        'Storable'              => '2.08',  #lib/Storable.pm
        'strict'                => '1.03',  #lib/strict.pm
        'subs'                  => '1.00',  #lib/subs.pm
        'Switch'                => '2.10',  #lib/Switch.pm
        'Symbol'                => '1.05',  #lib/Symbol.pm
        'Sys::Hostname'         => '1.11',  #lib/Sys/Hostname.pm
        'Sys::Syslog'           => '0.04',  #lib/Sys/Syslog.pm
        'Term::ANSIColor'       => '1.07',  #lib/Term/ANSIColor.pm
        'Term::Cap'             => '1.08',  #lib/Term/Cap.pm
        'Term::Complete'        => '1.401',  #lib/Term/Complete.pm
        'Term::ReadLine'        => '1.01',  #lib/Term/ReadLine.pm
        'Test'                  => '1.24',  #lib/Test.pm
        'Test::Builder'         => '0.17',  #lib/Test/Builder.pm
        'Test::Harness'         => '2.30',  #lib/Test/Harness.pm
        'Test::Harness::Assert' => '0.01',  #lib/Test/Harness/Assert.pm
        'Test::Harness::Iterator'=> '0.01',  #lib/Test/Harness/Iterator.pm
        'Test::Harness::Straps' => '0.15',  #lib/Test/Harness/Straps.pm
        'Test::More'            => '0.47',  #lib/Test/More.pm
        'Test::Simple'          => '0.47',  #lib/Test/Simple.pm
        'Text::Abbrev'          => '1.01',  #lib/Text/Abbrev.pm
        'Text::Balanced'        => '1.95',  #lib/Text/Balanced.pm
        'Text::ParseWords'      => '3.21',  #lib/Text/ParseWords.pm
        'Text::Soundex'         => '1.01',  #lib/Text/Soundex.pm
        'Text::Tabs'            => '98.112801',  #lib/Text/Tabs.pm
        'Text::Wrap'            => '2001.09291',  #lib/Text/Wrap.pm
        'Thread'                => '2.00',  #lib/Thread.pm
        'Thread::Queue'         => '2.00',  #lib/Thread/Queue.pm
        'Thread::Semaphore'     => '2.01',  #lib/Thread/Semaphore.pm
        'Thread::Signal'        => '1.00', #./ext/Thread/Thread/Signal.pm
        'Thread::Specific'      => '1.00', #./ext/Thread/Thread/Specific.pm
        'threads'               => '1.00',  #lib/threads.pm
        'threads::shared'       => '0.91',  #lib/threads/shared.pm
        'Tie::Array'            => '1.03',  #lib/Tie/Array.pm
        'Tie::File'             => '0.97',  #lib/Tie/File.pm
        'Tie::Handle'           => '4.1',  #lib/Tie/Handle.pm
        'Tie::Hash'             => '1.00',  #lib/Tie/Hash.pm
        'Tie::Memoize'          => '1.0',  #lib/Tie/Memoize.pm
        'Tie::RefHash'          => '1.31',  #lib/Tie/RefHash.pm
        'Tie::Scalar'           => '1.00',  #lib/Tie/Scalar.pm
        'Tie::SubstrHash'       => '1.00',  #lib/Tie/SubstrHash.pm
        'Time::gmtime'          => '1.02',  #lib/Time/gmtime.pm
        'Time::HiRes'           => '1.51',  #lib/Time/HiRes.pm
        'Time::Local'           => '1.07',  #lib/Time/Local.pm
        'Time::localtime'       => '1.02',  #lib/Time/localtime.pm
        'Time::tm'              => '1.00',  #lib/Time/tm.pm
        'Unicode::Collate'      => '0.28',  #lib/Unicode/Collate.pm
        'Unicode::Normalize'    => '0.23',  #lib/Unicode/Normalize.pm
        'Unicode::UCD'          => '0.21',  #lib/Unicode/UCD.pm
        'UNIVERSAL'             => '1.01',  #lib/UNIVERSAL.pm
        'User::grent'           => '1.00',  #lib/User/grent.pm
        'User::pwent'           => '1.00',  #lib/User/pwent.pm
        'utf8'                  => '1.02',  #lib/utf8.pm
        'vars'                  => '1.01',  #lib/vars.pm
        'version'               => '0.29',  #lib/version.pm
        'VMS::DCLsym'           => '1.02',  #vms/ext/DCLsym/DCLsym.pm
        'VMS::Filespec'         => '1.11',  #vms/ext/Filespec.pm
        'VMS::Stdio'            => '2.3',  #vms/ext/Stdio/Stdio.pm
        'vmsish'                => '1.01',  #lib/vmsish.pm
        'warnings'              => '1.03',  #lib/warnings.pm
        'warnings::register'    => '1.00',  #lib/warnings/register.pm
        'XS::APItest'           => '0.02',  #lib/XS/APItest.pm
        'XS::Typemap'           => '0.01',  #lib/XS/Typemap.pm
        'XSLoader'              => '0.03',  #lib/XSLoader.pm
       },

    5.009001 => {
        'AnyDBM_File'           => '1.00',  #lib/AnyDBM_File.pm
        'assertions'            => '0.01',  #lib/assertions.pm
        'assertions::activate'  => '0.01',  #lib/assertions/activate.pm
        'Attribute::Handlers'   => '0.78_01',  #lib/Attribute/Handlers.pm
        'attributes'            => '0.06',  #lib/attributes.pm
        'attrs'                 => '1.01',  #lib/attrs.pm
        'AutoLoader'            => '5.60',  #lib/AutoLoader.pm
        'AutoSplit'             => '1.04',  #lib/AutoSplit.pm
        'autouse'               => '1.03',  #lib/autouse.pm
        'B'                     => '1.05',  #lib/B.pm
        'base'                  => '2.04',  #lib/base.pm
        'B::Asmdata'            => '1.01',  #lib/B/Asmdata.pm
        'B::Assembler'          => '0.06',  #lib/B/Assembler.pm
        'B::Bblock'             => '1.02',  #lib/B/Bblock.pm
        'B::Bytecode'           => '1.01',  #lib/B/Bytecode.pm
        'B::C'                  => '1.04',  #lib/B/C.pm
        'B::CC'                 => '1.00',  #lib/B/CC.pm
        'B::Concise'            => '0.59',  #lib/B/Concise.pm
        'B::Debug'              => '1.02',  #lib/B/Debug.pm
        'B::Deparse'            => '0.65',  #lib/B/Deparse.pm
        'B::Disassembler'       => '1.03',  #lib/B/Disassembler.pm
        'Benchmark'             => '1.06',  #lib/Benchmark.pm
        'bigint'                => '0.05',  #lib/bigint.pm
        'bignum'                => '0.15',  #lib/bignum.pm
        'bigrat'                => '0.06',  #lib/bigrat.pm
        'blib'                  => '1.02',  #lib/blib.pm
        'B::Lint'               => '1.02',  #lib/B/Lint.pm
        'B::Showlex'            => '1.00',  #lib/B/Showlex.pm
        'B::Stackobj'           => '1.00',  #lib/B/Stackobj.pm
        'B::Stash'              => '1.00',  #lib/B/Stash.pm
        'B::Terse'              => '1.02',  #lib/B/Terse.pm
        'B::Xref'               => '1.01',  #lib/B/Xref.pm
        'ByteLoader'            => '0.05',  #lib/ByteLoader.pm
        'bytes'                 => '1.01',  #lib/bytes.pm
        'Carp'                  => '1.02',  #lib/Carp.pm
        'Carp::Heavy'           => '1.01',  #lib/Carp/Heavy.pm
        'CGI'                   => '3.04',  #lib/CGI.pm
        'CGI::Apache'           => '1.00',  #lib/CGI/Apache.pm
        'CGI::Carp'             => '1.27',  #lib/CGI/Carp.pm
        'CGI::Cookie'           => '1.24',  #lib/CGI/Cookie.pm
        'CGI::Fast'             => '1.05',  #lib/CGI/Fast.pm
        'CGI::Pretty'           => '1.08',  #lib/CGI/Pretty.pm
        'CGI::Push'             => '1.04',  #lib/CGI/Push.pm
        'CGI::Switch'           => '1.00',  #lib/CGI/Switch.pm
        'CGI::Util'             => '1.4',  #lib/CGI/Util.pm
        'charnames'             => '1.03',  #lib/charnames.pm
        'Class::ISA'            => '0.32',  #lib/Class/ISA.pm
        'Class::Struct'         => '0.63',  #lib/Class/Struct.pm
        'Config'                => undef,  #lib/Config.pm
        'constant'              => '1.04',  #lib/constant.pm
        'CPAN'                  => '1.76_01',  #lib/CPAN.pm
        'CPAN::FirstTime'       => '1.60 ',  #lib/CPAN/FirstTime.pm
        'CPAN::Nox'             => '1.03',  #lib/CPAN/Nox.pm
        'Cwd'                   => '2.17',  #lib/Cwd.pm
        'Data::Dumper'          => '2.121',  #lib/Data/Dumper.pm
        'DB'                    => '1.0',  #lib/DB.pm
        'DB_File'               => '1.808_01',  #lib/DB_File.pm
        'DBM_Filter'            => '0.01',  #lib/DBM_Filter.pm
        'DBM_Filter::compress'  => '0.01',  #lib/DBM_Filter/compress.pm
        'DBM_Filter::encode'    => '0.01',  #lib/DBM_Filter/encode.pm
        'DBM_Filter::int32'     => '0.01',  #lib/DBM_Filter/int32.pm
        'DBM_Filter::null'      => '0.01',  #lib/DBM_Filter/null.pm
        'DBM_Filter::utf8'      => '0.01',  #lib/DBM_Filter/utf8.pm
        'Devel::DProf'          => '20030813.00',  #lib/Devel/DProf.pm
        'Devel::Peek'           => '1.01',  #lib/Devel/Peek.pm
        'Devel::PPPort'         => '2.011_01',  #lib/Devel/PPPort.pm
        'Devel::SelfStubber'    => '1.03',  #lib/Devel/SelfStubber.pm
        'diagnostics'           => '1.12',  #lib/diagnostics.pm
        'Digest'                => '1.05',  #lib/Digest.pm
        'Digest::base'          => '1.00',  #lib/Digest/base.pm
        'Digest::MD5'           => '2.33',  #lib/Digest/MD5.pm
        'DirHandle'             => '1.00',  #lib/DirHandle.pm
        'Dumpvalue'             => '1.11',  #lib/Dumpvalue.pm
        'DynaLoader'            => '1.04',  #lib/DynaLoader.pm
        'Encode'                => '1.99_01',  #lib/Encode.pm
        'Encode::Alias'         => '1.38',  #lib/Encode/Alias.pm
        'Encode::Byte'          => '1.23',  #lib/Encode/Byte.pm
        'Encode::CJKConstants'  => '1.02',  #lib/Encode/CJKConstants.pm
        'Encode::CN'            => '1.24',  #lib/Encode/CN.pm
        'Encode::CN::HZ'        => '1.0501',  #lib/Encode/CN/HZ.pm
        'Encode::Config'        => '1.07',  #lib/Encode/Config.pm
        'Encode::EBCDIC'        => '1.21',  #lib/Encode/EBCDIC.pm
        'Encode::Encoder'       => '0.07',  #lib/Encode/Encoder.pm
        'Encode::Encoding'      => '1.33',  #lib/Encode/Encoding.pm
        'Encode::Guess'         => '1.09',  #lib/Encode/Guess.pm
        'Encode::JP'            => '1.25',  #lib/Encode/JP.pm
        'Encode::JP::H2Z'       => '1.02',  #lib/Encode/JP/H2Z.pm
        'Encode::JP::JIS7'      => '1.12',  #lib/Encode/JP/JIS7.pm
        'Encode::KR'            => '1.23',  #lib/Encode/KR.pm
        'Encode::KR::2022_KR'   => '1.06',  #lib/Encode/KR/2022_KR.pm
        'Encode::MIME::Header'  => '1.09',  #lib/Encode/MIME/Header.pm
        'Encode::Symbol'        => '1.22',  #lib/Encode/Symbol.pm
        'Encode::TW'            => '1.26',  #lib/Encode/TW.pm
        'Encode::Unicode'       => '1.40',  #lib/Encode/Unicode.pm
        'Encode::Unicode::UTF7' => '0.02',  #lib/Encode/Unicode/UTF7.pm
        'encoding'              => '1.48',  #lib/encoding.pm
        'English'               => '1.02',  #lib/English.pm
        'Env'                   => '1.00',  #lib/Env.pm
        'Errno'                 => '1.09_00',  #lib/Errno.pm
        'Exporter'              => '5.58',  #lib/Exporter.pm
        'Exporter::Heavy'       => '5.567',  #lib/Exporter/Heavy.pm
        'ExtUtils::Command'     => '1.07',  #lib/ExtUtils/Command.pm
        'ExtUtils::Command::MM' => '0.03',  #lib/ExtUtils/Command/MM.pm
        'ExtUtils::Constant'    => '0.14',  #lib/ExtUtils/Constant.pm
        'ExtUtils::Embed'       => '1.250601',  #lib/ExtUtils/Embed.pm
        'ExtUtils::Install'     => '1.32',  #lib/ExtUtils/Install.pm
        'ExtUtils::Installed'   => '0.08',  #lib/ExtUtils/Installed.pm
        'ExtUtils::Liblist'     => '1.01',  #lib/ExtUtils/Liblist.pm
        'ExtUtils::Liblist::Kid'=> '1.3',  #lib/ExtUtils/Liblist/Kid.pm
        'ExtUtils::MakeMaker'   => '6.21_02',  #lib/ExtUtils/MakeMaker.pm
        'ExtUtils::MakeMaker::bytes'=> '0.01',  #lib/ExtUtils/MakeMaker/bytes.pm
        'ExtUtils::MakeMaker::vmsish'=> '0.01',  #lib/ExtUtils/MakeMaker/vmsish.pm
        'ExtUtils::Manifest'    => '1.43',  #lib/ExtUtils/Manifest.pm
        'ExtUtils::Miniperl'    => undef,  #lib/ExtUtils/Miniperl.pm
        'ExtUtils::Mkbootstrap' => '1.15',  #lib/ExtUtils/Mkbootstrap.pm
        'ExtUtils::Mksymlists'  => '1.19',  #lib/ExtUtils/Mksymlists.pm
        'ExtUtils::MM'          => '0.04',  #lib/ExtUtils/MM.pm
        'ExtUtils::MM_Any'      => '0.0901',  #lib/ExtUtils/MM_Any.pm
        'ExtUtils::MM_BeOS'     => '1.04',  #lib/ExtUtils/MM_BeOS.pm
        'ExtUtils::MM_Cygwin'   => '1.07',  #lib/ExtUtils/MM_Cygwin.pm
        'ExtUtils::MM_DOS'      => '0.02',  #lib/ExtUtils/MM_DOS.pm
        'ExtUtils::MM_MacOS'    => '1.07',  #lib/ExtUtils/MM_MacOS.pm
        'ExtUtils::MM_NW5'      => '2.07_01',  #lib/ExtUtils/MM_NW5.pm
        'ExtUtils::MM_OS2'      => '1.04',  #lib/ExtUtils/MM_OS2.pm
        'ExtUtils::MM_Unix'     => '1.45_01',  #lib/ExtUtils/MM_Unix.pm
        'ExtUtils::MM_UWIN'     => '0.02',  #lib/ExtUtils/MM_UWIN.pm
        'ExtUtils::MM_VMS'      => '5.71_01',  #lib/ExtUtils/MM_VMS.pm
        'ExtUtils::MM_Win32'    => '1.10_01',  #lib/ExtUtils/MM_Win32.pm
        'ExtUtils::MM_Win95'    => '0.03',  #lib/ExtUtils/MM_Win95.pm
        'ExtUtils::MY'          => '0.01',  #lib/ExtUtils/MY.pm
        'ExtUtils::Packlist'    => '0.04',  #lib/ExtUtils/Packlist.pm
        'ExtUtils::testlib'     => '1.15',  #lib/ExtUtils/testlib.pm
        'ExtUtils::XSSymSet'    => '1.0',  #vms/ext/XSSymSet.pm
        'Fatal'                 => '1.04',  #lib/Fatal.pm
        'Fcntl'                 => '1.05',  #lib/Fcntl.pm
        'fields'                => '2.03',  #lib/fields.pm
        'File::Basename'        => '2.72',  #lib/File/Basename.pm
        'FileCache'             => '1.03',  #lib/FileCache.pm
        'File::CheckTree'       => '4.3',  #lib/File/CheckTree.pm
        'File::Compare'         => '1.1003',  #lib/File/Compare.pm
        'File::Copy'            => '2.07',  #lib/File/Copy.pm
        'File::DosGlob'         => '1.00',  #lib/File/DosGlob.pm
        'File::Find'            => '1.07',  #lib/File/Find.pm
        'File::Glob'            => '1.02',  #lib/File/Glob.pm
        'FileHandle'            => '2.01',  #lib/FileHandle.pm
        'File::Path'            => '1.06',  #lib/File/Path.pm
        'File::Spec'            => '0.87',  #lib/File/Spec.pm
        'File::Spec::Cygwin'    => '1.1',  #lib/File/Spec/Cygwin.pm
        'File::Spec::Epoc'      => '1.1',  #lib/File/Spec/Epoc.pm
        'File::Spec::Functions' => '1.3',  #lib/File/Spec/Functions.pm
        'File::Spec::Mac'       => '1.4',  #lib/File/Spec/Mac.pm
        'File::Spec::OS2'       => '1.2',  #lib/File/Spec/OS2.pm
        'File::Spec::Unix'      => '1.5',  #lib/File/Spec/Unix.pm
        'File::Spec::VMS'       => '1.4',  #lib/File/Spec/VMS.pm
        'File::Spec::Win32'     => '1.4',  #lib/File/Spec/Win32.pm
        'File::stat'            => '1.00',  #lib/File/stat.pm
        'File::Temp'            => '0.14',  #lib/File/Temp.pm
        'filetest'              => '1.01',  #lib/filetest.pm
        'Filter::Simple'        => '0.78',  #lib/Filter/Simple.pm
        'Filter::Util::Call'    => '1.0601',  #lib/Filter/Util/Call.pm
        'FindBin'               => '1.44',  #lib/FindBin.pm
        'GDBM_File'             => '1.07',  #lib/GDBM_File.pm
        'Getopt::Long'          => '2.3401',  #lib/Getopt/Long.pm
        'Getopt::Std'           => '1.05',  #lib/Getopt/Std.pm
        'Hash::Util'            => '0.05',  #lib/Hash/Util.pm
        'I18N::Collate'         => '1.00',  #lib/I18N/Collate.pm
        'I18N::Langinfo'        => '0.02',  #lib/I18N/Langinfo.pm
        'I18N::LangTags'        => '0.29',  #lib/I18N/LangTags.pm
        'I18N::LangTags::List'  => '0.29',  #lib/I18N/LangTags/List.pm
        'if'                    => '0.0401',  #lib/if.pm
        'integer'               => '1.00',  #lib/integer.pm
        'IO'                    => '1.21',  #lib/IO.pm
        'IO::Dir'               => '1.04',  #lib/IO/Dir.pm
        'IO::File'              => '1.10',  #lib/IO/File.pm
        'IO::Handle'            => '1.23',  #lib/IO/Handle.pm
        'IO::Pipe'              => '1.122',  #lib/IO/Pipe.pm
        'IO::Poll'              => '0.06',  #lib/IO/Poll.pm
        'IO::Seekable'          => '1.09',  #lib/IO/Seekable.pm
        'IO::Select'            => '1.16',  #lib/IO/Select.pm
        'IO::Socket'            => '1.28',  #lib/IO/Socket.pm
        'IO::Socket::INET'      => '1.27',  #lib/IO/Socket/INET.pm
        'IO::Socket::UNIX'      => '1.21',  #lib/IO/Socket/UNIX.pm
        'IPC::Msg'              => '1.02',  #lib/IPC/Msg.pm
        'IPC::Open2'            => '1.01',  #lib/IPC/Open2.pm
        'IPC::Open3'            => '1.0105',  #lib/IPC/Open3.pm
        'IPC::Semaphore'        => '1.02',  #lib/IPC/Semaphore.pm
        'IPC::SysV'             => '1.04',  #lib/IPC/SysV.pm
        'JNI'                   => '0.2',  #jpl/JNI/JNI.pm
        'JPL::AutoLoader'       => undef,  #jpl/JPL/AutoLoader.pm
        'JPL::Class'            => undef,  #jpl/JPL/Class.pm
        'JPL::Compile'          => undef,  #jpl/JPL/Compile.pm
        'less'                  => '0.01',  #lib/less.pm
        'lib'                   => '0.5565',  #lib/lib.pm
        'List::Util'            => '1.13',  #lib/List/Util.pm
        'locale'                => '1.00',  #lib/locale.pm
        'Locale::Constants'     => '2.01',  #lib/Locale/Constants.pm
        'Locale::Country'       => '2.61',  #lib/Locale/Country.pm
        'Locale::Currency'      => '2.21',  #lib/Locale/Currency.pm
        'Locale::Language'      => '2.21',  #lib/Locale/Language.pm
        'Locale::Maketext'      => '1.08',  #lib/Locale/Maketext.pm
        'Locale::Maketext::GutsLoader'=> undef,  #lib/Locale/Maketext/GutsLoader.pm
        'Locale::Maketext::Guts'=> undef,  #lib/Locale/Maketext/Guts.pm
        'Locale::Script'        => '2.21',  #lib/Locale/Script.pm
        'Math::BigFloat'        => '1.44',  #lib/Math/BigFloat.pm
        'Math::BigFloat::Trace' => '0.01',  #lib/Math/BigFloat/Trace.pm
        'Math::BigInt'          => '1.70',  #lib/Math/BigInt.pm
        'Math::BigInt::Calc'    => '0.40',  #lib/Math/BigInt/Calc.pm
        'Math::BigInt::CalcEmu' => '0.04',  #lib/Math/BigInt/CalcEmu.pm
        'Math::BigInt::Trace'   => '0.01',  #lib/Math/BigInt/Trace.pm
        'Math::BigRat'          => '0.12',  #lib/Math/BigRat.pm
        'Math::Complex'         => '1.34',  #lib/Math/Complex.pm
        'Math::Trig'            => '1.02',  #lib/Math/Trig.pm
        'Memoize'               => '1.01_01',  #lib/Memoize.pm
        'Memoize::AnyDBM_File'  => '0.65',  #lib/Memoize/AnyDBM_File.pm
        'Memoize::Expire'       => '1.00',  #lib/Memoize/Expire.pm
        'Memoize::ExpireFile'   => '1.01',  #lib/Memoize/ExpireFile.pm
        'Memoize::ExpireTest'   => '0.65',  #lib/Memoize/ExpireTest.pm
        'Memoize::NDBM_File'    => '0.65',  #lib/Memoize/NDBM_File.pm
        'Memoize::SDBM_File'    => '0.65',  #lib/Memoize/SDBM_File.pm
        'Memoize::Storable'     => '0.65',  #lib/Memoize/Storable.pm
        'MIME::Base64'          => '3.00_01',  #lib/MIME/Base64.pm
        'MIME::QuotedPrint'     => '3.00',  #lib/MIME/QuotedPrint.pm
        'NDBM_File'             => '1.05',  #lib/NDBM_File.pm
        'Net::Cmd'              => '2.24',  #lib/Net/Cmd.pm
        'Net::Config'           => '1.10',  #lib/Net/Config.pm
        'Net::Domain'           => '2.19',  #lib/Net/Domain.pm
        'Net::FTP'              => '2.72',  #lib/Net/FTP.pm
        'Net::FTP::A'           => '1.16',  #lib/Net/FTP/A.pm
        'Net::FTP::dataconn'    => '0.11',  #lib/Net/FTP/dataconn.pm
        'Net::FTP::E'           => '0.01',  #lib/Net/FTP/E.pm
        'Net::FTP::I'           => '1.12',  #lib/Net/FTP/I.pm
        'Net::FTP::L'           => '0.01',  #lib/Net/FTP/L.pm
        'Net::hostent'          => '1.01',  #lib/Net/hostent.pm
        'Net::netent'           => '1.00',  #lib/Net/netent.pm
        'Net::Netrc'            => '2.12',  #lib/Net/Netrc.pm
        'Net::NNTP'             => '2.22',  #lib/Net/NNTP.pm
        'Net::Ping'             => '2.31',  #lib/Net/Ping.pm
        'Net::POP3'             => '2.24',  #lib/Net/POP3.pm
        'Net::protoent'         => '1.00',  #lib/Net/protoent.pm
        'Net::servent'          => '1.01',  #lib/Net/servent.pm
        'Net::SMTP'             => '2.26',  #lib/Net/SMTP.pm
        'Net::Time'             => '2.09',  #lib/Net/Time.pm
        'NEXT'                  => '0.60',  #lib/NEXT.pm
        'O'                     => '1.00',  #lib/O.pm
        'ODBM_File'             => '1.04',  #ext/ODBM_File/ODBM_File.pm
        'Opcode'                => '1.06',  #lib/Opcode.pm
        'open'                  => '1.02',  #lib/open.pm
        'ops'                   => '1.00',  #lib/ops.pm
        'OS2::DLL'              => '1.02',  #os2/OS2/REXX/DLL/DLL.pm
        'OS2::ExtAttr'          => '0.02',  #os2/OS2/ExtAttr/ExtAttr.pm
        'OS2::PrfDB'            => '0.03',  #os2/OS2/PrfDB/PrfDB.pm
        'OS2::Process'          => '1.01',  #os2/OS2/Process/Process.pm
        'OS2::REXX'             => '1.02',  #os2/OS2/REXX/REXX.pm
        'overload'              => '1.02',  #lib/overload.pm
        'PerlIO'                => '1.03',  #lib/PerlIO.pm
        'PerlIO::encoding'      => '0.07',  #lib/PerlIO/encoding.pm
        'PerlIO::scalar'        => '0.02',  #lib/PerlIO/scalar.pm
        'PerlIO::via'           => '0.02',  #lib/PerlIO/via.pm
        'PerlIO::via::QuotedPrint'=> '0.06',  #lib/PerlIO/via/QuotedPrint.pm
        'Pod::Checker'          => '1.41',  #lib/Pod/Checker.pm
        'Pod::Find'             => '0.24',  #lib/Pod/Find.pm
        'Pod::Functions'        => '1.02',  #lib/Pod/Functions.pm
        'Pod::Html'             => '1.0502',  #lib/Pod/Html.pm
        'Pod::InputObjects'     => '1.14',  #lib/Pod/InputObjects.pm
        'Pod::LaTeX'            => '0.55',  #lib/Pod/LaTeX.pm
        'Pod::Man'              => '1.37',  #lib/Pod/Man.pm
        'Pod::ParseLink'        => '1.06',  #lib/Pod/ParseLink.pm
        'Pod::Parser'           => '1.14',  #lib/Pod/Parser.pm
        'Pod::ParseUtils'       => '0.3',  #lib/Pod/ParseUtils.pm
        'Pod::Perldoc'          => '3.12',  #lib/Pod/Perldoc.pm
        'Pod::Perldoc::BaseTo'  => undef,  #lib/Pod/Perldoc/BaseTo.pm
        'Pod::Perldoc::GetOptsOO'=> undef,  #lib/Pod/Perldoc/GetOptsOO.pm
        'Pod::Perldoc::ToChecker'=> undef,  #lib/Pod/Perldoc/ToChecker.pm
        'Pod::Perldoc::ToMan'   => undef,  #lib/Pod/Perldoc/ToMan.pm
        'Pod::Perldoc::ToNroff' => undef,  #lib/Pod/Perldoc/ToNroff.pm
        'Pod::Perldoc::ToPod'   => undef,  #lib/Pod/Perldoc/ToPod.pm
        'Pod::Perldoc::ToRtf'   => undef,  #lib/Pod/Perldoc/ToRtf.pm
        'Pod::Perldoc::ToText'  => undef,  #lib/Pod/Perldoc/ToText.pm
        'Pod::Perldoc::ToTk'    => 'undef',  #lib/Pod/Perldoc/ToTk.pm
        'Pod::Perldoc::ToXml'   => undef,  #lib/Pod/Perldoc/ToXml.pm
        'Pod::Plainer'          => '0.01',  #lib/Pod/Plainer.pm
        'Pod::PlainText'        => '2.02',  #lib/Pod/PlainText.pm
        'Pod::Select'           => '1.13',  #lib/Pod/Select.pm
        'Pod::Text'             => '2.21',  #lib/Pod/Text.pm
        'Pod::Text::Color'      => '1.04',  #lib/Pod/Text/Color.pm
        'Pod::Text::Overstrike' => '1.1',  #lib/Pod/Text/Overstrike.pm
        'Pod::Text::Termcap'    => '1.11',  #lib/Pod/Text/Termcap.pm
        'Pod::Usage'            => '1.16',  #lib/Pod/Usage.pm
        'POSIX'                 => '1.07',  #lib/POSIX.pm
        're'                    => '0.04',  #lib/re.pm
        'Safe'                  => '2.10',  #lib/Safe.pm
        'Scalar::Util'          => '1.13',  #lib/Scalar/Util.pm
        'SDBM_File'             => '1.04',  #lib/SDBM_File.pm
        'Search::Dict'          => '1.02',  #lib/Search/Dict.pm
        'SelectSaver'           => '1.00',  #lib/SelectSaver.pm
        'SelfLoader'            => '1.0904',  #lib/SelfLoader.pm
        'Shell'                 => '0.5.2',  #lib/Shell.pm
        'sigtrap'               => '1.02',  #lib/sigtrap.pm
        'Socket'                => '1.77',  #lib/Socket.pm
        'sort'                  => '1.02',  #lib/sort.pm
        'Storable'              => '2.11',  #lib/Storable.pm
        'strict'                => '1.03',  #lib/strict.pm
        'subs'                  => '1.00',  #lib/subs.pm
        'Switch'                => '2.10',  #lib/Switch.pm
        'Symbol'                => '1.05',  #lib/Symbol.pm
        'Sys::Hostname'         => '1.11',  #lib/Sys/Hostname.pm
        'Sys::Syslog'           => '0.05',  #lib/Sys/Syslog.pm
        'Term::ANSIColor'       => '1.08',  #lib/Term/ANSIColor.pm
        'Term::Cap'             => '1.08',  #lib/Term/Cap.pm
        'Term::Complete'        => '1.401',  #lib/Term/Complete.pm
        'Term::ReadLine'        => '1.01',  #lib/Term/ReadLine.pm
        'Test'                  => '1.24',  #lib/Test.pm
        'Test::Builder'         => '0.17',  #lib/Test/Builder.pm
        'Test::Harness'         => '2.40',  #lib/Test/Harness.pm
        'Test::Harness::Assert' => '0.02',  #lib/Test/Harness/Assert.pm
        'Test::Harness::Iterator'=> '0.02',  #lib/Test/Harness/Iterator.pm
        'Test::Harness::Straps' => '0.19',  #lib/Test/Harness/Straps.pm
        'Test::More'            => '0.47',  #lib/Test/More.pm
        'Test::Simple'          => '0.47',  #lib/Test/Simple.pm
        'Text::Abbrev'          => '1.01',  #lib/Text/Abbrev.pm
        'Text::Balanced'        => '1.95',  #lib/Text/Balanced.pm
        'Text::ParseWords'      => '3.21',  #lib/Text/ParseWords.pm
        'Text::Soundex'         => '1.01',  #lib/Text/Soundex.pm
        'Text::Tabs'            => '98.112801',  #lib/Text/Tabs.pm
        'Text::Wrap'            => '2001.09291',  #lib/Text/Wrap.pm
        'Thread'                => '2.00',  #lib/Thread.pm
        'Thread::Queue'         => '2.00',  #lib/Thread/Queue.pm
        'threads'               => '1.02',  #lib/threads.pm
        'Thread::Semaphore'     => '2.01',  #lib/Thread/Semaphore.pm
        'Thread::Signal'        => '1.00', #./ext/Thread/Thread/Signal.pm
        'Thread::Specific'      => '1.00', #./ext/Thread/Thread/Specific.pm
        'threads::shared'       => '0.92',  #lib/threads/shared.pm
        'Tie::Array'            => '1.03',  #lib/Tie/Array.pm
        'Tie::File'             => '0.97',  #lib/Tie/File.pm
        'Tie::Handle'           => '4.1',  #lib/Tie/Handle.pm
        'Tie::Hash'             => '1.01',  #lib/Tie/Hash.pm
        'Tie::Memoize'          => '1.0',  #lib/Tie/Memoize.pm
        'Tie::RefHash'          => '1.31',  #lib/Tie/RefHash.pm
        'Tie::Scalar'           => '1.00',  #lib/Tie/Scalar.pm
        'Tie::SubstrHash'       => '1.00',  #lib/Tie/SubstrHash.pm
        'Time::gmtime'          => '1.02',  #lib/Time/gmtime.pm
        'Time::HiRes'           => '1.56',  #lib/Time/HiRes.pm
        'Time::Local'           => '1.07_94',  #lib/Time/Local.pm
        'Time::localtime'       => '1.02',  #lib/Time/localtime.pm
        'Time::tm'              => '1.00',  #lib/Time/tm.pm
        'Unicode::Collate'      => '0.33',  #lib/Unicode/Collate.pm
        'Unicode::Normalize'    => '0.28',  #lib/Unicode/Normalize.pm
        'Unicode::UCD'          => '0.21',  #lib/Unicode/UCD.pm
        'UNIVERSAL'             => '1.02',  #lib/UNIVERSAL.pm
        'User::grent'           => '1.00',  #lib/User/grent.pm
        'User::pwent'           => '1.00',  #lib/User/pwent.pm
        'utf8'                  => '1.02',  #lib/utf8.pm
        'vars'                  => '1.01',  #lib/vars.pm
        'version'               => '0.36',  #lib/version.pm
        'VMS::DCLsym'           => '1.02',  #vms/ext/DCLsym/DCLsym.pm
        'VMS::Filespec'         => '1.11',  #vms/ext/Filespec.pm
        'VMS::Stdio'            => '2.3',  #vms/ext/Stdio/Stdio.pm
        'vmsish'                => '1.01',  #lib/vmsish.pm
        'warnings'              => '1.03',  #lib/warnings.pm
        'warnings::register'    => '1.00',  #lib/warnings/register.pm
        'XS::APItest'           => '0.03',  #lib/XS/APItest.pm
        'XSLoader'              => '0.03',  #lib/XSLoader.pm
        'XS::Typemap'           => '0.01',  #lib/XS/Typemap.pm
    },
    5.008004 => {
        'AnyDBM_File'           => '1.00',  #lib/AnyDBM_File.pm
        'attributes'            => '0.06',  #lib/attributes.pm
        'AutoLoader'            => '5.60',  #lib/AutoLoader.pm
        'AutoSplit'             => '1.04',  #lib/AutoSplit.pm
        'autouse'               => '1.03',  #lib/autouse.pm
        'base'                  => '2.05',  #lib/base.pm
        'Benchmark'             => '1.06',  #lib/Benchmark.pm
        'bigint'                => '0.05',  #lib/bigint.pm
        'bignum'                => '0.15',  #lib/bignum.pm
        'bigrat'                => '0.06',  #lib/bigrat.pm
        'blib'                  => '1.02',  #lib/blib.pm
        'bytes'                 => '1.01',  #lib/bytes.pm
        'Carp'                  => '1.02',  #lib/Carp.pm
        'CGI'                   => '3.04',  #lib/CGI.pm
        'charnames'             => '1.03',  #lib/charnames.pm
        'constant'              => '1.04',  #lib/constant.pm
        'CPAN'                  => '1.76_01',  #lib/CPAN.pm
        'Cwd'                   => '2.17',  #lib/Cwd.pm
        'DBM_Filter'            => '0.01',  #lib/DBM_Filter.pm
        'DB'                    => '1.0',  #lib/DB.pm
        'diagnostics'           => '1.12',  #lib/diagnostics.pm
        'Digest'                => '1.06',  #lib/Digest.pm
        'DirHandle'             => '1.00',  #lib/DirHandle.pm
        'Dumpvalue'             => '1.11',  #lib/Dumpvalue.pm
        'English'               => '1.01',  #lib/English.pm
        'Env'                   => '1.00',  #lib/Env.pm
        'Exporter'              => '5.58',  #lib/Exporter.pm
        'Fatal'                 => '1.03',  #lib/Fatal.pm
        'fields'                => '2.03',  #lib/fields.pm
        'FileCache'             => '1.03',  #lib/FileCache.pm
        'FileHandle'            => '2.01',  #lib/FileHandle.pm
        'filetest'              => '1.01',  #lib/filetest.pm
        'FindBin'               => '1.44',  #lib/FindBin.pm
        'if'                    => '0.03',  #lib/if.pm
        'integer'               => '1.00',  #lib/integer.pm
        'less'                  => '0.01',  #lib/less.pm
        'locale'                => '1.00',  #lib/locale.pm
        'Memoize'               => '1.01',  #lib/Memoize.pm
        'NEXT'                  => '0.60',  #lib/NEXT.pm
        'open'                  => '1.03',  #lib/open.pm
        'overload'              => '1.01',  #lib/overload.pm
        'PerlIO'                => '1.03',  #lib/PerlIO.pm
        'SelectSaver'           => '1.00',  #lib/SelectSaver.pm
        'SelfLoader'            => '1.0904',  #lib/SelfLoader.pm
        'Shell'                 => '0.5.2',  #lib/Shell.pm
        'sigtrap'               => '1.02',  #lib/sigtrap.pm
        'sort'                  => '1.02',  #lib/sort.pm
        'strict'                => '1.03',  #lib/strict.pm
        'subs'                  => '1.00',  #lib/subs.pm
        'Switch'                => '2.10',  #lib/Switch.pm
        'Symbol'                => '1.05',  #lib/Symbol.pm
        'Test'                  => '1.24',  #lib/Test.pm
        'Thread'                => '2.00',  #lib/Thread.pm
        'UNIVERSAL'             => '1.01',  #lib/UNIVERSAL.pm
        'utf8'                  => '1.03',  #lib/utf8.pm
        'vars'                  => '1.01',  #lib/vars.pm
        'vmsish'                => '1.01',  #lib/vmsish.pm
        'warnings'              => '1.03',  #lib/warnings.pm
        'Config'                => undef,  #lib/Config.pm
        'lib'                   => '0.5565',  #lib/lib.pm
        're'                    => '0.04',  #lib/re.pm
        'XSLoader'              => '0.02',  #lib/XSLoader.pm
        'DynaLoader'            => '1.05',  #lib/DynaLoader.pm
        'attrs'                 => '1.01',  #lib/attrs.pm
        'B'                     => '1.02',  #lib/B.pm
        'O'                     => '1.00',  #lib/O.pm
        'ByteLoader'            => '0.05',  #lib/ByteLoader.pm
        'DB_File'               => '1.808',  #lib/DB_File.pm
        'Encode'                => '1.99_01',  #lib/Encode.pm
        'encoding'              => '1.48',  #lib/encoding.pm
        'Fcntl'                 => '1.05',  #lib/Fcntl.pm
        'GDBM_File'             => '1.07',  #lib/GDBM_File.pm
        'IO'                    => '1.21',  #lib/IO.pm
        'NDBM_File'             => '1.05',  #lib/NDBM_File.pm
        'Safe'                  => '2.10',  #lib/Safe.pm
        'Opcode'                => '1.05',  #lib/Opcode.pm
        'ops'                   => '1.00',  #lib/ops.pm
        'POSIX'                 => '1.08',  #lib/POSIX.pm
        'SDBM_File'             => '1.04',  #lib/SDBM_File.pm
        'Socket'                => '1.77',  #lib/Socket.pm
        'Storable'              => '2.12',  #lib/Storable.pm
        'threads'               => '1.03',  #lib/threads.pm
        'Errno'                 => '1.09_00',  #lib/Errno.pm
        'Attribute::Handlers'   => '0.78_01',  #lib/Attribute/Handlers.pm
        'Carp::Heavy'           => '1.01',  #lib/Carp/Heavy.pm
        'CGI::Apache'           => '1.00',  #lib/CGI/Apache.pm
        'CGI::Carp'             => '1.27',  #lib/CGI/Carp.pm
        'CGI::Cookie'           => '1.24',  #lib/CGI/Cookie.pm
        'CGI::Fast'             => '1.05',  #lib/CGI/Fast.pm
        'CGI::Pretty'           => '1.08',  #lib/CGI/Pretty.pm
        'CGI::Push'             => '1.04',  #lib/CGI/Push.pm
        'CGI::Switch'           => '1.00',  #lib/CGI/Switch.pm
        'CGI::Util'             => '1.4',  #lib/CGI/Util.pm
        'Class::ISA'            => '0.32',  #lib/Class/ISA.pm
        'Class::Struct'         => '0.63',  #lib/Class/Struct.pm
        'CPAN::FirstTime'       => '1.60 ',  #lib/CPAN/FirstTime.pm
        'CPAN::Nox'             => '1.03',  #lib/CPAN/Nox.pm
        'DBM_Filter::compress'  => '0.01',  #lib/DBM_Filter/compress.pm
        'DBM_Filter::encode'    => '0.01',  #lib/DBM_Filter/encode.pm
        'DBM_Filter::int32'     => '0.01',  #lib/DBM_Filter/int32.pm
        'DBM_Filter::null'      => '0.01',  #lib/DBM_Filter/null.pm
        'DBM_Filter::utf8'      => '0.01',  #lib/DBM_Filter/utf8.pm
        'Devel::SelfStubber'    => '1.03',  #lib/Devel/SelfStubber.pm
        'Devel::DProf'          => '20030813.00',  #lib/Devel/DProf.pm
        'Devel::Peek'           => '1.01',  #lib/Devel/Peek.pm
        'Devel::PPPort'         => '2.011',  #lib/Devel/PPPort.pm
        'Digest::base'          => '1.00',  #lib/Digest/base.pm
        'Digest::MD5'           => '2.33',  #lib/Digest/MD5.pm
        'Exporter::Heavy'       => '5.57',  #lib/Exporter/Heavy.pm
        'ExtUtils::Command'     => '1.05',  #lib/ExtUtils/Command.pm
        'ExtUtils::Constant'    => '0.14',  #lib/ExtUtils/Constant.pm
        'ExtUtils::Embed'       => '1.250601',  #lib/ExtUtils/Embed.pm
        'ExtUtils::Installed'   => '0.08',  #lib/ExtUtils/Installed.pm
        'ExtUtils::Install'     => '1.32',  #lib/ExtUtils/Install.pm
        'ExtUtils::Liblist'     => '1.01',  #lib/ExtUtils/Liblist.pm
        'ExtUtils::MakeMaker'   => '6.17',  #lib/ExtUtils/MakeMaker.pm
        'ExtUtils::Manifest'    => '1.42',  #lib/ExtUtils/Manifest.pm
        'ExtUtils::Mkbootstrap' => '1.15',  #lib/ExtUtils/Mkbootstrap.pm
        'ExtUtils::Mksymlists'  => '1.19',  #lib/ExtUtils/Mksymlists.pm
        'ExtUtils::MM_Any'      => '0.07',  #lib/ExtUtils/MM_Any.pm
        'ExtUtils::MM_BeOS'     => '1.04',  #lib/ExtUtils/MM_BeOS.pm
        'ExtUtils::MM_Cygwin'   => '1.06',  #lib/ExtUtils/MM_Cygwin.pm
        'ExtUtils::MM_DOS'      => '0.02',  #lib/ExtUtils/MM_DOS.pm
        'ExtUtils::MM_MacOS'    => '1.07',  #lib/ExtUtils/MM_MacOS.pm
        'ExtUtils::MM_NW5'      => '2.07_02',  #lib/ExtUtils/MM_NW5.pm
        'ExtUtils::MM'          => '0.04',  #lib/ExtUtils/MM.pm
        'ExtUtils::MM_OS2'      => '1.04',  #lib/ExtUtils/MM_OS2.pm
        'ExtUtils::MM_Unix'     => '1.42',  #lib/ExtUtils/MM_Unix.pm
        'ExtUtils::MM_UWIN'     => '0.02',  #lib/ExtUtils/MM_UWIN.pm
        'ExtUtils::MM_VMS'      => '5.70',  #lib/ExtUtils/MM_VMS.pm
        'ExtUtils::MM_Win32'    => '1.09',  #lib/ExtUtils/MM_Win32.pm
        'ExtUtils::MM_Win95'    => '0.0301',  #lib/ExtUtils/MM_Win95.pm
        'ExtUtils::MY'          => '0.01',  #lib/ExtUtils/MY.pm
        'ExtUtils::Packlist'    => '0.04',  #lib/ExtUtils/Packlist.pm
        'ExtUtils::testlib'     => '1.15',  #lib/ExtUtils/testlib.pm
        'ExtUtils::Miniperl'    => undef,  #lib/ExtUtils/Miniperl.pm
        'ExtUtils::Command::MM' => '0.03',  #lib/ExtUtils/Command/MM.pm
        'ExtUtils::Liblist::Kid'=> '1.3001',  #lib/ExtUtils/Liblist/Kid.pm
        'ExtUtils::MakeMaker::bytes'=> '0.01',  #lib/ExtUtils/MakeMaker/bytes.pm
        'ExtUtils::MakeMaker::vmsish'=> '0.01',  #lib/ExtUtils/MakeMaker/vmsish.pm
        'File::Basename'        => '2.72',  #lib/File/Basename.pm
        'File::CheckTree'       => '4.3',  #lib/File/CheckTree.pm
        'File::Compare'         => '1.1003',  #lib/File/Compare.pm
        'File::Copy'            => '2.07',  #lib/File/Copy.pm
        'File::DosGlob'         => '1.00',  #lib/File/DosGlob.pm
        'File::Find'            => '1.07',  #lib/File/Find.pm
        'File::Path'            => '1.06',  #lib/File/Path.pm
        'File::Spec'            => '0.87',  #lib/File/Spec.pm
        'File::stat'            => '1.00',  #lib/File/stat.pm
        'File::Temp'            => '0.14',  #lib/File/Temp.pm
        'File::Glob'            => '1.02',  #lib/File/Glob.pm
        'File::Spec::Cygwin'    => '1.1',  #lib/File/Spec/Cygwin.pm
        'File::Spec::Epoc'      => '1.1',  #lib/File/Spec/Epoc.pm
        'File::Spec::Functions' => '1.3',  #lib/File/Spec/Functions.pm
        'File::Spec::Mac'       => '1.4',  #lib/File/Spec/Mac.pm
        'File::Spec::OS2'       => '1.2',  #lib/File/Spec/OS2.pm
        'File::Spec::Unix'      => '1.5',  #lib/File/Spec/Unix.pm
        'File::Spec::VMS'       => '1.4',  #lib/File/Spec/VMS.pm
        'File::Spec::Win32'     => '1.4',  #lib/File/Spec/Win32.pm
        'Filter::Simple'        => '0.78',  #lib/Filter/Simple.pm
        'Filter::Util::Call'    => '1.0601',  #lib/Filter/Util/Call.pm
        'Getopt::Long'          => '2.34',  #lib/Getopt/Long.pm
        'Getopt::Std'           => '1.05',  #lib/Getopt/Std.pm
        'Hash::Util'            => '0.05',  #lib/Hash/Util.pm
        'I18N::Collate'         => '1.00',  #lib/I18N/Collate.pm
        'I18N::LangTags'        => '0.29',  #lib/I18N/LangTags.pm
        'I18N::Langinfo'        => '0.02',  #lib/I18N/Langinfo.pm
        'I18N::LangTags::List'  => '0.29',  #lib/I18N/LangTags/List.pm
        'IPC::Open2'            => '1.01',  #lib/IPC/Open2.pm
        'IPC::Open3'            => '1.0106',  #lib/IPC/Open3.pm
        'IPC::Msg'              => '1.02',  #lib/IPC/Msg.pm
        'IPC::Semaphore'        => '1.02',  #lib/IPC/Semaphore.pm
        'IPC::SysV'             => '1.04',  #lib/IPC/SysV.pm
        'Locale::Constants'     => '2.01',  #lib/Locale/Constants.pm
        'Locale::Country'       => '2.61',  #lib/Locale/Country.pm
        'Locale::Currency'      => '2.21',  #lib/Locale/Currency.pm
        'Locale::Language'      => '2.21',  #lib/Locale/Language.pm
        'Locale::Maketext'      => '1.08',  #lib/Locale/Maketext.pm
        'Locale::Script'        => '2.21',  #lib/Locale/Script.pm
        'Locale::Maketext::GutsLoader'=> undef,  #lib/Locale/Maketext/GutsLoader.pm
        'Locale::Maketext::Guts'=> undef,  #lib/Locale/Maketext/Guts.pm
        'Math::BigFloat'        => '1.44',  #lib/Math/BigFloat.pm
        'Math::BigInt'          => '1.70',  #lib/Math/BigInt.pm
        'Math::BigRat'          => '0.12',  #lib/Math/BigRat.pm
        'Math::Complex'         => '1.34',  #lib/Math/Complex.pm
        'Math::Trig'            => '1.02',  #lib/Math/Trig.pm
        'Math::BigFloat::Trace' => '0.01',  #lib/Math/BigFloat/Trace.pm
        'Math::BigInt::CalcEmu' => '0.04',  #lib/Math/BigInt/CalcEmu.pm
        'Math::BigInt::Calc'    => '0.40',  #lib/Math/BigInt/Calc.pm
        'Math::BigInt::Trace'   => '0.01',  #lib/Math/BigInt/Trace.pm
        'Memoize::AnyDBM_File'  => '0.65',  #lib/Memoize/AnyDBM_File.pm
        'Memoize::ExpireFile'   => '1.01',  #lib/Memoize/ExpireFile.pm
        'Memoize::Expire'       => '1.00',  #lib/Memoize/Expire.pm
        'Memoize::ExpireTest'   => '0.65',  #lib/Memoize/ExpireTest.pm
        'Memoize::NDBM_File'    => '0.65',  #lib/Memoize/NDBM_File.pm
        'Memoize::SDBM_File'    => '0.65',  #lib/Memoize/SDBM_File.pm
        'Memoize::Storable'     => '0.65',  #lib/Memoize/Storable.pm
        'Net::Cmd'              => '2.24',  #lib/Net/Cmd.pm
        'Net::Config'           => '1.10',  #lib/Net/Config.pm
        'Net::Domain'           => '2.19',  #lib/Net/Domain.pm
        'Net::FTP'              => '2.72',  #lib/Net/FTP.pm
        'Net::hostent'          => '1.01',  #lib/Net/hostent.pm
        'Net::netent'           => '1.00',  #lib/Net/netent.pm
        'Net::Netrc'            => '2.12',  #lib/Net/Netrc.pm
        'Net::NNTP'             => '2.22',  #lib/Net/NNTP.pm
        'Net::Ping'             => '2.31',  #lib/Net/Ping.pm
        'Net::POP3'             => '2.24',  #lib/Net/POP3.pm
        'Net::protoent'         => '1.00',  #lib/Net/protoent.pm
        'Net::servent'          => '1.01',  #lib/Net/servent.pm
        'Net::SMTP'             => '2.26',  #lib/Net/SMTP.pm
        'Net::Time'             => '2.09',  #lib/Net/Time.pm
        'Net::FTP::A'           => '1.16',  #lib/Net/FTP/A.pm
        'Net::FTP::dataconn'    => '0.11',  #lib/Net/FTP/dataconn.pm
        'Net::FTP::E'           => '0.01',  #lib/Net/FTP/E.pm
        'Net::FTP::I'           => '1.12',  #lib/Net/FTP/I.pm
        'Net::FTP::L'           => '0.01',  #lib/Net/FTP/L.pm
        'PerlIO::encoding'      => '0.07',  #lib/PerlIO/encoding.pm
        'PerlIO::scalar'        => '0.02',  #lib/PerlIO/scalar.pm
        'PerlIO::via'           => '0.02',  #lib/PerlIO/via.pm
        'PerlIO::via::QuotedPrint'=> '0.06',  #lib/PerlIO/via/QuotedPrint.pm
        'Pod::Checker'          => '1.41',  #lib/Pod/Checker.pm
        'Pod::Find'             => '0.24',  #lib/Pod/Find.pm
        'Pod::Functions'        => '1.02',  #lib/Pod/Functions.pm
        'Pod::Html'             => '1.0502',  #lib/Pod/Html.pm
        'Pod::InputObjects'     => '1.14',  #lib/Pod/InputObjects.pm
        'Pod::LaTeX'            => '0.55',  #lib/Pod/LaTeX.pm
        'Pod::Man'              => '1.37',  #lib/Pod/Man.pm
        'Pod::ParseLink'        => '1.06',  #lib/Pod/ParseLink.pm
        'Pod::Parser'           => '1.14',  #lib/Pod/Parser.pm
        'Pod::ParseUtils'       => '0.3',  #lib/Pod/ParseUtils.pm
        'Pod::Perldoc'          => '3.12',  #lib/Pod/Perldoc.pm
        'Pod::Plainer'          => '0.01',  #lib/Pod/Plainer.pm
        'Pod::PlainText'        => '2.02',  #lib/Pod/PlainText.pm
        'Pod::Select'           => '1.13',  #lib/Pod/Select.pm
        'Pod::Text'             => '2.21',  #lib/Pod/Text.pm
        'Pod::Usage'            => '1.16',  #lib/Pod/Usage.pm
        'Pod::Perldoc::BaseTo'  => undef,  #lib/Pod/Perldoc/BaseTo.pm
        'Pod::Perldoc::GetOptsOO'=> undef,  #lib/Pod/Perldoc/GetOptsOO.pm
        'Pod::Perldoc::ToChecker'=> undef,  #lib/Pod/Perldoc/ToChecker.pm
        'Pod::Perldoc::ToMan'   => undef,  #lib/Pod/Perldoc/ToMan.pm
        'Pod::Perldoc::ToNroff' => undef,  #lib/Pod/Perldoc/ToNroff.pm
        'Pod::Perldoc::ToPod'   => undef,  #lib/Pod/Perldoc/ToPod.pm
        'Pod::Perldoc::ToRtf'   => undef,  #lib/Pod/Perldoc/ToRtf.pm
        'Pod::Perldoc::ToText'  => undef,  #lib/Pod/Perldoc/ToText.pm
        'Pod::Perldoc::ToTk'    => 'undef',  #lib/Pod/Perldoc/ToTk.pm
        'Pod::Perldoc::ToXml'   => undef,  #lib/Pod/Perldoc/ToXml.pm
        'Pod::Text::Color'      => '1.04',  #lib/Pod/Text/Color.pm
        'Pod::Text::Overstrike' => '1.1',  #lib/Pod/Text/Overstrike.pm
        'Pod::Text::Termcap'    => '1.11',  #lib/Pod/Text/Termcap.pm
        'Search::Dict'          => '1.02',  #lib/Search/Dict.pm
        'Term::ANSIColor'       => '1.08',  #lib/Term/ANSIColor.pm
        'Term::Cap'             => '1.08',  #lib/Term/Cap.pm
        'Term::Complete'        => '1.401',  #lib/Term/Complete.pm
        'Term::ReadLine'        => '1.01',  #lib/Term/ReadLine.pm
        'Test::Builder'         => '0.17',  #lib/Test/Builder.pm
        'Test::Harness'         => '2.40',  #lib/Test/Harness.pm
        'Test::More'            => '0.47',  #lib/Test/More.pm
        'Test::Simple'          => '0.47',  #lib/Test/Simple.pm
        'Test::Harness::Assert' => '0.02',  #lib/Test/Harness/Assert.pm
        'Test::Harness::Iterator'=> '0.02',  #lib/Test/Harness/Iterator.pm
        'Test::Harness::Straps' => '0.19',  #lib/Test/Harness/Straps.pm
        'Text::Abbrev'          => '1.01',  #lib/Text/Abbrev.pm
        'Text::Balanced'        => '1.95',  #lib/Text/Balanced.pm
        'Text::ParseWords'      => '3.21',  #lib/Text/ParseWords.pm
        'Text::Soundex'         => '1.01',  #lib/Text/Soundex.pm
        'Text::Tabs'            => '98.112801',  #lib/Text/Tabs.pm
        'Text::Wrap'            => '2001.09291',  #lib/Text/Wrap.pm
        'Thread::Queue'         => '2.00',  #lib/Thread/Queue.pm
        'Thread::Semaphore'     => '2.01',  #lib/Thread/Semaphore.pm
        'Tie::Array'            => '1.03',  #lib/Tie/Array.pm
        'Tie::File'             => '0.97',  #lib/Tie/File.pm
        'Tie::Handle'           => '4.1',  #lib/Tie/Handle.pm
        'Tie::Hash'             => '1.01',  #lib/Tie/Hash.pm
        'Tie::Memoize'          => '1.0',  #lib/Tie/Memoize.pm
        'Tie::RefHash'          => '1.31',  #lib/Tie/RefHash.pm
        'Tie::Scalar'           => '1.00',  #lib/Tie/Scalar.pm
        'Tie::SubstrHash'       => '1.00',  #lib/Tie/SubstrHash.pm
        'Time::gmtime'          => '1.02',  #lib/Time/gmtime.pm
        'Time::Local'           => '1.07',  #lib/Time/Local.pm
        'Time::localtime'       => '1.02',  #lib/Time/localtime.pm
        'Time::tm'              => '1.00',  #lib/Time/tm.pm
        'Time::HiRes'           => '1.59',  #lib/Time/HiRes.pm
        'Unicode::Collate'      => '0.33',  #lib/Unicode/Collate.pm
        'Unicode::UCD'          => '0.22',  #lib/Unicode/UCD.pm
        'Unicode::Normalize'    => '0.28',  #lib/Unicode/Normalize.pm
        'User::grent'           => '1.00',  #lib/User/grent.pm
        'User::pwent'           => '1.00',  #lib/User/pwent.pm
        'warnings::register'    => '1.00',  #lib/warnings/register.pm
        'B::Stash'              => '1.00',  #lib/B/Stash.pm
        'B::Asmdata'            => '1.01',  #lib/B/Asmdata.pm
        'B::C'                  => '1.02',  #lib/B/C.pm
        'B::Deparse'            => '0.66',  #lib/B/Deparse.pm
        'B::Debug'              => '1.01',  #lib/B/Debug.pm
        'B::Bblock'             => '1.02',  #lib/B/Bblock.pm
        'B::Assembler'          => '0.07',  #lib/B/Assembler.pm
        'B::Terse'              => '1.02',  #lib/B/Terse.pm
        'B::CC'                 => '1.00',  #lib/B/CC.pm
        'B::Concise'            => '0.60',  #lib/B/Concise.pm
        'B::Lint'               => '1.02',  #lib/B/Lint.pm
        'B::Showlex'            => '1.00',  #lib/B/Showlex.pm
        'B::Bytecode'           => '1.01',  #lib/B/Bytecode.pm
        'B::Disassembler'       => '1.03',  #lib/B/Disassembler.pm
        'B::Xref'               => '1.01',  #lib/B/Xref.pm
        'B::Stackobj'           => '1.00',  #lib/B/Stackobj.pm
        'Data::Dumper'          => '2.121',  #lib/Data/Dumper.pm
        'Encode::Alias'         => '1.38',  #lib/Encode/Alias.pm
        'Encode::Encoding'      => '1.33',  #lib/Encode/Encoding.pm
        'Encode::Guess'         => '1.09',  #lib/Encode/Guess.pm
        'Encode::Config'        => '1.07',  #lib/Encode/Config.pm
        'Encode::Encoder'       => '0.07',  #lib/Encode/Encoder.pm
        'Encode::CJKConstants'  => '1.02',  #lib/Encode/CJKConstants.pm
        'Encode::Byte'          => '1.23',  #lib/Encode/Byte.pm
        'Encode::CN'            => '1.24',  #lib/Encode/CN.pm
        'Encode::EBCDIC'        => '1.21',  #lib/Encode/EBCDIC.pm
        'Encode::JP'            => '1.25',  #lib/Encode/JP.pm
        'Encode::KR'            => '1.23',  #lib/Encode/KR.pm
        'Encode::Symbol'        => '1.22',  #lib/Encode/Symbol.pm
        'Encode::TW'            => '1.26',  #lib/Encode/TW.pm
        'Encode::Unicode'       => '1.40',  #lib/Encode/Unicode.pm
        'Encode::JP::H2Z'       => '1.02',  #lib/Encode/JP/H2Z.pm
        'Encode::JP::JIS7'      => '1.12',  #lib/Encode/JP/JIS7.pm
        'Encode::Unicode::UTF7' => '0.02',  #lib/Encode/Unicode/UTF7.pm
        'Encode::KR::2022_KR'   => '1.06',  #lib/Encode/KR/2022_KR.pm
        'Encode::MIME::Header'  => '1.09',  #lib/Encode/MIME/Header.pm
        'Encode::CN::HZ'        => '1.0501',  #lib/Encode/CN/HZ.pm
        'IO::Pipe'              => '1.123',  #lib/IO/Pipe.pm
        'IO::File'              => '1.10',  #lib/IO/File.pm
        'IO::Select'            => '1.16',  #lib/IO/Select.pm
        'IO::Socket'            => '1.28',  #lib/IO/Socket.pm
        'IO::Poll'              => '0.06',  #lib/IO/Poll.pm
        'IO::Dir'               => '1.04',  #lib/IO/Dir.pm
        'IO::Handle'            => '1.24',  #lib/IO/Handle.pm
        'IO::Seekable'          => '1.09',  #lib/IO/Seekable.pm
        'IO::Socket::INET'      => '1.27',  #lib/IO/Socket/INET.pm
        'IO::Socket::UNIX'      => '1.21',  #lib/IO/Socket/UNIX.pm
        'List::Util'            => '1.13',  #lib/List/Util.pm
        'Scalar::Util'          => '1.13',  #lib/Scalar/Util.pm
        'MIME::QuotedPrint'     => '3.01',  #lib/MIME/QuotedPrint.pm
        'MIME::Base64'          => '3.01',  #lib/MIME/Base64.pm
        'Sys::Hostname'         => '1.11',  #lib/Sys/Hostname.pm
        'Sys::Syslog'           => '0.05',  #lib/Sys/Syslog.pm
        'XS::APItest'           => '0.03',  #lib/XS/APItest.pm
        'XS::Typemap'           => '0.01',  #lib/XS/Typemap.pm
        'threads::shared'       => '0.92',  #lib/threads/shared.pm
        'ExtUtils::XSSymSet'    => '1.0',  #vms/ext/XSSymSet.pm
        'JNI'                   => '0.2',  #jpl/JNI/JNI.pm
        'JPL::AutoLoader'       => undef,  #jpl/JPL/AutoLoader.pm
        'JPL::Class'            => undef,  #jpl/JPL/Class.pm
        'JPL::Compile'          => undef,  #jpl/JPL/Compile.pm
        'ODBM_File'             => '1.05',  #ext/ODBM_File/ODBM_File.pm
        'OS2::DLL'              => '1.02',  #os2/OS2/REXX/DLL/DLL.pm
        'OS2::ExtAttr'          => '0.02',  #os2/OS2/ExtAttr/ExtAttr.pm
        'OS2::PrfDB'            => '0.03',  #os2/OS2/PrfDB/PrfDB.pm
        'OS2::Process'          => '1.01',  #os2/OS2/Process/Process.pm
        'OS2::REXX'             => '1.02',  #os2/OS2/REXX/REXX.pm
        'Thread::Signal'        => '1.00', #./ext/Thread/Thread/Signal.pm
        'Thread::Specific'      => '1.00', #./ext/Thread/Thread/Specific.pm
        'VMS::DCLsym'           => '1.02',  #vms/ext/DCLsym/DCLsym.pm
        'VMS::Filespec'         => '1.11',  #vms/ext/Filespec.pm
        'VMS::Stdio'            => '2.3',  #vms/ext/Stdio/Stdio.pm
    },
    5.008005 => {

        'AnyDBM_File'           => '1.00',  #lib/AnyDBM_File.pm
        'attributes'            => '0.06',  #lib/attributes.pm
        'AutoLoader'            => '5.60',  #lib/AutoLoader.pm
        'AutoSplit'             => '1.04',  #lib/AutoSplit.pm
        'autouse'               => '1.04',  #lib/autouse.pm
        'base'                  => '2.06',  #lib/base.pm
        'Benchmark'             => '1.06',  #lib/Benchmark.pm
        'bigint'                => '0.05',  #lib/bigint.pm
        'bignum'                => '0.15',  #lib/bignum.pm
        'bigrat'                => '0.06',  #lib/bigrat.pm
        'blib'                  => '1.02',  #lib/blib.pm
        'bytes'                 => '1.01',  #lib/bytes.pm
        'Carp'                  => '1.03',  #lib/Carp.pm
        'CGI'                   => '3.05',  #lib/CGI.pm
        'charnames'             => '1.04',  #lib/charnames.pm
        'constant'              => '1.04',  #lib/constant.pm
        'CPAN'                  => '1.76_01',  #lib/CPAN.pm
        'Cwd'                   => '2.19',  #lib/Cwd.pm
        'DBM_Filter'            => '0.01',  #lib/DBM_Filter.pm
        'DB'                    => '1.0',  #lib/DB.pm
        'diagnostics'           => '1.13',  #lib/diagnostics.pm
        'Digest'                => '1.08',  #lib/Digest.pm
        'DirHandle'             => '1.00',  #lib/DirHandle.pm
        'Dumpvalue'             => '1.11',  #lib/Dumpvalue.pm
        'English'               => '1.01',  #lib/English.pm
        'Env'                   => '1.00',  #lib/Env.pm
        'Exporter'              => '5.58',  #lib/Exporter.pm
        'Fatal'                 => '1.03',  #lib/Fatal.pm
        'fields'                => '2.03',  #lib/fields.pm
        'FileCache'             => '1.04_01',  #lib/FileCache.pm
        'FileHandle'            => '2.01',  #lib/FileHandle.pm
        'filetest'              => '1.01',  #lib/filetest.pm
        'FindBin'               => '1.44',  #lib/FindBin.pm
        'if'                    => '0.03',  #lib/if.pm
        'integer'               => '1.00',  #lib/integer.pm
        'less'                  => '0.01',  #lib/less.pm
        'locale'                => '1.00',  #lib/locale.pm
        'Memoize'               => '1.01',  #lib/Memoize.pm
        'NEXT'                  => '0.60',  #lib/NEXT.pm
        'open'                  => '1.03',  #lib/open.pm
        'overload'              => '1.01',  #lib/overload.pm
        'PerlIO'                => '1.03',  #lib/PerlIO.pm
        'SelectSaver'           => '1.00',  #lib/SelectSaver.pm
        'SelfLoader'            => '1.0904',  #lib/SelfLoader.pm
        'Shell'                 => '0.6',  #lib/Shell.pm
        'sigtrap'               => '1.02',  #lib/sigtrap.pm
        'sort'                  => '1.02',  #lib/sort.pm
        'strict'                => '1.03',  #lib/strict.pm
        'subs'                  => '1.00',  #lib/subs.pm
        'Switch'                => '2.10',  #lib/Switch.pm
        'Symbol'                => '1.05',  #lib/Symbol.pm
        'Test'                  => '1.25',  #lib/Test.pm
        'Thread'                => '2.00',  #lib/Thread.pm
        'UNIVERSAL'             => '1.01',  #lib/UNIVERSAL.pm
        'utf8'                  => '1.04',  #lib/utf8.pm
        'vars'                  => '1.01',  #lib/vars.pm
        'vmsish'                => '1.01',  #lib/vmsish.pm
        'warnings'              => '1.03',  #lib/warnings.pm
        'Config'                => undef,  #lib/Config.pm
        'lib'                   => '0.5565',  #lib/lib.pm
        're'                    => '0.04',  #lib/re.pm
        'XSLoader'              => '0.02',  #lib/XSLoader.pm
        'DynaLoader'            => '1.05',  #lib/DynaLoader.pm
        'attrs'                 => '1.01',  #lib/attrs.pm
        'B'                     => '1.02',  #lib/B.pm
        'O'                     => '1.00',  #lib/O.pm
        'ByteLoader'            => '0.05',  #lib/ByteLoader.pm
        'DB_File'               => '1.809',  #lib/DB_File.pm
        'Encode'                => '2.01',  #lib/Encode.pm
        'encoding'              => '2.00',  #lib/encoding.pm
        'Fcntl'                 => '1.05',  #lib/Fcntl.pm
        'GDBM_File'             => '1.07',  #lib/GDBM_File.pm
        'IO'                    => '1.21',  #lib/IO.pm
        'NDBM_File'             => '1.05',  #lib/NDBM_File.pm
        'Safe'                  => '2.11',  #lib/Safe.pm
        'Opcode'                => '1.05',  #lib/Opcode.pm
        'ops'                   => '1.00',  #lib/ops.pm
        'POSIX'                 => '1.08',  #lib/POSIX.pm
        'SDBM_File'             => '1.04',  #lib/SDBM_File.pm
        'Socket'                => '1.77',  #lib/Socket.pm
        'Storable'              => '2.13',  #lib/Storable.pm
        'threads'               => '1.05',  #lib/threads.pm
        'Errno'                 => '1.09_00',  #lib/Errno.pm
        'Attribute::Handlers'   => '0.78_01',  #lib/Attribute/Handlers.pm
        'Carp::Heavy'           => '1.01',  #lib/Carp/Heavy.pm
        'CGI::Apache'           => '1.00',  #lib/CGI/Apache.pm
        'CGI::Carp'             => '1.28',  #lib/CGI/Carp.pm
        'CGI::Cookie'           => '1.24',  #lib/CGI/Cookie.pm
        'CGI::Fast'             => '1.05',  #lib/CGI/Fast.pm
        'CGI::Pretty'           => '1.08',  #lib/CGI/Pretty.pm
        'CGI::Push'             => '1.04',  #lib/CGI/Push.pm
        'CGI::Switch'           => '1.00',  #lib/CGI/Switch.pm
        'CGI::Util'             => '1.5',  #lib/CGI/Util.pm
        'Class::ISA'            => '0.32',  #lib/Class/ISA.pm
        'Class::Struct'         => '0.63',  #lib/Class/Struct.pm
        'CPAN::FirstTime'       => '1.60 ',  #lib/CPAN/FirstTime.pm
        'CPAN::Nox'             => '1.03',  #lib/CPAN/Nox.pm
        'DBM_Filter::compress'  => '0.01',  #lib/DBM_Filter/compress.pm
        'DBM_Filter::encode'    => '0.01',  #lib/DBM_Filter/encode.pm
        'DBM_Filter::int32'     => '0.01',  #lib/DBM_Filter/int32.pm
        'DBM_Filter::null'      => '0.01',  #lib/DBM_Filter/null.pm
        'DBM_Filter::utf8'      => '0.01',  #lib/DBM_Filter/utf8.pm
        'Devel::SelfStubber'    => '1.03',  #lib/Devel/SelfStubber.pm
        'Devel::DProf'          => '20030813.00',  #lib/Devel/DProf.pm
        'Devel::Peek'           => '1.01',  #lib/Devel/Peek.pm
        'Devel::PPPort'         => '2.011',  #lib/Devel/PPPort.pm
        'Digest::base'          => '1.00',  #lib/Digest/base.pm
        'Digest::MD5'           => '2.33',  #lib/Digest/MD5.pm
        'Exporter::Heavy'       => '5.57',  #lib/Exporter/Heavy.pm
        'ExtUtils::Command'     => '1.05',  #lib/ExtUtils/Command.pm
        'ExtUtils::Constant'    => '0.14',  #lib/ExtUtils/Constant.pm
        'ExtUtils::Embed'       => '1.250601',  #lib/ExtUtils/Embed.pm
        'ExtUtils::Installed'   => '0.08',  #lib/ExtUtils/Installed.pm
        'ExtUtils::Install'     => '1.32',  #lib/ExtUtils/Install.pm
        'ExtUtils::Liblist'     => '1.01',  #lib/ExtUtils/Liblist.pm
        'ExtUtils::MakeMaker'   => '6.17',  #lib/ExtUtils/MakeMaker.pm
        'ExtUtils::Manifest'    => '1.42',  #lib/ExtUtils/Manifest.pm
        'ExtUtils::Mkbootstrap' => '1.15',  #lib/ExtUtils/Mkbootstrap.pm
        'ExtUtils::Mksymlists'  => '1.19',  #lib/ExtUtils/Mksymlists.pm
        'ExtUtils::MM_Any'      => '0.07',  #lib/ExtUtils/MM_Any.pm
        'ExtUtils::MM_BeOS'     => '1.04',  #lib/ExtUtils/MM_BeOS.pm
        'ExtUtils::MM_Cygwin'   => '1.06',  #lib/ExtUtils/MM_Cygwin.pm
        'ExtUtils::MM_DOS'      => '0.02',  #lib/ExtUtils/MM_DOS.pm
        'ExtUtils::MM_MacOS'    => '1.07',  #lib/ExtUtils/MM_MacOS.pm
        'ExtUtils::MM_NW5'      => '2.07_02',  #lib/ExtUtils/MM_NW5.pm
        'ExtUtils::MM'          => '0.04',  #lib/ExtUtils/MM.pm
        'ExtUtils::MM_OS2'      => '1.04',  #lib/ExtUtils/MM_OS2.pm
        'ExtUtils::MM_Unix'     => '1.42',  #lib/ExtUtils/MM_Unix.pm
        'ExtUtils::MM_UWIN'     => '0.02',  #lib/ExtUtils/MM_UWIN.pm
        'ExtUtils::MM_VMS'      => '5.70',  #lib/ExtUtils/MM_VMS.pm
        'ExtUtils::MM_Win32'    => '1.09',  #lib/ExtUtils/MM_Win32.pm
        'ExtUtils::MM_Win95'    => '0.0301',  #lib/ExtUtils/MM_Win95.pm
        'ExtUtils::MY'          => '0.01',  #lib/ExtUtils/MY.pm
        'ExtUtils::Packlist'    => '0.04',  #lib/ExtUtils/Packlist.pm
        'ExtUtils::testlib'     => '1.15',  #lib/ExtUtils/testlib.pm
        'ExtUtils::Miniperl'    => undef,  #lib/ExtUtils/Miniperl.pm
        'ExtUtils::Command::MM' => '0.03',  #lib/ExtUtils/Command/MM.pm
        'ExtUtils::Liblist::Kid'=> '1.3001',  #lib/ExtUtils/Liblist/Kid.pm
        'ExtUtils::MakeMaker::bytes'=> '0.01',  #lib/ExtUtils/MakeMaker/bytes.pm
        'ExtUtils::MakeMaker::vmsish'=> '0.01',  #lib/ExtUtils/MakeMaker/vmsish.pm
        'File::Basename'        => '2.73',  #lib/File/Basename.pm
        'File::CheckTree'       => '4.3',  #lib/File/CheckTree.pm
        'File::Compare'         => '1.1003',  #lib/File/Compare.pm
        'File::Copy'            => '2.08',  #lib/File/Copy.pm
        'File::DosGlob'         => '1.00',  #lib/File/DosGlob.pm
        'File::Find'            => '1.07',  #lib/File/Find.pm
        'File::Path'            => '1.06',  #lib/File/Path.pm
        'File::Spec'            => '0.87',  #lib/File/Spec.pm
        'File::stat'            => '1.00',  #lib/File/stat.pm
        'File::Temp'            => '0.14',  #lib/File/Temp.pm
        'File::Glob'            => '1.03',  #lib/File/Glob.pm
        'File::Spec::Cygwin'    => '1.1',  #lib/File/Spec/Cygwin.pm
        'File::Spec::Epoc'      => '1.1',  #lib/File/Spec/Epoc.pm
        'File::Spec::Functions' => '1.3',  #lib/File/Spec/Functions.pm
        'File::Spec::Mac'       => '1.4',  #lib/File/Spec/Mac.pm
        'File::Spec::OS2'       => '1.2',  #lib/File/Spec/OS2.pm
        'File::Spec::Unix'      => '1.5',  #lib/File/Spec/Unix.pm
        'File::Spec::VMS'       => '1.4',  #lib/File/Spec/VMS.pm
        'File::Spec::Win32'     => '1.4',  #lib/File/Spec/Win32.pm
        'Filter::Simple'        => '0.78',  #lib/Filter/Simple.pm
        'Filter::Util::Call'    => '1.0601',  #lib/Filter/Util/Call.pm
        'Getopt::Long'          => '2.34',  #lib/Getopt/Long.pm
        'Getopt::Std'           => '1.05',  #lib/Getopt/Std.pm
        'Hash::Util'            => '0.05',  #lib/Hash/Util.pm
        'I18N::Collate'         => '1.00',  #lib/I18N/Collate.pm
        'I18N::LangTags'        => '0.33',  #lib/I18N/LangTags.pm
        'I18N::Langinfo'        => '0.02',  #lib/I18N/Langinfo.pm
        'I18N::LangTags::Detect'=> '1.03',  #lib/I18N/LangTags/Detect.pm
        'I18N::LangTags::List'  => '0.29',  #lib/I18N/LangTags/List.pm
        'IPC::Open2'            => '1.01',  #lib/IPC/Open2.pm
        'IPC::Open3'            => '1.0106',  #lib/IPC/Open3.pm
        'IPC::Msg'              => '1.02',  #lib/IPC/Msg.pm
        'IPC::Semaphore'        => '1.02',  #lib/IPC/Semaphore.pm
        'IPC::SysV'             => '1.04',  #lib/IPC/SysV.pm
        'Locale::Constants'     => '2.07',  #lib/Locale/Constants.pm
        'Locale::Country'       => '2.07',  #lib/Locale/Country.pm
        'Locale::Currency'      => '2.07',  #lib/Locale/Currency.pm
        'Locale::Language'      => '2.07',  #lib/Locale/Language.pm
        'Locale::Maketext'      => '1.09',  #lib/Locale/Maketext.pm
        'Locale::Script'        => '2.07',  #lib/Locale/Script.pm
        'Locale::Maketext::GutsLoader'=> undef,  #lib/Locale/Maketext/GutsLoader.pm
        'Locale::Maketext::Guts'=> undef,  #lib/Locale/Maketext/Guts.pm
        'Math::BigFloat'        => '1.44',  #lib/Math/BigFloat.pm
        'Math::BigInt'          => '1.70',  #lib/Math/BigInt.pm
        'Math::BigRat'          => '0.12',  #lib/Math/BigRat.pm
        'Math::Complex'         => '1.34',  #lib/Math/Complex.pm
        'Math::Trig'            => '1.02',  #lib/Math/Trig.pm
        'Math::BigFloat::Trace' => '0.01',  #lib/Math/BigFloat/Trace.pm
        'Math::BigInt::CalcEmu' => '0.04',  #lib/Math/BigInt/CalcEmu.pm
        'Math::BigInt::Calc'    => '0.40',  #lib/Math/BigInt/Calc.pm
        'Math::BigInt::Trace'   => '0.01',  #lib/Math/BigInt/Trace.pm
        'Memoize::AnyDBM_File'  => '0.65',  #lib/Memoize/AnyDBM_File.pm
        'Memoize::ExpireFile'   => '1.01',  #lib/Memoize/ExpireFile.pm
        'Memoize::Expire'       => '1.00',  #lib/Memoize/Expire.pm
        'Memoize::ExpireTest'   => '0.65',  #lib/Memoize/ExpireTest.pm
        'Memoize::NDBM_File'    => '0.65',  #lib/Memoize/NDBM_File.pm
        'Memoize::SDBM_File'    => '0.65',  #lib/Memoize/SDBM_File.pm
        'Memoize::Storable'     => '0.65',  #lib/Memoize/Storable.pm
        'Net::Cmd'              => '2.26',  #lib/Net/Cmd.pm
        'Net::Config'           => '1.10',  #lib/Net/Config.pm
        'Net::Domain'           => '2.19',  #lib/Net/Domain.pm
        'Net::FTP'              => '2.75',  #lib/Net/FTP.pm
        'Net::hostent'          => '1.01',  #lib/Net/hostent.pm
        'Net::netent'           => '1.00',  #lib/Net/netent.pm
        'Net::Netrc'            => '2.12',  #lib/Net/Netrc.pm
        'Net::NNTP'             => '2.23',  #lib/Net/NNTP.pm
        'Net::Ping'             => '2.31',  #lib/Net/Ping.pm
        'Net::POP3'             => '2.28',  #lib/Net/POP3.pm
        'Net::protoent'         => '1.00',  #lib/Net/protoent.pm
        'Net::servent'          => '1.01',  #lib/Net/servent.pm
        'Net::SMTP'             => '2.29',  #lib/Net/SMTP.pm
        'Net::Time'             => '2.10',  #lib/Net/Time.pm
        'Net::FTP::A'           => '1.16',  #lib/Net/FTP/A.pm
        'Net::FTP::dataconn'    => '0.11',  #lib/Net/FTP/dataconn.pm
        'Net::FTP::E'           => '0.01',  #lib/Net/FTP/E.pm
        'Net::FTP::I'           => '1.12',  #lib/Net/FTP/I.pm
        'Net::FTP::L'           => '0.01',  #lib/Net/FTP/L.pm
        'PerlIO::encoding'      => '0.07',  #lib/PerlIO/encoding.pm
        'PerlIO::scalar'        => '0.02',  #lib/PerlIO/scalar.pm
        'PerlIO::via'           => '0.02',  #lib/PerlIO/via.pm
        'PerlIO::via::QuotedPrint'=> '0.06',  #lib/PerlIO/via/QuotedPrint.pm
        'Pod::Checker'          => '1.42',  #lib/Pod/Checker.pm
        'Pod::Find'             => '0.2401',  #lib/Pod/Find.pm
        'Pod::Functions'        => '1.02',  #lib/Pod/Functions.pm
        'Pod::Html'             => '1.0502',  #lib/Pod/Html.pm
        'Pod::InputObjects'     => '1.14',  #lib/Pod/InputObjects.pm
        'Pod::LaTeX'            => '0.56',  #lib/Pod/LaTeX.pm
        'Pod::Man'              => '1.37',  #lib/Pod/Man.pm
        'Pod::ParseLink'        => '1.06',  #lib/Pod/ParseLink.pm
        'Pod::Parser'           => '1.14',  #lib/Pod/Parser.pm
        'Pod::ParseUtils'       => '1.2',  #lib/Pod/ParseUtils.pm
        'Pod::Perldoc'          => '3.13',  #lib/Pod/Perldoc.pm
        'Pod::Plainer'          => '0.01',  #lib/Pod/Plainer.pm
        'Pod::PlainText'        => '2.02',  #lib/Pod/PlainText.pm
        'Pod::Select'           => '1.13',  #lib/Pod/Select.pm
        'Pod::Text'             => '2.21',  #lib/Pod/Text.pm
        'Pod::Usage'            => '1.16',  #lib/Pod/Usage.pm
        'Pod::Perldoc::BaseTo'  => undef,  #lib/Pod/Perldoc/BaseTo.pm
        'Pod::Perldoc::GetOptsOO'=> undef,  #lib/Pod/Perldoc/GetOptsOO.pm
        'Pod::Perldoc::ToChecker'=> undef,  #lib/Pod/Perldoc/ToChecker.pm
        'Pod::Perldoc::ToMan'   => undef,  #lib/Pod/Perldoc/ToMan.pm
        'Pod::Perldoc::ToNroff' => undef,  #lib/Pod/Perldoc/ToNroff.pm
        'Pod::Perldoc::ToPod'   => undef,  #lib/Pod/Perldoc/ToPod.pm
        'Pod::Perldoc::ToRtf'   => undef,  #lib/Pod/Perldoc/ToRtf.pm
        'Pod::Perldoc::ToText'  => undef,  #lib/Pod/Perldoc/ToText.pm
        'Pod::Perldoc::ToTk'    => 'undef',  #lib/Pod/Perldoc/ToTk.pm
        'Pod::Perldoc::ToXml'   => undef,  #lib/Pod/Perldoc/ToXml.pm
        'Pod::Text::Color'      => '1.04',  #lib/Pod/Text/Color.pm
        'Pod::Text::Overstrike' => '1.1',  #lib/Pod/Text/Overstrike.pm
        'Pod::Text::Termcap'    => '1.11',  #lib/Pod/Text/Termcap.pm
        'Search::Dict'          => '1.02',  #lib/Search/Dict.pm
        'Term::ANSIColor'       => '1.08',  #lib/Term/ANSIColor.pm
        'Term::Cap'             => '1.09',  #lib/Term/Cap.pm
        'Term::Complete'        => '1.401',  #lib/Term/Complete.pm
        'Term::ReadLine'        => '1.01',  #lib/Term/ReadLine.pm
        'Test::Builder'         => '0.17',  #lib/Test/Builder.pm
        'Test::Harness'         => '2.42',  #lib/Test/Harness.pm
        'Test::More'            => '0.47',  #lib/Test/More.pm
        'Test::Simple'          => '0.47',  #lib/Test/Simple.pm
        'Test::Harness::Assert' => '0.02',  #lib/Test/Harness/Assert.pm
        'Test::Harness::Iterator'=> '0.02',  #lib/Test/Harness/Iterator.pm
        'Test::Harness::Straps' => '0.19',  #lib/Test/Harness/Straps.pm
        'Text::Abbrev'          => '1.01',  #lib/Text/Abbrev.pm
        'Text::Balanced'        => '1.95',  #lib/Text/Balanced.pm
        'Text::ParseWords'      => '3.22',  #lib/Text/ParseWords.pm
        'Text::Soundex'         => '1.01',  #lib/Text/Soundex.pm
        'Text::Tabs'            => '98.112801',  #lib/Text/Tabs.pm
        'Text::Wrap'            => '2001.09292',  #lib/Text/Wrap.pm
        'Thread::Queue'         => '2.00',  #lib/Thread/Queue.pm
        'Thread::Semaphore'     => '2.01',  #lib/Thread/Semaphore.pm
        'Tie::Array'            => '1.03',  #lib/Tie/Array.pm
        'Tie::File'             => '0.97',  #lib/Tie/File.pm
        'Tie::Handle'           => '4.1',  #lib/Tie/Handle.pm
        'Tie::Hash'             => '1.01',  #lib/Tie/Hash.pm
        'Tie::Memoize'          => '1.0',  #lib/Tie/Memoize.pm
        'Tie::RefHash'          => '1.31',  #lib/Tie/RefHash.pm
        'Tie::Scalar'           => '1.00',  #lib/Tie/Scalar.pm
        'Tie::SubstrHash'       => '1.00',  #lib/Tie/SubstrHash.pm
        'Time::gmtime'          => '1.02',  #lib/Time/gmtime.pm
        'Time::Local'           => '1.10',  #lib/Time/Local.pm
        'Time::localtime'       => '1.02',  #lib/Time/localtime.pm
        'Time::tm'              => '1.00',  #lib/Time/tm.pm
        'Time::HiRes'           => '1.59',  #lib/Time/HiRes.pm
        'Unicode::Collate'      => '0.40',  #lib/Unicode/Collate.pm
        'Unicode::UCD'          => '0.22',  #lib/Unicode/UCD.pm
        'Unicode::Normalize'    => '0.30',  #lib/Unicode/Normalize.pm
        'User::grent'           => '1.00',  #lib/User/grent.pm
        'User::pwent'           => '1.00',  #lib/User/pwent.pm
        'warnings::register'    => '1.00',  #lib/warnings/register.pm
        'B::Stash'              => '1.00',  #lib/B/Stash.pm
        'B::Asmdata'            => '1.01',  #lib/B/Asmdata.pm
        'B::C'                  => '1.02',  #lib/B/C.pm
        'B::Deparse'            => '0.67',  #lib/B/Deparse.pm
        'B::Debug'              => '1.01',  #lib/B/Debug.pm
        'B::Bblock'             => '1.02',  #lib/B/Bblock.pm
        'B::Assembler'          => '0.07',  #lib/B/Assembler.pm
        'B::Terse'              => '1.02',  #lib/B/Terse.pm
        'B::CC'                 => '1.00',  #lib/B/CC.pm
        'B::Concise'            => '0.61',  #lib/B/Concise.pm
        'B::Lint'               => '1.02',  #lib/B/Lint.pm
        'B::Showlex'            => '1.00',  #lib/B/Showlex.pm
        'B::Bytecode'           => '1.01',  #lib/B/Bytecode.pm
        'B::Disassembler'       => '1.03',  #lib/B/Disassembler.pm
        'B::Xref'               => '1.01',  #lib/B/Xref.pm
        'B::Stackobj'           => '1.00',  #lib/B/Stackobj.pm
        'Data::Dumper'          => '2.121',  #lib/Data/Dumper.pm
        'Encode::Alias'         => '2.00',  #lib/Encode/Alias.pm
        'Encode::Encoding'      => '2.00',  #lib/Encode/Encoding.pm
        'Encode::Guess'         => '2.00',  #lib/Encode/Guess.pm
        'Encode::Config'        => '2.00',  #lib/Encode/Config.pm
        'Encode::Encoder'       => '2.00',  #lib/Encode/Encoder.pm
        'Encode::CJKConstants'  => '2.00',  #lib/Encode/CJKConstants.pm
        'Encode::Byte'          => '2.00',  #lib/Encode/Byte.pm
        'Encode::CN'            => '2.00',  #lib/Encode/CN.pm
        'Encode::EBCDIC'        => '2.00',  #lib/Encode/EBCDIC.pm
        'Encode::JP'            => '2.00',  #lib/Encode/JP.pm
        'Encode::KR'            => '2.00',  #lib/Encode/KR.pm
        'Encode::Symbol'        => '2.00',  #lib/Encode/Symbol.pm
        'Encode::TW'            => '2.00',  #lib/Encode/TW.pm
        'Encode::Unicode'       => '2.00',  #lib/Encode/Unicode.pm
        'Encode::JP::H2Z'       => '2.00',  #lib/Encode/JP/H2Z.pm
        'Encode::JP::JIS7'      => '2.00',  #lib/Encode/JP/JIS7.pm
        'Encode::Unicode::UTF7' => '2.01',  #lib/Encode/Unicode/UTF7.pm
        'Encode::KR::2022_KR'   => '2.00',  #lib/Encode/KR/2022_KR.pm
        'Encode::MIME::Header'  => '2.00',  #lib/Encode/MIME/Header.pm
        'Encode::CN::HZ'        => '2.01',  #lib/Encode/CN/HZ.pm
        'IO::Pipe'              => '1.123',  #lib/IO/Pipe.pm
        'IO::File'              => '1.10',  #lib/IO/File.pm
        'IO::Select'            => '1.16',  #lib/IO/Select.pm
        'IO::Socket'            => '1.28',  #lib/IO/Socket.pm
        'IO::Poll'              => '0.06',  #lib/IO/Poll.pm
        'IO::Dir'               => '1.04',  #lib/IO/Dir.pm
        'IO::Handle'            => '1.24',  #lib/IO/Handle.pm
        'IO::Seekable'          => '1.09',  #lib/IO/Seekable.pm
        'IO::Socket::INET'      => '1.27',  #lib/IO/Socket/INET.pm
        'IO::Socket::UNIX'      => '1.21',  #lib/IO/Socket/UNIX.pm
        'List::Util'            => '1.14',  #lib/List/Util.pm
        'Scalar::Util'          => '1.14',  #lib/Scalar/Util.pm
        'MIME::QuotedPrint'     => '3.01',  #lib/MIME/QuotedPrint.pm
        'MIME::Base64'          => '3.01',  #lib/MIME/Base64.pm
        'Sys::Hostname'         => '1.11',  #lib/Sys/Hostname.pm
        'Sys::Syslog'           => '0.05',  #lib/Sys/Syslog.pm
        'XS::APItest'           => '0.04',  #lib/XS/APItest.pm
        'XS::Typemap'           => '0.01',  #lib/XS/Typemap.pm
        'threads::shared'       => '0.92',  #lib/threads/shared.pm
        'ExtUtils::XSSymSet'    => '1.0',  #vms/ext/XSSymSet.pm
        'JNI'                   => '0.2',  #jpl/JNI/JNI.pm
        'JPL::AutoLoader'       => undef,  #jpl/JPL/AutoLoader.pm
        'JPL::Class'            => undef,  #jpl/JPL/Class.pm
        'JPL::Compile'          => undef,  #jpl/JPL/Compile.pm
        'ODBM_File'             => '1.05',  #ext/ODBM_File/ODBM_File.pm
        'OS2::DLL'              => '1.02',  #os2/OS2/REXX/DLL/DLL.pm
        'OS2::ExtAttr'          => '0.02',  #os2/OS2/ExtAttr/ExtAttr.pm
        'OS2::PrfDB'            => '0.03',  #os2/OS2/PrfDB/PrfDB.pm
        'OS2::Process'          => '1.01',  #os2/OS2/Process/Process.pm
        'OS2::REXX'             => '1.02',  #os2/OS2/REXX/REXX.pm
        'Thread::Signal'        => '1.00', #./ext/Thread/Thread/Signal.pm
        'Thread::Specific'      => '1.00', #./ext/Thread/Thread/Specific.pm
        'VMS::DCLsym'           => '1.02',  #vms/ext/DCLsym/DCLsym.pm
        'VMS::Filespec'         => '1.11',  #vms/ext/Filespec.pm
        'VMS::Stdio'            => '2.3',  #vms/ext/Stdio/Stdio.pm
    },
    5.008006 => {
        'AnyDBM_File'           => '1.00',  #lib/AnyDBM_File.pm
        'Attribute::Handlers'   => '0.78_01',  #lib/Attribute/Handlers.pm
        'attributes'            => '0.06',  #lib/attributes.pm
        'attrs'                 => '1.01',  #lib/attrs.pm
        'AutoLoader'            => '5.60',  #lib/AutoLoader.pm
        'AutoSplit'             => '1.04',  #lib/AutoSplit.pm
        'autouse'               => '1.04',  #lib/autouse.pm
        'B'                     => '1.07',  #lib/B.pm
        'base'                  => '2.06',  #lib/base.pm
        'B::Asmdata'            => '1.01',  #lib/B/Asmdata.pm
        'B::Assembler'          => '0.07',  #lib/B/Assembler.pm
        'B::Bblock'             => '1.02',  #lib/B/Bblock.pm
        'B::Bytecode'           => '1.01',  #lib/B/Bytecode.pm
        'B::C'                  => '1.04',  #lib/B/C.pm
        'B::CC'                 => '1.00',  #lib/B/CC.pm
        'B::Concise'            => '0.64',  #lib/B/Concise.pm
        'B::Debug'              => '1.02',  #lib/B/Debug.pm
        'B::Deparse'            => '0.69',  #lib/B/Deparse.pm
        'B::Disassembler'       => '1.03',  #lib/B/Disassembler.pm
        'Benchmark'             => '1.06',  #lib/Benchmark.pm
        'bigint'                => '0.05',  #lib/bigint.pm
        'bignum'                => '0.15',  #lib/bignum.pm
        'bigrat'                => '0.06',  #lib/bigrat.pm
        'blib'                  => '1.02',  #lib/blib.pm
        'B::Lint'               => '1.03',  #lib/B/Lint.pm
        'B::Showlex'            => '1.02',  #lib/B/Showlex.pm
        'B::Stackobj'           => '1.00',  #lib/B/Stackobj.pm
        'B::Stash'              => '1.00',  #lib/B/Stash.pm
        'B::Terse'              => '1.02',  #lib/B/Terse.pm
        'B::Xref'               => '1.01',  #lib/B/Xref.pm
        'ByteLoader'            => '0.05',  #lib/ByteLoader.pm
        'bytes'                 => '1.01',  #lib/bytes.pm
        'Carp'                  => '1.03',  #lib/Carp.pm
        'Carp::Heavy'           => '1.03',  #lib/Carp/Heavy.pm
        'CGI'                   => '3.05',  #lib/CGI.pm
        'CGI::Apache'           => '1.00',  #lib/CGI/Apache.pm
        'CGI::Carp'             => '1.28',  #lib/CGI/Carp.pm
        'CGI::Cookie'           => '1.24',  #lib/CGI/Cookie.pm
        'CGI::Fast'             => '1.05',  #lib/CGI/Fast.pm
        'CGI::Pretty'           => '1.08',  #lib/CGI/Pretty.pm
        'CGI::Push'             => '1.04',  #lib/CGI/Push.pm
        'CGI::Switch'           => '1.00',  #lib/CGI/Switch.pm
        'CGI::Util'             => '1.5',  #lib/CGI/Util.pm
        'charnames'             => '1.04',  #lib/charnames.pm
        'Class::ISA'            => '0.32',  #lib/Class/ISA.pm
        'Class::Struct'         => '0.63',  #lib/Class/Struct.pm
        'Config'                => undef,  #lib/Config.pm
        'constant'              => '1.04',  #lib/constant.pm
        'CPAN'                  => '1.76_01',  #lib/CPAN.pm
        'CPAN::FirstTime'       => '1.60 ',  #lib/CPAN/FirstTime.pm
        'CPAN::Nox'             => '1.03',  #lib/CPAN/Nox.pm
        'Cwd'                   => '3.01',  #lib/Cwd.pm
        'Data::Dumper'          => '2.121_02',  #lib/Data/Dumper.pm
        'DB'                    => '1.0',  #lib/DB.pm
        'DB_File'               => '1.810',  #lib/DB_File.pm
        'DBM_Filter'            => '0.01',  #lib/DBM_Filter.pm
        'DBM_Filter::compress'  => '0.01',  #lib/DBM_Filter/compress.pm
        'DBM_Filter::encode'    => '0.01',  #lib/DBM_Filter/encode.pm
        'DBM_Filter::int32'     => '0.01',  #lib/DBM_Filter/int32.pm
        'DBM_Filter::null'      => '0.01',  #lib/DBM_Filter/null.pm
        'DBM_Filter::utf8'      => '0.01',  #lib/DBM_Filter/utf8.pm
        'Devel::DProf'          => '20030813.00',  #lib/Devel/DProf.pm
        'Devel::Peek'           => '1.02',  #lib/Devel/Peek.pm
        'Devel::PPPort'         => '3.03',  #lib/Devel/PPPort.pm
        'Devel::SelfStubber'    => '1.03',  #lib/Devel/SelfStubber.pm
        'diagnostics'           => '1.14',  #lib/diagnostics.pm
        'Digest'                => '1.08',  #lib/Digest.pm
        'Digest::base'          => '1.00',  #lib/Digest/base.pm
        'Digest::MD5'           => '2.33',  #lib/Digest/MD5.pm
        'DirHandle'             => '1.00',  #lib/DirHandle.pm
        'Dumpvalue'             => '1.11',  #lib/Dumpvalue.pm
        'DynaLoader'            => '1.05',  #lib/DynaLoader.pm
        'Encode'                => '2.08',  #lib/Encode.pm
        'Encode::Alias'         => '2.02',  #lib/Encode/Alias.pm
        'Encode::Byte'          => '2.00',  #lib/Encode/Byte.pm
        'Encode::CJKConstants'  => '2.00',  #lib/Encode/CJKConstants.pm
        'Encode::CN'            => '2.00',  #lib/Encode/CN.pm
        'Encode::CN::HZ'        => '2.01',  #lib/Encode/CN/HZ.pm
        'Encode::Config'        => '2.00',  #lib/Encode/Config.pm
        'Encode::EBCDIC'        => '2.00',  #lib/Encode/EBCDIC.pm
        'Encode::Encoder'       => '2.00',  #lib/Encode/Encoder.pm
        'Encode::Encoding'      => '2.02',  #lib/Encode/Encoding.pm
        'Encode::Guess'         => '2.00',  #lib/Encode/Guess.pm
        'Encode::JP'            => '2.01',  #lib/Encode/JP.pm
        'Encode::JP::H2Z'       => '2.00',  #lib/Encode/JP/H2Z.pm
        'Encode::JP::JIS7'      => '2.00',  #lib/Encode/JP/JIS7.pm
        'Encode::KR'            => '2.00',  #lib/Encode/KR.pm
        'Encode::KR::2022_KR'   => '2.00',  #lib/Encode/KR/2022_KR.pm
        'Encode::MIME::Header'  => '2.00',  #lib/Encode/MIME/Header.pm
        'Encode::Symbol'        => '2.00',  #lib/Encode/Symbol.pm
        'Encode::TW'            => '2.00',  #lib/Encode/TW.pm
        'Encode::Unicode'       => '2.02',  #lib/Encode/Unicode.pm
        'Encode::Unicode::UTF7' => '2.01',  #lib/Encode/Unicode/UTF7.pm
        'encoding'              => '2.01',  #lib/encoding.pm
        'English'               => '1.01',  #lib/English.pm
        'Env'                   => '1.00',  #lib/Env.pm
        'Errno'                 => '1.09_00',  #lib/Errno.pm
        'Exporter'              => '5.58',  #lib/Exporter.pm
        'Exporter::Heavy'       => '5.58',  #lib/Exporter/Heavy.pm
        'ExtUtils::Command'     => '1.05',  #lib/ExtUtils/Command.pm
        'ExtUtils::Command::MM' => '0.03',  #lib/ExtUtils/Command/MM.pm
        'ExtUtils::Constant'    => '0.1401',  #lib/ExtUtils/Constant.pm
        'ExtUtils::Embed'       => '1.250601',  #lib/ExtUtils/Embed.pm
        'ExtUtils::Install'     => '1.32',  #lib/ExtUtils/Install.pm
        'ExtUtils::Installed'   => '0.08',  #lib/ExtUtils/Installed.pm
        'ExtUtils::Liblist'     => '1.01',  #lib/ExtUtils/Liblist.pm
        'ExtUtils::Liblist::Kid'=> '1.3001',  #lib/ExtUtils/Liblist/Kid.pm
        'ExtUtils::MakeMaker'   => '6.17',  #lib/ExtUtils/MakeMaker.pm
        'ExtUtils::MakeMaker::bytes'=> '0.01',  #lib/ExtUtils/MakeMaker/bytes.pm
        'ExtUtils::MakeMaker::vmsish'=> '0.01',  #lib/ExtUtils/MakeMaker/vmsish.pm
        'ExtUtils::Manifest'    => '1.42',  #lib/ExtUtils/Manifest.pm
        'ExtUtils::Miniperl'    => undef,  #lib/ExtUtils/Miniperl.pm
        'ExtUtils::Mkbootstrap' => '1.15',  #lib/ExtUtils/Mkbootstrap.pm
        'ExtUtils::Mksymlists'  => '1.19',  #lib/ExtUtils/Mksymlists.pm
        'ExtUtils::MM'          => '0.04',  #lib/ExtUtils/MM.pm
        'ExtUtils::MM_Any'      => '0.07',  #lib/ExtUtils/MM_Any.pm
        'ExtUtils::MM_BeOS'     => '1.04',  #lib/ExtUtils/MM_BeOS.pm
        'ExtUtils::MM_Cygwin'   => '1.06',  #lib/ExtUtils/MM_Cygwin.pm
        'ExtUtils::MM_DOS'      => '0.02',  #lib/ExtUtils/MM_DOS.pm
        'ExtUtils::MM_MacOS'    => '1.07',  #lib/ExtUtils/MM_MacOS.pm
        'ExtUtils::MM_NW5'      => '2.07_02',  #lib/ExtUtils/MM_NW5.pm
        'ExtUtils::MM_OS2'      => '1.04',  #lib/ExtUtils/MM_OS2.pm
        'ExtUtils::MM_Unix'     => '1.42',  #lib/ExtUtils/MM_Unix.pm
        'ExtUtils::MM_UWIN'     => '0.02',  #lib/ExtUtils/MM_UWIN.pm
        'ExtUtils::MM_VMS'      => '5.70',  #lib/ExtUtils/MM_VMS.pm
        'ExtUtils::MM_Win32'    => '1.09',  #lib/ExtUtils/MM_Win32.pm
        'ExtUtils::MM_Win95'    => '0.0301',  #lib/ExtUtils/MM_Win95.pm
        'ExtUtils::MY'          => '0.01',  #lib/ExtUtils/MY.pm
        'ExtUtils::Packlist'    => '0.04',  #lib/ExtUtils/Packlist.pm
        'ExtUtils::testlib'     => '1.15',  #lib/ExtUtils/testlib.pm
        'ExtUtils::XSSymSet'    => '1.0',  #vms/ext/XSSymSet.pm
        'Fatal'                 => '1.03',  #lib/Fatal.pm
        'Fcntl'                 => '1.05',  #lib/Fcntl.pm
        'fields'                => '2.03',  #lib/fields.pm
        'File::Basename'        => '2.73',  #lib/File/Basename.pm
        'FileCache'             => '1.04_01',  #lib/FileCache.pm
        'File::CheckTree'       => '4.3',  #lib/File/CheckTree.pm
        'File::Compare'         => '1.1003',  #lib/File/Compare.pm
        'File::Copy'            => '2.08',  #lib/File/Copy.pm
        'File::DosGlob'         => '1.00',  #lib/File/DosGlob.pm
        'File::Find'            => '1.07',  #lib/File/Find.pm
        'File::Glob'            => '1.03',  #lib/File/Glob.pm
        'FileHandle'            => '2.01',  #lib/FileHandle.pm
        'File::Path'            => '1.06',  #lib/File/Path.pm
        'File::Spec'            => '3.01',  #lib/File/Spec.pm
        'File::Spec::Cygwin'    => '1.1',  #lib/File/Spec/Cygwin.pm
        'File::Spec::Epoc'      => '1.1',  #lib/File/Spec/Epoc.pm
        'File::Spec::Functions' => '1.3',  #lib/File/Spec/Functions.pm
        'File::Spec::Mac'       => '1.4',  #lib/File/Spec/Mac.pm
        'File::Spec::OS2'       => '1.2',  #lib/File/Spec/OS2.pm
        'File::Spec::Unix'      => '1.5',  #lib/File/Spec/Unix.pm
        'File::Spec::VMS'       => '1.4',  #lib/File/Spec/VMS.pm
        'File::Spec::Win32'     => '1.5',  #lib/File/Spec/Win32.pm
        'File::stat'            => '1.00',  #lib/File/stat.pm
        'File::Temp'            => '0.14',  #lib/File/Temp.pm
        'filetest'              => '1.01',  #lib/filetest.pm
        'Filter::Simple'        => '0.78',  #lib/Filter/Simple.pm
        'Filter::Util::Call'    => '1.0601',  #lib/Filter/Util/Call.pm
        'FindBin'               => '1.44',  #lib/FindBin.pm
        'GDBM_File'             => '1.07',  #lib/GDBM_File.pm
        'Getopt::Long'          => '2.34',  #lib/Getopt/Long.pm
        'Getopt::Std'           => '1.05',  #lib/Getopt/Std.pm
        'Hash::Util'            => '0.05',  #lib/Hash/Util.pm
        'I18N::Collate'         => '1.00',  #lib/I18N/Collate.pm
        'I18N::Langinfo'        => '0.02',  #lib/I18N/Langinfo.pm
        'I18N::LangTags'        => '0.35',  #lib/I18N/LangTags.pm
        'I18N::LangTags::Detect'=> '1.03',  #lib/I18N/LangTags/Detect.pm
        'I18N::LangTags::List'  => '0.35',  #lib/I18N/LangTags/List.pm
        'if'                    => '0.03',  #lib/if.pm
        'integer'               => '1.00',  #lib/integer.pm
        'IO'                    => '1.21',  #lib/IO.pm
        'IO::Dir'               => '1.04',  #lib/IO/Dir.pm
        'IO::File'              => '1.10',  #lib/IO/File.pm
        'IO::Handle'            => '1.24',  #lib/IO/Handle.pm
        'IO::Pipe'              => '1.123',  #lib/IO/Pipe.pm
        'IO::Poll'              => '0.06',  #lib/IO/Poll.pm
        'IO::Seekable'          => '1.09',  #lib/IO/Seekable.pm
        'IO::Select'            => '1.16',  #lib/IO/Select.pm
        'IO::Socket'            => '1.28',  #lib/IO/Socket.pm
        'IO::Socket::INET'      => '1.27',  #lib/IO/Socket/INET.pm
        'IO::Socket::UNIX'      => '1.21',  #lib/IO/Socket/UNIX.pm
        'IPC::Msg'              => '1.02',  #lib/IPC/Msg.pm
        'IPC::Open2'            => '1.01',  #lib/IPC/Open2.pm
        'IPC::Open3'            => '1.0106',  #lib/IPC/Open3.pm
        'IPC::Semaphore'        => '1.02',  #lib/IPC/Semaphore.pm
        'IPC::SysV'             => '1.04',  #lib/IPC/SysV.pm
        'JNI'                   => '0.2',  #jpl/JNI/JNI.pm
        'JPL::AutoLoader'       => undef,  #jpl/JPL/AutoLoader.pm
        'JPL::Class'            => undef,  #jpl/JPL/Class.pm
        'JPL::Compile'          => undef,  #jpl/JPL/Compile.pm
        'less'                  => '0.01',  #lib/less.pm
        'lib'                   => '0.5565',  #lib/lib.pm
        'List::Util'            => '1.14',  #lib/List/Util.pm
        'locale'                => '1.00',  #lib/locale.pm
        'Locale::Constants'     => '2.07',  #lib/Locale/Constants.pm
        'Locale::Country'       => '2.07',  #lib/Locale/Country.pm
        'Locale::Currency'      => '2.07',  #lib/Locale/Currency.pm
        'Locale::Language'      => '2.07',  #lib/Locale/Language.pm
        'Locale::Maketext'      => '1.09',  #lib/Locale/Maketext.pm
        'Locale::Maketext::GutsLoader'=> undef,  #lib/Locale/Maketext/GutsLoader.pm
        'Locale::Maketext::Guts'=> undef,  #lib/Locale/Maketext/Guts.pm
        'Locale::Script'        => '2.07',  #lib/Locale/Script.pm
        'Math::BigFloat'        => '1.47',  #lib/Math/BigFloat.pm
        'Math::BigFloat::Trace' => '0.01',  #lib/Math/BigFloat/Trace.pm
        'Math::BigInt'          => '1.73',  #lib/Math/BigInt.pm
        'Math::BigInt::Calc'    => '0.43',  #lib/Math/BigInt/Calc.pm
        'Math::BigInt::CalcEmu' => '0.04',  #lib/Math/BigInt/CalcEmu.pm
        'Math::BigInt::Trace'   => '0.01',  #lib/Math/BigInt/Trace.pm
        'Math::BigRat'          => '0.13',  #lib/Math/BigRat.pm
        'Math::Complex'         => '1.34',  #lib/Math/Complex.pm
        'Math::Trig'            => '1.02',  #lib/Math/Trig.pm
        'Memoize'               => '1.01',  #lib/Memoize.pm
        'Memoize::AnyDBM_File'  => '0.65',  #lib/Memoize/AnyDBM_File.pm
        'Memoize::Expire'       => '1.00',  #lib/Memoize/Expire.pm
        'Memoize::ExpireFile'   => '1.01',  #lib/Memoize/ExpireFile.pm
        'Memoize::ExpireTest'   => '0.65',  #lib/Memoize/ExpireTest.pm
        'Memoize::NDBM_File'    => '0.65',  #lib/Memoize/NDBM_File.pm
        'Memoize::SDBM_File'    => '0.65',  #lib/Memoize/SDBM_File.pm
        'Memoize::Storable'     => '0.65',  #lib/Memoize/Storable.pm
        'MIME::Base64'          => '3.05',  #lib/MIME/Base64.pm
        'MIME::QuotedPrint'     => '3.03',  #lib/MIME/QuotedPrint.pm
        'NDBM_File'             => '1.05',  #lib/NDBM_File.pm
        'Net::Cmd'              => '2.26',  #lib/Net/Cmd.pm
        'Net::Config'           => '1.10',  #lib/Net/Config.pm
        'Net::Domain'           => '2.19',  #lib/Net/Domain.pm
        'Net::FTP'              => '2.75',  #lib/Net/FTP.pm
        'Net::FTP::A'           => '1.16',  #lib/Net/FTP/A.pm
        'Net::FTP::dataconn'    => '0.11',  #lib/Net/FTP/dataconn.pm
        'Net::FTP::E'           => '0.01',  #lib/Net/FTP/E.pm
        'Net::FTP::I'           => '1.12',  #lib/Net/FTP/I.pm
        'Net::FTP::L'           => '0.01',  #lib/Net/FTP/L.pm
        'Net::hostent'          => '1.01',  #lib/Net/hostent.pm
        'Net::netent'           => '1.00',  #lib/Net/netent.pm
        'Net::Netrc'            => '2.12',  #lib/Net/Netrc.pm
        'Net::NNTP'             => '2.23',  #lib/Net/NNTP.pm
        'Net::Ping'             => '2.31',  #lib/Net/Ping.pm
        'Net::POP3'             => '2.28',  #lib/Net/POP3.pm
        'Net::protoent'         => '1.00',  #lib/Net/protoent.pm
        'Net::servent'          => '1.01',  #lib/Net/servent.pm
        'Net::SMTP'             => '2.29',  #lib/Net/SMTP.pm
        'Net::Time'             => '2.10',  #lib/Net/Time.pm
        'NEXT'                  => '0.60',  #lib/NEXT.pm
        'O'                     => '1.00',  #lib/O.pm
        'ODBM_File'             => '1.05',  #ext/ODBM_File/ODBM_File.pm
        'Opcode'                => '1.05',  #lib/Opcode.pm
        'open'                  => '1.04',  #lib/open.pm
        'ops'                   => '1.00',  #lib/ops.pm
        'OS2::DLL'              => '1.02',  #os2/OS2/REXX/DLL/DLL.pm
        'OS2::ExtAttr'          => '0.02',  #os2/OS2/ExtAttr/ExtAttr.pm
        'OS2::PrfDB'            => '0.03',  #os2/OS2/PrfDB/PrfDB.pm
        'OS2::Process'          => '1.01',  #os2/OS2/Process/Process.pm
        'OS2::REXX'             => '1.02',  #os2/OS2/REXX/REXX.pm
        'overload'              => '1.02',  #lib/overload.pm
        'PerlIO'                => '1.03',  #lib/PerlIO.pm
        'PerlIO::encoding'      => '0.07',  #lib/PerlIO/encoding.pm
        'PerlIO::scalar'        => '0.02',  #lib/PerlIO/scalar.pm
        'PerlIO::via'           => '0.02',  #lib/PerlIO/via.pm
        'PerlIO::via::QuotedPrint'=> '0.06',  #lib/PerlIO/via/QuotedPrint.pm
        'Pod::Checker'          => '1.42',  #lib/Pod/Checker.pm
        'Pod::Find'             => '0.2401',  #lib/Pod/Find.pm
        'Pod::Functions'        => '1.02',  #lib/Pod/Functions.pm
        'Pod::Html'             => '1.0502',  #lib/Pod/Html.pm
        'Pod::InputObjects'     => '1.14',  #lib/Pod/InputObjects.pm
        'Pod::LaTeX'            => '0.56',  #lib/Pod/LaTeX.pm
        'Pod::Man'              => '1.37',  #lib/Pod/Man.pm
        'Pod::ParseLink'        => '1.06',  #lib/Pod/ParseLink.pm
        'Pod::Parser'           => '1.14',  #lib/Pod/Parser.pm
        'Pod::ParseUtils'       => '1.2',  #lib/Pod/ParseUtils.pm
        'Pod::Perldoc'          => '3.13',  #lib/Pod/Perldoc.pm
        'Pod::Perldoc::BaseTo'  => undef,  #lib/Pod/Perldoc/BaseTo.pm
        'Pod::Perldoc::GetOptsOO'=> undef,  #lib/Pod/Perldoc/GetOptsOO.pm
        'Pod::Perldoc::ToChecker'=> undef,  #lib/Pod/Perldoc/ToChecker.pm
        'Pod::Perldoc::ToMan'   => undef,  #lib/Pod/Perldoc/ToMan.pm
        'Pod::Perldoc::ToNroff' => undef,  #lib/Pod/Perldoc/ToNroff.pm
        'Pod::Perldoc::ToPod'   => undef,  #lib/Pod/Perldoc/ToPod.pm
        'Pod::Perldoc::ToRtf'   => undef,  #lib/Pod/Perldoc/ToRtf.pm
        'Pod::Perldoc::ToText'  => undef,  #lib/Pod/Perldoc/ToText.pm
        'Pod::Perldoc::ToTk'    => 'undef',  #lib/Pod/Perldoc/ToTk.pm
        'Pod::Perldoc::ToXml'   => undef,  #lib/Pod/Perldoc/ToXml.pm
        'Pod::Plainer'          => '0.01',  #lib/Pod/Plainer.pm
        'Pod::PlainText'        => '2.02',  #lib/Pod/PlainText.pm
        'Pod::Select'           => '1.13',  #lib/Pod/Select.pm
        'Pod::Text'             => '2.21',  #lib/Pod/Text.pm
        'Pod::Text::Color'      => '1.04',  #lib/Pod/Text/Color.pm
        'Pod::Text::Overstrike' => '1.1',  #lib/Pod/Text/Overstrike.pm
        'Pod::Text::Termcap'    => '1.11',  #lib/Pod/Text/Termcap.pm
        'Pod::Usage'            => '1.16',  #lib/Pod/Usage.pm
        'POSIX'                 => '1.08',  #lib/POSIX.pm
        're'                    => '0.04',  #lib/re.pm
        'Safe'                  => '2.11',  #lib/Safe.pm
        'Scalar::Util'          => '1.14',  #lib/Scalar/Util.pm
        'SDBM_File'             => '1.04',  #lib/SDBM_File.pm
        'Search::Dict'          => '1.02',  #lib/Search/Dict.pm
        'SelectSaver'           => '1.00',  #lib/SelectSaver.pm
        'SelfLoader'            => '1.0904',  #lib/SelfLoader.pm
        'Shell'                 => '0.6',  #lib/Shell.pm
        'sigtrap'               => '1.02',  #lib/sigtrap.pm
        'Socket'                => '1.77',  #lib/Socket.pm
        'sort'                  => '1.02',  #lib/sort.pm
        'Storable'              => '2.13',  #lib/Storable.pm
        'strict'                => '1.03',  #lib/strict.pm
        'subs'                  => '1.00',  #lib/subs.pm
        'Switch'                => '2.10',  #lib/Switch.pm
        'Symbol'                => '1.05',  #lib/Symbol.pm
        'Sys::Hostname'         => '1.11',  #lib/Sys/Hostname.pm
        'Sys::Syslog'           => '0.05',  #lib/Sys/Syslog.pm
        'Term::ANSIColor'       => '1.08',  #lib/Term/ANSIColor.pm
        'Term::Cap'             => '1.09',  #lib/Term/Cap.pm
        'Term::Complete'        => '1.401',  #lib/Term/Complete.pm
        'Term::ReadLine'        => '1.01',  #lib/Term/ReadLine.pm
        'Test'                  => '1.25',  #lib/Test.pm
        'Test::Builder'         => '0.17',  #lib/Test/Builder.pm
        'Test::Harness'         => '2.42',  #lib/Test/Harness.pm
        'Test::Harness::Assert' => '0.02',  #lib/Test/Harness/Assert.pm
        'Test::Harness::Iterator'=> '0.02',  #lib/Test/Harness/Iterator.pm
        'Test::Harness::Straps' => '0.19',  #lib/Test/Harness/Straps.pm
        'Test::More'            => '0.47',  #lib/Test/More.pm
        'Test::Simple'          => '0.47',  #lib/Test/Simple.pm
        'Text::Abbrev'          => '1.01',  #lib/Text/Abbrev.pm
        'Text::Balanced'        => '1.95',  #lib/Text/Balanced.pm
        'Text::ParseWords'      => '3.23',  #lib/Text/ParseWords.pm
        'Text::Soundex'         => '1.01',  #lib/Text/Soundex.pm
        'Text::Tabs'            => '98.112801',  #lib/Text/Tabs.pm
        'Text::Wrap'            => '2001.09292',  #lib/Text/Wrap.pm
        'Thread'                => '2.00',  #lib/Thread.pm
        'Thread::Queue'         => '2.00',  #lib/Thread/Queue.pm
        'threads'               => '1.05',  #lib/threads.pm
        'Thread::Semaphore'     => '2.01',  #lib/Thread/Semaphore.pm
        'Thread::Signal'        => '1.00',  #ext/Thread/Thread/Signal.pm
        'Thread::Specific'      => '1.00',  #ext/Thread/Thread/Specific.pm
        'threads::shared'       => '0.92',  #lib/threads/shared.pm
        'Tie::Array'            => '1.03',  #lib/Tie/Array.pm
        'Tie::File'             => '0.97',  #lib/Tie/File.pm
        'Tie::Handle'           => '4.1',  #lib/Tie/Handle.pm
        'Tie::Hash'             => '1.01',  #lib/Tie/Hash.pm
        'Tie::Memoize'          => '1.0',  #lib/Tie/Memoize.pm
        'Tie::RefHash'          => '1.31',  #lib/Tie/RefHash.pm
        'Tie::Scalar'           => '1.00',  #lib/Tie/Scalar.pm
        'Tie::SubstrHash'       => '1.00',  #lib/Tie/SubstrHash.pm
        'Time::gmtime'          => '1.02',  #lib/Time/gmtime.pm
        'Time::HiRes'           => '1.65',  #lib/Time/HiRes.pm
        'Time::Local'           => '1.10',  #lib/Time/Local.pm
        'Time::localtime'       => '1.02',  #lib/Time/localtime.pm
        'Time::tm'              => '1.00',  #lib/Time/tm.pm
        'Unicode::Collate'      => '0.40',  #lib/Unicode/Collate.pm
        'Unicode::Normalize'    => '0.30',  #lib/Unicode/Normalize.pm
        'Unicode::UCD'          => '0.22',  #lib/Unicode/UCD.pm
        'UNIVERSAL'             => '1.01',  #lib/UNIVERSAL.pm
        'User::grent'           => '1.00',  #lib/User/grent.pm
        'User::pwent'           => '1.00',  #lib/User/pwent.pm
        'utf8'                  => '1.04',  #lib/utf8.pm
        'vars'                  => '1.01',  #lib/vars.pm
        'VMS::DCLsym'           => '1.02',  #vms/ext/DCLsym/DCLsym.pm
        'VMS::Filespec'         => '1.11',  #vms/ext/Filespec.pm
        'vmsish'                => '1.01',  #lib/vmsish.pm
        'VMS::Stdio'            => '2.3',  #vms/ext/Stdio/Stdio.pm
        'warnings'              => '1.03',  #lib/warnings.pm
        'warnings::register'    => '1.00',  #lib/warnings/register.pm
        'XS::APItest'           => '0.05',  #lib/XS/APItest.pm
        'XSLoader'              => '0.02',  #lib/XSLoader.pm
        'XS::Typemap'           => '0.01',  #lib/XS/Typemap.pm
    },
    5.009002 => {
	'AnyDBM_File'           => '1.00',
	'Attribute::Handlers'   => '0.78_01',
	'AutoLoader'            => '5.60',
	'AutoSplit'             => '1.04',
	'B'                     => '1.07',
	'B::Asmdata'            => '1.01',
	'B::Assembler'          => '0.07',
	'B::Bblock'             => '1.02',
	'B::Bytecode'           => '1.01',
	'B::C'                  => '1.04',
	'B::CC'                 => '1.00',
	'B::Concise'            => '0.64',
	'B::Debug'              => '1.02',
	'B::Deparse'            => '0.69',
	'B::Disassembler'       => '1.03',
	'B::Lint'               => '1.03',
	'B::O'                  => '1.00',
	'B::Showlex'            => '1.02',
	'B::Stackobj'           => '1.00',
	'B::Stash'              => '1.00',
	'B::Terse'              => '1.02',
	'B::Xref'               => '1.01',
	'Benchmark'             => '1.07',
	'ByteLoader'            => '0.05',
	'CGI'                   => '3.07',
	'CGI::Apache'           => '1.00',
	'CGI::Carp'             => '1.29',
	'CGI::Cookie'           => '1.25',
	'CGI::Fast'             => '1.05',
	'CGI::Pretty'           => '1.08',
	'CGI::Push'             => '1.04',
	'CGI::Switch'           => '1.00',
	'CGI::Util'             => '1.5',
	'CPAN'                  => '1.76_01',
	'CPAN::FirstTime'       => '1.60 ',
	'CPAN::Nox'             => '1.03',
	'Carp'                  => '1.04',
	'Carp::Heavy'           => '1.04',
	'Class::ISA'            => '0.33',
	'Class::Struct'         => '0.63',
	'Config::Extensions'    => '0.01',
	'Cwd'                   => '3.05',
	'DB'                    => '1.0',
	'DBM_Filter'            => '0.01',
	'DBM_Filter::compress'  => '0.01',
	'DBM_Filter::encode'    => '0.01',
	'DBM_Filter::int32'     => '0.01',
	'DBM_Filter::null'      => '0.01',
	'DBM_Filter::utf8'      => '0.01',
	'DB_File'               => '1.811',
	'Data::Dumper'          => '2.121_04',
	'Devel::DProf'          => '20030813.00',
	'Devel::PPPort'         => '3.06',
	'Devel::Peek'           => '1.02',
	'Devel::SelfStubber'    => '1.03',
	'Digest'                => '1.10',
	'Digest::MD5'           => '2.33',
	'Digest::base'          => '1.00',
	'Digest::file'          => '0.01',
	'DirHandle'             => '1.00',
	'Dumpvalue'             => '1.11',
	'DynaLoader'            => '1.07',
	'Encode'                => '2.09',
	'Encode::Alias'         => '2.02',
	'Encode::Byte'          => '2.00',
	'Encode::CJKConstants'  => '2.00',
	'Encode::CN'            => '2.00',
	'Encode::CN::HZ'        => '2.01',
	'Encode::Config'        => '2.00',
	'Encode::EBCDIC'        => '2.00',
	'Encode::Encoder'       => '2.00',
	'Encode::Encoding'      => '2.02',
	'Encode::Guess'         => '2.00',
	'Encode::JP'            => '2.01',
	'Encode::JP::H2Z'       => '2.00',
	'Encode::JP::JIS7'      => '2.00',
	'Encode::KR'            => '2.00',
	'Encode::KR::2022_KR'   => '2.00',
	'Encode::MIME::Header'  => '2.00',
	'Encode::Symbol'        => '2.00',
	'Encode::TW'            => '2.00',
	'Encode::Unicode'       => '2.02',
	'Encode::Unicode::UTF7' => '2.01',
	'English'               => '1.03',
	'Env'                   => '1.00',
	'Errno'                 => '1.09_01',
	'Exporter'              => '5.59',
	'Exporter::Heavy'       => '5.59',
	'ExtUtils::Command'     => '1.07',
	'ExtUtils::Command::MM' => '0.03_01',
	'ExtUtils::Constant'    => '0.16',
	'ExtUtils::Constant::Base'=> '0.01',
	'ExtUtils::Constant::Utils'=> '0.01',
	'ExtUtils::Constant::XS'=> '0.01',
	'ExtUtils::Embed'       => '1.26',
	'ExtUtils::Install'     => '1.32',
	'ExtUtils::Installed'   => '0.08',
	'ExtUtils::Liblist'     => '1.01',
	'ExtUtils::Liblist::Kid'=> '1.3',
	'ExtUtils::MM'          => '0.04',
	'ExtUtils::MM_Any'      => '0.10',
	'ExtUtils::MM_BeOS'     => '1.04',
	'ExtUtils::MM_Cygwin'   => '1.07',
	'ExtUtils::MM_DOS'      => '0.02',
	'ExtUtils::MM_MacOS'    => '1.08',
	'ExtUtils::MM_NW5'      => '2.07',
	'ExtUtils::MM_OS2'      => '1.04',
	'ExtUtils::MM_UWIN'     => '0.02',
	'ExtUtils::MM_Unix'     => '1.46_01',
	'ExtUtils::MM_VMS'      => '5.71',
	'ExtUtils::MM_Win32'    => '1.10',
	'ExtUtils::MM_Win95'    => '0.03',
	'ExtUtils::MY'          => '0.01',
	'ExtUtils::MakeMaker'   => '6.25',
	'ExtUtils::MakeMaker::bytes'=> '0.01',
	'ExtUtils::MakeMaker::vmsish'=> '0.01',
	'ExtUtils::Manifest'    => '1.44',
	'ExtUtils::Mkbootstrap' => '1.15',
	'ExtUtils::Mksymlists'  => '1.19',
	'ExtUtils::Packlist'    => '0.04',
	'ExtUtils::testlib'     => '1.15',
	'Fatal'                 => '1.04',
	'Fcntl'                 => '1.05',
	'File::Basename'        => '2.73',
	'File::CheckTree'       => '4.3',
	'File::Compare'         => '1.1003',
	'File::Copy'            => '2.08',
	'File::DosGlob'         => '1.00',
	'File::Find'            => '1.09',
	'File::Glob'            => '1.04',
	'File::Path'            => '1.06',
	'File::Spec'            => '3.05',
	'File::Spec::Cygwin'    => '1.1',
	'File::Spec::Epoc'      => '1.1',
	'File::Spec::Functions' => '1.3',
	'File::Spec::Mac'       => '1.4',
	'File::Spec::OS2'       => '1.2',
	'File::Spec::Unix'      => '1.5',
	'File::Spec::VMS'       => '1.4',
	'File::Spec::Win32'     => '1.5',
	'File::Temp'            => '0.16',
	'File::stat'            => '1.00',
	'FileCache'             => '1.04_01',
	'FileHandle'            => '2.01',
	'Filter::Simple'        => '0.78',
	'Filter::Util::Call'    => '1.0601',
	'FindBin'               => '1.44',
	'GDBM_File'             => '1.07',
	'Getopt::Long'          => '2.3401',
	'Getopt::Std'           => '1.05',
	'Hash::Util'            => '0.05',
	'I18N::Collate'         => '1.00',
	'I18N::LangTags'        => '0.35',
	'I18N::LangTags::Detect'=> '1.03',
	'I18N::LangTags::List'  => '0.35',
	'I18N::Langinfo'        => '0.02',
	'IO'                    => '1.21',
	'IO::Dir'               => '1.04',
	'IO::File'              => '1.10',
	'IO::Handle'            => '1.24',
	'IO::Pipe'              => '1.123',
	'IO::Poll'              => '0.06',
	'IO::Seekable'          => '1.09',
	'IO::Select'            => '1.16',
	'IO::Socket'            => '1.28',
	'IO::Socket::INET'      => '1.27',
	'IO::Socket::UNIX'      => '1.21',
	'IPC::Open2'            => '1.01',
	'IPC::Open3'            => '1.0106',
	'IPC::SysV'             => '1.04',
	'IPC::SysV::Msg'        => '1.02',
	'IPC::SysV::Semaphore'  => '1.02',
	'List::Util'            => '1.14',
	'Locale::Constants'     => '2.07',
	'Locale::Country'       => '2.07',
	'Locale::Currency'      => '2.07',
	'Locale::Language'      => '2.07',
	'Locale::Maketext'      => '1.09',
	'Locale::Maketext::Guts'=> undef,
	'Locale::Maketext::GutsLoader'=> undef,
	'Locale::Script'        => '2.07',
	'MIME::Base64'          => '3.05',
	'MIME::QuotedPrint'     => '3.03',
	'Math::BigFloat'        => '1.49',
	'Math::BigFloat::Trace' => '0.01',
	'Math::BigInt'          => '1.75',
	'Math::BigInt::Calc'    => '0.45',
	'Math::BigInt::CalcEmu' => '0.05',
	'Math::BigInt::Trace'   => '0.01',
	'Math::BigRat'          => '0.14',
	'Math::Complex'         => '1.34',
	'Math::Trig'            => '1.02',
	'Memoize'               => '1.01_01',
	'Memoize::AnyDBM_File'  => '0.65',
	'Memoize::Expire'       => '1.00',
	'Memoize::ExpireFile'   => '1.01',
	'Memoize::ExpireTest'   => '0.65',
	'Memoize::NDBM_File'    => '0.65',
	'Memoize::SDBM_File'    => '0.65',
	'Memoize::Storable'     => '0.65',
	'Module::CoreList'      => '1.99',
	'NDBM_File'             => '1.05',
	'NEXT'                  => '0.60_01',
	'Net::Cmd'              => '2.26',
	'Net::Config'           => '1.10',
	'Net::Domain'           => '2.19',
	'Net::FTP'              => '2.75',
	'Net::FTP::A'           => '1.16',
	'Net::FTP::E'           => '0.01',
	'Net::FTP::I'           => '1.12',
	'Net::FTP::L'           => '0.01',
	'Net::FTP::dataconn'    => '0.11',
	'Net::NNTP'             => '2.23',
	'Net::Netrc'            => '2.12',
	'Net::POP3'             => '2.28',
	'Net::Ping'             => '2.31',
	'Net::SMTP'             => '2.29',
	'Net::Time'             => '2.10',
	'Net::hostent'          => '1.01',
	'Net::netent'           => '1.00',
	'Net::protoent'         => '1.00',
	'Net::servent'          => '1.01',
	'ODBM_File'             => '1.05',
	'Opcode'                => '1.06',
	'Opcode::Safe'          => '2.11',
	'Opcode::ops'           => '1.00',
	'POSIX'                 => '1.08',
	'PerlIO'                => '1.03',
	'PerlIO::encoding'      => '0.07',
	'PerlIO::scalar'        => '0.02',
	'PerlIO::via'           => '0.02',
	'PerlIO::via::QuotedPrint'=> '0.06',
	'Pod::Checker'          => '1.42',
	'Pod::Find'             => '1.3',
	'Pod::Functions'        => '1.02',
	'Pod::Html'             => '1.0502',
	'Pod::InputObjects'     => '1.3',
	'Pod::LaTeX'            => '0.58',
	'Pod::Man'              => '1.37',
	'Pod::ParseLink'        => '1.06',
	'Pod::ParseUtils'       => '1.3',
	'Pod::Parser'           => '1.3',
	'Pod::Perldoc'          => '3.14',
	'Pod::Perldoc::BaseTo'  => undef,
	'Pod::Perldoc::GetOptsOO'=> undef,
	'Pod::Perldoc::ToChecker'=> undef,
	'Pod::Perldoc::ToMan'   => undef,
	'Pod::Perldoc::ToNroff' => undef,
	'Pod::Perldoc::ToPod'   => undef,
	'Pod::Perldoc::ToRtf'   => undef,
	'Pod::Perldoc::ToText'  => undef,
	'Pod::Perldoc::ToTk'    => undef,
	'Pod::Perldoc::ToXml'   => undef,
	'Pod::PlainText'        => '2.02',
	'Pod::Plainer'          => '0.01',
	'Pod::Select'           => '1.3',
	'Pod::Text'             => '2.21',
	'Pod::Text::Color'      => '1.04',
	'Pod::Text::Overstrike' => '1.1',
	'Pod::Text::Termcap'    => '1.11',
	'Pod::Usage'            => '1.3',
	'SDBM_File'             => '1.04',
	'Scalar::Util'          => '1.14_1',
	'Search::Dict'          => '1.02',
	'SelectSaver'           => '1.01',
	'SelfLoader'            => '1.0904',
	'Shell'                 => '0.6',
	'Socket'                => '1.77',
	'Storable'              => '2.14',
	'Switch'                => '2.10',
	'Symbol'                => '1.05',
	'Sys::Hostname'         => '1.11',
	'Sys::Syslog'           => '0.06',
	'Term::ANSIColor'       => '1.09',
	'Term::Cap'             => '1.09',
	'Term::Complete'        => '1.402',
	'Term::ReadLine'        => '1.01',
	'Test'                  => '1.25',
	'Test::Builder'         => '0.22',
	'Test::Harness'         => '2.46',
	'Test::Harness::Assert' => '0.02',
	'Test::Harness::Iterator'=> '0.02',
	'Test::Harness::Straps' => '0.20_01',
	'Test::More'            => '0.54',
	'Test::Simple'          => '0.54',
	'Text::Abbrev'          => '1.01',
	'Text::Balanced'        => '1.95_01',
	'Text::ParseWords'      => '3.24',
	'Text::Soundex'         => '1.01',
	'Text::Tabs'            => '98.112801',
	'Text::Wrap'            => '2001.09292',
	'Thread'                => '2.00',
	'Thread::Queue'         => '2.00',
	'Thread::Semaphore'     => '2.01',
	'Thread::Signal'        => '1.00',
	'Thread::Specific'      => '1.00',
	'Tie::Array'            => '1.03',
	'Tie::File'             => '0.97',
	'Tie::Handle'           => '4.1',
	'Tie::Hash'             => '1.01',
	'Tie::Memoize'          => '1.0',
	'Tie::RefHash'          => '1.32',
	'Tie::Scalar'           => '1.00',
	'Tie::SubstrHash'       => '1.00',
	'Time::HiRes'           => '1.66',
	'Time::Local'           => '1.11',
	'Time::gmtime'          => '1.02',
	'Time::localtime'       => '1.02',
	'Time::tm'              => '1.00',
	'UNIVERSAL'             => '1.02',
	'Unicode::Collate'      => '0.40',
	'Unicode::Normalize'    => '0.30',
	'Unicode::UCD'          => '0.22',
	'User::grent'           => '1.00',
	'User::pwent'           => '1.00',
	'XS::APItest'           => '0.05',
	'XS::Typemap'           => '0.01',
	'XSLoader'              => '0.03',
	'assertions'            => '0.01',
	'assertions::activate'  => '0.01',
	'attributes'            => '0.06',
	'attrs'                 => '1.01',
	'autouse'               => '1.04',
	'base'                  => '2.06',
	'bigint'                => '0.06',
	'bignum'                => '0.16',
	'bigrat'                => '0.07',
	'blib'                  => '1.02',
	'bytes'                 => '1.01',
	'charnames'             => '1.04',
	'constant'              => '1.05',
	'diagnostics'           => '1.14',
	'encoding'              => '2.01',
	'encoding::warnings'    => '0.05',
	'fields'                => '2.03',
	'filetest'              => '1.01',
	'if'                    => '0.0401',
	'integer'               => '1.00',
	'less'                  => '0.01',
	'lib'                   => '0.5565',
	'locale'                => '1.00',
	'open'                  => '1.04',
	'overload'              => '1.03',
	're'                    => '0.05',
	'sigtrap'               => '1.02',
	'sort'                  => '1.02',
	'strict'                => '1.03',
	'subs'                  => '1.00',
	'threads'               => '1.05',
	'threads::shared'       => '0.92',
	'utf8'                  => '1.04',
	'vars'                  => '1.01',
	'version'               => '0.42',
	'vmsish'                => '1.01',
	'warnings'              => '1.04',
	'warnings::register'    => '1.00',
    },
    5.008007 => {
	'AnyDBM_File'           => '1.00',
	'Attribute::Handlers'   => '0.78_01',
	'AutoLoader'            => '5.60',
	'AutoSplit'             => '1.04',
	'B'                     => '1.09',
	'B::Asmdata'            => '1.01',
	'B::Assembler'          => '0.07',
	'B::Bblock'             => '1.02',
	'B::Bytecode'           => '1.01',
	'B::C'                  => '1.04',
	'B::CC'                 => '1.00',
	'B::Concise'            => '0.65',
	'B::Debug'              => '1.02',
	'B::Deparse'            => '0.7',
	'B::Disassembler'       => '1.04',
	'B::Lint'               => '1.03',
	'B::O'                  => '1.00',
	'B::Showlex'            => '1.02',
	'B::Stackobj'           => '1.00',
	'B::Stash'              => '1.00',
	'B::Terse'              => '1.03',
	'B::Xref'               => '1.01',
	'Benchmark'             => '1.07',
	'ByteLoader'            => '0.05',
	'CGI'                   => '3.10',
	'CGI::Apache'           => '1.00',
	'CGI::Carp'             => '1.29',
	'CGI::Cookie'           => '1.25',
	'CGI::Fast'             => '1.05',
	'CGI::Pretty'           => '1.08',
	'CGI::Push'             => '1.04',
	'CGI::Switch'           => '1.00',
	'CGI::Util'             => '1.5',
	'CPAN'                  => '1.76_01',
	'CPAN::FirstTime'       => '1.60 ',
	'CPAN::Nox'             => '1.03',
	'Carp'                  => '1.04',
	'Carp::Heavy'           => '1.04',
	'Class::ISA'            => '0.33',
	'Class::Struct'         => '0.63',
	'Cwd'                   => '3.05',
	'DB'                    => '1.0',
	'DBM_Filter'            => '0.01',
	'DBM_Filter::compress'  => '0.01',
	'DBM_Filter::encode'    => '0.01',
	'DBM_Filter::int32'     => '0.01',
	'DBM_Filter::null'      => '0.01',
	'DBM_Filter::utf8'      => '0.01',
	'DB_File'               => '1.811',
	'Data::Dumper'          => '2.121_04',
	'Devel::DProf'          => '20050310.00',
	'Devel::PPPort'         => '3.06',
	'Devel::Peek'           => '1.02',
	'Devel::SelfStubber'    => '1.03',
	'Digest'                => '1.10',
	'Digest::MD5'           => '2.33',
	'Digest::base'          => '1.00',
	'Digest::file'          => '0.01',
	'DirHandle'             => '1.00',
	'Dumpvalue'             => '1.11',
	'DynaLoader'            => '1.05',
	'Encode'                => '2.10',
	'Encode::Alias'         => '2.03',
	'Encode::Byte'          => '2.00',
	'Encode::CJKConstants'  => '2.00',
	'Encode::CN'            => '2.00',
	'Encode::CN::HZ'        => '2.01',
	'Encode::Config'        => '2.00',
	'Encode::EBCDIC'        => '2.00',
	'Encode::Encoder'       => '2.00',
	'Encode::Encoding'      => '2.02',
	'Encode::Guess'         => '2.00',
	'Encode::JP'            => '2.01',
	'Encode::JP::H2Z'       => '2.00',
	'Encode::JP::JIS7'      => '2.00',
	'Encode::KR'            => '2.00',
	'Encode::KR::2022_KR'   => '2.00',
	'Encode::MIME::Header'  => '2.00',
	'Encode::Symbol'        => '2.00',
	'Encode::TW'            => '2.00',
	'Encode::Unicode'       => '2.02',
	'Encode::Unicode::UTF7' => '2.01',
	'English'               => '1.01',
	'Env'                   => '1.00',
	'Errno'                 => '1.09_01',
	'Exporter'              => '5.58',
	'Exporter::Heavy'       => '5.59',
	'ExtUtils::Command'     => '1.05',
	'ExtUtils::Command::MM' => '0.03',
	'ExtUtils::Constant'    => '0.16',
	'ExtUtils::Constant::Base'=> '0.01',
	'ExtUtils::Constant::Utils'=> '0.01',
	'ExtUtils::Constant::XS'=> '0.01',
	'ExtUtils::Embed'       => '1.250601',
	'ExtUtils::Install'     => '1.32',
	'ExtUtils::Installed'   => '0.08',
	'ExtUtils::Liblist'     => '1.01',
	'ExtUtils::Liblist::Kid'=> '1.3001',
	'ExtUtils::MM'          => '0.04',
	'ExtUtils::MM_Any'      => '0.07',
	'ExtUtils::MM_BeOS'     => '1.04',
	'ExtUtils::MM_Cygwin'   => '1.06',
	'ExtUtils::MM_DOS'      => '0.02',
	'ExtUtils::MM_MacOS'    => '1.07',
	'ExtUtils::MM_NW5'      => '2.07_02',
	'ExtUtils::MM_OS2'      => '1.04',
	'ExtUtils::MM_UWIN'     => '0.02',
	'ExtUtils::MM_Unix'     => '1.42',
	'ExtUtils::MM_VMS'      => '5.70',
	'ExtUtils::MM_Win32'    => '1.09',
	'ExtUtils::MM_Win95'    => '0.0301',
	'ExtUtils::MY'          => '0.01',
	'ExtUtils::MakeMaker'   => '6.17',
	'ExtUtils::MakeMaker::bytes'=> '0.01',
	'ExtUtils::MakeMaker::vmsish'=> '0.01',
	'ExtUtils::Manifest'    => '1.42',
	'ExtUtils::Mkbootstrap' => '1.15',
	'ExtUtils::Mksymlists'  => '1.19',
	'ExtUtils::Packlist'    => '0.04',
	'ExtUtils::testlib'     => '1.15',
	'Fatal'                 => '1.03',
	'Fcntl'                 => '1.05',
	'File::Basename'        => '2.73',
	'File::CheckTree'       => '4.3',
	'File::Compare'         => '1.1003',
	'File::Copy'            => '2.08',
	'File::DosGlob'         => '1.00',
	'File::Find'            => '1.09',
	'File::Glob'            => '1.04',
	'File::Path'            => '1.07',
	'File::Spec'            => '3.05',
	'File::Spec::Cygwin'    => '1.1',
	'File::Spec::Epoc'      => '1.1',
	'File::Spec::Functions' => '1.3',
	'File::Spec::Mac'       => '1.4',
	'File::Spec::OS2'       => '1.2',
	'File::Spec::Unix'      => '1.5',
	'File::Spec::VMS'       => '1.4',
	'File::Spec::Win32'     => '1.5',
	'File::Temp'            => '0.16',
	'File::stat'            => '1.00',
	'FileCache'             => '1.05',
	'FileHandle'            => '2.01',
	'Filter::Simple'        => '0.78',
	'Filter::Util::Call'    => '1.0601',
	'FindBin'               => '1.44',
	'GDBM_File'             => '1.07',
	'Getopt::Long'          => '2.34',
	'Getopt::Std'           => '1.05',
	'Hash::Util'            => '0.05',
	'I18N::Collate'         => '1.00',
	'I18N::LangTags'        => '0.35',
	'I18N::LangTags::Detect'=> '1.03',
	'I18N::LangTags::List'  => '0.35',
	'I18N::Langinfo'        => '0.02',
	'IO'                    => '1.21',
	'IO::Dir'               => '1.04',
	'IO::File'              => '1.11',
	'IO::Handle'            => '1.24',
	'IO::Pipe'              => '1.123',
	'IO::Poll'              => '0.06',
	'IO::Seekable'          => '1.09',
	'IO::Select'            => '1.16',
	'IO::Socket'            => '1.28',
	'IO::Socket::INET'      => '1.28',
	'IO::Socket::UNIX'      => '1.21',
	'IPC::Open2'            => '1.01',
	'IPC::Open3'            => '1.0106',
	'IPC::SysV'             => '1.04',
	'IPC::SysV::Msg'        => '1.02',
	'IPC::SysV::Semaphore'  => '1.02',
	'List::Util'            => '1.14',
	'Locale::Constants'     => '2.07',
	'Locale::Country'       => '2.07',
	'Locale::Currency'      => '2.07',
	'Locale::Language'      => '2.07',
	'Locale::Maketext'      => '1.09',
	'Locale::Maketext::Guts'=> undef,
	'Locale::Maketext::GutsLoader'=> undef,
	'Locale::Script'        => '2.07',
	'MIME::Base64'          => '3.05',
	'MIME::QuotedPrint'     => '3.03',
	'Math::BigFloat'        => '1.51',
	'Math::BigFloat::Trace' => '0.01',
	'Math::BigInt'          => '1.77',
	'Math::BigInt::Calc'    => '0.47',
	'Math::BigInt::CalcEmu' => '0.05',
	'Math::BigInt::Trace'   => '0.01',
	'Math::BigRat'          => '0.15',
	'Math::Complex'         => '1.34',
	'Math::Trig'            => '1.02',
	'Memoize'               => '1.01',
	'Memoize::AnyDBM_File'  => '0.65',
	'Memoize::Expire'       => '1.00',
	'Memoize::ExpireFile'   => '1.01',
	'Memoize::ExpireTest'   => '0.65',
	'Memoize::NDBM_File'    => '0.65',
	'Memoize::SDBM_File'    => '0.65',
	'Memoize::Storable'     => '0.65',
	'NDBM_File'             => '1.05',
	'NEXT'                  => '0.60',
	'Net::Cmd'              => '2.26',
	'Net::Config'           => '1.10',
	'Net::Domain'           => '2.19',
	'Net::FTP'              => '2.75',
	'Net::FTP::A'           => '1.16',
	'Net::FTP::E'           => '0.01',
	'Net::FTP::I'           => '1.12',
	'Net::FTP::L'           => '0.01',
	'Net::FTP::dataconn'    => '0.11',
	'Net::NNTP'             => '2.23',
	'Net::Netrc'            => '2.12',
	'Net::POP3'             => '2.28',
	'Net::Ping'             => '2.31',
	'Net::SMTP'             => '2.29',
	'Net::Time'             => '2.10',
	'Net::hostent'          => '1.01',
	'Net::netent'           => '1.00',
	'Net::protoent'         => '1.00',
	'Net::servent'          => '1.01',
	'ODBM_File'             => '1.05',
	'Opcode'                => '1.05',
	'Opcode::Safe'          => '2.11',
	'Opcode::ops'           => '1.00',
	'POSIX'                 => '1.08',
	'PerlIO'                => '1.03',
	'PerlIO::encoding'      => '0.07',
	'PerlIO::scalar'        => '0.02',
	'PerlIO::via'           => '0.02',
	'PerlIO::via::QuotedPrint'=> '0.06',
	'Pod::Checker'          => '1.42',
	'Pod::Find'             => '1.3',
	'Pod::Functions'        => '1.02',
	'Pod::Html'             => '1.0503',
	'Pod::InputObjects'     => '1.3',
	'Pod::LaTeX'            => '0.58',
	'Pod::Man'              => '1.37',
	'Pod::ParseLink'        => '1.06',
	'Pod::ParseUtils'       => '1.3',
	'Pod::Parser'           => '1.3',
	'Pod::Perldoc'          => '3.14',
	'Pod::Perldoc::BaseTo'  => undef,
	'Pod::Perldoc::GetOptsOO'=> undef,
	'Pod::Perldoc::ToChecker'=> undef,
	'Pod::Perldoc::ToMan'   => undef,
	'Pod::Perldoc::ToNroff' => undef,
	'Pod::Perldoc::ToPod'   => undef,
	'Pod::Perldoc::ToRtf'   => undef,
	'Pod::Perldoc::ToText'  => undef,
	'Pod::Perldoc::ToTk'    => undef,
	'Pod::Perldoc::ToXml'   => undef,
	'Pod::PlainText'        => '2.02',
	'Pod::Plainer'          => '0.01',
	'Pod::Select'           => '1.3',
	'Pod::Text'             => '2.21',
	'Pod::Text::Color'      => '1.04',
	'Pod::Text::Overstrike' => '1.1',
	'Pod::Text::Termcap'    => '1.11',
	'Pod::Usage'            => '1.3',
	'SDBM_File'             => '1.04',
	'Scalar::Util'          => '1.14',
	'Search::Dict'          => '1.02',
	'SelectSaver'           => '1.01',
	'SelfLoader'            => '1.0904',
	'Shell'                 => '0.6',
	'Socket'                => '1.77',
	'Storable'              => '2.13',
	'Switch'                => '2.10',
	'Symbol'                => '1.06',
	'Sys::Hostname'         => '1.11',
	'Sys::Syslog'           => '0.06',
	'Term::ANSIColor'       => '1.09',
	'Term::Cap'             => '1.09',
	'Term::Complete'        => '1.402',
	'Term::ReadLine'        => '1.01',
	'Test'                  => '1.25',
	'Test::Builder'         => '0.22',
	'Test::Harness'         => '2.48',
	'Test::Harness::Assert' => '0.02',
	'Test::Harness::Iterator'=> '0.02',
	'Test::Harness::Point'  => '0.01',
	'Test::Harness::Straps' => '0.23',
	'Test::More'            => '0.54',
	'Test::Simple'          => '0.54',
	'Text::Abbrev'          => '1.01',
	'Text::Balanced'        => '1.95',
	'Text::ParseWords'      => '3.24',
	'Text::Soundex'         => '1.01',
	'Text::Tabs'            => '98.112801',
	'Text::Wrap'            => '2001.09293',
	'Thread'                => '2.00',
	'Thread::Queue'         => '2.00',
	'Thread::Semaphore'     => '2.01',
	'Thread::Signal'        => '1.00',
	'Thread::Specific'      => '1.00',
	'Tie::Array'            => '1.03',
	'Tie::File'             => '0.97',
	'Tie::Handle'           => '4.1',
	'Tie::Hash'             => '1.01',
	'Tie::Memoize'          => '1.0',
	'Tie::RefHash'          => '1.32',
	'Tie::Scalar'           => '1.00',
	'Tie::SubstrHash'       => '1.00',
	'Time::HiRes'           => '1.66',
	'Time::Local'           => '1.11',
	'Time::gmtime'          => '1.02',
	'Time::localtime'       => '1.02',
	'Time::tm'              => '1.00',
	'UNIVERSAL'             => '1.01',
	'Unicode::Collate'      => '0.40',
	'Unicode::Normalize'    => '0.32',
	'Unicode::UCD'          => '0.23',
	'User::grent'           => '1.00',
	'User::pwent'           => '1.00',
	'XS::APItest'           => '0.06',
	'XS::Typemap'           => '0.01',
	'XSLoader'              => '0.02',
	'attributes'            => '0.06',
	'attrs'                 => '1.01',
	'autouse'               => '1.04',
	'base'                  => '2.07',
	'bigint'                => '0.07',
	'bignum'                => '0.17',
	'bigrat'                => '0.08',
	'blib'                  => '1.02',
	'bytes'                 => '1.02',
	'charnames'             => '1.04',
	'constant'              => '1.05',
	'diagnostics'           => '1.14',
	'encoding'              => '2.01',
	'fields'                => '2.03',
	'filetest'              => '1.01',
	'if'                    => '0.03',
	'integer'               => '1.00',
	'less'                  => '0.01',
	'lib'                   => '0.5565',
	'locale'                => '1.00',
	'open'                  => '1.04',
	'overload'              => '1.03',
	're'                    => '0.04',
	'sigtrap'               => '1.02',
	'sort'                  => '1.02',
	'strict'                => '1.03',
	'subs'                  => '1.00',
	'threads'               => '1.05',
	'threads::shared'       => '0.93',
	'utf8'                  => '1.05',
	'vars'                  => '1.01',
	'vmsish'                => '1.01',
	'warnings'              => '1.03',
	'warnings::register'    => '1.00',
    },
);

1;
__END__
